# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates a plug-in manager for the
text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE

class ScribesPluginManager(GObject):
	"""
	This class creates an object that manages plug-ins for the text
	editor. The core function of the object is to load and unload
	plug-ins.
	"""

	__gsignals__ = {
		"processed": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, editor):
		"""
		Initialize the plug-in manager.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		GObject.__init__(self)
		self.__init_attributes(editor)
		self.signal_id_1 = self.connect("processed", self.__manager_processed_cb)
		self.signal_id_2 = self.__editor.connect("loading-plugins", self.__manager_loading_plugins_cb)
		self.signal_id_3 = self.__editor.connect("loaded-plugins", self.__manager_loaded_plugins_cb)
		try:
			# Check for plug-in folders.
			if self.__plugin_folders_exist() is False:
				raise Exception
			# Check for plug-in modules.
			if self.__plugin_modules_exist() is False:
				raise Exception
			self.__set_plugin_search_path()
			# Load plug-in modules.
			self.__load_plugins()
			# Monitor plug-in folders for changes.
			from info import home_plugin_folder, core_plugin_folder
			from gnomevfs import monitor_add, MONITOR_DIRECTORY
			from gnomevfs import get_uri_from_local_path
			home_plugin_folder_uri = get_uri_from_local_path(home_plugin_folder)
			core_plugin_folder_uri = get_uri_from_local_path(core_plugin_folder)
			self.__monitor_id = monitor_add(home_plugin_folder_uri, MONITOR_DIRECTORY,
					self.__plugin_folder_changed_cb)
		except:
			self.disconnect(self.signal_id_1)
			self.__editor.disconnect(self.signal_id_2)
			self.__editor.disconnect(self.signal_id_3)
			print "Plug-ins were not loaded"

	def __init_attributes(self, editor):
		"""
		Initialize the object's data attributes.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		# Reference to the editor.
		self.__editor = editor
		# List of active plug-ins.
		self.__active_plugins = []
		# List of inactive plug-ins.
		self.__inactive_plugins = []
		# Modules with loaded plug-in.
		self.__active_modules = []
		# Modules with unloaded plug-ins.
		self.__inactive_modules = []
		# Modules with plug-ins that failed to load.
		self.__failed_modules = []
		# Third party modules found (~/.gnome2/scribes/plugins).
		self.__found_home_modules = []
		# Core modules found (${prefix}/share/scribes/plugins).
		self.__found_core_modules = []
		# Total number of modules.
		self.__number_of_modules = 0
		# Number of modules processed.
		self.__processed_modules = 0
		# Status ID.
		self.__status_id = None
		self.__monitor_id = None
		return

	def __plugin_folders_exist(self):
		"""
		Check if plug-in folders exist.

		There are two plug-in folders: ~/.gnome2/scribes/plugins and
		${prefix}/share/scribes/plugins. Each plug-in folders should
		contain __init__.py. If __init__.py is not found, it is created
		when possible. If __init__.py cannot be created, plug-ins
		will not be loaded.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@return: True if plug-in folders are found.
		@rtype: A Boolean object.
		"""
		from info import home_plugin_folder, core_plugin_folder
		from os import path
		try:
			filename = home_plugin_folder + "/__init__.py"
			if not path.exists(filename):
				from os import makedirs
				makedirs(home_plugin_folder)
				try:
					fd = open(filename, "w")
					fd.close()
				except IOError:
					return False
			filename = core_plugin_folder + "/__init__.py"
			if not path.exists(filename):
				#raise Exception
				pass
		except:
			return False
		return True

	def __plugin_modules_exist(self):
		"""
		Account for found plug-in modules.

		Found plug-in modules are place in the
		__found_module_names list.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@return: True if plug-in modules are found.
		@rtype: A Boolean object.
		"""
		from info import home_plugin_folder, core_plugin_folder
		found_modules = self.__get_plugin_modules(home_plugin_folder)
		if found_modules:
			self.__found_home_modules = found_modules
			self.__number_of_modules += len(found_modules)
		#found_modules = self.__get_plugin_modules(core_plugin_folder)
		#if found_modules:
		#	self.__found_core_modules = found_modules
		#	self.__number_of_modules += len(found_modules)
		if self.__number_of_modules:
			return True
		return False

	def __get_plugin_modules(self, plugin_folder):
		"""
		Get all plug-in modules in a folder.

		Valid plug-in modules are python files that start with "Plugin"
		and end with ".py".

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param plugin_folder: A plug-in folder.
		@type plugin_folder: A String object.

		@return: A list of strings representing plug-in modules.
		@rtype: A List object.
		"""
		from os import listdir
		plugin_files = []
		for filename in listdir(plugin_folder):
			if filename in plugin_files:
				continue
			if filename.startswith("Plugin") is False:
				continue
			if filename.endswith(".py") is False:
				continue
			plugin_files.append(filename)
		if not plugin_files:
			return None
		return [filename.rstrip(".py") for filename in plugin_files]

	def __set_plugin_search_path(self):
		from info import home_plugin_folder, core_plugin_folder
		from sys import path
		if not home_plugin_folder in path:
			path.insert(0, home_plugin_folder)
		if not core_plugin_folder in path:
			path.insert(0, core_plugin_folder)
		return

	def __load_plugins(self):
		"""
		Load found plug-ins.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.
		"""
		self.__editor.emit("loading-plugins")
		from info import home_plugin_folder, core_plugin_folder
		from thread import start_new_thread
		if self.__found_core_modules:
			start_new_thread(self.__process_modules, (self.__found_core_modules, core_plugin_folder))
		if self.__found_home_modules:
			start_new_thread(self.__process_modules, (self.__found_home_modules, home_plugin_folder))
		return

	def __process_modules(self, modules, plugin_folder):
		"""
		Process modules.

		@param modules: A list of modules to be processed.
		@type modules: A List object.

		@param plugin_folder: Path to the plug-in to process.
		@type plugin_folder: A String object.
		"""
		from thread import start_new_thread
		for module in modules:
			start_new_thread(self.__initialize_plugin, (module, plugin_folder))
		return

	def __initialize_plugin(self, module_name, plugin_folder):
		"""
		Initialize necessary plug-ins.

		This function determines what plug-ins to activate. Checks are
		also performed to filter invalid modules or problematic
		plug-ins.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param module: A plug-in module.
		@type module: A Module object.

		@param home_plugin_folder: Third party plug-in folder.
		@type home_plugin_folder: A String object.
		"""
		try:
			from imp import load_source
			file_path = plugin_folder + "/" + module_name + ".py"
			module = load_source(module_name, file_path)
			if hasattr(module, "autoload") is False:
				raise Exception
			if getattr(module, "autoload"):
				if self.__activate_plugin(module) is False:
					raise Exception
			else:
				self.__inactive_modules.append(module)
		except:
			print "Failed to load: ", module_name
			self.__failed_modules.append(module)
		self.emit("processed")
		return

	def __activate_plugin(self, module):
		"""
		Load a plug-in.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param module: Module containing a plug-in initialization class/
		@type module: A Module object.

		@return: True if initialization is successful.
		@rtype: A Boolean object.
		"""
		try:
			if self.__can_activate_plugin(module) is False:
				raise Exception
			version = getattr(module, "version")
			class_name = getattr(module, "class_name")
			if self.__duplicate_plugin_found(class_name):
				if self.__unload_duplicate_plugin(class_name, version) is False:
					raise Exception
			PluginClass = getattr(module, class_name)
			plugin_object = PluginClass(self.__editor)
			plugin_object.load()
			plugin_object.scribes_plugin_module = module
			plugin_object.scribes_plugin_name = class_name
			plugin_object.scribes_plugin_version = version
			self.__active_plugins.append(plugin_object)
			self.__active_modules.append(module)
		except:
			return False
		return True

	def __can_activate_plugin(self, module):
		"""
		Check if a plug-in can be initialized.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param module: Modules containing a plug-in initialization class.
		@type module: A Module object.

		@return: True if a plug-in can be initialized.
		@rtype: A Boolean object.
		"""
		try:
			if hasattr(module, "version") is False:
				raise Exception
			version = getattr(module, "version")
			if hasattr(module, "class_name") is False:
				raise Exception
			class_name = getattr(module, "class_name")
			if hasattr(module, class_name) is False:
				raise Exception
			PluginClass = getattr(module, class_name)
			if hasattr(PluginClass, "__init__") is False:
				raise Exception
			if hasattr(PluginClass, "load") is False:
				raise Exception
			if hasattr(PluginClass, "unload") is False:
				raise Exception
		except:
			return False
		return True

	def __duplicate_plugin_found(self, name):
		"""
		Check for possible duplicate plug-ins.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param name: Name of a plug-in class.
		@type name: A String object.

		@return: True if duplicate plug-ins are found.
		@rtype: A Boolean object.
		"""
		for plugin_object in self.__active_plugins:
			if name == plugin_object.scribes_plugin_name:
				return True
		return False

	def __unload_duplicate_plugin(self, name, version):
		"""
		Try to unload duplicate active plug-in.

		The plug-in with the newer version number will be loaded. The
		plug-in with the lower version number will be unloaded.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param version: The version of the module to be loaded.
		@type version: A String object.

		@return: True if duplicate active module is successfully unloaded.
		@rtype: A Boolean object.
		"""
		for plugin_object in self.__active_plugins:
			if name == plugin_object.scribes_plugin_name:
				if float(version) >= float(plugin_object.scribes_plugin_version):
					self.__remove_plugin(plugin_object.scribes_plugin_module)
					return True
				break
		return False

	def __deactivate_plugin(self, module):
		"""
		Unload a plug-in.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param module: Module containing a plug-in initialization class/
		@type module: A Module object.

		@return: True if initialization is successful.
		@rtype: A Boolean object.
		"""
		try:
			for plugin_object in self.__active_plugins:
				if plugin_object.scribes_plugin_module == module:
					plugin_object.unload()
					self.__active_plugins.remove(plugin_object)
					self.__active_modules.remove(module)
					self.__inactive_modules.append(module)
					self.__inactive_plugins.append(plugin_object)
					break
		except:
			return False
		return True

	def __remove_plugin(self, module):
		"""
		Unload a plug-in.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param module: Module containing a plug-in initialization class/
		@type module: A Module object.

		@return: True if initialization is successful.
		@rtype: A Boolean object.
		"""
		try:
			for plugin_object in self.__active_plugins:
				if plugin_object.scribes_plugin_module == module:
					plugin_object.unload()
					self.__active_plugins.remove(plugin_object)
					self.__active_modules.remove(module)
					del plugin_object
					del module
					plugin_object = None
					module = None
					break
		except:
			return False
		return True

########################################################################
#
# 					 Event and Signal Handlers
#
########################################################################

	def __manager_processed_cb(self, plugin_manager):
		"""
		Handles callback when "processed" signal is emitted.

		@param self: Reference to the ScribesPluginManager instance.
		@type self: A ScribesPluginManager object.

		@param plugin_manager: Reference to the ScribesPluginManager.
		@type plugin_manager: A ScribesPluginManager object.
		"""
		self.__processed_modules += 1
		if self.__processed_modules == self.__number_of_modules:
			self.__editor.emit("loaded-plugins")
		return

	def __manager_loading_plugins_cb(self, editor):
		self.__status_id =	self.__editor.feedback.set_modal_message("Loading plug-ins", "run")
		return

	def __manager_loaded_plugins_cb(self, editor):
		self.__editor.feedback.unset_modal_message(self.__status_id, False)
		self.__editor.feedback.update_status_message("Loaded plug-ins", "info")
		return

	def __plugin_folder_changed_cb(self, monitor_uri, info_uri, event_type):
		from gnomevfs import MONITOR_EVENT_DELETED
		from gnomevfs import MONITOR_EVENT_CREATED
		from gnomevfs import get_local_path_from_uri
		filename = get_local_path_from_uri(info_uri)
		if filename.endswith(".py") is False:
			return
		from os import path
		module_name = path.split(filename)[1]
		if module_name.startswith("Plugin") is False:
			return
		module_name = path.splitext(module_name)[0]
		from imp  import load_source
		module = load_source(module_name, filename)
		if event_type == MONITOR_EVENT_DELETED:
			print "'%s' was deleted in plugin folder" % filename
		elif event_type == MONITOR_EVENT_CREATED:
			print "'%s' was created in plugin folder" % filename
		return
