# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Actions are GTK+ objects that perform a single predefined function. Once an
action has been created, it can be called anytime and from anywhere to perform
operations it is designed to accomplish.

User operations are performed via actions within the text editor. Actions
represent functions such as clicking toolbar buttons or activating operations
through shortcut keys. All user operations must be implemented via actions.

This module houses all actions supported by the text editor.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

################################################################################
#
#						Action Callbacks
#
################################################################################

def close_file_cb(action, editor):
	"""
	Handles callback to close the text editor's window.

	@param action: Reference to an action that closes the text editor's
				window.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor instance.
	@type editor: An Editor object.
	"""
	editor.emit("close-document")
	return True

def new_file_cb(action, editor):
	"""
	Handles callback to create a new instance of the editor.

	@param action: Reference to an action that spawns a new instance of the
				text editor.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	# Create a new GTK+ mainloop. At any point in time, there must be at least
	# one GTK+ mainloop running. When an instance of the text editor is
	# destroyed, its GTK+ mainloop is destroyed along with it. Thus, a new
	# mainloop should be created as long as an editor object exists.
	from gtk import quit_add, main
	quit_add(0, main)
	from main import launch_new_editor
	from gobject import timeout_add
	timeout_add(7, launch_new_editor)
#	from subprocess import Popen
#	Popen(["scribesng"])
	return True

def save_file_cb(action, editor):
	"""
	Handles callback to save a document contained in the text editor's buffer.

	@param action: Reference to an action that shows the text editor's save dialog
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	editor.emit("save-document")
	return

def toggle_readonly_cb(action, editor):
	"""
	Toggle readonly mode on or off.

	@param action: An action to toggle readonly on or off.
	@type action: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: An Editor object.
	"""
	if editor.uri is None:
		# Do not toggle readonly mode when the editor contains no document.
		from internationalization import msg0323
		editor.feedback.update_status_message(msg0323, "fail")
		return
	if editor.is_readonly:
		from utils import check_uri_permission
		result = check_uri_permission(editor.uri)
		if result:
			editor.emit("disable-readonly")
		else:
			from internationalization import msg0325
			editor.feedback.update_status_message(msg0325, "fail")
	else:
		if editor.saved_document is False:
			editor.triggermanager.trigger("save_file")
		editor.emit("enable-readonly")
	return

def toggle_fullscreen_cb(action, editor):
	"""
	Toggle fullscreen mode on or off

	@param action: An action to toggle readonly on or off.
	@type action: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: An Editor object.
	"""
	if editor.window.is_fullscreen:
		editor.emit("disable-fullscreen")
	else:
		editor.emit("enable-fullscreen")
	return

def undo_edits_cb(action, editor):
	"""
	Handles callback to undo an operation in the text editor's buffer.

	@param action: Reference to an action that performs an undo operation in the
				text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	if editor.textbuffer.can_undo():
		editor.textbuffer.undo()
		from internationalization import msg0122
		editor.feedback.update_status_message(msg0122, "undo")
		from cursor import move_view_to_cursor
		move_view_to_cursor(editor.textview)
	else:
		from internationalization import msg0002
		editor.feedback.update_status_message(msg0002, "fail")
	return

def redo_edits_cb(action, editor):
	"""
	Handles callback to redo an undone operation in the text editor's buffer.

	@param action: Reference to an action that performs a redo operation in the
				text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	if editor.textbuffer.can_redo():
		editor.textbuffer.redo()
		from internationalization import msg0123
		editor.feedback.update_status_message(msg0123, "redo")
		from cursor import move_view_to_cursor
		move_view_to_cursor(editor.textview)
	else:
		from internationalization import msg0003
		editor.feedback.update_status_message(msg0003, "fail")
	return

def show_gotobar_cb(action, editor):
	"""
	Handles callback to show the text editor's goto bar.

	@param action: Reference to an action that shows the text editor's goto bar.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	try:
		if editor.bar_is_visible:
			return
		editor.gotobar.show_bar()
	except:
		from gotobar import ScribesGotoBar
		editor.gotobar = ScribesGotoBar(editor)
		editor.gotobar.show_bar()
	return

def show_findbar_cb(action, editor):
	"""
	Handles callback to show the text editor's find bar.

	@param action: Reference to an action that shows the text editor's
				find bar.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	try:
		if editor.bar_is_visible:
			return
		editor.searchbar.show_bar()
	except:
		from searchbar import ScribesSearchBar
		editor.searchbar = ScribesSearchBar(editor)
		editor.searchbar.show_bar()
	return

def show_regexbar_cb(action, editor):
	"""
	Handles callback to show the text editor's find bar.

	@param action: Reference to an action that shows the text editor's
				find bar.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	try:
		if editor.bar_is_visible:
			return
		editor.regexbar.show_bar()
	except:
		from regexbar import ScribesRegexBar
		editor.regexbar = ScribesRegexBar(editor)
		editor.regexbar.show_bar()
	return

def show_incsearchbar_cb(action, editor):
	"""
	Handles callback to show the text editor's find bar.

	@param action: Reference to an action that shows the text editor's
				find bar.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	try:
		if editor.bar_is_visible:
			return
		editor.incsearchbar.show_bar()
	except:
		from incsearchbar import ScribesIncrementalSearchBar
		editor.incsearchbar = ScribesIncrementalSearchBar(editor)
		editor.incsearchbar.show_bar()
	return

def show_replacebar_cb(action, editor):
	"""
	Handles callback to show the text editor's replace bar.

	@param action: Reference to an action that shows the text editor's
				replace bar.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	try:
		if editor.bar_is_visible:
			return
		editor.replacebar.show_bar()
	except:
		from replacebar import ScribesReplaceBar
		editor.replacebar = ScribesReplaceBar(editor)
		editor.replacebar.show_bar()
	return

def preferences_cb(action, editor):
	"""
	Handles callback to show the text editor's preferences dialog.

	@param action: Reference to an action that shows the text editor's
				preferences dialog.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	return True

def help_cb(action, editor):
	"""
	Handles callback to launch the GNOME help browser.

	@param action: Reference to an action that launches the GNOME help browser.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	return True

def indent_line_cb(action, editor):
	"""
	Handles callback to indent line(s) is the text editor's buffer

	@param action: Reference to an action that indents lines in the text
		editor's buffer.
	@type action: A Trigger object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from internationalization import msg0179
	status_id = editor.feedback.set_modal_message(msg0179, "run")
	from cursor import show_busy_textview_cursor, show_textview_cursor
	show_busy_textview_cursor(editor.textview)
	from indent import indent
	lines_indented = indent(editor.textview)
	show_textview_cursor(editor.textview)
	editor.feedback.unset_modal_message(status_id, False)
	if len(lines_indented) > 1:
		from internationalization import msg0059
		editor.feedback.update_status_message(msg0059, "succeed")
	else:
		from internationalization import msg0146
		value = lines_indented[0] + 1
		message = msg0146 % (value)
		editor.feedback.update_status_message(message, "succeed")
	return

def unindent_line_cb(action, editor):
	"""
	Handles callback to unindent line(s) is the text editor's buffer

	@param action: Reference to an action that indents lines in the text
		editor's buffer.
	@type action: A Trigger object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from internationalization import msg0179
	status_id = editor.feedback.set_modal_message(msg0179, "run")
	from cursor import show_busy_textview_cursor, show_textview_cursor
	show_busy_textview_cursor(editor.textview)
	from unindent import unindent
	lines_unindented = unindent(editor.textview)
	show_textview_cursor(editor.textview)
	editor.feedback.unset_modal_message(status_id, False)
	if not lines_unindented:
		from internationalization import msg0060
		editor.feedback.update_status_message(msg0060, "fail")
	elif len(lines_unindented) > 1:
		from internationalization import msg0061
		editor.feedback.update_status_message(msg0061, "succeed")
	else:
		from internationalization import msg0147
		value = lines_unindented[0] + 1
		message = msg0147 % (value)
		editor.feedback.update_status_message(message, "succeed")
	return

def spaces_to_tabs_cb(action, editor):
	"""
	Handles callback to convert space characters to tab characters in  the text
	editor's buffer

	@param action: Reference to an action that converts spaces to tabs text
		editor's buffer.
	@type action: A Trigger object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from internationalization import msg0381
	status_id = editor.feedback.set_modal_message(msg0381, "run")
	from cursor import show_busy_textview_cursor, show_textview_cursor
	show_busy_textview_cursor(editor.textview)
	from spaces import convert_spaces_to_tabs
	converted_lines = convert_spaces_to_tabs(editor.textview)
	show_textview_cursor(editor.textview)
	editor.feedback.unset_modal_message(status_id, False)
	if not converted_lines:
		from internationalization import msg0383
		editor.feedback.update_status_message(msg0383, "fail")
	else:
		from internationalization import msg0382
		editor.feedback.update_status_message(msg0382, "succeed")
	return

def tabs_to_spaces_cb(action, editor):
	"""
	Handles callback to convert space characters to tab characters in  the text
	editor's buffer

	@param action: Reference to an action that converts spaces to tabs text
		editor's buffer.
	@type action: A Trigger object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from internationalization import msg0384
	status_id = editor.feedback.set_modal_message(msg0384, "run")
	from cursor import show_busy_textview_cursor, show_textview_cursor
	show_busy_textview_cursor(editor.textview)
	from spaces import convert_tabs_to_spaces
	converted_lines = convert_tabs_to_spaces(editor.textview)
	show_textview_cursor(editor.textview)
	editor.feedback.unset_modal_message(status_id, False)
	if not converted_lines:
		from internationalization import msg0386
		editor.feedback.update_status_message(msg0386, "fail")
	else:
		from internationalization import msg0385
		editor.feedback.update_status_message(msg0385, "succeed")
	return

def remove_trailing_spaces_cb(action, editor):
	"""
	Handles callback to remove spaces at end of lines in the text editor's
	buffer.

	@param action: Reference to an action that removes spaces.
	@type action: A Trigger object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.
	"""
	if editor.is_readonly:
		# Prevent save operations when the text editor is in readonly mode.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from internationalization import msg0178
	status_id = editor.feedback.set_modal_message(msg0178, "run")
	from cursor import show_busy_textview_cursor, show_textview_cursor
	show_busy_textview_cursor(editor.textview)
	from spaces import remove_trailing_spaces
	affected_lines = remove_trailing_spaces(editor.textview)
	show_textview_cursor(editor.textview)
	editor.feedback.unset_modal_message(status_id, False)
	if not affected_lines:
		from internationalization import msg0134
		editor.feedback.update_status_message(msg0134, "fail")
	else:
		from internationalization import msg0387
		editor.feedback.update_status_message(msg0387, "succeed")
	return

def select_word_cb(action, editor):
	"""
	Handles callback to select a word in the text editor's buffer.

	@param action: Reference to an action to select a word in the tex editor's
				buffer.
	@type action: A gtk.Action

	@param editor: Reference to the editor instance
	@type editor: An editor object
	"""
	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from cursor import get_cursor_iterator
	iterator = get_cursor_iterator(editor.textbuffer)
	from word import select_word
	result = select_word(editor.textbuffer, iterator)
	if result:
		from internationalization import msg0007
		editor.feedback.update_status_message(msg0007, "succeed")
	else:
		from internationalization import msg0008
		editor.feedback.update_status_message(msg0008, "fail")
	return

def select_line_cb(trigger, editor):
	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from cursor import get_cursor_line
	cursor_line = get_cursor_line(editor.textbuffer)
	from lines import select_line
	result = select_line(editor.textbuffer)
	if result:
		from internationalization import msg0010
		message = msg0010 % (cursor_line + 1)
		editor.feedback.update_status_message(message, "succeed")
	else:
		from internationalization import msg0009
		message = msg0009 % (cursor_line + 1)
		editor.feedback.update_status_message(message, "fail")
	return

def delete_line_cb(trigger, editor):
	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from cursor import get_cursor_line
	cursor_line = get_cursor_line(editor.textbuffer)
	from lines import delete_line
	delete_line(editor.textbuffer)
	from internationalization import msg0004
	message = msg0004 % (cursor_line + 1)
	editor.feedback.update_status_message(message, "succeed")
	return

def join_line_cb(trigger, editor):
	"""
	Handles callback to join lines in the text editor's buffer.

	@param trigger: An object representing a join line operation.
	@type trigger: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: A Editor object.
	"""
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from cursor import get_cursor_line
	cursor_line = get_cursor_line(editor.textbuffer)
	from lines import join_line
	result = join_line(editor.textbuffer)
	if result:
		from internationalization import msg0005
		message = msg0005 % (cursor_line+1, cursor_line+2)
		editor.feedback.update_status_message(message, "succeed")
	else:
		from internationalization import msg0006
		editor.feedback.update_status_message(msg0006, "fail")
	return

def free_line_above_cb(trigger, editor):
	"""
	Handles callback to free the current line in the text editor's buffer.

	@param trigger: An object representing a join line operation.
	@type trigger: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: A Editor object.
	"""
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from lines import free_line_above
	line_number = free_line_above(editor.textbuffer)
	from cursor import move_view_to_cursor
	move_view_to_cursor(editor.textview)
	from internationalization import msg0136
	message = msg0136 % (line_number+1)
	editor.feedback.update_status_message(message, "succeed")
	return

def free_line_below_cb(trigger, editor):
	"""
	Handles callback to free the current line in the text editor's buffer.

	@param trigger: An object representing a join line operation.
	@type trigger: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: A Editor object.
	"""
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from lines import free_line_below
	line_number = free_line_below(editor.textbuffer)
	from cursor import move_view_to_cursor
	move_view_to_cursor(editor.textview)
	from internationalization import msg0136
	message = msg0136 % (line_number+1)
	editor.feedback.update_status_message(message, "succeed")
	return

def delete_cursor_to_line_end_cb(trigger, editor):
	"""
	Handles callback to join lines in the text editor's buffer.

	@param trigger: An object representing a join line operation.
	@type trigger: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: A Editor object.
	"""
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from cursor import get_cursor_line
	cursor_line = get_cursor_line(editor.textbuffer)
	from lines import delete_cursor_to_line_end
	result = delete_cursor_to_line_end(editor.textbuffer)
	if result:
		from internationalization import msg0137
		message = msg0137 % (cursor_line+1)
		editor.feedback.update_status_message(message, "succeed")
	else:
		from internationalization import msg0138
		editor.feedback.update_status_message(msg0138, "fail")
	return

def delete_cursor_to_line_begin_cb(trigger, editor):
	"""
	Handles callback to join lines in the text editor's buffer.

	@param trigger: An object representing a join line operation.
	@type trigger: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: A Editor object.
	"""
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from cursor import get_cursor_line
	cursor_line = get_cursor_line(editor.textbuffer)
	from lines import delete_cursor_to_line_begin
	result = delete_cursor_to_line_begin(editor.textbuffer)
	if result:
		from internationalization import msg0139
		message = msg0139 % (cursor_line+1)
		editor.feedback.update_status_message(message, "succeed")
	else:
		from internationalization import msg0140
		editor.feedback.update_status_message(msg0140, "fail")
	return

def select_sentence_cb(trigger, editor):
	# Prevent this action from occurring when the text editor is in
	# readonly mode.
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from sentence import select_sentence
	result = select_sentence(editor.textbuffer)
	if result:
		from internationalization import msg0013
		editor.feedback.update_status_message(msg0013, "succeed")
	else:
		from internationalization import msg0014
		editor.feedback.update_status_message(msg0014, "fail")
	return

def select_paragraph_cb(trigger, editor):
	# Prevent this action from occurring when the text editor is in
	# readonly mode.
	if editor.is_readonly:
		# Feedback to the status bar indicating the operation cannot be
		# performed.
		from internationalization import msg0145
		editor.feedback.update_status_message(msg0145, "fail")
		return
	from paragraph import select_paragraph
	result = select_paragraph(editor.textbuffer)
	if result:
		from internationalization import msg0011
		editor.feedback.update_status_message(msg0011, "succeed")
	else:
		from internationalization import msg0012
		editor.feedback.update_status_message(msg0012, "fail")
	return

def reply_handler_cb(uris):
	print uris
	return

def error_handler_cb(error):
	print error
	return

def show_user_guide_cb(trigger, editor):
	"""
	Handles callback to show the text editor's manual.

	@param trigger: An action to launch the editor's help browser.
	@type trigger: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: An Editor object.
	"""
	from gnome import help_display
	if help_display("/scribes.xml"):
		from internationalization import msg0057
		editor.feedback.update_status_message(msg0057, "help", 10)
	else:
		from internationalization import msg0429
		editor.feedback.update_status_message(msg0429, "fail", 10)
	return

def toggle_spell_checking_cb(trigger, editor):
	"""
	Handles callback to toggle spell checking.

	@param trigger: An action to toggle spell checking.
	@type trigger: A Trigger object.

	@param editor: Reference to the text editor.
	@type editor: An Editor object.
	"""
	from gconf import client_get_default
	client = client_get_default()
	value = client.get_bool("/apps/scribes/spell_check")
	if value:
		client.set_bool("/apps/scribes/spell_check", False)
		from internationalization import msg0022
		editor.feedback.update_status_message(msg0022, "warning")
	else:
		client.set_bool("/apps/scribes/spell_check", True)
		from internationalization import msg0023
		editor.feedback.update_status_message(msg0023, "succeed")
	return

def close_all_windows_cb(trigger, editor):
	editor.service_proxy_object.close_windows(dbus_interface="net.sourceforge.ScribesService",
			reply_handler=reply_handler_cb,
			error_handler=error_handler_cb)
	return

def reply_handler_cb(*args):
	return

def error_handler_cb(error):
	print error
	return

################################################################################
#
#								Actions
#
################################################################################

def create_actions(editor):
	"""
	Create an action group.

	Actions are unique functions that the editor can perform.

	@param editor: Reference to the editor instance.
	@type editor: An Editor object.

	@return: An action group containing all the functions the editor can
		perform.
	@rtype: A gtk.ActionGroup object
	"""
	from triggermanager import TriggerManager
	manager = TriggerManager(editor.window)
	close_file_trigger = ("close_file", "ctrl - w", close_file_cb)
	new_file_trigger = ("new_file", "ctrl - n", new_file_cb)
	save_file_trigger = ("save_file", "ctrl - s", save_file_cb)
	toggle_readonly_trigger = ("toggle_readonly", "F3", toggle_readonly_cb)
	toggle_fullscreen_trigger = ("toggle_fullscreen", "F11", toggle_fullscreen_cb)
	undo_action_trigger = ("undo_action", "ctrl - z", undo_edits_cb)
	redo_action_trigger = ("redo_action", "ctrl - Z", redo_edits_cb)
	indent_line_trigger = ("indent_line", "ctrl - t", indent_line_cb)
	unindent_line_trigger = ("unindent_line", "ctrl - T", unindent_line_cb)
	spaces_to_tabs_trigger = ("spaces_to_tabs", "alt - t", spaces_to_tabs_cb)
	tabs_to_spaces_trigger = ("tabs_to_spaces", "alt - T", tabs_to_spaces_cb)
	remove_trailing_spaces_trigger = ("remove_trailing_spaces", "alt - r", remove_trailing_spaces_cb)
	select_word_trigger = ("select_word", "alt - w", select_word_cb)
	select_line_trigger = ("select_line", "alt - l", select_line_cb)
	select_sentence_trigger = ("select_sentence", "alt - s", select_sentence_cb)
	select_paragraph_trigger = ("select_paragraph", "alt - p", select_paragraph_cb)
	delete_line_trigger = ("delete_line", "alt - d", delete_line_cb)
	join_line_trigger = ("join_line", "alt - j", join_line_cb)
	free_line_above_trigger = ("free_line_above", "alt - O", free_line_above_cb)
	free_line_below_trigger = ("free_line_below", "alt - o", free_line_below_cb)
	delete_cursor_to_line_end_trigger = ("delete_cursor_to_end", "alt - End", delete_cursor_to_line_end_cb)
	delete_cursor_to_line_begin_trigger = ("delete_cursor_to_begin", "alt - Home", delete_cursor_to_line_begin_cb)
	help_trigger = ("show_user_guide", "F1", show_user_guide_cb)
	toggle_spell_checking_trigger = ("toggle_spell_checking", "F6", toggle_spell_checking_cb)
	close_all_windows_trigger = ("close_all_windows", "ctrl - Q", close_all_windows_cb)
	# Trigger Entries.
	entries = [
		new_file_trigger,
		close_file_trigger, save_file_trigger,
		toggle_readonly_trigger, toggle_fullscreen_trigger,
		undo_action_trigger, redo_action_trigger,
		indent_line_trigger, unindent_line_trigger,
		spaces_to_tabs_trigger, tabs_to_spaces_trigger,
		remove_trailing_spaces_trigger, select_word_trigger,
		select_line_trigger, delete_line_trigger, join_line_trigger,
		free_line_above_trigger, free_line_below_trigger,
		delete_cursor_to_line_end_trigger, help_trigger,
		delete_cursor_to_line_begin_trigger, select_sentence_trigger,
		select_paragraph_trigger, toggle_spell_checking_trigger,
		close_all_windows_trigger
	]
	# Create Actions for actiongroup
	manager.add_triggers(entries, data=editor)
	return manager
