# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

from dialog import ScribesDialog

class BookmarkBrowser(ScribesDialog):

	def __init__(self, bookmark_manager):
		"""
		Initialize the browser's data attributes.

		@param self: Reference to the BookmarkBrowser instance.
		@type self: A BookmarkBrowser object.

		@param bookmark_manager: The text editor's bookmark manager.
		@type bookmark_manager: A BookmarkManager object.
		"""
		ScribesDialog.__init__(self, bookmark_manager.editor)
		self.__init_attributes(bookmark_manager)
		self.__set_dialog_properties()
		self.__create_dialog_widgets()
		self.__arrange_widgets()

	def __init_attributes(self, bookmark_manager):
		"""
		Initialize the browser's data attributes.

		@param self: Reference to the BookmarkBrowser instance.
		@type self: A BookmarkBrowser object.

		@param bookmark_manager: The text editor's bookmark manager.
		@type bookmark_manager: A BookmarkManager object.
		"""
		bookmark_manager.browser = self
		self.editor = bookmark_manager.editor
		self.manager = bookmark_manager
		self.status_id = None
		return

	def __set_dialog_properties(self):
		"""
		Customize the dialog properties.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.
		"""
		from internationalization import msg0336
		self.set_property("title", msg0336)
		self.set_property("role", "bookmark_browser")
		from gtk.gdk import WINDOW_TYPE_HINT_DIALOG
		self.set_property("type-hint", WINDOW_TYPE_HINT_DIALOG)
		self.set_property("modal", False)
		self.set_property("skip-taskbar-hint", True)
		self.set_property("skip-pager-hint", True)
		self.set_property("icon", self.manager.bookmark_image)
		from gtk import WIN_POS_CENTER_ON_PARENT
		self.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		self.set_property("resizable", True)
		self.set_property("has-separator", False)
		from gtk import STOCK_CLOSE, RESPONSE_CLOSE
		self.add_button(STOCK_CLOSE, RESPONSE_CLOSE)
		self.vbox.set_property("spacing", 10)
		self.set_property("border-width", 10)
		# Calculate resolution independent width and height of the preferences
		# dialog window.
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self.editor.window,
												1.679790026, 1.865209472)
		self.set_property("width-request", width)
		self.set_property("height-request", height)
		self.set_transient_for(self.editor.window)
		return

	def __create_dialog_widgets(self):
		"""
		Create key dialog widgets.

		@param self: Reference to the BookmarkBrowser instance.
		@type self: A BookmarkBrowser object.
		"""
		from bookmarkbrowserview import BookmarkBrowserView
		self.treeview = BookmarkBrowserView(self)
		return

	def __arrange_widgets(self):
		"""
		Arrange the graphic user interface components for the browser.

		@param self: Reference to the BookmarkBrowser instance.
		@type self: A BookmarkBrowser object.
		"""
		from utils import create_scrollwin
		scrollwin = create_scrollwin()
		scrollwin.add(self.treeview)
		self.vbox.pack_start(scrollwin, True, True, 0)
		return

	def show_dialog(self):
		"""
		Show the bookmark browser.

		@param self: Reference to the BookmarkBrowser instance.
		@type self: A BookmarkBrowser object.
		"""
		from internationalization import msg0417
		self.status_id = self.editor.feedback.set_modal_message(msg0417, "info")
		ScribesDialog.show_dialog(self)
		return

	def hide_dialog(self):
		"""
		Hide the bookmark browser.

		@param self: Reference to the BookmarkBrowser instance.
		@type self: A BookmarkBrowser object.
		"""
		self.editor.feedback.unset_modal_message(self.status_id, False)
		ScribesDialog.hide_dialog(self)
		return
