# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

from gtk import TreeView

class BookmarkBrowserView(TreeView):
	"""
	This class creates the TreeView object for the text editor's
	bookmark browser. The default properties and behavior of the view
	object is defined by this class.
	"""

	def __init__(self, bookmark_browser):
		"""
		Initialize the tree view.

		@param self: Reference to the BookmarkBrowserView instance.
		@type self: A BookmarkBrowserView object.

		@param bookmark_browser: The text editor's bookmark browser.
		@type bookmark_browser: A BookmarkBrowser object.
		"""
		TreeView.__init__(self)
		self.__init_attributes(bookmark_browser)
		self.__set_properties()
		self.connect("map-event", self.__view_map_event_cb)
		self.connect("row-activated", self.__view_row_activated_cb)

	def __init_attributes(self, bookmark_browser):
		"""
		Initialize the tree view's data attributes.

		@param self: Reference to the BookmarkBrowserView instance.
		@type self: A BookmarkBrowserView object.

		@param bookmark_browser: The text editor's bookmark browser.
		@type bookmark_browser: A BookmarkBrowser object.
		"""
		self.browser = bookmark_browser
		self.editor = self.browser.editor
		self.manager = self.browser.manager
		from bookmarkbrowsermodel import BookmarkBrowserModel
		self.liststore = BookmarkBrowserModel(self)
		self.__first_time_flag = True
		return

	def __set_properties(self):
		"""
		Set the tree view's properties.

		@param self: Reference to the BookmarkBrowserView instance.
		@type self: A BookmarkBrowserView object.
		"""
		from gtk import CellRendererText
		renderer = CellRendererText()
		renderer.set_property('cell-background', 'yellow')
		renderer.set_property("xalign", 1.0)
		from gtk import TreeViewColumn
		from internationalization import msg0337
		column = TreeViewColumn(msg0337, renderer, text=0)
		self.append_column(column)
		column.set_expand(False)
		from gtk import TREE_VIEW_COLUMN_AUTOSIZE, SORT_DESCENDING
		column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)
		column.set_attributes(renderer, text=0, cell_background_set=2)
		column.set_sort_indicator(True)
		column.set_sort_order(SORT_DESCENDING)
		column.set_sort_column_id(0)

		renderer = CellRendererText()
		renderer.set_property('cell-background', 'yellow')
		from internationalization import msg0338
		column = TreeViewColumn(msg0338, renderer, text=1)
		self.append_column(column)
		column.set_expand(True)
		column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)
		column.set_attributes(renderer, text=1, cell_background_set=2)
		column.set_sort_order(SORT_DESCENDING)
		column.set_sort_column_id(1)

		self.set_model(self.liststore)
		self.columns_autosize()
		self.set_enable_search(True)
		self.set_search_column(0)
		self.set_property("rules-hint", True)
		return

	def __view_map_event_cb(self, treeview, event):
		"""
		Handles callback when the "map-event" signal is emitted.

		@param self: Reference to the BookmarkBrowserView instance.
		@type self: A BookmarkBrowserView object.

		@param treeview: The bookmark browser's view.
		@type treeview: A BookmarkBrowserView object.

		@param event: An event that occurs when the view is displayed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.__first_time_flag:
			# Show the sort indicator when the dialog is initially
			# displayed.
			column = treeview.get_columns()
			column[0].clicked()
			self.__first_time_flag = False
		return True

	def __view_row_activated_cb(self, treeview, path, column):
		"""
		Handles callback when the "row-activated" signal is emitted.

		@param treeview: The bookmark browser's treeview.
		@type treeview: A BookmarkBrowserView object.

		@param path: A row in a treeview.
		@type path: A row object.

		@param column: A column in a treeview.
		@type column: A gtk.TreeViewColumn object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		# Get the selected line in the dialog's treeview.
		selection = treeview.get_selection()
		model, iterator = selection.get_selected()
		line = model.get_value(iterator, 0)

		# Move cursor to the selected line in the dialog's treeview.
		iterator = self.editor.textbuffer.get_iter_at_line(int(line)-1)
		self.editor.textbuffer.place_cursor(iterator)
		from cursor import move_view_to_cursor
		move_view_to_cursor(self.editor.textview)
		self.browser.hide_dialog()
		from internationalization import msg0171
		message = msg0171 % int(line)
		self.editor.feedback.update_status_message(message, "succeed")
		return True
