# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
Bookmark actions for the text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class BookmarkTriggers(object):
	"""
	This class implements an object that allows users of the text editor
	to perform bookmark operations via the text editor's standard
	shortcut keys.
	"""

	def __init__(self, editor):
		"""
		Initialize the bookmark trigger object.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__init_attributes(editor)
		self.__create_bookmark_triggers()
		self.editor.connect("loaded-document", self.__trigger_loaded_document_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the bookmark triggers data attributes.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.
		"""
		self.editor = editor
		self.__manager = None
		self.__browser = None
		return

	def __create_bookmark_triggers(self):
		"""
		Create the bookmark triggers.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.
		"""
		# Trigger to bookmark a line.
		from trigger import Trigger
		trigger = Trigger("bookmark_line")
		trigger.connect("activate", self.__toggle_bookmark_cb)
		self.editor.triggermanager.add_trigger(trigger, "ctrl - d")

		trigger = Trigger("remove_bookmark")
		trigger.connect("activate", self.__toggle_bookmark_cb)
		self.editor.triggermanager.add_trigger(trigger)

		# Trigger to remove all bookmarks from the document.
		trigger = Trigger("remove_all_bookmarks")
		trigger.connect("activate", self.__remove_all_bookmarks_cb)
		self.editor.triggermanager.add_trigger(trigger, "alt - ctrl - Delete")

		# Trigger to move to the next bookmark in the document.
		trigger = Trigger("next_bookmark")
		trigger.connect("activate", self.__next_bookmark_cb)
		self.editor.triggermanager.add_trigger(trigger, "alt - Down")

		# Trigger to move to the next bookmark in the document.
		trigger = Trigger("previous_bookmark")
		trigger.connect("activate", self.__previous_bookmark_cb)
		self.editor.triggermanager.add_trigger(trigger, "alt - Up")

		# Trigger to move to the first bookmark in the document
		trigger = Trigger("first_bookmark")
		trigger.connect("activate", self.__first_bookmark_cb)
		self.editor.triggermanager.add_trigger(trigger, "alt - ctrl - Home")

		# Trigger to move to the first bookmark in the document
		trigger = Trigger("last_bookmark")
		trigger.connect("activate", self.__last_bookmark_cb)
		self.editor.triggermanager.add_trigger(trigger, "alt - ctrl - End")

		# Trigger to show the bookmark browser.
		trigger = Trigger("show_bookmark_browser")
		trigger.connect("activate", self.__show_bookmark_browser_cb)
		self.editor.triggermanager.add_trigger(trigger, "ctrl - b")
		return

	def __toggle_bookmark_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		This function toggles bookmarks on a line in the text editor's
		buffer.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param trigger: An action to bookmark a line in the text editor's
			buffer.
		@type trigger: A Trigger object.
		"""
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)
		from cursor import get_cursor_line, move_view_to_cursor
		line = get_cursor_line(self.editor.textbuffer)
		if self.__manager.line_is_bookmarked(line):
			self.__manager.remove_bookmark_on_line(line)
			from internationalization import msg0167
			message = msg0167 % (line+1)
			self.editor.feedback.update_status_message(message, "succeed")
			return
		self.__manager.bookmark_line(line)
		move_view_to_cursor(self.editor.textview)
		from internationalization import msg0165
		message = msg0165 % (line+1)
		self.editor.feedback.update_status_message(message, "succeed")
		return

	def __remove_bookmark_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		This function removes a bookmark from a line in the text
		editor's buffer.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param trigger: An action to remove a bookmark from a line in
			the text editor's buffer.
		@type trigger: A Trigger object.
		"""
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)
		from cursor import get_cursor_line
		line = get_cursor_line(self.editor.textbuffer)
		if self.__manager.line_is_bookmarked(line) is False:
			from internationalization import msg0166
			message = msg0166 % (line+1)
			self.editor.feedback.update_status_message(message, "fail")
			return
		self.__manager.remove_bookmark_on_line(line)
		from internationalization import msg0167
		message = msg0167 % (line+1)
		self.editor.feedback.update_status_message(message, "succeed")
		return

	def __remove_all_bookmarks_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param trigger: Remove all bookmarks from the buffer.
		@type trigger: A Trigger object.
		"""
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)
		from cursor import get_cursor_line
		if not self.__manager.get_bookmarked_lines():
			from internationalization import msg0168
			self.editor.feedback.update_status_message(msg0168, "fail")
			return
		self.__manager.remove_all_bookmarks()
		from internationalization import msg0169
		self.editor.feedback.update_status_message(msg0169, "succeed")
		return

	def __next_bookmark_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param trigger: Move to next bookmark.
		@type trigger: A Trigger object.
		"""
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)
		if not self.__manager.get_bookmarked_lines():
			from internationalization import msg0168
			self.editor.feedback.update_status_message(msg0168, "fail")
			return
		result = self.__manager.move_to_next_bookmark()
		if result:
			from internationalization import msg0171
			from cursor import get_cursor_line
			line = get_cursor_line(self.editor.textbuffer)
			message = msg0171 % (line+1)
			self.editor.feedback.update_status_message(message, "succeed")
		else:
			from internationalization import msg0170
			self.editor.feedback.update_status_message(msg0170, "fail")
		return

	def __previous_bookmark_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param trigger: Move to previous bookmark in the buffer.
		@type trigger: A Trigger object.
		"""
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)
		if not self.__manager.get_bookmarked_lines():
			from internationalization import msg0168
			self.editor.feedback.update_status_message(msg0168, "fail")
			return
		result = self.__manager.move_to_previous_bookmark()
		if result:
			from internationalization import msg0171
			from cursor import get_cursor_line
			line = get_cursor_line(self.editor.textbuffer)
			message = msg0171 % (line+1)
			self.editor.feedback.update_status_message(message, "succeed")
		else:
			from internationalization import msg0172
			self.editor.feedback.update_status_message(msg0172, "fail")
		return

	def __first_bookmark_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param trigger: Move to first document in the buffer.
		@type trigger: A Trigger object.
		"""
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)
		if not self.__manager.get_bookmarked_lines():
			from internationalization import msg0168
			self.editor.feedback.update_status_message(msg0168, "fail")
			return
		if len(self.__manager.get_bookmarked_lines()) == 1:
			from internationalization import msg0415
			self.editor.feedback.update_status_message(msg0415, "fail")
			return
		self.__manager.move_to_first_bookmark()
		from internationalization import msg0413
		self.editor.feedback.update_status_message(msg0413, "succeed")
		return

	def __last_bookmark_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param trigger: Move to the last bookmark in the buffer.
		@type trigger: A Trigger object.
		"""
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)

		if not self.__manager.get_bookmarked_lines():
			from internationalization import msg0168
			self.editor.feedback.update_status_message(msg0168, "fail")
			return
		if len(self.__manager.get_bookmarked_lines()) == 1:
			from internationalization import msg0416
			self.editor.feedback.update_status_message(msg0416, "fail")
			return
		self.__manager.move_to_last_bookmark()
		from internationalization import msg0414
		self.editor.feedback.update_status_message(msg0414, "succeed")
		return

	def __show_bookmark_browser_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param trigger: An action to show the bookmark browser.
		@type trigger: A Trigger object.
		"""
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)

		if not self.__manager.get_bookmarked_lines():
			from internationalization import msg0428
			self.editor.feedback.update_status_message(msg0428, "fail")
			return
		try:
			self.__browser.show_dialog()
		except AttributeError:
			from bookmarkbrowser import BookmarkBrowser
			self.__browser = BookmarkBrowser(self.__manager)
			self.__browser.show_dialog()
		return

	def __trigger_loaded_document_cb(self, editor):
		"""
		Handles callback when the "loaded-document" signal is emitted.

		@param self: Reference to the BookmarkTriggers instance.
		@type self: A BookmarkTriggers object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from bookmark_metadata import get_bookmarks_from_database
		bookmarked_lines = get_bookmarks_from_database(str(editor.uri))
		if bookmarked_lines is None:
			return
		try:
			self.__manager.is_initialized
		except AttributeError:
			from bookmark import BookmarkManager
			self.__manager = BookmarkManager(self.editor)
		for line in bookmarked_lines:
			self.__manager.bookmark_line(line)
		return
