# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that implements bracket operations for the
text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class BracketManager(object):
	"""
	This class implements bracket operations for the text editor.
	Closing characters for most pair characters are automatically
	inserted into the textbuffer.
	"""

	def __init__(self, editor):
		"""
		Initialize the manager object.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param textview: The text editor's buffer container.
		@type textview: A ScribesTextView object.
		"""
		self.__init_attributes(editor)
		self.editor.triggermanager.add_trigger(self.trigger, "alt - b")
		self.textview.connect("key-press-event", self.__bracket_key_press_event_cb)
		self.editor.connect("loaded-document", self.__bracket_loaded_document_cb)
		self.editor.connect("cursor-moved", self.__bracket_cursor_moved_cb)
		self.trigger.connect("activate", self.__bracket_activate_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the manager's data attributes.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param textview: The text editor's buffer container.
		@type textview: A ScribesTextView object.
		"""
		self.textview = editor.textview
		self.editor = editor
		self.textbuffer = editor.textview.get_buffer()
		from gtksourceview import source_iter_find_matching_bracket as match
		self.match = match
		self.monitor_list = []
		# Trigger to bookmark a line.
		from trigger import Trigger
		self.trigger = Trigger("select_text_within_characters")
		from gtk import keysyms
		self.open_pair_characters = [keysyms.quotedbl,
			keysyms.braceleft, keysyms.bracketleft,
			keysyms.parenleft, keysyms.leftdoublequotemark,
			keysyms.guillemotleft, keysyms.guillemotright,
			keysyms.leftsinglequotemark, keysyms.leftmiddlecurlybrace,
			keysyms.lowleftcorner, keysyms.topleftparens,
			keysyms.topleftsqbracket, keysyms.upleftcorner,
			keysyms.botleftparens, keysyms.botleftsqbracket,
			keysyms.apostrophe]
		self.open_pair_characters_for_enclosement = self.open_pair_characters + [keysyms.less, keysyms.apostrophe]
		return

	def __bracket_key_press_event_cb(self, textview, event):
		"""
		Handles callback when "key-press-event" signal is emitted.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param textview: The text editor's buffer container.
		@type textview: A ScribesTextView object.

		@param event: An event that occurs when keys on the keyboard are pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		#from gtk.gdk import keyval_name
		#print keyval_name(event.keyval)
		selection = self.textbuffer.get_selection_bounds()
		if selection and event.keyval in self.open_pair_characters_for_enclosement:
			self.__enclose_selection(event.keyval)
			return True
		if self.monitor_list:
			from gtk import keysyms
			if event.keyval == keysyms.BackSpace:
				result = self.__remove_closing_pair_character()
				return result
			if event.keyval in (keysyms.Escape, self.monitor_list[-1][0]):
				self.__move_cursor_out_of_bracket_region()
				return True
		if event.keyval in self.open_pair_characters:
			self.__insert_closing_pair_character(event.keyval)
			return True
		return False

	def __bracket_cursor_moved_cb(self, editor):
		"""
		Handles callback when the "mark-set" signal is emitted.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.

		@param iterator: The position of the cursor.
		@type iterator: A gtk.TextIter object.

		@param mark: The position of the cursor.
		@type mark: A gtk.TextMark object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if not self.monitor_list:
			return False
		begin_mark = self.monitor_list[-1][1][0]
		end_mark = self.monitor_list[-1][1][1]
		begin = self.textbuffer.get_iter_at_mark(begin_mark)
		end = self.textbuffer.get_iter_at_mark(end_mark)
		from cursor import get_cursor_iterator
		cursor_position = get_cursor_iterator(self.textbuffer)
		if cursor_position.equal(begin) or cursor_position.equal(end):
			self.__stop_monitoring()
		elif cursor_position.in_range(begin, end) is False:
			self.__stop_monitoring()
		return False

	def __insert_closing_pair_character(self, keyval):
		"""
		Insert closing pair characters into the text editor's buffer.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param keyval: A value representing a keyboard symbol
		@type keyval: An Integer object.
		"""
		from gtk import keysyms
		if keyval == keysyms.quotedbl:
			self.__insert_pair_characters(keyval, keysyms.quotedbl)
		elif keyval == keysyms.braceleft:
			self.__insert_pair_characters(keyval, keysyms.braceright)
		elif keyval == keysyms.bracketleft:
			self.__insert_pair_characters(keyval, keysyms.bracketright)
		elif keyval == keysyms.parenleft:
			self.__insert_pair_characters(keyval, keysyms.parenright)
		elif keyval == keysyms.leftdoublequotemark:
			self.__insert_pair_characters(keyval, keysyms.righdoublequotemark)
		elif keyval == keysyms.guillemotleft:
			self.__insert_pair_characters(keyval, keysyms.guillemotright)
		elif keyval == keysyms.guillemotright:
			self.__insert_pair_characters(keyval, keysyms.guillemotleft)
		elif keyval == keysyms.leftsinglequotemark:
			self.__insert_pair_characters(keyval, keysyms.rightsinglequotemark)
		elif keyval == keysyms.leftmiddlecurlybrace:
			self.__insert_pair_characters(keyval, keysyms.rightmiddlecurlybrace)
		elif keyval == keysyms.lowleftcorner:
			self.__insert_pair_characters(keyval, keysyms.lowrightcorner)
		elif keyval == keysyms.topleftparens:
			self.__insert_pair_characters(keyval, keysyms.toprightparens)
		elif keyval == keysyms.topleftsqbracket:
			self.__insert_pair_characters(keyval, keysyms.toprightsqbracket)
		elif keyval == keysyms.upleftcorner:
			self.__insert_pair_characters(keyval, keysyms.uprightcorner)
		elif keyval == keysyms.botleftparens:
			self.__insert_pair_characters(keyval, keysyms.botrightparens)
		elif keyval == keysyms.botleftsqbracket:
			self.__insert_pair_characters(keyval, keysyms.botrightsqbracket)
		elif keyval == keysyms.less:
			self.__insert_pair_characters(keyval, keysyms.greater)
		elif keyval == keysyms.dollar:
			self.__insert_pair_characters(keyval, keysyms.dollar)
		elif keyval == keysyms.apostrophe:
			if self.__can_insert_apostrophe():
				self.__insert_pair_characters(keyval, keysyms.apostrophe)
			else:
				self.__insert_apostrophe()
		return

	def __bracket_loaded_document_cb(self, editor):
		"""
		Handles callback when the "loaded-document" signal is emitted.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from gnomevfs import get_mime_type
		mimetype = get_mime_type(str(self.editor.uri))
		if mimetype in ["text/x-tex"]:
			from gtk import keysyms
			self.open_pair_characters.append(keysyms.dollar)
		elif mimetype in ["text/html", "application/xml", "text/xml",
			"application/docbook+xml"]:
			from gtk import keysyms
			self.open_pair_characters.append(keysyms.less)
		return

	def __bracket_activate_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param trigger: A trigger for the pair character selection function.
		@type trigger: A Trigger object.
		"""
		if self.__select_text_within_pair_characters():
			from internationalization import msg0206
			self.editor.feedback.update_status_message(msg0206, "succeed")
		else:
			from internationalization import msg0205
			self.editor.feedback.update_status_message(msg0205, "fail")
		return

	def __insert_pair_characters(self, open_keyval, close_keyval):
		"""
		Insert pair characters into the text editor's buffer.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param open_keyval: A value representing the openning character.
		@type open_keyval: An Integer object.

		@param close_keyval: A value representing the closing character.
		@type close_keyval: An Integer object.
		"""
		from gtk.gdk import keyval_to_unicode
		utf8_open_character = unichr(keyval_to_unicode(open_keyval)).encode("utf-8")
		utf8_closing_character = unichr(keyval_to_unicode(close_keyval)).encode("utf-8")
		from cursor import get_cursor_iterator
		cursor_position = get_cursor_iterator(self.textbuffer)
		begin_mark = self.textbuffer.create_mark(None, cursor_position, True)
		self.textbuffer.begin_user_action()
		self.textbuffer.insert_at_cursor(utf8_open_character+utf8_closing_character)
		self.textbuffer.end_user_action()
		from cursor import get_cursor_iterator
		cursor_position = get_cursor_iterator(self.textbuffer)
		end_mark = self.textbuffer.create_mark(None, cursor_position, False)
		cursor_position.backward_char()
		self.textbuffer.place_cursor(cursor_position)
		self.monitor_list.append((close_keyval, (begin_mark, end_mark)))
		from internationalization import msg0154
		self.editor.feedback.update_status_message(msg0154, "succeed")
		return

	def __enclose_selection(self, keyval):
		"""
		Enclose selection in pair characters.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@param keyval: The value of a pair character.
		@type keyval: An Integer object.

		@param begin: The begining position of a selected text.
		@type begin: A gtk.TextIter object.

		@param end: The end position of a selected text.
		@type end: A gtk.TextIter object.
		"""
		from gtk import keysyms
		if keyval == keysyms.quotedbl:
			self.__insert_enclosed_selection(keysyms.quotedbl, keysyms.quotedbl)
		elif keyval == keysyms.braceleft:
			self.__insert_enclosed_selection(keysyms.braceleft, keysyms.braceright)
		elif keyval == keysyms.bracketleft:
			self.__insert_enclosed_selection(keysyms.bracketleft, keysyms.bracketright)
		elif keyval == keysyms.parenleft:
			self.__insert_enclosed_selection(keysyms.parenleft, keysyms.parenright)
		elif keyval == keysyms.leftdoublequotemark:
			self.__insert_enclosed_selection(keysyms.leftdoublequotemark, keysyms.rightdoublequotemark)
		elif keyval == keysyms.guillemotleft:
			self.__insert_enclosed_selection(keysyms.guillemotleft, keysyms.guillemotright)
		elif keyval == keysyms.guillemotright:
			self.__insert_enclosed_selection(keysyms.guillemotright, keysyms.guillemotleft)
		elif keyval == keysyms.leftsinglequotemark:
			self.__insert_enclosed_selection(keysyms.leftsinglequotemark, keysyms.rightsinglequotemark)
		elif keyval == keysyms.leftmiddlecurlybrace:
			self.__insert_enclosed_selection(keysyms.leftmiddlecurlybrace, keysyms.rightmiddlecurlybrace)
		elif keyval == keysyms.lowleftcorner:
			self.__insert_enclosed_selection(keysyms.lowleftcorner, keysyms.lowrightcorner)
		elif keyval == keysyms.topleftparens:
			self.__insert_enclosed_selection(keysyms.topleftparens, keysyms.toprightparens)
		elif keyval == keysyms.topleftsqbracket:
			self.__insert_enclosed_selection(keysyms.topleftsqbracket, keysyms.toprightsqbracket)
		elif keyval == keysyms.upleftcorner:
			self.__insert_enclosed_selection(keysyms.upleftcorner, keysyms.uprightcorner)
		elif keyval == keysyms.botleftparens:
			self.__insert_enclosed_selection(keysyms.botleftparens, keysyms.botrightparens)
		elif keyval == keysyms.botleftsqbracket:
			self.__insert_enclosed_selection(keysyms.botleftsqbracket, keysyms.botrightsqbracket)
		elif keyval == keysyms.less:
			self.__insert_enclosed_selection(keysyms.less, keysyms.greater)
		elif keyval == keysyms.dollar:
			self.__insert_enclosed_selection(keysyms.dollar, keysyms.dollar)
		elif keyval == keysyms.apostrophe:
			self.__insert_enclosed_selection(keysyms.apostrophe, keysyms.apostrophe)
		return

	def __insert_enclosed_selection(self, open_keyval, close_keyval):
		from gtk.gdk import keyval_to_unicode
		utf8_open_character = unichr(keyval_to_unicode(open_keyval)).encode("utf-8")
		utf8_closing_character = unichr(keyval_to_unicode(close_keyval)).encode("utf-8")
		selection = self.textbuffer.get_selection_bounds()
		string = self.textbuffer.get_text(selection[0], selection[1])
		text = utf8_open_character + string + utf8_closing_character
		self.textbuffer.begin_user_action()
		self.textbuffer.delete(selection[0], selection[1])
		self.textbuffer.insert_at_cursor(text)
		self.textbuffer.end_user_action()
		from internationalization import msg0207
		self.editor.feedback.update_status_message(msg0207, "succeed")
		return

	def __move_cursor_out_of_bracket_region(self):
		"""
		Move the cursor to the end of the closing pair character.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.
		"""
		end_mark = self.monitor_list[-1][1][1]
		iterator = self.textbuffer.get_iter_at_mark(end_mark)
		self.textbuffer.place_cursor(iterator)
		from cursor import move_view_to_cursor
		move_view_to_cursor(self.textview)
		return

	def __stop_monitoring(self):
		"""
		Stop monitoring the most recently inserted bracket region.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.
		"""
		begin_mark = self.monitor_list[-1][1][0]
		end_mark = self.monitor_list[-1][1][1]
		if begin_mark.get_deleted() is False:
			self.textbuffer.delete_mark(begin_mark)
		if end_mark.get_deleted() is False:
			self.textbuffer.delete_mark(end_mark)
		del self.monitor_list[-1]
		return

	def __remove_closing_pair_character(self):
		"""
		Remove closing pair character if it was automatically entered.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@return: True to handle keyboard events.
		@rtype: A Boolean object.
		"""
		begin_mark = self.monitor_list[-1][1][0]
		end_mark = self.monitor_list[-1][1][1]
		begin = self.textbuffer.get_iter_at_mark(begin_mark)
		end = self.textbuffer.get_iter_at_mark(end_mark)
		if len(self.textbuffer.get_text(begin, end)) != 2:
			return False
		begin.forward_char()
		from gtk.gdk import keyval_to_unicode
		close_keyval = self.monitor_list[-1][0]
		character = unichr(keyval_to_unicode(close_keyval)).encode("utf-8")
		if begin.get_char() != character:
			return False
		begin.backward_char()
		self.textbuffer.begin_user_action()
		self.textbuffer.delete(begin, end)
		self.textbuffer.end_user_action()
		from internationalization import msg0156
		self.editor.feedback.update_status_message(msg0156, "succeed")
		return True

	def __select_text_within_pair_characters(self):
		"""
		Select text within pair characters if possible.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@return: True if the operation succeeded.
		@rtype: A Boolean object.
		"""
		open_pair_characters = ["(", "[", "{", "<"]
		from cursor import get_cursor_iterator
		cursor_iterator = get_cursor_iterator(self.editor.textbuffer)
		transition_iterator = cursor_iterator.copy()
		while True:
			if transition_iterator.is_start():
				break
			transition_iterator.backward_char()
			if transition_iterator.get_char() in open_pair_characters:
				begin = transition_iterator.copy()
				if self.match(begin):
					if begin.compare(cursor_iterator) >= 0:
						transition_iterator.forward_char()
						self.editor.textbuffer.select_range(transition_iterator, begin)
						return True
		return False

	def __can_insert_apostrophe(self):
		"""
		Check whether or not a closing apostrophe can be automatically
		inserted into the text editor's buffer.

		Automatic completion will not be performed for apostrophes
		following an alphabet.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.

		@return: True to insert automatically insert apostrophe.
		@rtype: A Boolean object.
		"""
		from cursor import get_cursor_iterator
		iterator = get_cursor_iterator(self.editor.textbuffer)
		if iterator.starts_line():
			return True
		iterator.backward_char()
		character = iterator.get_char()
		if character.isalpha():
			return False
		return True

	def __insert_apostrophe(self):
		"""
		Insert a single apostrophe into the buffer.

		@param self: Reference to the BracketManager instance.
		@type self: A BracketManager object.
		"""
		from gtk import keysyms
		from gtk.gdk import keyval_to_unicode
		utf8_apostrophe_character = unichr(keyval_to_unicode(keysyms.apostrophe)).encode("utf-8")
		self.editor.textbuffer.insert_at_cursor(utf8_apostrophe_character)
		return
