# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that performs case changing operations for
the text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class CaseProcessor(object):
	"""
	This class implements an object responsible for changing the case of
	selected text in the text editor.
	"""

	def __init__(self, editor):
		"""
		Initialize the object.

		@param self: Reference to the CaseProcessor instance.
		@type self: A CaseProcessor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__init_attributes(editor)

	def __init_attributes(self, editor):
		"""
		Initialize the object's data attributes.

		@param self: Reference to the CaseProcessor instance.
		@type self: A CaseProcessor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		return

	def __replace_selection_with(self, string):
		"""
		Replace a selection in the text editor's buffer with string.

		@param self: Reference to the CaseProcessor instance.
		@type self: A CaseProcessor object.

		@param string: A string to insert into the text editor's buffer.
		@type string: A String object.
		"""
		begin_iterator, end_iterator = self.__editor.textbuffer.get_selection_bounds()
		begin_offset = begin_iterator.get_offset()
		end_offset = end_iterator.get_offset()
		self.__editor.textbuffer.begin_user_action()
		self.__editor.textbuffer.delete(begin_iterator, end_iterator)
		begin_iterator = self.__editor.textbuffer.get_iter_at_offset(begin_offset)
		self.__editor.textbuffer.insert(begin_iterator, string)
		self.__editor.textbuffer.end_user_action()
		begin_iterator = self.__editor.textbuffer.get_iter_at_offset(begin_offset)
		end_iterator = self.__editor.textbuffer.get_iter_at_offset(end_offset)
		self.__editor.textbuffer.select_range(begin_iterator, end_iterator)
		return

	def upper(self):
		"""
		Convert selected string in the text editor's buffer to uppercase.

		@param self: Reference to the CaseProcessor instance.
		@type self: A CaseProcessor object.
		"""
		selection = self.__editor.textbuffer.get_selection_bounds()
		if selection is None:
			from internationalization import msg0015
			self.__editor.feedback.update_status_message(msg0015, "fail")
			return
		text = self.__editor.textbuffer.get_text(selection[0], selection[1])
		unicode_text = text.decode("utf-8")
		if unicode_text.isupper():
			self.lower()
			return
		text = unicode_text.upper()
		text = text.encode("utf-8")
		self.__replace_selection_with(text)
		from internationalization import msg0017
		self.__editor.feedback.update_status_message(msg0017, "suceed")
		return

	def lower(self):
		"""
		Convert selected string in the text editor's buffer to lowercase.

		@param self: Reference to the CaseProcessor instance.
		@type self: A CaseProcessor object.
		"""
		selection = self.__editor.textbuffer.get_selection_bounds()
		if selection is None:
			from internationalization import msg0015
			self.__editor.feedback.update_status_message(msg0015, "fail")
			return
		text = self.__editor.textbuffer.get_text(selection[0], selection[1])
		unicode_text = text.decode("utf-8")
		if unicode_text.islower():
			self.upper()
			return
		text = unicode_text.lower()
		text = text.encode("utf-8")
		self.__replace_selection_with(text)
		from internationalization import msg0019
		self.__editor.feedback.update_status_message(msg0019, "suceed")
		return

	def title(self):
		"""
		Convert selected string in the text editor's buffer to lowercase.

		@param self: Reference to the CaseProcessor instance.
		@type self: A CaseProcessor object.
		"""
		selection = self.__editor.textbuffer.get_selection_bounds()
		if selection is None:
			from internationalization import msg0015
			self.__editor.feedback.update_status_message(msg0015, "fail")
			return
		text = self.__editor.textbuffer.get_text(selection[0], selection[1])
		unicode_text = text.decode("utf-8")
		if unicode_text.istitle():
			from internationalization import msg0151
			self.__editor.feedback.update_status_message(msg0151, "fail")
			return
		text = unicode_text.title()
		text = text.encode("utf-8")
		self.__replace_selection_with(text)
		from internationalization import msg0150
		self.__editor.feedback.update_status_message(msg0150, "suceed")
		return

	def swapcase(self):
		"""
		Convert selected string in the text editor's buffer to swapcase.

		@param self: Reference to the CaseProcessor instance.
		@type self: A CaseProcessor object.
		"""
		selection = self.__editor.textbuffer.get_selection_bounds()
		if selection is None:
			from internationalization import msg0015
			self.__editor.feedback.update_status_message(msg0015, "fail")
			return
		text = self.__editor.textbuffer.get_text(selection[0], selection[1])
		unicode_text = text.decode("utf-8")
		text = unicode_text.swapcase()
		text = text.encode("utf-8")
		self.__replace_selection_with(text)
		from internationalization import msg0149
		self.__editor.feedback.update_status_message(msg0149, "suceed")
		return
