# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates the background color button
for the text editor's color editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import ColorButton

class BackgroundButton(ColorButton):
	"""
	This class creates a color button for the text editor's color
	editor. The color button allows users to set the background color of
	text editor's buffer.
	"""

	def __init__(self, editor):
		"""
		Initialize the button.

		@param self: Reference to the BackgroundButton instance.
		@type self: A BackgroundButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		ColorButton.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__client.notify_add("/apps/scribes/bgcolor", self.__bgcolor_cb)
		self.__client.notify_add("/apps/scribes/use_theme_colors", self.__use_theme_colors_cb)
		self.connect("color-set", self.__color_set_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the button's data attributes.

		@param self: Reference to the BackgroundButton instance.
		@type self: A BackgroundButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		from gconf import client_get_default
		self.__client = client_get_default()
		return

	def __set_properties(self):
		"""
		Define the default behavior of the button.

		@param self: Reference to the BackgroundButton instance.
		@type self: A BackgroundButton object.
		"""
		bgcolor = self.__client.get_string("/apps/scribes/bgcolor")
		if bgcolor is None:
			bgcolor = "#ffffff"
		from gtk.gdk import color_parse
		bgcolor = color_parse(bgcolor)
		self.set_color(bgcolor)
		from internationalization import msg0439
		self.set_title(msg0439)
		from tooltips import foreground_button_tip
		self.__editor.tip.set_tip(self, foreground_button_tip)
		use_theme_colors = self.__client.get_bool("/apps/scribes/use_theme_colors")
		if use_theme_colors is None:
			use_theme_colors = True
		self.set_property("sensitive", not use_theme_colors)
		return

	def __bgcolor_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when background color changes.

		@param self: Reference to the BackgroundButton instance.
		@type self: A BackgroundButton object.
		"""
		bgcolor = client.get_string("/apps/scribes/bgcolor")
		color = self.get_color()
		from utils import convert_color_to_spec
		color = convert_color_to_spec(color)
		if bgcolor != color:
			from gtk.gdk import color_parse
			self.set_color(color_parse(bgcolor))
		return

	def __color_set_cb(self, button):
		"""
		Handles callback when the "toggled" signal is emitted.

		@param self: Reference to the BackgroundButton instance.
		@type self: A BackgroundButton object.

		@param button: Reference to the BackgroundButton.
		@type button: A BackgroundButton object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		bgcolor = self.get_color()
		from utils import convert_color_to_spec
		bgcolor = convert_color_to_spec(bgcolor)
		color = self.__client.get_string("/apps/scribes/bgcolor")
		if bgcolor != color:
			self.__client.set_string("/apps/scribes/bgcolor", bgcolor)
		return True

	def __use_theme_colors_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when background color changes.

		@param self: Reference to the BackgroundButton instance.
		@type self: A BackgroundButton object.
		"""
		use_theme_colors = self.__client.get_bool("/apps/scribes/use_theme_colors")
		if use_theme_colors is None:
			use_theme_colors = True
		if use_theme_colors:
			if self.get_property("sensitive"):
				self.set_property("sensitive", False)
		else:
			if self.get_property("sensitive") is False:
				self.set_property("sensitive", True)
		return
