# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class creates the text editor's color editor.
The color editor allows users to customize the foreground, background
and syntax colors of the text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from sdialog import Dialog

class ColorEditor(Dialog):
	"""
	This class creates a dialog that allows users to customize the
	foreground, background and syntax colors of the text editor. The
	dialog is frontend to entries in the GConf database.
	"""

	def __init__(self, editor):
		"""
		Initialize the dialog.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		Dialog.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__arrange_widgets()
		self.__close_button.connect("clicked", self.__clicked_cb)

	def __get__treeview(self):
		"""
		Get the treeview for the color editor.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.

		@return: Treeview for the color editor.
		@rtype: A ColorEditorTreeView object.
		"""
		return self.__treeview

########################################################################
#
#							Public API
#
########################################################################

	treeview = property(__get__treeview, doc="Treeview for the color editor")

########################################################################

	def __init_attributes(self, editor):
		"""
		Initialize the dialog's data attributes.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		self.__status_id = None
		self.__trigger = None
		from gtk import STOCK_CLOSE, Button
		self.__close_button = Button(stock=STOCK_CLOSE, use_underline=True)
		self.__label = self.__create_labels()
		from colorthemecheckbutton import ThemeCheckButton
		self.__theme_check_button = ThemeCheckButton(editor)
		from colorfgbutton import ForegroundButton
		self.__foreground_color_button = ForegroundButton(editor)
		from colorbgbutton import BackgroundButton
		self.__background_color_button = BackgroundButton(editor)
		from colortreeview import ColorEditorTreeView
		self.__treeview = ColorEditorTreeView(editor)
		from colorfgsyntaxbutton import ForegroundSyntaxButton
		self.__foreground_syntax_color_button = ForegroundSyntaxButton(editor, self)
		from colorbgsyntaxbutton import BackgroundSyntaxButton
		self.__background_syntax_color_button = BackgroundSyntaxButton(editor, self)
		from colorboldcheckbutton import BoldCheckButton
		self.__bold_check_button = BoldCheckButton(editor, self)
		from coloritaliccheckbutton import ItalicCheckButton
		self.__italic_check_button = ItalicCheckButton(editor, self)
		from colorunderlinecheckbutton import UnderlineCheckButton
		self.__underline_check_button = UnderlineCheckButton(editor, self)
		from colorresetbutton import ResetButton
		self.__reset_button = ResetButton(editor, self)
		return

	def show_dialog(self):
		"""
		Show the preferences dialog.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.
		"""
		self.__editor.emit("show-dialog", self)
		from internationalization import msg0321
		self.__status_id = self.__editor.feedback.set_modal_message(msg0321, "color")
		Dialog.show_dialog(self)
		return

	def hide_dialog(self):
		"""
		Hide the preferences dialog.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.
		"""
		self.__editor.emit("hide-dialog", self)
		self.__editor.feedback.unset_modal_message(self.__status_id)
		Dialog.hide_dialog(self)
		return

	def __set_properties(self):
		"""
		Define the default behavior of the dialog.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.
		"""
		self.set_property("name", "scribes color editor")
		from internationalization import msg0296
		self.set_property("title", msg0296)
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self.__editor.window, 2, 2.133333333)
		self.set_property("default-width", width)
		self.set_property("default-height", height)
		self.set_transient_for(self.__editor.window)
		return

	def __arrange_widgets(self):
		"""
		Arrange the widgets of the dialog.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.
		"""
		self.main_area.pack_start(self.__theme_check_button, False, False, 0)
		from gtk import HBox, VBox, Label, Alignment
		maincontainer = HBox(homogeneous=False, spacing=10)
		self.main_area.pack_start(maincontainer, False, False, 0)
		container = HBox(homogeneous=False, spacing=10)
		container.pack_start(self.__label[0], False, False, 0)
		container.pack_start(self.__foreground_color_button, True, True, 0)
		maincontainer.pack_start(container, True, True, 0)
		container = HBox(homogeneous=False, spacing=10)
		container.pack_start(self.__label[1], False, False, 0)
		container.pack_start(self.__background_color_button, True, True, 0)
		maincontainer.pack_start(container, True, True, 0)
		from utils import create_scrollwin
		scrollwin = create_scrollwin()
		scrollwin.add(self.__treeview)
		container = HBox(homogeneous=False, spacing=10)
		self.main_area.pack_start(container, True, True, 0)
		container.pack_start(scrollwin, True, True, 0)
		vcontainer = VBox(homogeneous=False, spacing=10)
		alignment = Alignment(yalign=0.5)
		alignment.add(vcontainer)
		container.pack_start(alignment, False, False, 0)
		container = HBox(homogeneous=False, spacing=10)
		container.pack_start(self.__label[2], False, False, 0)
		container.pack_start(self.__foreground_syntax_color_button, False, False, 0)
		vcontainer.pack_start(container, False, False, 0)
		container = HBox(homogeneous=False, spacing=10)
		container.pack_start(self.__label[3], False, False, 0)
		container.pack_start(self.__background_syntax_color_button, False, False, 0)
		vcontainer.pack_start(container, False, False, 0)
		vcontainer.pack_start(self.__bold_check_button, False, False, 0)
		vcontainer.pack_start(self.__italic_check_button, False, False, 0)
		vcontainer.pack_start(self.__underline_check_button, False, False, 0)
		vcontainer.pack_start(self.__reset_button, False, False, 0)
		self.button_area.pack_start(self.__close_button, False, False, 0)
		return

	def __clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.

		@param button: The close button.
		@type button: A Button object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_dialog()
		return True

	def __create_labels(self):
		"""
		Create all labels for the preferences dialog.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.

		@return: A list of labels for the dialog.
		@rtype: A List object.
		"""
		from internationalization import msg0298, msg0299, msg0442, msg0443
		string_list = [msg0298, msg0299, msg0442, msg0443]
		labels = []
		from gtk import Label
		for string in string_list:
			label = Label(string)
			label.set_use_underline(True)
			label.set_use_markup(True)
			labels.append(label)
		return labels
