# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that generates a list (dictionary to be
precise) of words eligible for automatic word completion.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_OBJECT

class WordCompletionListGenerator(GObject):
	"""
	This class creates an object responsible for generating word
	completion dictionary for the text editor. The dictionary's keys are
	words eligible for completion, while its value are the number of
	occurence of the words in the text editor's buffer.
	"""

	__gsignals__ = {
		"started-indexing": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"finished-indexing": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, editor):
		"""
		Initialize the word completion list generator.

		@param self: Reference to the WordCompletionListGenerator instance.
		@type self: A WordCompletionListGenerator object.

		@param textbuffer: The text editor's buffer
		@type textbuffer: A ScribesTextBuffer object.
		"""
		GObject.__init__(self)
		self.__init_attributes(editor)
		self.connect("started-indexing", self.__generator_started_indexing_cb)
		self.connect("finished-indexing", self.__generator_finished_indexing_cb)
		self.textbuffer.connect("changed", self.__generator_changed_cb)
		self.editor.connect("loaded-document", self.__generator_changed_cb)
		self.editor.connect("saved-document", self.__generator_saved_document_cb)
		self.editor.connect("renamed-document", self.__generator_renamed_document_cb)
		from gobject import timeout_add, PRIORITY_LOW
		timeout_add(7000, self.__generate_word_completion_dictionary, priority=PRIORITY_LOW)

	def __init_attributes(self, editor):
		"""
		Initialize the generator's attributes.

		@param self: Reference to the WordCompletionListGenerator
			instance.
		@type self: A WordCompletionListGenerator object.

		@param textbuffer: The text editor's buffer
		@type textbuffer: A ScribesTextBuffer object.
		"""
		from re import UNICODE, compile
		self.pattern = compile(r"[^-\w]", UNICODE)
		self.textbuffer = editor.textbuffer
		self.editor = editor
		self.is_indexing = False
		self.indexed = True
		self.file_size = 0
		return

	def __generator_started_indexing_cb(self, generator):
		"""
		Handles callback when the "started-indexing" signal is emitted.

		@param self: Reference to the WordCompletionListGenerator
			instance.
		@type self: A WordCompletionListGenerator object.
		"""
		self.__set_file_size()
		self.is_indexing = True
		return

	def __generator_finished_indexing_cb(self, generator):
		"""
		Handles callback when the "finished-indexing" signal is emitted.

		@param self: Reference to the WordCompletionListGenerator
			instance.
		@type self: A WordCompletionListGenerator object.
		"""
		self.is_indexing = False
		self.indexed = True
		return

	def __generator_changed_cb(self, textbuffer):
		"""
		Handles callback when the "changed" signal is emitted.

		@param textbuffer: The text editor's buffer
		@type textbuffer: A ScribesTextBuffer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.indexed = False
		return True

	def __generate_word_completion_dictionary(self):
		"""
		Generate the word completion dictionary.

		@param self: Reference to the WordCompletionListGenerator instance.
		@type self: A WordCompletionListGenerator object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		if self.indexed or self.is_indexing or self.editor.is_readonly:
			return True
		self.emit("started-indexing")
		begin_position, end_position = self.textbuffer.get_bounds()
		text = self.textbuffer.get_text(begin_position, end_position)
		if text:
			from re import split
			from utils import response
			if self.file_size > 100000:
				response()
			text_list = split(self.pattern, text.decode("utf-8"))
			word_list = []
			for items in text_list:
				if self.file_size > 100000:
					response()
				if len(items) > 3:
					word_list.append(items)
			if word_list:
				dictionary = {}
				# Index strings based on their occurence in the editor's
				# buffer and place them in a dictionary.
				for string in word_list:
					# Ensure the text editor's graphic user interface
					# does not lock during indexing operations,
					# especially, when indexing very large documents.
					# Ignore strings that are likely to be ASCII
					# embellishments.
					if self.file_size > 100000:
						response()
					if string.startswith("---") or string.startswith("___"):
						continue
					if string in dictionary.keys():
						dictionary[string] += 1
					else:
						dictionary[string] = 1
				self.textbuffer.word_completion_dictionary = dictionary
		self.emit("finished-indexing")
		return True

	def __generator_saved_document_cb(self, editor):
		"""
		Handles callback when the "saved-document" signal is emitted.

		@param self: Reference to the WordCompletionListGenerator instance.
		@type self: A WordCompletionListGenerator object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__set_file_size()
		return

	def __generator_renamed_document_cb(self, editor):
		"""
		Handles callback when the "renamed-document" signal is emitted.

		@param self: Reference to the WordCompletionListGenerator instance.
		@type self: A WordCompletionListGenerator object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__set_file_size()
		return

	def __set_file_size(self):
		"""
		Update the file size attribute.

		@param self: Reference to the WordCompletionListGenerator instance.
		@type self: A WordCompletionListGenerator object.
		"""
		if self.editor.uri:
			from utils import get_file_size
			self.file_size = get_file_size(str(self.editor.uri))
		return
