# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents the text editor's completion window's view object.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import SIGNAL_RUN_LAST, TYPE_NONE
from gtk import TreeView

class CompletionView(TreeView):
	"""
	This class implements an object that show a list of words elligible
	for word completion. Selecting and activating any item in the list
	inserts the selected word into the text editor's buffer.
	"""

	__gsignals__ = {
		"populated-model": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"inserted-text": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, completion):
		"""
		Intialize the view object.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.
		"""
		TreeView.__init__(self)
		self.__init_attributes(completion)
		self.__set_properties()
		self.completion.connect("matches-found", self.__view_matches_found_cb)
		self.connect("row-activated", self.__view_row_activated_cb)
		self.connect("button-press-event", self.__view_button_press_event)
		self.manager.connect("show-completion-window", self.__view_show_cb)
		self.manager.connect("hide-completion-window", self.__view_hide_cb)
		self.editor.textview.connect("key-press-event", self.__view_key_press_event_cb)

	def __init_attributes(self, completion_manager):
		"""
		Initialize the view object's data attributes.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.

		@param completion_manager: The text editor's completion manager
			object.
		@type completion_manager: A CompletionManager object.
		"""
		self.manager = completion_manager
		self.completion = completion_manager.completion
		self.editor = completion_manager.editor
		self.model = self.__create_model()
		self.renderer = self.__create_renderer()
		self.column = self.__create_column()
		self.word_list = []
		self.completion_list = []
		self.is_visible = False
		return

	def __set_properties(self):
		"""
		Define the view object's properties.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.
		"""
		self.append_column(self.column)
		self.set_headers_visible(False)
		self.set_property("rules-hint", True)
		self.set_property("hover-selection", True)
		self.set_property("model", self.model)
		from gtk.gdk import color_parse
		from gtk import STATE_SELECTED, STATE_ACTIVE
		style = self.editor.textview.get_style()
		color = style.base[STATE_SELECTED]
		self.modify_base(STATE_ACTIVE, color)
		return

	def __create_model(self):
		"""
		Create the view's model, or database.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.
		"""
		from gtk import ListStore
		model = ListStore(str)
		return model

	def __create_renderer(self):
		"""
		Create the view's text renderer.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.
		"""
		from gtk import CellRendererText
		renderer = CellRendererText()
		return renderer

	def __create_column(self):
		"""
		Create the view's column.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.
		"""
		from gtk import TreeViewColumn
		column = TreeViewColumn("", self.renderer, text=0)
		column.set_expand(False)
		return column

	def __populate_model(self):
		"""
		Populate the view's data model.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.
		"""
		if self.completion_list != self.word_list:
			self.word_list = self.completion_list
			self.model.clear()
			for word in self.word_list:
				self.model.append([word])
			self.realize()
			self.columns_autosize()
			self.get_selection().select_path(0)
		self.emit("populated-model")
		return

	def __view_matches_found_cb(self, completion):
		"""
		Handles callback when the "matches-found" signal is emitted.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.

		@param completion: The text editor's word completion object.
		@type completion: A WordCompletion object.
		"""
		self.completion_list = completion.completion_list
		self.__populate_model()
		return

	def __view_row_activated_cb(self, treeview, path, column):
		"""
		Handles callback when the "row-activated" signal is emitted.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.

		@param treeview: The text editor's completion window's view.
		@type treeview: A CompletionView object.

		@param path: An object representing a row in the view.
		@type path: A Path object.

		@param column: The text editor's completion window's view's column.
		@type column: A gtk.TreeViewColumn object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.__insert_word_completion(path)
		self.emit("inserted-text")
		return True

	def __view_button_press_event(self, treeview, event):
		"""
		Handles callback when the "button-press-event" signal is emitted.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.

		@param treeview: The text editor's completion window's view.
		@type treeview: A CompletionView object.

		@param event: An event that occurs when the right mouse button is pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		# Get the selected item on the completion window.
		selection = treeview.get_selection()

		# Get the model and iterator of the selected item.
		model, iterator = selection.get_selected()

		# Get the path and column of the selected item.
		path = model.get_path(iterator)
		column = treeview.get_column(0)

		# Activate the selected item to trigger word completion.
		treeview.row_activated(path, column)
		return True

	def __view_key_press_event_cb(self, treeview, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.

		@param treeview: The text editor's completion view.
		@type treeview: A CompletionView object.

		@param event: An event that occurs when keys are pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.is_visible is False:
			return False
		# Get the selected item on the completion window.
		selection = self.get_selection()

		# Get the model and iterator of the selected item.
		model, iterator = selection.get_selected()

		# If for whatever reason the selection is lost, select the first row
		# automactically when the up or down arrow key is pressed.
		if not iterator:
			selection.select_path((0,))
			model, iterator = selection.get_selected()
		path = model.get_path(iterator)
		from gtk import keysyms
		if event.keyval == keysyms.Return:
			# Insert the selected item into the editor's buffer when the enter key
			# event is detected.
			self.row_activated(path, self.get_column(0))
		elif event.keyval == keysyms.Up:
			# If the up key is pressed check to see if the first row is selected.
			# If it is, select the last row. Otherwise, get the path to the row
			# above and select it.
			if not path[0]:
				number_of_rows = len(model)
				selection.select_path(number_of_rows-1)
				self.scroll_to_cell(number_of_rows-1)
			else:
				selection.select_path((path[0]-1,))
				self.scroll_to_cell((path[0]-1,))
		elif event.keyval == keysyms.Down:
			# Get the iterator of the next row.
			next_iterator = model.iter_next(iterator)
			# If the next row exists, select it, if not select the first row.
			if next_iterator:
				selection.select_iter(next_iterator)
				path = model.get_path(next_iterator)
				self.scroll_to_cell(path)
			else:
				selection.select_path(0)
				self.scroll_to_cell(0)
		else:
			return False
		return True

	def __view_show_cb(self, manager):
		"""
		Handles callback when the "show-completion-window" signal is emitted.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.

		@param manager: The text editor's completion manager
		@type manager: A CompletionManager object.
		"""
		self.is_visible = True
		return

	def __view_hide_cb(self, manager):
		"""
		Handles callback when the "hide-completion-window" signal is emitted.

		@param self: Reference to the CompletionView instance.
		@type self: A CompletionView object.

		@param manager: The text editor's completion manager
		@type manager: A CompletionManager object.
		"""
		self.is_visible = False
		return

	def __insert_word_completion(self, path):
		"""
		Insert items selected in the completion window into the text editor's
		buffer.

		@param path: The selected row in the completion window.
		@type path: A gtk.TreeRow object.

		@param editor: Reference to the editor object.
		@type editor: An editor object.
		"""
		# Get the database containing potential completion string matches.
		model = self.get_model()

		# Get the selected completion string.
		completion_string = model[path[0]][0].decode("utf8")

		# Index to split completion string for insertion into the text editor's
		# buffer. Encode to utf8 before insertion.
		from cursor import get_word_before_cursor
		index = len(get_word_before_cursor(self.editor.textbuffer).decode("utf8"))
		string = completion_string[index:]

		# Split completion_string at the right index and insert into the editor's
		# buffer.
		self.editor.textbuffer.begin_user_action()
		self.editor.textbuffer.insert_at_cursor(string)
		self.editor.textbuffer.end_user_action()
		# Feedback to the status bar indicating word completion occurred.
		#update_status_info(editor, msg0155, "yes")
		from internationalization import msg0155
		self.editor.feedback.update_status_message(msg0155, "succeed")
		return
