# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents the implementation of the completion window object
for the text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import Window

class CompletionWindow(Window):
	"""
	This class implements a completion window for the text editor. The
	object determines when to show and hide the completion window as
	well as the size and position of the window.
	"""

	def __init__(self, completion_manager):
		"""
		Initialize the completion window object.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param completion_manager: Reference to the completion manager.
		@type completion_manager: A CompletionManager object.
		"""
		from gtk import WINDOW_POPUP
		Window.__init__(self, WINDOW_POPUP)
		self.__init_attributes(completion_manager)
		self.__set_properties()
		self.__signal_id_1 = self.view.connect("inserted-text", self.__window_inserted_text_cb)
		self.__signal_id_2 = self.completion.connect("no-matches-found", self.__window_no_matches_found_cb)
		self.__signal_id_3 = self.scrolledwindow.connect("policy-determined", self.__window_policy_determined_cb)
		self.__signal_id_4 = self.editor.window.connect("key-press-event", self.__window_key_press_event_cb)
		self.__signal_id_5 = self.editor.textview.connect("focus-out-event", self.__window_focus_out_cb)
		self.__signal_id_6 = self.editor.textbuffer.connect("delete-range", self.__window_delete_range_cb)
		self.__block_signals()

	def __init_attributes(self, completion_manager):
		"""
		Initialize the completion window object's data attributes

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param completion_manager: Reference to the completion manager.
		@type completion_manager: A CompletionManager object.
		"""
		self.manager = completion_manager
		self.__signals_are_blocked = True
		self.view = completion_manager.view
		self.scrolledwindow = completion_manager.scrolledwindow
		self.completion = completion_manager.completion
		self.editor = completion_manager.editor
		self.__signal_id_1 = None
		self.__signal_id_2 = None
		self.__signal_id_3 = None
		self.__signal_id_4 = None
		self.__signal_id_5 = None
		self.__signal_id_6 = None
		return

	def __set_properties(self):
		"""
		Define the completion window's default properties.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.
		"""
		from gtk.gdk import WINDOW_TYPE_HINT_MENU
		self.set_property("type-hint", WINDOW_TYPE_HINT_MENU)
		self.set_property("width-request", 200)
		self.set_property("height-request", 200)
		return

	def show_window(self):
		"""
		Show the completion window.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.
		"""
		from utils import response
		if self.__signals_are_blocked:
			self.__unblock_signals()
		self.show_all()
		self.manager.emit("show-completion-window")
		response()
		return

	def hide_window(self):
		"""
		Hide the completion window.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.
		"""
		if self.__signals_are_blocked is False:
			self.__block_signals()
		self.manager.emit("hide-completion-window")
		self.hide_all()
		return

########################################################################
#
#					Signal and Event Handlers
#
########################################################################

	def __window_policy_determined_cb(self, scrolled_window):
		"""
		Handles callback when the "policy-determined" signal is emitted.

		This method is called when the completion window's scrolled
		window finishes determining whether or not it should be
		displayed.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param scrolled_window: Reference to the completion window's
			scrolled window.
		@type scrolled_window: A CompletionScrolledWindow object.
		"""
		width, height = self.view.size_request()
		width = self.__calculate_window_width(width) + 3
		height = self.__calculate_window_height(height) + 3
		self.resize(width, height)
		self.set_property("width-request", width)
		self.set_property("height-request", height)
		self.__position_window(width, height)
		self.show_window()
		return

	def __window_no_matches_found_cb(self, completion):
		"""
		Handles called when the "no-matches-found" signal is emitted.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param completion: The text editor's word completion object.
		@type completion: A WordCompletion object.
		"""
		self.hide_window()
		return

	def __window_inserted_text_cb(self, view):
		"""
		Handles callback when the "inserted-text" signal is emitted.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param view: The completion window's view.
		@type view: A CompletionView object.
		"""
		self.hide_window()
		return

	def __window_delete_range_cb(self, textbuffer, start, end):
		"""
		Handles callback when the "delete-range" signal is emitted.

		This function is called when text is deleted from the text
		editor's buffer.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.

		@param start: The beginning position of the text removed.
		@type start: A gtk.TextIter object.

		@param end: The end position of the text removed.
		@type end: A gtk.TextIter object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_window()
		return True

	def __window_key_press_event_cb(self, window, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param window: The text editor's window.
		@type window: A ScribesWindow object.

		@param event: An event that occurs when keys are pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		from gtk import keysyms
		if event.keyval == keysyms.Escape:
			self.hide_window()
			return True
		if event.keyval == keysyms.Tab:
			self.hide_window()
#		if event.keyval in [keysyms.Return, keysyms.Down, keysyms.Up,
#			keysyms.Right, keysyms.Left]:
#			return True
		if event.keyval in [keysyms.Right, keysyms.Left]:
			self.hide_window()
		return False

	def __window_focus_out_cb(self, textview, event):
		"""
		Handles callback when the "focus-out-event" signal is emitted.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param textview: The text editor's buffer container.
		@type textview: A ScribesTextView object.

		@param event: An event the editor's view looses focus.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_window()
		return False

########################################################################
#
#							Helper Methods
#
########################################################################

	def __position_window(self, width, height):
		"""
		Position the completion window in the text editor's buffer.

		@param width: The completion window's width.
		@type width: An Integer object.

		@param height: The completion window's height.
		@type height: An Integer object.
		"""
		# Get the cursor's coordinate and size.
		from cursor import get_cursor_window_coordinates, get_cursor_size
		cursor_x, cursor_y = get_cursor_window_coordinates(self.editor.textview)
		cursor_height = get_cursor_size(self.editor.textview)[1]

		# Get the text editor's textview coordinate and size.
		from gtk import TEXT_WINDOW_TEXT
		window = self.editor.textview.get_window(TEXT_WINDOW_TEXT)
		rectangle = self.editor.textview.get_visible_rect()
		window_x, window_y = window.get_origin()
		window_width, window_height = rectangle.width, rectangle.height

		# Determine where to position the completion window.
		position_x = window_x + cursor_x
		position_y = window_y + cursor_y + cursor_height

		# If the completion window extends past the text editor's buffer,
		# reposition the completion window inside the text editor's buffer area.
		if (position_x + width) > (window_x + window_width):
			position_x = (window_x + window_width) - width
		if (position_y + height) > (window_y + window_height):
			position_y = (window_y + cursor_y) - height
		if self.__signals_are_blocked is False:
			if position_y != self.get_position()[1]:
				position_x = self.get_position()[0]
				self.move(position_x, position_y)
			return
		# Set the window's new position.
		self.move(position_x, position_y)
		return

	def __calculate_window_width(self, width):
		"""
		Calculate the completion window's height.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param height: The completion window's view's height.
		@type height: An Integer object.

		@return: The height of the completion window.
		@rtype: An Integer object.
		"""
		if width <= 200:
			width = 200
		else:
			width += 28
		return width

	def __calculate_window_height(self, height):
		"""
		Calculate the completion window's height.

		@param self: Reference to the CompletionWindow instance.
		@type self: A CompletionWindow object.

		@param height: The completion window's view's height.
		@type height: An Integer object.

		@return: The height of the completion window.
		@rtype: An Integer object.
		"""
		if height > 200:
			height = 200
		return height

	def __block_signals(self):
		self.view.handler_block(self.__signal_id_1)
		self.completion.handler_block(self.__signal_id_2)
#		self.scrolledwindow.handler_block(self.__signal_id_3)
		self.editor.window.handler_block(self.__signal_id_4)
		self.editor.textview.handler_block(self.__signal_id_5)
		self.editor.textbuffer.handler_block(self.__signal_id_6)
		self.__signals_are_blocked = True
		return

	def __unblock_signals(self):
		self.view.handler_unblock(self.__signal_id_1)
		self.completion.handler_unblock(self.__signal_id_2)
#		self.scrolledwindow.handler_unblock(self.__signal_id_3)
		self.editor.window.handler_unblock(self.__signal_id_4)
		self.editor.textview.handler_unblock(self.__signal_id_5)
		self.editor.textbuffer.handler_unblock(self.__signal_id_6)
		self.__signals_are_blocked = False
		return
