# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

class CursorMovement(object):
	"""
	This class implements an object that monitors the cursor's movement.
	"""

	def __init__(self, editor):
		"""
		Initialize the cursor tracking object.

		@param self: Reference to the CursorMovement instance.
		@type self: A CursorMovement object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__init_attributes(editor)
		from gobject import timeout_add
		self.__timeout_id = timeout_add(50, self.__check_cursor_position)
		self.textbuffer.connect("delete-range", self.__cursor_delete_range_cb)
		self.textbuffer.connect_after("delete-range", self.__cursor_delete_range_cb)
		self.editor.textview.connect("focus-in-event", self.__cursor_focus_in_event_cb)
		self.editor.textview.connect("focus-out-event", self.__cursor_focus_out_event_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the cursor tracking object's data attributes.

		@param self: Reference to the CursorMovement instance.
		@type self: A CursorMovement object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from utils import response
		self.__response = response
		self.editor = editor
		self.textbuffer = editor.textbuffer
		self.__timeout_id = None
		self.__cursor_position = self.textbuffer.get_property("cursor_position")
		return

	def __check_cursor_position(self):
		"""
		Emit the "cursor-moved" signal when the cursor position changes.

		@param self: Reference to the CursorMovement instance.
		@type self: A CursorMovement object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		if self.__cursor_position == self.textbuffer.get_property("cursor_position"):
			return True
		self.__response()
		self.editor.emit("cursor-moved")
		self.__cursor_position = self.textbuffer.get_property("cursor_position")
		self.__response()
		return True

	def __cursor_delete_range_cb(self, *args):
		"""
		Handles callback when the "delete-range" signal is emitted.

		@param self: Reference to the CursorMovement instance.
		@type self: A CursorMovement object.

		@param *args: Event handler arguments.
		@type *args: A List object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.editor.emit("cursor-moved")
		return False

	def __cursor_focus_in_event_cb(self, textview, event):
		"""
		Handles callback when the "focus-in-event" signal is emitted.

		@param self: Reference to the CursorMovement instance.
		@type self: A CursorMovement object.

		@param textview: The text editor's textview.
		@type textview: A ScribesTextView object.

		@param event: An event that occurs when the textview loses focus.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.__timeout_id is None:
			from gobject import timeout_add
			self.__timeout_id = timeout_add(50, self.__check_cursor_position)
		return False

	def __cursor_focus_out_event_cb(self, textview, event):
		"""
		Handles callback when the "focus-out-event" signal is emitted.

		@param self: Reference to the CursorMovement instance.
		@type self: A CursorMovement object.

		@param textview: The text editor's textview.
		@type textview: A ScribesTextView object.

		@param event: An event that occurs when the textview gains focus.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.__timeout_id:
			from gobject import source_remove
			source_remove(self.__timeout_id)
			self.__timeout_id = None
		return False
