# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents functions that initialize the text editor's
service object. The service object runs in a different process. Only
one instance of the service object can be run.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

def main(argv=None):
	"""
	Main entry point to the text editor.

	This function initializes the text editor's service. The service
	is a D-Bus object run on a separate process. The purpose of the
	service is to manage instances of the text editor. New instances of
	the text editor are spawned from the service. The service ensures
	that the text editor uses only one process. Multiple instances of
	the text editor can be spawned from the single process.

	This function also takes care of command line operations. Valid
	arguments are filenames. Valid options include --version(-v),
	--help(-h), --newfile(-n). If no arguments or options are passed to
	the text editor via the shell terminal, a single instance of the
	text editor is displayed.

	@param argv: Arguments passed to the text editor from the shell terminal.
	@type argv:	A List object.
	"""
	from info import dbus_iface
	services = dbus_iface.ListNames()
	scribes_service = "net.sourceforge.ScribesService"
	if argv:
		from commandline import CommandLineProcessor
		arguments = CommandLineProcessor(argv)
		uri_list = arguments.uri_list
		if uri_list:
			if scribes_service in services:
				from info import session_bus
				proxy_object = session_bus.get_object(scribes_service, "/net/sourceforge/ScribesService")
				proxy_object.new_files(uri_list, dbus_interface=scribes_service)
				raise SystemExit
			else:
				from gobject import timeout_add
				timeout_add(3, start_dbus_server)
				parameters = " ".join(uri_list)
				timeout_add(10, start_scribes, parameters)
				from main import fork_scribes
				fork_scribes()
		else:
			raise SystemExit
	else:
		if scribes_service in services:
			from info import session_bus
			proxy_object = session_bus.get_object(scribes_service, "/net/sourceforge/ScribesService")
			proxy_object.new_window(dbus_interface=scribes_service)
			raise SystemExit
		else:
			from gobject import timeout_add
			timeout_add(3, start_dbus_server)
			timeout_add(10, start_scribes)
			from main import fork_scribes
			fork_scribes()
	from gobject import MainLoop
	timeout_add(21000, start_psyco)
	mainloop = MainLoop()
	mainloop.run()
	return

def start_dbus_server():
	"""
	Start Scribes D-Bus server.
	"""
	from scribesservice import ScribesService
	ScribesService()
	return False

def start_scribes(parameters=None):
	"""
	Start the text editor.
	"""
	from subprocess import Popen
	from info import scribes_executable_client
	if parameters:
		Popen([scribes_executable_client, parameters])
	else:
		Popen([scribes_executable_client])
	return False

def start_psyco():
	"""
	Start the Psycho compiler.
	"""
	from utils import start_psyco
	start_psyco()
	return False
