# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates a D-Bus object for the text
editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from dbus.service import Object, method, signal

class ScribesDBusObject(Object):
	"""
	This class creates a D-Bus object for the text editor. The object
	implements, methods, signals and services that D-Bus capable
	applications, including the text editor, can interact with.
	"""

	def __init__(self, editor, dbus_name, object_path="/net/sourceforge/Scribes"):
		"""
		Initialize the D-Bus object.

		@param self: Reference to the ScribesDBusObject instance.
		@type self: A ScribesDBusObject object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param dbus_name: The D-Bus service name for the text editor.
		@type dbus_name: A String object.

		@param object_path: The D-Bus path for the text editor.
		@type object_path: A String object.
		"""
		Object.__init__(self, dbus_name, object_path)
		self.__init_attributes(editor)
		self.__session_bus.add_signal_receiver(self.__focus_window_cb,
						"focus_window",
						"net.sourceforge.ScribesService",
						"net.sourceforge.ScribesService",
						"/net/sourceforge/ScribesService")
		self.__session_bus.add_signal_receiver(self.__close_all_windows_cb,
						"close_all_windows",
						"net.sourceforge.ScribesService",
						"net.sourceforge.ScribesService",
						"/net/sourceforge/ScribesService")
		self.__session_bus.add_signal_receiver(self.__name_change_cb,
						'NameOwnerChanged',
						'org.freedesktop.DBus',
						'org.freedesktop.DBus',
						'/org/freedesktop/DBus',
						arg0='net.sourceforge.ScribesService')

	def __init_attributes(self, editor):
		"""
		Initialize the object's data attributes.

		@param self: Reference to the ScribesDBusObject instance.
		@type self: A ScribesDBusObject object.
		"""
		self.__editor = editor
		from dbus import SessionBus
		self.__session_bus = SessionBus()
		return

	@method("net.sourceforge.ScribesIFace")
	def get_uri(self):
		return self.__get_uri()

	@method("net.sourceforge.ScribesIFace")
	def open_new_window(self):
		return self.__open_new_window()

	@method("net.sourceforge.ScribesIFace")
	def open_new_files(self, uris):
		return self.__open_new_files(uris)

	@signal("net.sourceforge.ScribesIFace")
	def focus_window(self, uri):
		pass

	def __get_uri(self):
		if self.__editor.uri:
			return str(self.__editor.uri)
		return ""

	def __focus_window_cb(self, uri):
		"""
		Handles callback when the "focus_window" signal is received from
		D-Bus.

		@param self: Reference to the ScribesDBusObject instance.
		@type self: A ScribesDBusObject object.

		@param uri: The uri content in the text editor.
		@type uri: A String object.
		"""
		if self.__editor.uri is None:
			return
		if uri == str(self.__editor.uri):
			self.__editor.window.set_focus_on_map(True)
			xcoordinate, ycoordinate = self.__editor.window.get_position()
			window_is_maximized = self.__editor.window.is_maximized
			self.__editor.window.hide()
			if window_is_maximized is False:
				self.__editor.window.move(xcoordinate, ycoordinate)
			self.__editor.window.show_all()
		return

	def __name_change_cb(self, *args):
		from info import dbus_iface
		if not "net.sourceforge.ScribesService" in dbus_iface.ListNames():
			self.__editor.triggermanager.trigger("close_file")
		return

	def __open_new_window(self):
		self.__editor.triggermanager.trigger("new_file")
		return

	def __open_new_files(self, uris):
		self.__editor.fileloader.load_uris(uris)
		return

	def __close_all_windows_cb(self):
		self.__editor.triggermanager.trigger("close_file")
		return
