# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates a document browser for the
text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from sdialog import Dialog

class DocumentBrowser(Dialog):
	"""
	This class creates a document browser. The browser is a dialog that
	lists all documents opened by the text editor. Selecting and clicking
	a row in the dialog focuses the text editor window associated with
	the information on the row.
	"""

	def __init__(self, editor):
		"""
		Initialize the dialog.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		Dialog.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__arrange_widgets()
		self.__close_button.connect("clicked", self.__clicked_cb)
		self.__treeview.connect("row-activated", self.__row_activated_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the dialog's data attributes.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		self.__status_id = None
		from documentbrowsertreeview import DocumentBrowserTreeView
		self.__treeview = DocumentBrowserTreeView(editor)
		from gtk import STOCK_CLOSE, Button
		self.__close_button = Button(stock=STOCK_CLOSE, use_underline=True)
		return

	def __set_properties(self):
		"""
		Define the default behavior of the dialog.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.
		"""
		self.set_property("name", "scribes document browser")
		from internationalization import msg0449
		self.set_property("title", msg0449)
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self.__editor.window, 1.6, 2.5)
		self.set_property("default-width", width)
		self.set_property("default-height", height)
		self.set_transient_for(self.__editor.window)
		return

	def __arrange_widgets(self):
		"""
		Arrange the widgets of the dialog.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.
		"""
		from utils import create_scrollwin
		scrollwin = create_scrollwin()
		scrollwin.add(self.__treeview)
		self.main_area.pack_start(scrollwin, True, True, 0)
		self.button_area.pack_start(self.__close_button, False, False, 0)
		return

	def show_dialog(self):
		"""
		Determine whether or not to show the dialog.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.
		"""
		self.__editor.service_proxy_object.get_uris(dbus_interface="net.sourceforge.ScribesService",
					reply_handler=self.__reply_handler_cb,
					error_handler=self.__error_handler_cb)
		return

	def hide_dialog(self):
		"""
		Hide the dialog.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.
		"""
		self.__editor.emit("hide-dialog", self)
		self.__editor.feedback.unset_modal_message(self.__status_id)
		Dialog.hide_dialog(self)
		return

	def __clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.

		@param button: The close button.
		@type button: A Button object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_dialog()
		return True

	def __reply_handler_cb(self, uris):
		"""
		Handles callback when the service_proxy_object returns URIs.

		This function determines whether or not to show the document
		browser.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.

		@param uris: A list of URIs or nothing.
		@type uris: A List object.
		"""
		if not uris:
			from internationalization import msg0453
			self.__editor.feedback.update_status_message(msg0453, "warning")
			return
		if len(uris) < 2:
			if uris[0] in [str(self.__editor.uri)]:
				from internationalization import msg0454
				self.__editor.feedback.update_status_message(msg0454, "warning")
				return
		self.__treeview.populate_model(uris)
		self.__show_dialog()
		return

	def __show_dialog(self):
		"""
		Show the dialog.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.
		"""
		self.__editor.emit("show-dialog", self)
		from internationalization import msg0448
		self.__status_id = self.__editor.feedback.set_modal_message(msg0448, "open")
		Dialog.show_dialog(self)
		self.__treeview.grab_focus()
		return

	def __error_handler_cb(self, error):
		"""
		Handles callback when the service_proxy_object returns an error.

		@param self: Reference to the DocumentBrowser
		@type self: A DocumentBrowser object.

		@param error: An error message.
		@type error: A String object.
		"""
		print error
		return

	def __row_activated_cb(self, treeview, path, view_column):
		"""
		Handles callback when the "row-activated" signal is emitted.

		@param self: Reference to the DocumentBrowser instance.
		@type self: A DocumentBrowser object.

		@param treeview: The treeview for the document browser.
		@type treeview: A DocumentBrowserTreeView object.

		@param path: A row in the treeview.
		@type path: A Tuple object.

		@param view_column: A column in the treeview
		@type view_column: A gtk.TreeViewColumn object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_dialog()
		return True
