# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This modules documents a class that implements the treeview object for
the text editor's document browser.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import TreeView

class DocumentBrowserTreeView(TreeView):
	"""
	This class implements the treeview for the document browser. The
	view list all documents opened by the text editor.
	"""

	def __init__(self, editor):
		"""
		Initialize the treeview.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		TreeView.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__editor.connect("show-dialog", self.__treeview_show_dialog_cb)
		self.connect("row-activated", self.__treeview_row_activated_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the treeview's data attributes.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		self.__uri_list = []
		self.__model = self.__create_model()
		from internationalization import msg0450, msg0451, msg0452
		self.__column_one = self.__create_column(msg0451, 0)
		self.__column_two = self.__create_column(msg0450, 1)
		self.__column_three = self.__create_column(msg0452, 2)
		return

	def __set_properties(self):
		"""
		Defines the default property and behavior of the editor.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.
		"""
		self.set_model(self.__model)
		self.append_column(self.__column_one)
		self.append_column(self.__column_two)
		self.append_column(self.__column_three)
		self.columns_autosize()
		self.set_enable_search(True)
		self.set_search_column(0)
		self.set_property("rules-hint", True)
		self.__column_one.clicked()
		return

	def __create_model(self):
		"""
		Create the model for the treeview.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.

		@return: A model containing open URI information.
		@rtype: A gtk.ListStore object.
		"""
		# The model has four columns. One for the file type.
		# Another for the file name. Yet another for the full path name.
		# And the last for the file URI. Only the firt tree columns are
		# visible to users.
		from gtk import ListStore
		model = ListStore(str, str, str, str)
		return model

	def __create_column(self, title, column_position):
		"""
		Create a column for the treeview.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.

		@return: A column for the treeview.
		@rtype: A gtk.TreeViewColumn object.
		"""
		from gtk import TreeViewColumn, TREE_VIEW_COLUMN_GROW_ONLY
		from gtk import SORT_ASCENDING, CellRendererText
		column = TreeViewColumn(title, CellRendererText(), text=column_position)
		column.set_property("expand", False)
		column.set_property("sizing", TREE_VIEW_COLUMN_GROW_ONLY)
		column.set_property("clickable", True)
		column.set_sort_column_id(column_position)
		column.set_property("sort-indicator", True)
		column.set_property("sort-order", SORT_ASCENDING)
		return column

	def populate_model(self, uris):
		"""
		Populate the treeview with information about open documents.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.

		@param uris: A list of open URIs
		@type uris: A List object.
		"""
		uris.sort()
		self.__uri_list.sort()
		if uris == self.__uri_list:
			return
		self.__uri_list = uris
		self.__model.clear()
		for uri in self.__uri_list:
			file_type, filename, pathname, fileuri= self.__process_uri(uri)
			self.__model.append([filename, file_type, pathname, fileuri])
		return

	def __process_uri(self, uri):
		"""
		Split the uri into type, name and pathname.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.

		@param uri: Reference to a document.
		@type uri: A String object.
		"""
		value = ("", "", "", "")
		from utils import get_language
		language = get_language(uri)
		if language:
			file_type = language.get_name()
		else:
			file_type = "Plain Text"
		from gnomevfs import URI
		uri_object = URI(uri)
		filename = uri_object.short_name
		pathname = uri_object.path
		fileuri = uri
		return file_type, filename, pathname, fileuri

	def __treeview_show_dialog_cb(self, editor, dialog):
		"""
		Handles callback when the "show-dialog" signal is emitted.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param dialog: The dialog being displayed.
		@type dialog: A Dialog object.
		"""
		if not dialog.get_property("name") in ["scribes document browser"]:
			return
		self.grab_focus()
		return

	def __treeview_row_activated_cb(self, treeview, path, view_column):
		"""
		Handles callback when the "row-activated" signal is emitted.

		@param self: Reference to the DocumentBrowserTreeView instance.
		@type self: A DocumentBrowserTreeView object.

		@param treeview: The treeview for the document browser.
		@type treeview: A DocumentBrowserTreeView object.

		@param path: A row in the treeview.
		@type path: A Tuple object.

		@param view_column: A column in the treeview
		@type view_column: A gtk.TreeViewColumn object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		iterator = self.__model.get_iter(path)
		uri = self.__model.get_value(iterator, 3)
		self.__editor.service_proxy_object.focus_uri_window(uri,
					dbus_interface="net.sourceforge.ScribesService",
					reply_handler=self.__reply_handler_cb,
					error_handler=self.__error_handler_cb)
		return True

	def __reply_handler_cb(self, *args):
		return

	def __error_handler_cb(self, error):
		print error
		return
