# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or
# modify it under the # terms of the GNU General Public
# License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option) any
# later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public
# License along with # Scribes; if not, write to the Free
# Software Foundation, Inc., 51 Franklin St, ifth Floor,
# Boston, MA  02110-1301  USA

"""
This module exposes a class responsible for creating the graphic user interface
components of the text editor. The class also manages state information about
text editor instances.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_OBJECT
from gobject import type_register

class Editor(GObject):
	"""
	The Editor class can be described as an information center. It serves
	two distinct purposes. It creates graphic user interface widgets for the
	text editor, and it provides information about the attributes and general
	state of the text editor.

	In brief, an editor instance exists to expose its attributes (e.g. widgets)
	and state to appropriate parties. Auxiliary modules, classes or functions
	can then perform specialized operations on editor instances based on
	information the instances expose. For instance, the indentation module
	provides functions that operate on editor instances to perform indentation
	operations.

	Thus an editor instance is a library of information that allows modules,
	functions or classes to perform actions, usually text processing but not
	necessarily, on the text editor.
	"""

	__gsignals__ = {
		"checking-document": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"loading-document": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"loaded-document": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"load-error": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"enable-readonly": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"disable-readonly": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"saving-document": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"saved-document": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"save-document": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"save-error": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"gui-created": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"show-dialog": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_OBJECT,)),
		"hide-dialog": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_OBJECT,)),
		"renamed-document": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"close-document": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"close-document-no-save": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"created-widgets": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"enable-fullscreen": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"disable-fullscreen": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"not-yet-implemented": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"show-bar": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_OBJECT,)),
		"hide-bar": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_OBJECT,)),
		"updated-template-database": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"cursor-moved": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"initialized-trigger-manager": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"loading-plugins": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"loaded-plugins": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"activate-plugin": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_OBJECT,)),
		"deactivate-plugin": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_OBJECT,)),
		"initialized-attributes": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, uri_list=None):
		"""
		Initialize an instance of a text editor.

		This function creates the graphic user interface of the text
		editor, initializes the default state of the text editor and the
		GNOME libraries.

		@param self: Reference to the text editor instance.
		@type self: An Editor object.

		@param will_load_document: True if the editor will be created
			while loading a document.
		@type will_load_document: A Boolean object.
		"""
		GObject.__init__(self)
		from gobject import timeout_add, PRIORITY_HIGH
		# Initialize all the editor's attributes.
		timeout_add(3, self.__init_editor_attributes, uri_list, priority=PRIORITY_HIGH)
		# Connect the text editor to signals.
		self.connect("close-document", self.__editor_close_document_cb)
		self.connect("created-widgets", self.__editor_created_widgets_cb)
		self.connect("checking-document", self.__editor_checking_document_cb)
		self.connect("loading-document", self.__editor_loading_document_cb)
		self.connect("loaded-document", self.__editor_loaded_document_cb)
		self.connect("renamed-document", self.__editor_renamed_document_cb)
		self.connect("load-error", self.__editor_load_error_cb)
		self.connect("saving-document", self.__editor_saving_document_cb)
		self.connect("saved-document", self.__editor_saved_document_cb)
		self.connect("save-error", self.__editor_save_error_cb)
		self.connect("enable-readonly", self.__editor_enable_readonly_cb)
		self.connect("disable-readonly", self.__editor_disable_readonly_cb)
		self.connect("show-bar", self.__editor_show_bar_cb)
		self.connect("hide-bar", self.__editor_hide_bar_cb)
		self.connect("gui-created", self.__editor_gui_created_cb)
		self.connect_after("gui-created", self.__editor_gui_created_after_cb)
		self.connect("close-document-no-save", self.__editor_close_document_no_save_cb)
		self.connect("loaded-plugins", self.__editor_loaded_plugins_cb)
		self.connect("initialized-attributes", self.__editor_initialized_attributes_cb)
		from signal import signal, SIGHUP, SIGTERM, SIGSEGV
		signal(SIGHUP, self.__editor_kernel_signals_cb)
		#signal(SIGKILL, self.__editor_kernel_signals_cb)
		signal(SIGTERM, self.__editor_kernel_signals_cb)

	def __get_error_dialog(self):
		"""
		Return the error dialog for the text editor.

		@param self: Reference to the text editor.
		@type self: An Editor object.

		@return: An error dialog object for the text editor.
		@rtype: A ScribesErrorDialog object.
		"""
		return self.__error_dialog

	def __get_message_dialog(self):
		"""
		Return the error dialog for the text editor.

		@param self: Reference to the text editor.
		@type self: An Editor object.

		@return: An error dialog object for the text editor.
		@rtype: A ScribesErrorDialog object.
		"""
		return self.__message_dialog

	def __get_tooltip(self):
		"""
		Return the tooltip object for the text editor.

		@param self: Reference to the Editor instance.
		@type self: A Editor object.

		@return: A tooltip object.
		@rtype: A gtk.Tooltip object.
		"""
		return self.__tip

	def __get_preference_menu(self):
		"""
		Return the preference menu for the toolbar.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@return: Preference menu for the toolbar.
		@rtype: A PreferenceMenu object.
		"""
		return self.__preference_menu

	def __get_fileloader(self):
		"""
		Return an object that loads files.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@return: An object that loads files.
		@rtype: A LoadURI object.
		"""
		return self.__fileloader

	def __get_recent_manager(self):
		return self.__recent_manager

	def __get_recent_menu(self):
		return self.__recent_menu

########################################################################
#
#					Public API Properties
#
########################################################################

	error_dialog = property(__get_error_dialog, doc="Error dialog for the text editor.")
	message_dialog = property(__get_message_dialog, doc="Message dialog for the text editor.")
	tip = property(__get_tooltip, doc="Tooltip object for the text editor.")
	preference_menu = property(__get_preference_menu, doc="Preference menu for the toolbar.")
	fileloader = property(__get_fileloader, doc="Object used to load files.")
	recent_manager = property(__get_recent_manager, doc="Recent manager for the text editor.")
	recent_menu = property(__get_recent_menu, doc="Recent menu for the toolbar.")

########################################################################

	def __init_editor_attributes(self, uri_list):
		"""
		Initialize the text editor's attributes.

		This function sets the default values of the text editor's attributes.
		The attributes can be used by modules, classes or functions to interact
		with, or manipulate, instances of the text editor.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param will_load_document: True if the editor will load a document.
		@type will_load_document: A Boolean object.
		"""
		from sys import setcheckinterval
		setcheckinterval(10)
		from utils import response
		response()
		self.__uri_list = uri_list
		# True if the editor will load a document.
		self.will_load_document = False
		if uri_list:
			self.will_load_document = True
		# True if the editor's graphic user interface wigdets have been created.
		self.created_widgets = False
		# True if the editor's container objects have been created.
		self.created_containers = False
		# True if the editor's graphic user interface has been packed and
		# arranged.
		self.is_arranged = False
		# True if the editor is in readonly mode.
		self.is_readonly = False
		# True if the editor is in the process of loading a document.
		self.is_loading_document = False
		# True if the editor has successfully finished loading a document.
		self.loaded_document = False
		# True if the editor is in the process of saving a document.
		self.is_saving_document = False
		# True if the editor has successfully finished saving a document.
		self.saved_document = True
		# True to enable automatic saving.
		self.enable_automatic_saving = True
		# True if the editor is preparing to load a document.
		self.is_checking_document = False
		# True if the editor contains a document.
		self.contains_document = False
		self.bar_is_visible = False
		# True if all the graphic user interface components have been
		# created.
		self.gui_is_created = False
		self.load_error = False
		# Initialize the editor's widgets.
		self.window = None
		self.toolbar = None
		self.textbuffer = None
		self.textview = None
		self.statusone = None
		self.statustwo = None
		self.statusthree = None
		self.status_image_frame = None
		self.status_image = None

		# Create the tooltip object for the editor.
		from tooltips import create_tooltips
		self.__tip = create_tooltips()
		# Initialize the editor's containers.
		self.maincontainer = None
		self.toolbarcontainer = None
		self.viewcontainer = None
		self.statuscontainer = None

		from edialog import ScribesErrorDialog
		self.__error_dialog = ScribesErrorDialog(self)
		from mdialog import ScribesMessageDialog
		self.__message_dialog = ScribesMessageDialog(self)
		# Initialize the editor's feedback manager.
		self.feedback = None

		# Initialize the editor's trigger manager.
		self.triggermanager = None

		# The search manager allows clients to search the editor's buffer for
		# text. See search.py for implementation details.
		self.searchmanager = None

		from prefmenu import PreferenceMenu
		self.__preference_menu = PreferenceMenu(self)

		from recentmanager import RecentManager
		self.__recent_manager = RecentManager(self)()
		from recentmenu import RecentMenu
		self.__recent_menu = RecentMenu(self)

		# Initialize the text editor's default URI.
		#self.__uri = uri
		#self.__readonly = readonly
		self.uri = None
		# The encoding of the file to open or save.
		self.encoding = None

		# An object that loads files.
		from fileloader import LoadURI
		self.__fileloader = LoadURI(self)
		# The text editor quits only after the termination queue is empty. This
		# allows objects that need to perform cleanup or metadata operation
		# to register a unique number in the queue. Registration should occur
		# during the creation of the object. The object is responsible for
		# removing it identification from the queue.
		self.termination_queue = []
		# Register a unique number with the editor's termination queue
		self.termination_id = self.register_termination_id()
		from info import session_bus
		self.service_proxy_object = session_bus.get_object("net.sourceforge.ScribesService", "/net/sourceforge/ScribesService")
		self.emit("initialized-attributes")
		return

	def __create_editor_widgets(self):
		"""
		Create key graphic user interface components for the text editor.

		This function creates attributes of the text editor that represent
		graphic user interface widgets. Modules, classes or functions accross
		the project can interact with, or manipulate, these widgets to perform
		specialized operations. This function is called repeatedly unless its
		return value is False.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.
		"""
		from utils import response
		response()
		from window import ScribesWindow
		self.window = ScribesWindow(self)
		from maincontainer import ScribesMainContainer
		self.maincontainer = ScribesMainContainer(self)
		from toolbarcontainer import ScribesToolbarContainer
		self.toolbarcontainer = ScribesToolbarContainer(self)
		from viewcontainer import ScribesViewContainer
		self.viewcontainer = ScribesViewContainer(self)
		from statuscontainer import ScribesStatusContainer
		self.statuscontainer = ScribesStatusContainer(self)
		# Create the text editor's buffer.
		from textbuffer import ScribesTextBuffer
		self.textbuffer = ScribesTextBuffer(self)
		# Create the text editor's view. The view is a container for the
		# text editor's buffer.
		from textview import ScribesTextView
		self.textview = ScribesTextView(self)
		# Create the text editor's toolbar.
		from toolbar import ScribesToolbar
		self.toolbar = ScribesToolbar(self)
		# Create statusbar image.
		from gtk import Image
		self.status_image  = Image()
		# A container to hold the statusbar image.
		from gtk import Frame
		self.status_image_frame = Frame()
		# Create the text editor's statusbars.
		from statusbar import StatusOne, StatusTwo, StatusThree
		self.statusone  = StatusOne(self)
		self.statustwo = StatusTwo(self)
		self.statusthree = StatusThree(self)
		self.emit("created-widgets")
		response()
		return False

	def __arrange_editor_widgets(self):
		"""
		Arrange text editor widgets then display it.

		After the text editor graphic user interface components are created,
		this function arranges the widgets by packing them into the editor's
		main containers. After which, the text editor is displayed. This
		function is called repeatedly unless its return value is False.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.
		"""
		from utils import response
		response()
		# Arrange the editor's containers.
		#self.window.add(self.maincontainer)
		self.maincontainer.pack_start(self.toolbarcontainer, False, False, 0)
		self.maincontainer.pack_start(self.viewcontainer, True, True, 1)
		self.maincontainer.pack_start(self.statuscontainer, False, False, 0)

		# Scroll window for the text editor.
		from utils import create_scrollwin
		scrollwin = create_scrollwin()
		scrollwin.add(self.textview)

		# Frame that holds the statusbar image.
		from gtk import SHADOW_IN
		self.status_image_frame.set_shadow_type(SHADOW_IN)
		self.status_image_frame.add(self.status_image)
		self.status_image.clear()
		self.status_image.hide_all()
		self.status_image_frame.hide_all()

		# Pack widgets into the editor's containers.
		self.toolbarcontainer.pack_start(self.toolbar, True, True, 0)
		self.viewcontainer.pack_start(scrollwin, True, True, 0)
		self.statuscontainer.pack_start(self.status_image_frame, False, False,0)
		self.statuscontainer.pack_start(self.statusone, True, True, 0)
		self.statuscontainer.pack_start(self.statustwo, False, True, 0)
		self.statuscontainer.pack_start(self.statusthree, False, True, 0)
		self.maincontainer.resize_children()
		self.emit("gui-created")
		response()
		return False

########################################################################
#
#					Object Termination Register Methods
#
########################################################################

	def register_termination_id(self):
		"""
		Register a unique identification with the text editor.

		Objects that need to perform special operations before the text editor
		quits, need to call this method when they are initially created. This
		function generates a unique identification number for objects. The
		identification number should be used as an argument to the
		unregister_termination_id method after the object has finished performing
		its special operation, so that the text editor can quit successfully.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@return: A unique identification number.
		@rtype: A Float object.
		"""
		from utils import generate_random_number
		termination_id = generate_random_number(self.termination_queue)
		self.termination_queue.append(termination_id)
		return termination_id

	def unregister_termination_id(self, termination_id):
		"""
		Remove unique identification number from the text editor's termination
		registration queue.

		This function disassociates an object from the text editor so that the
		editor can proceed to quit successfully.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param termination_id: A unique identification number
		@type termination_id: A Float object.
		"""
		if termination_id in self.termination_queue:
			self.termination_queue.remove(termination_id)
		return

################################################################################
#
#	 					GNOME Library Registration
#
################################################################################

	def __init_gnome_libs(self):
		"""
		Initialize the GNOME libraries.

		The text editor is a GNOME application. This function initializes the
		GNOME libraries as a matter of protocol required by all GNOME
		applications.

		@note: This feature is broken. There are no good documentations on how
			to properly initialize the GNOME libraries for Python applications.
			As a result I'm not sure if I'm doing things correctly. I'll look
			into this later. I hope to focus on better GNOME integration in
			future versions of the text editor.
		@param self: Reference to the Editor instance.
		@type self: A Editor object.
		"""
		from gnome import init, PARAM_ENABLE_SOUND, PARAM_APP_DATADIR
		from info import version, name, scribes_data_path
		from gnome.ui import libgnomeui_module_info_get
		properties = {
				PARAM_ENABLE_SOUND: True,
				PARAM_APP_DATADIR: scribes_data_path,
			}
		init(name, version, properties=properties)
		return False

########################################################################
#
#	 						Helper Methods
#
########################################################################

	def __quit_editor(self):
		"""
		Wait for objects to perform their final operations then quit the
		application.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		if self.termination_queue:
			return True
		self.service_proxy_object.unregister_object_path(self.dbus_object_path,
						dbus_interface="net.sourceforge.ScribesService",
						reply_handler=self.__unregister_reply_handler,
						error_handler=self.__error_handler)
		return False

	def __quit_interpreter(self, number_of_instances):
		from gtk import main_quit
		if number_of_instances:
			self.__destroy_widgets()
			self.__delete_editor_object()
			main_quit()
		else:
			main_quit()
			from os import _exit
			_exit(0)
		return

	def __destroy_widgets(self):
		self.textview.destroy()
		self.toolbar.destroy()
		self.statusone.destroy()
		self.statustwo.destroy()
		self.statusthree.destroy()
		self.toolbarcontainer.destroy()
		self.viewcontainer.destroy()
		self.statuscontainer.destroy()
		self.maincontainer.destroy()
		self.window.destroy()
		del self.textview, self.toolbar, self.statusone, self.statustwo
		del self.statusthree, self.toolbarcontainer, self.viewcontainer
		del self.statuscontainer, self.maincontainer, self.window
		return

	def __delete_editor_object(self):
		from gc import collect #, get_referrers, get_referents
#		for objects in get_referrers(self):
#			print
#			print "----------------------------------------------------"
#			print objects
#			print "----------------------------------------------------"
#			print
#			del objects
#		for objects in get_referents(self):
#			print
#			print "----------------------------------------------------"
#			print objects
#			print "----------------------------------------------------"
#			print
#			del objects
		del self
		collect()
		return

	def __update_encoding_metadata(self, uri):
		"""
		Update the encoding database with information about the encoding of the
		text editor's content.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param uri: A universal resource identifier representing, or pointing
			to, a text document.
		@type uri: A String object.
		"""
		if self.uri:
			from encoding_metadata import update_encoding_in_database
			update_encoding_in_database(uri, self.encoding)
		self.unregister_termination_id(self.termination_id)
		return False

################################################################################
#
#						Signal and Event Handlers
#
################################################################################

	def __editor_close_document_cb(self, editor):
		"""
		Handles callback when the "close-document" signal is emitted.

		This function allow
		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from gobject import timeout_add
		timeout_add(10, self.__update_encoding_metadata, str(self.uri))
		timeout_add(50, self.__quit_editor)
		return

	def __editor_close_document_no_save_cb(self, editor):
		"""
		Handles callback when the "close-document-no-save" signal is
		emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.termination_queue = []
		self.__quit_editor()
		return

	def __editor_initialized_attributes_cb(self, editor):
		from gobject import timeout_add, PRIORITY_HIGH
		timeout_add(3, self.__create_editor_widgets, priority=PRIORITY_HIGH)
		return

	def __editor_created_widgets_cb(self, editor):
		"""
		Handles callback when the "created-widgets" signal is emitted.

		This function arranges the text editor wigdets after they have
		been created.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from gobject import timeout_add, PRIORITY_HIGH, PRIORITY_LOW
		# Create D-Bus object for the text editor.
		timeout_add(7, self.__init_dbus_object, priority=PRIORITY_HIGH)
		from feedback import ScribesFeedbackManager
		self.feedback = ScribesFeedbackManager(self)
		timeout_add(3, self.__arrange_editor_widgets, priority=PRIORITY_HIGH)
		# Initialize the GNOME libraries.
		timeout_add(21, self.__init_gnome_libs, priority=PRIORITY_LOW)
		if self.__uri_list:
			from thread import start_new_thread
			start_new_thread(self.__fileloader.load_uris, (self.__uri_list,))
		return

	def __editor_checking_document_cb(self, editor):
		"""
		Handles callback when the "checking-document" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_checking_document = True
		return

	def __editor_loading_document_cb(self, editor):
		"""
		Handles callback when the "loading-document" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_loading_document = True
		self.loaded_document = False
		self.is_checking_document = False
		return

	def __editor_loaded_document_cb(self, editor):
		"""
		Handles callback when the "loaded-document" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.loaded_document = True
		self.contains_document = True
		self.is_loading_document = False
		self.is_checking_document = False
		self.service_proxy_object.update_uris(
						dbus_interface="net.sourceforge.ScribesService",
						reply_handler=self.__update_uris_reply_handler,
						error_handler=self.__error_handler)
		return

	def __editor_load_error_cb(self, editor):
		"""
		Handles callback when the "load-error" signal is emitted.

		This function resets a few text editor attributes to their
		default state. See the __init_editor_attributes method for
		commentary on the attributes.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.load_error = True
		self.uri = None
		self.is_readonly = None
		self.opendialog = None
		self.savedialog = None
		self.will_load_document = False
		self.filesaver = None
		self.is_loading_document = False
		self.loaded_document = False
		self.is_saving_document = False
		self.saved_document = True
		self.is_checking_document = False
		return

	def __editor_saving_document_cb(self, editor):
		"""
		Handles callback when the "saving-document" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_saving_document = True
		self.saved_document = False
		return

	def __editor_saved_document_cb(self, editor):
		"""
		Handles callback when the "saved-document" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.saved_document = True
		self.is_saving_document = False
		self.enable_automatic_saving = True
		return

	def __editor_save_error_cb(self, editor):
		"""
		Handles callback when the "save-error" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.enable_automatic_saving = False
		self.saved_document = False
		self.is_saving_document = False
		return

	def __editor_renamed_document_cb(self, editor):
		"""
		Handles callback when the "renamed-document" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.saved_document = True
		self.is_saving_document = False
		self.enable_automatic_saving = True
		self.contains_document = True
		self.service_proxy_object.update_uris(dbus_interface="net.sourceforge.ScribesService",
						reply_handler=self.__update_uris_reply_handler,
						error_handler=self.__error_handler)
		return

	def __editor_enable_readonly_cb(self, editor):
		"""
		Handles callback when the "enable-readonly" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_readonly = True
		self.enable_automatic_saving = False
		return

	def __editor_disable_readonly_cb(self, editor):
		"""
		Handles callback when the "disable-readonly" signal is emitted.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_readonly = False
		self.enable_automatic_saving = True
		return

	def __editor_loaded_plugins_cb(self, editor):
		from utils import response
		from gobject import timeout_add, PRIORITY_LOW
		timeout_add(3, response)
		# Initialize Psyco after 21 seconds.
		timeout_add(21000, self.__start_psyco, priority=PRIORITY_LOW)
		timeout_add(3, response)
		return

	def __editor_gui_created_cb(self, editor):
		# True if the graphic user interface components are successfully
		# created.
		self.gui_is_created = True
		return

	def __editor_gui_created_after_cb(self, editor):
		# Initialize the trigger manager.
		from actions import create_actions
		editor.triggermanager = create_actions(editor)
		editor.emit("initialized-trigger-manager")

		# Initialize auxiliary services.
		from services import init_services
		from thread import start_new_thread
		start_new_thread(init_services, (self,))

		from gobject import timeout_add, PRIORITY_LOW
		from utils import response
		# Initialize Psyco after 21 seconds.
		timeout_add(21000, self.__start_psyco, priority=PRIORITY_LOW)
		timeout_add(3, response)

		# Initialize plug-ins.
#		from PluginManager import ScribesPluginManager
#		start_new_thread(ScribesPluginManager, (self,))

		# Garbage collector.
		from gc import collect
		collect()
		return

	def __editor_show_bar_cb(self, editor, bar):
		self.bar_is_visible = True
		return

	def __editor_hide_bar_cb(self, editor, bar):
		self.bar_is_visible = False
		return

	def __start_psyco(self):
		"""
		Start Psyco, the Python Just in Time compiler.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		self.service_proxy_object.can_start_psyco(
						dbus_interface="net.sourceforge.ScribesService",
						reply_handler=self.__can_start_psyco_cb,
						error_handler=self.__error_handler)
		return False

	def __can_start_psyco_cb(self, can_start_psyco):
		if can_start_psyco is False:
			return
		self.service_proxy_object.started_psyco(
						dbus_interface="net.sourceforge.ScribesService",
						reply_handler=self.__started_psyco_cb,
						error_handler=self.__started_psyco_error_cb)
		# Start the pysco virtual matchine if available.
		from utils import start_psyco
		start_psyco()
		return

	def __started_psyco_cb(self):
		return

	def __started_psyco_error_cb(self, error):
		print error
		return

	def __editor_kernel_signals_cb(self, *args):
		self.emit("close-document")
		return

	def __init_dbus_object(self):
		"""
		Initialize the D-Bus object for the text editor.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.
		"""
		from info import session_bus
		from dbus.service import BusName
		from time import time
		from random import random
		string = str(int(time())) + str(random()).lstrip("0.")
		name = self.dbus_name = "net.sourceforge.Scribes" #+ string
		self.dbus_object_path = "/net/sourceforge/Scribes/" + string
		bus_name = BusName(name, bus=session_bus)
		from dbusobject import ScribesDBusObject
		self.dbus_object = ScribesDBusObject(self, bus_name, self.dbus_object_path)
		self.service_proxy_object.register_object_path(self.dbus_object_path,
						dbus_interface="net.sourceforge.ScribesService",
						reply_handler=self.__register_reply_handler,
						error_handler=self.__error_handler)
		return False

	def __register_reply_handler(self, *args):
		return

	def __unregister_reply_handler(self, *args):
		self.service_proxy_object.update_uris(dbus_interface="net.sourceforge.ScribesService",
						reply_handler=self.__update_uris_reply_handler,
						error_handler=self.__error_handler)
		return

	def __update_uris_reply_handler(self, *args):
		if self.termination_queue:
			return
		self.service_proxy_object.get_number_of_instances(dbus_interface="net.sourceforge.ScribesService",
						reply_handler=self.__quit_interpreter,
						error_handler=self.__error_handler)
		return

	def __error_handler(self, error):
		print error
		print "In editor.py"
		return
