# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates the export dialog for the
text editor's template editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from sdialog import Dialog

class ExportDialog(Dialog):
	"""
	This class implements the export dialog for the template editor.
	"""

	def __init__(self, export_button):
		"""
		Initialize the dialog.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.

		@param export_button: The template dialog's export button.
		@type export_button: An TemplateEditorExportButton object.
		"""
		Dialog.__init__(self)
		self.__init_attributes(export_button)
		self.__set_properties()
		self.__arrange_widgets()
		self.connect("map-event", self.__dialog_map_event_cb)
		self.activate_button.connect("clicked", self.__dialog_clicked_cb)
		self.filechooser_entry.connect("activate", self.__dialog_file_activated_cb)
		self.filechooser_entry.connect("changed", self.__dialog_changed_cb)

	def __init_attributes(self, export_button):
		"""
		Initialize the dialog's data attributes.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.

		@param export_button: The template dialog's export button.
		@type export_button: An TemplateEditorExportButton object.
		"""
		self.template_editor = export_button.template_editor
		self.language_view = self.template_editor.language_view
		self.description_view = self.template_editor.description_view
		self.editor = self.template_editor.editor
		self.activate_button = self.__create_activate_button()
		self.filechooser_widget = self.__create_filechooser_widget()
		children = self.filechooser_widget.get_children()
		self.filechooser_entry = children[0].get_children()[0].get_children()[0].get_children()[2]
		return

	def __set_properties(self):
		"""
		Define the dialog's properties.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.
		"""
		from internationalization import msg0427
		self.set_property("title", msg0427)
		self.set_property("icon-name", "stock_save")
		self.set_transient_for(self.template_editor.window)
		return

	def __arrange_widgets(self):
		"""
		Arrange the widgets for the dialog.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.
		"""
		self.main_area.pack_start(self.filechooser_widget, True, True, 0)
		self.button_area.pack_start(self.cancel_button, False, False, 0)
		self.button_area.pack_start(self.activate_button, False, False, 0)
		return

	def __create_activate_button(self):
		"""
		Create the import button for the dialog.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.

		@return: An import button for the dialog.
		@rtype: A gtk.Button object.
		"""
		from gtk import STOCK_UNDO, Button
		from internationalization import msg0419
		from utils import create_button
		hbox = create_button(STOCK_UNDO, msg0419)
		button = Button()
		button.add(hbox)
		return button

	def __create_filechooser_widget(self):
		"""
		Create the wigdet used to select files.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.

		@return: A file chooser widget.
		@rtype: A gtk.FileChooserWidget object.
		"""
		from gtk import FileChooserWidget, FILE_CHOOSER_ACTION_SAVE
		widget = FileChooserWidget(FILE_CHOOSER_ACTION_SAVE)
		from dialogfilter import create_filter
		from internationalization import msg0351
		xml_filter = create_filter(msg0351, "text/xml")
		widget.add_filter(xml_filter)
		from info import home_folder, desktop_folder
		from os import path
		if path.exists(desktop_folder):
			widget.set_current_folder(desktop_folder)
		else:
			widget.set_current_folder(home_folder)
		return widget

	def __get_selected_templates(self):
		"""
		Get selected templates in the description view.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@return: Return list containing information about selected templates.
		@rtype: A List object.
		"""
		templates = []
		language_id = self.__get_language_id()
		selection = self.description_view.get_selection()
		model, paths = selection.get_selected_rows()
		for path in paths:
			trigger = self.__get_template_trigger(path)
			description = self.__get_template_description(path)
			template = self.__get_template(language_id+trigger)
			templates.append((language_id, trigger, description, template))
		return templates

	def __get_language_id(self, column=1):
		"""
		Get the language identification for the template.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@return: The language identification for the template.
		@rtype: A String object.
		"""
		selection = self.language_view.get_selection()
		model, iterator = selection.get_selected()
		language_id = model.get_value(iterator, column)
		return language_id

	def __get_template_trigger(self, path):
		"""
		Get the template trigger.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@param path: A row in the description view.
		@type path: A Path object.

		@return: The template trigger.
		@rtype: A String object.
		"""
		model = self.description_view.get_model()
		iterator = model.get_iter(path)
		trigger = model.get_value(iterator, 0)
		return trigger

	def __get_template_description(self, path):
		"""
		Get the template description.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@param path: A row in the description view.
		@type path: A Path object.

		@return: A description for the template.
		@rtype: A String object.
		"""
		model = self.description_view.get_model()
		iterator = model.get_iter(path)
		description = model.get_value(iterator, 1)
		return description

	def __get_template(self, template_key):
		"""
		Get the template from the template database.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@param template_key: The template key for a template value.
		@type template_key: A String object.

		@return: A template for the text editor.
		@rtype: A String object.
		"""
		from template_metadata import open_template_database
		from template_metadata import close_template_database
		database = open_template_database()
		template = database[template_key][1]
		close_template_database(database)
		return template

	def __dialog_map_event_cb(self, dialog, event):
		"""
		Handles callback when the "map-event" signal is emitted.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.

		@param dialog: Reference to the ExportDialog.
		@type dialog: An ExportDialog object.

		@param event: An event that occurs when the dialog is displayed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		name = self.__get_language_id(0) + "-" + "templates.xml"
		self.filechooser_widget.set_current_name(name)
		self.filechooser_entry.grab_focus()
		return True

	def __dialog_clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@param button: The dialog's export dialog.
		@type button: A gtk.Button object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_dialog()
		self.template_editor.window.set_property("sensitive", False)
		name = self.filechooser_widget.get_filename()
		if name:
			if name.endswith(".xml") is False:
				name = name + ".xml"
			if self.__can_write_file(name):
				from utils import response
				response()
				templates = self.__get_selected_templates()
				from writexmltemplate import create_template_file
				response()
				create_template_file(templates, name)
				response()
			else:
				self.editor.error_dialog.show_message("No permission to write file.", parent_window=self)
		else:
			self.editor.error_dialog.show_message("Name field cannot be empty.", parent_window=self)
		self.template_editor.window.set_property("sensitive", True)
		self.editor.emit("updated-template-database")
		return True

	def __dialog_changed_cb(self, filechooser_entry):
		"""
		Handles callback when the "changed" signal is emitted.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.

		@param filechooser_entry: The dialog's entry widget.
		@type filechooser_entry: A gtk.FileChooserEntry object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.activate_button.set_property("sensitive", False)
		if filechooser_entry.get_text():
			self.activate_button.set_property("sensitive", True)
		return True

	def __can_write_file(self, filename):
		"""
		Whether or not there is permission to write a file.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.

		@param filename: The path to check for permissions
		@type filename: A String object.

		@return: True if there is permission to write to the file path.
		@rtype: A Boolean object.
		"""
		try:
			from os import access, F_OK, W_OK, path
			folder = path.dirname(filename)
			if access(folder, F_OK) is False:
				raise Exception
			if access(folder, W_OK) is False:
				raise Exception
		except:
			return False
		return True

	def __dialog_file_activated_cb(self, filechooser_widget):
		"""
		Handles callback when the "file-activated" signal is emitted.

		@param self: Reference to the ExportDialog instance.
		@type self: An ExportDialog object.

		@param filechooser_widget: The dialog's filechooser_widget.
		@type filechooser_widget: A gtk.FileChooserWidget object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.activate_button.activate()
		return True
