# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module implements a class that manages feedback information relayed to
users via the text editor's statusbar.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class ScribesFeedbackManager(object):
	"""
	This class creates an object responsible for sending information about the
	state and operations of the text editor to users. Both visual and textual
	elements are used to convey feedback messages. Visual elements are
	represented by stock icons. While textual elements are just ordinary
	strings with informative messages.
	"""

	def __init__(self, editor):
		"""
		Initialize the feedback system.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: Reference to a text editor instance.
		@type editor: An Editor object.
		"""
		self.__init_attributes(editor)
		self.__remove_status_message()
		self.__signal_id_1 = self.editor.connect("loading-document", self.__feedback_loading_document_cb)
		self.__signal_id_2 = self.editor.connect("loaded-document", self.__feedback_loaded_document_cb)
#		self.editor.connect("saving-document", self.__feedback_saving_document_cb)
		self.__signal_id_3 = self.editor.connect("saved-document", self.__feedback_saved_document_cb)
		self.__signal_id_4 = self.editor.connect("renamed-document", self.__feedback_renamed_document_cb)
		self.__signal_id_5 = self.editor.connect("not-yet-implemented", self.__feedback_not_yet_implemented_cb)
		self.__signal_id_6 = self.editor.connect("enable-readonly", self.__feedback_enable_readonly_cb)
		self.__signal_id_7 = self.editor.connect("disable-readonly", self.__feedback_disable_readonly_cb)
		self.__signal_id_8 = self.editor.connect("gui-created", self.__feedback_gui_created_cb)
		self.__signal_id_9 = self.editor.connect("close-document", self.__feedback_close_document_cb)
		self.__signal_id_11 = self.editor.connect("close-document-no-save", self.__feedback_close_document_cb)
		self.__signal_id_10 = self.editor.textbuffer.connect("changed", self.__feedback_changed_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the feedback manager's attributes.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: Reference to a text editor instance.
		@type editor: An Editor object.
		"""
		self.editor = editor
		self.editor.feedback = self
		self.icon_dictionary = self.__generate_feedback_icons()
		self.timer = None
		self.modal_stack = []
		self.default_message = None
		self.is_enabled = True
		self.modal_message_id = 0
		self.spinner_is_active = False
		self.spinner_count = 0
		self.__signal_id_1 = None
		self.__signal_id_2 = None
		self.__signal_id_3 = None
		self.__signal_id_4 = None
		self.__signal_id_5 = None
		self.__signal_id_6 = None
		self.__signal_id_7 = None
		self.__signal_id_8 = None
		self.__signal_id_9 = None
		self.__signal_id_10 = None
		self.__signal_id_11 = None
		self.__registration_id = self.editor.register_termination_id()
		return

	def update_status_message(self, message=None, icon=None, timer=3):
		"""
		Update the statusbar area with feedback information.

		This function can be used as a callback to a GObject asynchronous
		timeout function. Thus, it can be called asynchronously. The function
		updates the statusbar with the current state of the text editor, or
		the operation the editor is performing. The statusbar image also changes
		to reflect these changes.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param message: Message informing users of the state of the text editor.
		@type message: A String object.

		@param icon: An image representing the operation the text editor is
			currently performing, or whether or not an operation suceeded.
		@type icon: A String object.

		@param timer: How long the feedback message should be displayed in
			seconds.
		@type timer: An Integer object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		from gobject import timeout_add, source_remove
		try:
			source_remove(self.timer)
		except:
			pass
		if message:
			timeout_add(10, self.__set_status_message, message)
		if icon in self.icon_dictionary.keys():
			stock_icon = self.icon_dictionary[icon]
			timeout_add(10, self.update_status_image, stock_icon)
		try:
			timer *= 1000
			self.timer = timeout_add(timer, self.__reset_status_message)
		except:
			self.timer = timeout_add(5000, self.__reset_status_message)
		if icon in ("fail", "warning"):
			from gtk.gdk import beep
			beep()
		self.is_busy = True
		return False

	def update_status_image(self, stock_icon):
		"""
		Update the status bar image with a stock icon.

		This function can be used as a callback to a GObject asynchronous
		timeout function. Thus, it can be called asynchronously.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param stock_icon: An stock icon representing the state of the text
			editor or an operation the editor is performing.
		@type stock_icon: A gtk.STOCK_* object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		# An object representing the statusbar image.
		image = self.editor.status_image
		image_frame = self.editor.status_image_frame

		# Remove the current stock icon and replace it with a new one.
		image.clear()
		from gtk import ICON_SIZE_MENU
		image.set_from_stock(stock_icon, ICON_SIZE_MENU)
		image_frame.show_all()
		return False

	def set_modal_message(self, message=None, icon=None):
		"""
		Update the statusbar area with feedback.

		This function updates the statusbar area with a feedback message and an
		icon complementing the message. The feedback remains in the statusbar
		area indefinitely until it is unset by calling the unset_modal_message
		method. The function returns a unique number which should be used as an
		argument when calling the unset_modal_message method.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param message: Feedback message displayed at the text editor's
			statusbar.
		@type message: A String object.

		@param icon: A string representing a GTK+ stock icon to display at the
			text editor's statusbar area.
		@type icon: A String object.

		@return: A unique number associated with the feedback.
		@rtype: A Float object.

		"""
		try:
			from gobject import source_remove, timeout_add
			source_remove(self.timer)
		except:
			pass
		status_id = self.__generate_random_number()
		self.modal_stack.append((message, icon, status_id))
		timeout_add(1, self.__set_status_message, message)
		stock_icon = self.icon_dictionary[icon]
		timeout_add(1, self.update_status_image, stock_icon)
		return status_id

	def unset_modal_message(self, status_id, reset=True):
		"""
		Removal modal feedback messages from the text editor's statusbar area.

		This function removes feedback messages and associated icons set using
		the set_modal_message method.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param status_id: A unique floating number associated with a modal
			feedback message and associated icon.
		@type status_id: A Float object.

		@param reset: True to reset the statusbar to its default message, False
			otherwise.
		@type reset: A Boolean object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		for item in self.modal_stack:
			if status_id in item:
				self.modal_stack.remove(item)
		if reset:
			from gobject import timeout_add
			timeout_add(1, self.__reset_status_message)
		return False

	def format_statusbar_filename_for_display(self):
		"""
		Format the filename of the document contained in the text editor for
		display.

		This function converts the URI of the document contained in the text
		editor to a human readable filename for display. It also shortens the
		filename if it is long. For example, /home/foo/bar will be be shortened
		to ~/bar. Only names of local files are formatted. Names of files
		situated at remote locations are left unformatted.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.
		"""
		statusbar_filename = uri = str(self.editor.uri)
		if self.editor.uri.is_local or uri.startswith("file:///"):
			from gnomevfs import get_local_path_from_uri
			local_path = get_local_path_from_uri(uri)
			from info import home_folder
			if local_path.startswith(home_folder):
				statusbar_filename = local_path.replace(home_folder, "~")
			else:
				statusbar_filename = local_path
		return statusbar_filename

	def __remove_status_image(self):
		"""
		Remove the statusbar image.

		This function can be used as a callback to a GObject asynchronous
		timeout function. Thus, it can be called asynchronously.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.

		"""
		try:
			image_frame = self.editor.status_image_frame
			image_frame.hide_all()
			image = self.editor.status_image
			image.clear()
		except:
			pass
		return False

	def __remove_status_message(self):
		"""
		Remove the message and icon in the text editor's statusbar area.

		This function can be used as a callback to a GObject asynchronous
		timeout function. Thus, it can be called asynchronously.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.

		"""
		self.is_busy = False
		from gobject import timeout_add
		timeout_add(10, self.__remove_status_image)
		statusbar = self.editor.statusone
		statusbar.pop(statusbar.context_id)
		statusbar.push(0, "")
		return False

	def __set_status_message(self, message):
		"""
		Push feedback message to the text editor's statusbar.

		This function can be used as a callback to a GObject asynchronous
		timeout function. Thus, it can be called asynchronously.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param message: Feedback message to relay to users.
		@type message: A String object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.

		"""
		statusbar = self.editor.statusone
		statusbar.pop(statusbar.context_id)
		context_id = statusbar.get_context_id(message)
		statusbar.push(context_id, message)
		return False

	def __reset_status_message(self):
		"""
		Automatically determine the default feedback message and image.

		This function can be used as a callback to a GObject asynchronous
		timeout function. Thus, it can be called asynchronously.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		try:
			from gobject import timeout_add
			if self.modal_stack and self.editor.is_readonly is False:
				message = self.modal_stack[-1][0]
				icon = self.modal_stack[-1][1]
				stock_icon = self.icon_dictionary[icon]
				timeout_add(1, self.update_status_image, stock_icon)
				timeout_add(1, self.__set_status_message, message)
				return False
			if self.editor.uri is None:
				self.__remove_status_message()
				return False
			filename = self.format_statusbar_filename_for_display()
			from internationalization import msg0044, msg0045
			message = self.default_message = msg0045 % filename
			if self.editor.is_readonly:
				stock_icon = self.icon_dictionary["readonly"]
				timeout_add(1, self.update_status_image, stock_icon)
				message = self.default_message = msg0044 % filename
				timeout_add(1, self.__set_status_message, message)
			elif self.editor.saved_document:
				stock_icon = self.icon_dictionary["new"]
				timeout_add(1, self.update_status_image, stock_icon)
				timeout_add(1, self.__set_status_message, message)
			else:
				stock_icon = self.icon_dictionary["edit"]
				timeout_add(1, self.update_status_image, stock_icon)
				timeout_add(1, self.__set_status_message, message)
		except AttributeError:
			pass
		return False

	def __generate_feedback_icons(self):
		"""
		Generate GTK+ stock icons to representing feedback images.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@return: A dictionary of stock icons.
		@rtype: A Dictionary object.
		"""
		from gtk import STOCK_YES, STOCK_NO, STOCK_OPEN, STOCK_EDIT, STOCK_SAVE
		from gtk import STOCK_SAVE_AS, STOCK_NEW, STOCK_PRINT, STOCK_UNDO
		from gtk import STOCK_REDO, STOCK_JUMP_TO, STOCK_FIND
		from gtk import STOCK_FIND_AND_REPLACE, STOCK_PREFERENCES
		from gtk import STOCK_DIALOG_WARNING, STOCK_DIALOG_AUTHENTICATION
		from gtk import STOCK_DIALOG_ERROR, STOCK_COPY, STOCK_CUT, STOCK_PASTE
		from gtk import STOCK_SPELL_CHECK, STOCK_HELP, STOCK_EXECUTE
		from gtk import STOCK_DIALOG_INFO, STOCK_ABOUT, STOCK_SELECT_COLOR
		from gtk import STOCK_STOP
		icon_dictionary = {
			"succeed": STOCK_YES,
			"suceed": STOCK_YES,
			"fail": STOCK_NO,
			"open": STOCK_OPEN,
			"edit": STOCK_EDIT,
			"save": STOCK_SAVE,
			"saveas": STOCK_SAVE_AS,
			"new": STOCK_NEW,
			"print": STOCK_PRINT,
			"undo": STOCK_UNDO,
			"redo": STOCK_REDO,
			"goto": STOCK_JUMP_TO,
			"find": STOCK_FIND,
			"replace": STOCK_FIND_AND_REPLACE,
			"preferences": STOCK_PREFERENCES,
			"warning": STOCK_DIALOG_WARNING,
			"readonly": STOCK_DIALOG_AUTHENTICATION,
			"error": STOCK_DIALOG_ERROR,
			"copy" : STOCK_COPY,
			"cut": STOCK_CUT,
			"paste": STOCK_PASTE,
			"spell": STOCK_SPELL_CHECK,
			"help": STOCK_HELP,
			"run": STOCK_EXECUTE,
			"info": STOCK_DIALOG_INFO,
			"about": STOCK_ABOUT,
			"color": STOCK_SELECT_COLOR,
			"stop": STOCK_STOP,
		}
		return icon_dictionary

	def __generate_random_number(self):
		"""
		Generate a random floating point number between 0.0 and 1.0.

		This function generates a unique number that is associated with a
		feedback message. The number is used to keep track modal feedback
		messages in a stack.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@return: A unique floating point number.
		@rtype: A Float object.

		"""
		from random import random
		while True:
			exit = True
			status_id = random()
			if self.modal_stack:
				for item in self.modal_stack:
					if status_id in item:
						exit = False
						break
			if exit:
				break
		return status_id

	def disable_feedback_system(self):
		return False

	def enable_feeback_system(self):
		return False

################################################################################
#
#						Signal and Event Handlers
#
################################################################################

	def __feedback_loading_document_cb(self, editor):
		"""
		Handles callback when the text editor is loading a document.

		This function informs users via the statusbar area that the editor is
		in the process of loading a document.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		from internationalization import msg0032
		self.modal_message_id = self.set_modal_message(msg0032, "run")
		from gobject import timeout_add
		timeout_add(10, self.start_spinner_animation)
		return

	def __feedback_loaded_document_cb(self, editor):
		"""
		Handles callback when the text editor has finished loading a document.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		from gobject import timeout_add
		from internationalization import msg0030
		filename = self.format_statusbar_filename_for_display()
		message = msg0030 % filename
		timeout_add(1, self.unset_modal_message, self.modal_message_id, False)
		timeout_add(10, self.update_status_message, message, "open")
		timeout_add(10, self.stop_spinner_animation)
		return

	def __feedback_saving_document_cb(self, editor):
		"""
		Handles callback when the text editor has saved the contents of its
		buffer.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: An instance of the text editor
		@type editor: An Editor object.
		"""
		from gobject import timeout_add
#		timeout_add(10, self.start_spinner_animation)
		return

	def __feedback_saved_document_cb(self, editor):
		"""
		Handles callback when the text editor has saved the contents of its
		buffer.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: An instance of the text editor
		@type editor: An Editor object.
		"""
		from gobject import timeout_add
		if editor.statusone.text == self.default_message:
			filename = self.format_statusbar_filename_for_display()
			from internationalization import msg0085
			message = msg0085 % filename

			timeout_add(1, self.update_status_message, message, "save")
#		timeout_add(10, self.stop_spinner_animation)
		return

	def __feedback_changed_cb(self, textbuffer):
		"""
		Handles callback when the editor's buffer's "changed" signal is emitted.

		This function changes the statusbar image to an edit stock icon when
		text is entered into the text editor's buffer. The icon is changed only
		if the default feedback message is displayed.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: An instance of the text editor
		@type editor: An Editor object.

		"""
		if self.editor.statusone.text == self.default_message:
			from gtk import STOCK_EDIT
			if self.editor.status_image.get_stock()[0] != STOCK_EDIT:
				self.update_status_image(STOCK_EDIT)
		return True

	def __feedback_renamed_document_cb(self, editor):
		"""
		Handles callback when the "renamed-document" signal is emitted.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__reset_status_message()
		return

	def __feedback_not_yet_implemented_cb(self, editor):
		"""
		Handles callback when the "not-yet-implemented" signal is emitted.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		message = "Not yet implemented"
		self.update_status_message(message, "warning")
		return

	def __feedback_enable_readonly_cb(self, editor):
		"""
		Handles callback when the "enabe-readonly" signal is emitted.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from internationalization import msg0322
		self.update_status_message(msg0322, "suceed")
		return

	def __feedback_disable_readonly_cb(self, editor):
		"""
		Handles callback when the "disable-readonly" signal is emitted.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from internationalization import msg0324
		self.update_status_message(msg0324, "suceed")
		return

	def __feedback_gui_created_cb(self, editor):
		"""
		Handles callback when the "gui-created" signal is emitted.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__remove_status_message()
		return

	def start_spinner_animation(self):
		"""
		Start the throbber animation on the text editor's toolbar.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		self.spinner_count += 1
		if self.spinner_is_active:
			return False
		self.spinner_is_active = True
		self.editor.spinner.inactive_spinner_image.clear()
		self.editor.spinner.inactive_spinner_image.set_from_animation(self.editor.spinner.animation)
		from utils import response
		response()
		return False

	def stop_spinner_animation(self):
		"""
		Stop the throbber animation on the text editor's toolbar.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		self.spinner_count -= 1
		if self.spinner_count < 1:
			self.spinner_count = 0
			self.spinner_is_active = False
			self.editor.spinner.inactive_spinner_image.clear()
			self.editor.spinner.inactive_spinner_image.set_from_pixbuf(self.editor.spinner.pixbuf)
		from utils import response
		response()
		return False

	def __feedback_close_document_cb(self, editor):
		"""
		Handles callback when the "close-document" signal is emitted.

		@param self: Reference to the ScribesFeedbackManager instance.
		@type self: A ScribesFeedbackManager object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from sys import path
#		self.__remove_status_image()
#		self.__remove_status_message()
		self.editor.disconnect(self.__signal_id_1)
		self.editor.disconnect(self.__signal_id_2)
		self.editor.disconnect(self.__signal_id_3)
		self.editor.disconnect(self.__signal_id_4)
		self.editor.disconnect(self.__signal_id_5)
		self.editor.disconnect(self.__signal_id_6)
		self.editor.disconnect(self.__signal_id_7)
		self.editor.disconnect(self.__signal_id_8)
		self.editor.disconnect(self.__signal_id_9)
		self.editor.disconnect(self.__signal_id_11)
		self.editor.textbuffer.disconnect(self.__signal_id_10)
		del self.icon_dictionary, self.editor, self.modal_stack
		del self.timer, self.default_message, self.is_enabled
		del self.modal_message_id, self.spinner_is_active
		del self.spinner_count
		editor.unregister_termination_id(self.__registration_id)
		del self
		return
