# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that can load multiple documents into the
text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class LoadURI(object):
	"""
	The class creates an object that can load multiple URIs into the
	text editor. The class ensures only one instance a document exists.
	"""

	def __init__(self, editor):
		"""
		Initialize the URI loader.

		@param self: Reference to the LoadURI instance.
		@type self: A LoadURI object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__init_attributes(editor)

	def __init_attributes(self, editor):
		"""
		Initialize the loader's data attributes.

		@param self: Reference to the LoadURI instance.
		@type self: A LoadURI object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__uri_list = None
		self.__editor = editor
		return

	def load_uris(self, uri_list):
		"""
		Load a list of URIs into the text editor.

		Each file is loaded in a separate instance of the text editor.o

		@param self: Reference to the LoadURI instance.
		@type self: A LoadURI object.

		@param uri_list: A list of files (URIs).
		@type uri_list: A List object.
		"""
		self.__uri_list = uri_list
		self.__editor.service_proxy_object.get_uris(dbus_interface="net.sourceforge.ScribesService",
					reply_handler=self.__reply_handler_cb,
					error_handler=self.__error_handler_cb)
		return

	def __load_uris(self):
		"""
		Load one or more files into the text editor's buffer.

		@param self: Reference to the LoadURI instance.
		@type self: A LoadURI object.
		"""
		from loader import Loader
		from gobject import timeout_add
		time = 10
		for uri in self.__uri_list:
			timeout_add(time, self.__load_uri, Loader, self.__editor, uri)
			time += 10
		return

	def __load_uri(self, Loader, editor, uri):
		"""
		Load a document into the text editor's buffer.

		This function is called repeatedly unless its return value is False.

		@param Loader: An object responsible for loaded text documents.
		@type Loader: A Loader object.

		@param editor: Reference to a text editor instance.
		@type editor: An Editor object.

		@param uri: A universal resource identifier representing, or pointing to, a
			document.
		@type uri: A String object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		Loader(editor, uri.strip())
		return False

	def __reply_handler_cb(self, uris):
		if not uris:
			self.__load_uris()
			return
		for uri in uris:
			if uri in self.__uri_list:
				self.__uri_list.remove(uri)
				self.__editor.service_proxy_object.focus_uri_window(uri,
					dbus_interface="net.sourceforge.ScribesService",
					reply_handler=self.__reply_handler_focus_cb,
					error_handler=self.__error_handler_cb)
		if self.__uri_list:
			self.__load_uris()
		return

	def __reply_handler_focus_cb(self, *args):
		return

	def __error_handler_cb(self, error):
		print error
		print "In fileloader.py"
		return
