# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module exposes a class that creates the search button for the text editor's
findbar.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import Button

class ScribesFindSearchButton(Button):
	"""
	This class creates a search button for the text editor's findbar. It defines
	the behavior and default properties of the button.
	"""

	def __init__(self, findbar):
		"""
		Initialize the button.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param findbar: The text editor's findbar.
		@type findbar: A ScribesFindBar object.
		"""
		from gtk import STOCK_FIND
		Button.__init__(self, stock=STOCK_FIND, use_underline=True)
		self.__init_attributes(findbar)
		self.__set_properties()
		self.connect("clicked", self.__searchbutton_clicked_cb)
		self.editor.connect("show-bar", self.__searchbutton_show_bar_cb)
		self.editor.connect("hide-bar", self.__searchbutton_hide_bar_cb)
		self.entry.connect("changed", self.__searchbutton_changed_cb)
		self.__signal_id_1 = self.searchmanager.connect_after("matches-found", self.__searchbutton_matches_found_cb)
		self.__signal_id_2 = self.searchmanager.connect_after("no-matches-found", self.__searchbutton_no_matches_found_cb)
		self.__signal_id_3 = self.searchmanager.connect("next", self.__searchbutton_next_cb)
		self.__signal_id_4 = self.searchmanager.connect("cancel", self.__searchbutton_cancel_cb)
		self.word_check_button.connect("toggled", self.__searchbutton_toggled_cb)
		self.case_check_button.connect("toggled", self.__searchbutton_toggled_cb)
		self.__block_search_replace_signals()

	def __init_attributes(self, findbar):
		"""
		Initialize the button's attributes.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param findbar: the text editor's findbar.
		@type findbar: A ScribesFindBar object.
		"""
		self.editor = findbar.editor
		self.entry = findbar.findbar_entry
		self.searchmanager = findbar.search_replace_manager
		self.status_id = None
		self.__signal_id_1 = None
		self.__signal_id_2 = None
		self.__signal_id_3 = None
		self.__signal_id_4 = None
		self.word_check_button = findbar.findbar_match_word_button
		self.case_check_button = findbar.findbar_match_case_button
		return

	def __set_properties(self):
		"""
		Set the button's default properties.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: S ScribesFindSearchButton object.
		"""
		self.set_property("sensitive", False)
		return

	def __searchbutton_show_bar_cb(self, editor, bar):
		"""
		Handles callback when the "show-bar" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: The text editor's findbar.
		@type bar: A ScribesFindBar object.
		"""
		self.__unblock_search_replace_signals()
		if self.entry.get_text():
			self.set_property("sensitive", True)
		else:
			self.set_property("sensitive", False)
		return

	def __searchbutton_hide_bar_cb(self, editor, bar):
		"""
		Handles callback when the "hide-bar" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: One of the text editor's bar object.
		@type bar: A ScribesBar object.
		"""
		self.__block_search_replace_signals()
		return

	def __searchbutton_changed_cb(self, entry):
		"""
		Handles callback when the "changed" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param entry: The findbar's entry object.
		@type entry: A ScribesFindEntry object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if entry.get_text():
			self.set_property("sensitive", True)
		else:
			self.set_property("sensitive", False)
		return True

	def __searchbutton_clicked_cb(self, button):
		"""
		Handles callback when the button's "clicked" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param button: The findbar's find button.
		@type button: A ScribesFindSearchButton object.
		"""
		self.searchmanager.search(self.entry.get_text())
		return True

	def __searchbutton_matches_found_cb(self, searchmanager):
		"""
		Handles callback when the "matches-found" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param searchmanager: The text editor's search processing object.
		@type searchmanager: A ScribesFindSearchButton object.
		"""
		return

	def __searchbutton_no_matches_found_cb(self, searchmanager):
		"""
		Handles callback when the "no-matches-found" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param searchmanager: The text editor's search processing object.
		@type searchmanager: A ScribesFindSearchButton object.
		"""
		return

	def __searchbutton_next_cb(self, searchmanager):
		"""
		Handles callback when the "next" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param searchmanager: The text editor's search processing object.
		@type searchmanager: A ScribesFindSearchButton object.
		"""
		return

	def __searchbutton_cancel_cb(self, searchmanager):
		"""
		Handles callback when the "cancel" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param searchmanager: The text editor's search processing object.
		@type searchmanager: A SearchProcessor object.
		"""
		if self.entry.get_text():
			self.set_property("sensitive", True)
		else:
			self.set_property("sensitive", False)
		return

	def __searchbutton_toggled_cb(self, togglebutton):
		"""
		Handles callback when the "toggled" signal is emitted.

		@param self: Reference to the ScribesFindSearchButton instance.
		@type self: A ScribesFindSearchButton object.

		@param togglebutton: The check buttons on the findbar.
		@type togglebutton: A gtk.CheckButton object.

		@return: True to prevent propagation of signals to parent widgets.
		@type: A Boolean Object.
		"""
		return True

	def __block_search_replace_signals(self):
		self.searchmanager.handler_block(self.__signal_id_1)
		self.searchmanager.handler_block(self.__signal_id_2)
		self.searchmanager.handler_block(self.__signal_id_3)
		self.searchmanager.handler_block(self.__signal_id_4)
		return

	def __unblock_search_replace_signals(self):
		self.searchmanager.handler_unblock(self.__signal_id_1)
		self.searchmanager.handler_unblock(self.__signal_id_2)
		self.searchmanager.handler_unblock(self.__signal_id_3)
		self.searchmanager.handler_unblock(self.__signal_id_4)
		return
