# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
The modules exposes a class responsible for creating the gotobar. The gotobar
allows users to move the cursor to a specific line.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from bar import ScribesBar

class ScribesGotoBar(ScribesBar):
	"""
	This class creates a gotobar object. The gotobar allows users to move the
	cursor to a specific line in the document contained by the text editor.
	"""

	def __init__(self, editor):
		"""
		Initialize the gotobar object.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		ScribesBar.__init__(self, editor)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__create_widgets()
		self.__arrange_widgets()
		self.editor.connect("show-bar", self.__scribesgotobar_show_bar_cb)
		self.editor.connect("hide-bar", self.__scribesgotobar_hide_bar_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the gotobar's attributes.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		self.status_id = None
		self.bar_is_visible = False
		return

	def __set_properties(self):
		"""
		Define the bar's properties.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.
		"""
		self.resize(rows=1, columns=3)
		return

	def __create_widgets(self):
		"""
		Create widgets for the gotobar object.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.
		"""
		self.label = self.__create_labels()
		from gotospinbutton import ScribesGotoSpinButton
		self.spinbutton = ScribesGotoSpinButton(self.editor)
		return

	def __arrange_widgets(self):
		"""
		Arrange the gotobar widgets.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.
		"""
		from gtk import SHRINK, FILL, EXPAND
		self.attach(child=self.label[0],
					left_attach=0,
					right_attach=1,
					top_attach=0,
					bottom_attach=1,
					xoptions=SHRINK|FILL,
					yoptions=EXPAND,
					xpadding=7,
					ypadding=0)

		self.attach(child=self.spinbutton,
					left_attach=1,
					right_attach=2,
					top_attach=0,
					bottom_attach=1,
					xoptions=SHRINK|FILL,
					yoptions=EXPAND,
					xpadding=0,
					ypadding=0)

		self.attach(child=self.label[1],
					left_attach=2,
					right_attach=3,
					top_attach=0,
					bottom_attach=1,
					xoptions=SHRINK|FILL,
					yoptions=EXPAND,
					xpadding=0,
					ypadding=0)
		return

	def show_bar(self):
		"""
		Show the gotobar.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.
		"""
		if self.bar_is_visible:
			return
		ScribesBar.show_bar(self)
		self.bar_is_visible = True
		from internationalization import msg0180
		self.status_id = self.editor.feedback.set_modal_message(msg0180, "goto")
		return

	def hide_bar(self):
		"""
		Hide the gotobar.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.
		"""
		if self.bar_is_visible is False:
			return
		ScribesBar.hide_bar(self)
		self.bar_is_visible = False
		self.editor.feedback.unset_modal_message(self.status_id, False)
		from internationalization import msg0182
		self.editor.feedback.update_status_message(msg0182, "info", 3)
		return

	def __create_labels(self):
		"""
		Create labels for the gotobar widget.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.

		@return: A list of labels for the gotobar.
		@rtype: A List object.
		"""
		from internationalization import msg0056, msg0055
		strings = [msg0055, msg0056]
		labels = []
		from gtk import Label
		for string in strings:
			label = Label(string)
			label.set_use_underline(True)
			label.set_use_markup(True)
			labels.append(label)
		return labels

	def __scribesgotobar_show_bar_cb(self, editor, bar):
		"""
		Handles callback when the "show-bar" signal is emitted.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param gotobar: The text editor's gotobar.
		@type gotobar: A ScribesGotoBar object.
		"""
		self.bar_is_visible = True
		from internationalization import msg0056
		message = msg0056 % self.editor.textbuffer.get_line_count()
		self.label[1].set_label(message)
		return

	def __scribesgotobar_hide_bar_cb(self, editor, bar):
		"""
		Handles callback when the "hide-bar" signal is emitted.

		@param self: Reference to the ScribesGotoBar instance.
		@type self: A ScribesGotoBar object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param gotobar: The text editor's gotobar.
		@type gotobar: A ScribesGotoBar object.
		"""
		self.bar_is_visible = False
		return
