# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module exposes a class responsible for creating the gotobar's spinbutton
object.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import SpinButton

class ScribesGotoSpinButton(SpinButton):
	"""
	This class creates an object that allows users to jump to a specific line in
	the text editor's buffer. The object is gtk.SpinButton. This class defines
	the behavior and property of the gotobar's spinbutton object.
	"""

	def __init__(self, editor):
		"""
		Initialize the gotobar's spinbutton.

		@param self: Reference to the ScribesGotoSpinButton instance.
		@type self: A ScribesGotoSpinButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		SpinButton.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.editor.connect("show-bar", self.__spinbutton_show_bar_cb)
		self.connect("value-changed", self.__spinbutton_value_changed_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the spinbutton's attributes

		@param self: Reference to the ScribesGotoSpinButton instance.
		@type self: A ScribesGotoSpinButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		from gtk import Adjustment
		self.adjustment = Adjustment()
		return

	def __set_properties(self):
		"""
		Define the default properties of the spinbutton.

		@param self: Reference to the ScribesGotoSpinButton instance.
		@type self: A ScribesGotoSpinButton object.
		"""
		from gtk import UPDATE_IF_VALID
		self.set_update_policy(UPDATE_IF_VALID)
		self.set_numeric(True)
		self.set_wrap(False)
		self.set_snap_to_ticks(True)
		self.set_digits(0)
		self.set_property("xalign", 1)
		self.set_adjustment(self.adjustment)
		return

	def __spinbutton_show_bar_cb(self, editor, bar):
		"""
		Handles callback when the "show-bar" signal is emitted.

		@param self: Reference to the ScribesGotoSpinButton instance.
		@type self: A ScribesGotoSpinButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: The gotospinbutton.
		@type bar: A ScribesGotoSpinButton object.
		"""
		from cursor import get_cursor_line, move_view_to_cursor
		cursor_line = get_cursor_line(self.editor.textbuffer)
		value = cursor_line + 1
		lower = 1
		upper =  self.editor.textbuffer.get_line_count()
		step_increment = 1
		page_increment = 5
		page_size = 11
		self.adjustment.set_all(value, lower, upper, step_increment,
								page_increment, page_size)
		move_view_to_cursor(self.editor.textview)
		self.grab_focus()
		self.connect("key-press-event", self.__spinbutton_key_press_event_cb, bar)
		return

	def __spinbutton_value_changed_cb(self, spinbutton):
		"""
		Handles callback when the "value-changed" signal is emitted.

		@param self: Reference to the ScribesGotoSpinButton instance.
		@type self: A ScribesGotoSpinButton object.

		@param spinbutton: The gotobar's spinbutton.
		@type spinbutton: A ScribesGotoSpinButton object.
		"""
		line = spinbutton.get_value() - 1
		iterator = self.editor.textbuffer.get_iter_at_line(int(line))
		self.editor.textbuffer.place_cursor(iterator)
		from cursor import move_view_to_cursor
		move_view_to_cursor(self.editor.textview)
		from internationalization import msg0181
		message = msg0181 % (line + 1)
		self.editor.feedback.update_status_message(message, "succeed")
		return True

	def __spinbutton_key_press_event_cb(self, spinbutton, event, bar):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the ScribesGotoSpinButton instance.
		@type self: A ScribesGotoSpinButton object.

		@param spinbutton: The text editor's spinbutton.
		@type spinbutton: A ScribesGotoSpinButton object.

		@param event: An event that occurs when keys are pressed.
		@type event: A gtk.Event object.

		@param bar: The gotobar for the text editor.
		@type bar: A ScribesGotoBar object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		from gtk import keysyms
		if event.keyval == keysyms.Return:
			bar.hide_bar()
			return True
		return False
