# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates the import dialog for the
text editor's template editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from sdialog import Dialog

class ImportDialog(Dialog):
	"""
	This class create the import dialog for the template editor.
	"""

	def __init__(self, import_button):
		"""
		Initialize the dialog.

		@param self: Reference to the ImportDialog instance.
		@type self: An ImportDialog object.

		@param import_button: The template editor's import button.
		@type import_button: A ImportButton object.
		"""
		Dialog.__init__(self)
		self.__init_attributes(import_button)
		self.__set_properties()
		self.__arrange_widgets()
		self.activate_button.connect("clicked", self.__dialog_clicked_cb)
		self.filechooser_widget.connect("selection-changed", self.__dialog_selection_changed_cb)
		self.filechooser_widget.connect("file-activated", self.__dialog_file_activated_cb)

	def __init_attributes(self, import_button):
		"""
		Initialize the dialog's data attributes.

		@param self: Reference to the ImportDialog instance.
		@type self: A ImportDialog object.

		@param import_button: The template editor's import button.
		@type import_button: A ImportButton object.
		"""
		self.template_editor = import_button.template_editor
		self.editor = self.template_editor.editor
		self.import_button = import_button
		self.import_button.dialog = self
		self.activate_button = self.__create_activate_button()
		self.filechooser_widget = self.__create_filechooser_widget()
		return

	def __set_properties(self):
		"""
		Define the dialog's properties.

		@param self: Reference to the ImportDialog instance.
		@type self: An ImportDialog object.
		"""
		from internationalization import msg0426
		self.set_property("title", msg0426)
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self.editor.window,
														1.6, 1.929648241)
		self.set_default_size(width, height)
		self.set_property("icon-name", "stock_open")
		self.set_transient_for(self.template_editor.window)
		return

	def __arrange_widgets(self):
		"""
		Arrange the widgets for the dialog.

		@param self: Reference to the ImportDialog instance.
		@type self: An ImportDialog object.
		"""
		self.main_area.pack_start(self.filechooser_widget, True, True, 0)
		self.button_area.pack_start(self.cancel_button, False, False, 0)
		self.button_area.pack_start(self.activate_button, False, False, 0)
		self.activate_button.set_property("sensitive", False)
		return

	def __create_activate_button(self):
		"""
		Create the import button for the dialog.

		@param self: Reference to the ImportDialog instance.
		@type self: An ImportDialog object.

		@return: An import button for the dialog.
		@rtype: A gtk.Button object.
		"""
		from gtk import STOCK_REDO, Button
		from internationalization import msg0420
		from utils import create_button
		hbox = create_button(STOCK_REDO, msg0420)
		button = Button()
		button.add(hbox)
		return button

	def __create_filechooser_widget(self):
		"""
		Create the wigdet used to select files.

		@param self: Reference to the ImportDialog instance.
		@type self: An ImportDialog object.

		@return: A file chooser widget.
		@rtype: A gtk.FileChooserWidget object.
		"""
		from gtk import FileChooserWidget
		widget = FileChooserWidget()
		from dialogfilter import create_filter
		from internationalization import msg0351
		xml_filter = create_filter(msg0351, "text/xml")
		widget.add_filter(xml_filter)
		from info import home_folder, desktop_folder
		from os import path
		if path.exists(desktop_folder):
			widget.set_current_folder(desktop_folder)
		else:
			widget.set_current_folder(home_folder)
		return widget

	def __dialog_clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the Dialog instance.
		@type self: A Dialog object.

		@param button: The import button for the dialog.
		@type button: A gtk.Button object.
		"""
		self.template_editor.window.set_property("sensitive", False)
		self.hide_dialog()
		from utils import response
		response()
		xml_template = self.filechooser_widget.get_filename()
		if xml_template is None:
			return True
		try:
			from importtemplate import import_template_from_file
			result = import_template_from_file(xml_template)
			if result is False:
				raise Exception
		except:
			from internationalization import msg0425
			self.editor.error_dialog.show_message(msg0425, parent_window=self)
		response()
		self.template_editor.window.set_property("sensitive", True)
		self.editor.emit("updated-template-database")
		return True

	def __dialog_selection_changed_cb(self, filechooser_widget):
		"""
		Handles callback when the "selection-changed" signal is emitted.

		@param self: Reference to the Dialog instance.
		@type self: A Dialog object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		filename = self.filechooser_widget.get_filename()
		if filename is None:
			return
		from os import path
		if path.isdir(filename):
			self.activate_button.set_property("sensitive", False)
		else:
			self.activate_button.set_property("sensitive", True)
		return True

	def __dialog_file_activated_cb(self, filechooser_widget):
		"""
		Handles callback when the "file-activated" signal is emitted.

		@param self: Reference to the Dialog instance.
		@type self: A Dialog object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.activate_button.activate()
		return True
