# -*- coding: utf8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module defines a class that loads URIs representing text files into
instances of text editors.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Loader(object):
	"""
	This class creates an object that loads a valid URI pointing to a text
	document into instances of text editors.
	"""

	def __init__(self, editor, uri, readonly=False):
		"""
		Initialize a Loader object and load a URI into the text editor.

		Before a URI can be loaded into a text editor, sufficient information
		about the URI needs to be obtained. If enough information about a URI
		cannot be obtained, the loading process is aborted and an error dialog
		is displayed indicating the result of the exceptional event.

		Only text files can be loaded into the text editor's buffer. Thus, only
		text files are regarded as valid files. After sufficient information
		about a URI has been obtained, checks are performed on a URI to ensure
		it points to a valid file. If the type of a file the URI points to
		cannot be determined, or the file is an invalid file, the loading
		process is aborted and an error dialog is displayed indicating the
		result of the exceptional event.

		Checks are performed to ensure that users have permission to read from
		or write to a URI. If users do not have permission to read from a file,
		the loading process is aborted and an error dialog is displayed
		indicating the result of the exceptional event. If users have permission
		to read from a file, but do not have permission to write to it, the
		file is loaded in readonly mode.

		After all checks and validations are successful, the URI is loaded into
		the text editor's buffer.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@param editor: Reference to a text editor instance.
		@type editor: An Editor object.

		@param uri: A Uniform Resource Identifier pointing to document,
			hopefully a text document.
		@type uri: A String object.

		@param readonly: Whether or not the buffer should be editable.
		@type readonly: A Boolean object.
		"""
		# Check to see if the file should be launched in a new instance of the
		# text editor.
		result = self.__needs_new_window(editor)
		if result:
			self.__load_file_in_new_window(uri)
		else:
			# Initialize the loader object.
			self.__init_attributes(editor, uri, readonly)
			# Emit a global signal indicating document checks have begun.
			self.editor.emit("checking-document")
			# Retrieve information about the file pointed to by the URI.
			self.fileinfo  = self.__get_uri_info()
			if self.fileinfo:
				# Check for correct file type.
				self.mimetype = self.__get_uri_type()
				if self.mimetype:
					# Check for permissions.
					result  = self.__check_uri_permission()
					if result:
						# Load URI into the text editor's buffer.
						from gobject import timeout_add
						timeout_add(10, self.__load_uri)

	def __init_attributes(self, editor, uri, readonly):
		"""
		Initialize the loader object's attributes.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@param editor: Reference to a text editor instance.
		@type editor: An Editor object.

		@param uri: A Uniform Resource Identifier representing a file.
		@type uri: A String object.

		@param readonly: Whether or not to launch the file in readonly mode.
		@type readonly: A Boolean object.
		"""
		# An instance of the text editor.
		self.editor = editor
		self.temp_buffer = ""
		self.encoding = None
		# Initialize to display GNOME's authentication dialog if needed.
		from gnome.ui import authentication_manager_init
		authentication_manager_init()
		# A URI object of the uri. Note, a URI object is not the same as a uri
		# string. See the GNOME-VFS manual for more information on the
		# properties of a URI object.
		from gnomevfs import URI
		self.uri = URI(uri)
		self.undo_flag = False
		# True to open the uri in readonly mode.
		self.readonly = readonly
		# An object containing information about a URI.
		self.fileinfo = None
		# The MIME Type of a URI.
		self.mimetype = None
		# A gtksourceview.SourceLanguage object used to activate syntax
		# highlight.
		self.language = None
		# Unique number associated with feedback messages relayed to the text
		# editor's statusbar.
		self.status_id = None
		# Unique number associated with an error feedback message relayed to
		# text editor's statusbar.
		self.status_error_id = None
		# Some MIME types are not properly recognized as text files by the
		# freedesktop.org MIME system. These MIME types are added to a white
		# list so they can be loaded by the text editor.
		self.mimetype_whitelist = ["image/svg+xml", "application/rtf",
						"application/x-shellscript", "application/x-glade",
						"application/x-php", "application/octet-stream",
						"application/x-desktop", "application/x-trash",
						"application/x-csh", "audio/x-mpegurl",
						"application/x-perl", "application/x-ruby",
						"application/x-javascript", "application/xml",
						"application/x-m4", "application/docbook+xml",
						"application/x-extension-rb",
						"application/x-xbel", "application/xslt+xml"]
		self.use_encoding_list = False
		self.unicode_error_flag = False
		self.encoding_list = [
			"utf8", "iso-8859-1", "iso-8859-15", "iso-8859-2", "iso-8859-3",
			"iso-8859-4", "iso-8859-5", "iso-8859-6", "iso-8859-7",
			"iso-8859-8", "iso-8859-9", "iso-8859-10", "iso-8859-11",
			"iso-8859-12", "iso-8859-13", "iso-8859-14",  "sjis","sjis2004",
			"sjisx0213", "csbig5", "hkscs", "ujis", "eucjis2004", "eucjisx0213",
			"utf16"
		]
		# URI schemes that can be written to for remote locations.
		self.writable_vfs_schemes = ["ssh", "sftp", "smb", "dav", "davs", "ftp"]
		return

	def __load_uri(self):
		"""
		Load the text file the URI points to into the editor's buffer.

		This function opens a URI, reads the content of the URI into the editor's
		buffer and sets the editor's window title to indicate the editor is in
		the process of loading a document.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.
		"""
		self.editor.uri = self.uri
		if self.editor.gui_is_created is False:
			return True
		# Emit a global signal indicating document loading has begun.
		self.editor.emit("loading-document")
		# Asynchronously open the file for loading.
		from gobject import timeout_add, PRIORITY_HIGH
		timeout_add(10, self.__open_uri, priority=PRIORITY_HIGH)
		return False

	def __open_uri(self):
		"""
		Asynchronously open a URI.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.
		"""
		from gnomevfs import OPEN_READ
		from gnomevfs.async import open
		try:
			open(self.uri, self.__open_cb, OPEN_READ, 10)
		except:
			from internationalization import msg0358
			self.__show_error_dialog(msg0358)
		return False

	def __open_cb(self, handle, result):
		"""
		Handles callback to read a URI after it has been opened asynchronously.

		This function reads the contents of a URI. If the URI is local, it reads
		all its content. If the URI is remote, the contents of the URI is read
		4096 bytes at a time.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@param handle: An object pointing to a URI.
		@type handle: A gnomevfs.Handle object.

		@param result: The result of a read operation.
		@type result: A gnomevfs.Error object.
		"""
		if self.uri.is_local or str(self.uri).startswith("file:///"):
			if not self.fileinfo.size:
				# Hardcode the file size information if it is zero, otherwise
				# GNOME-VFS will complain.
				self.fileinfo.size  = 4096
			handle.read(self.fileinfo.size, self.__read_cb)
		else:
			handle.read(4096, self.__read_cb)
		return

	def __read_cb(self, handle, buffer, result, bytes):
		"""
		Handles callback to insert text into the editor's buffer.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@param handle:
		@type handle: A gnomevfs.Handle object.

		@param buffer: An area in memory where text read from a URI is placed.
		@type buffer: A String object.

		@param result: The result of a read operation.
		@type result: A gnomevfs.Error object.

		@param bytes: Bytes read.
		@type bytes: An Integer object.
		"""
		try:
			if self.uri.is_local or str(self.uri).startswith("file:///"):
				self.insert_string_to_buffer(buffer, handle)
				handle.close(self.__close_cb)
			else:
				if not result:
					self.get_text_from_remote_location(buffer, handle)
				else:
					self.insert_string_to_buffer(self.temp_buffer, handle)
					handle.close(self.__close_cb)
		except ValueError:
			pass
		return

	def __close_cb(self, *args):
		"""
		Close the URI and finalize the editor's buffer for editing.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.
		"""
		self.editor.emit("loaded-document")
		if self.readonly:
			self.editor.triggermanager.trigger("toggle_readonly")
		if self.unicode_error_flag:
			self.editor.encoding = None
		else:
			self.editor.encoding = self.encoding
		return

################################################################################
#
#						URI Information Retrieval
#
################################################################################

	def __get_uri_info(self):
		"""
		Get metadata information about a URI.

		This function calls the GNOME-VFS get_file_info function which returns
		a gnomevfs.fileinfo object. The fileinfo object contains information
		about the URI that is needed for further processing. Information such as
		the MIME type, access permission rights, name of the URI, among many
		others are provided by the fileinfo object.

		This function, most importantly, makes provisions for times when errors
		may occur while retrieving information about a URI. Basically, if
		information about a URI cannot be retrieved for whatever reasons, the
		URI will not be loaded. Instead the loading process will be aborted and
		the Loader object destroyed.

		@param self: Reference to the Loader object.
		@type self: A Loader object.

		@return: An object containing metadata information about a URI, or
			nothing if an error occurs.
		@rtype: A gnomevfs.fileinfo object or None.
		"""
		# GNOME-VFS errors to handle.
		from gnomevfs import AccessDeniedError, NotFoundError, GenericError
		# Initialize a fileinfo object.
		fileinfo = None
		try:
			FILE_INFO_ACCESS_RIGHTS = 1 << 4
			from gnomevfs import get_file_info, FILE_INFO_DEFAULT
			from gnomevfs import FILE_INFO_GET_MIME_TYPE
			from gnomevfs import FILE_INFO_FORCE_SLOW_MIME_TYPE
			from gnomevfs import FILE_INFO_FOLLOW_LINKS
			fileinfo = get_file_info(self.uri, FILE_INFO_DEFAULT |
										FILE_INFO_GET_MIME_TYPE |
										FILE_INFO_FORCE_SLOW_MIME_TYPE |
										FILE_INFO_FOLLOW_LINKS |
										FILE_INFO_ACCESS_RIGHTS)
			if not fileinfo:
				fileinfo = None
				from internationalization import msg0359
				message = msg0359 % self.uri
				self.__show_error_dialog(message)
			# If no errors are detected, set the new uri.
		except AccessDeniedError:
			from internationalization import msg0360
			self.__show_error_dialog(message)
		except NotFoundError:
			from internationalization import msg0037
			self.__show_error_dialog(msg0037)
		except GenericError:
			from internationalization import msg0359
			message = msg0359 % self.uri
			self.__show_error_dialog(message)
		except:
			from internationalization import msg0359
			message = msg0359 % self.uri
			self.__show_error_dialog(message)
		return fileinfo

################################################################################
#
#						URI File Type Check
#
################################################################################

	def __get_uri_type(self):
		"""
		Retrieve the MIME type of a URI only if it is for a valid text file.

		This function performs MIME type checks on a URI. If the URI is not for
		a text file, the URI will not be loaded. If the checks confirm the URI
		is for a text file, the MIME type of the URI is returned. If the MIME
		type of the URI cannot be determined, the URI will not be loaded.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@return: The MIME type of the file or None if it can't be determined.
		@rtype: A String object.
		"""
		mimetype = None
		from gnomevfs import FILE_INFO_FIELDS_MIME_TYPE
		if self.fileinfo.valid_fields & FILE_INFO_FIELDS_MIME_TYPE:
			if self.fileinfo.mime_type.startswith("text/") or \
				self.fileinfo.mime_type in self.mimetype_whitelist:
				mimetype = self.fileinfo.mime_type
			else:
				# Show an error dialog if the MIME type of the file is not a
				# text, SVG, shellscript or rtf one.
				from internationalization import msg0192
				message = msg0192 % self.fileinfo.mime_type
				self.__show_error_dialog(message)
		else:
			# Show an error dialog if the MIME type of the file cannot be
			# determined.
			from internationalization import msg0191
			message = msg0191
			self.__show_error_dialog(message)
		return mimetype

################################################################################
#
#						URI Permission Check
#
################################################################################

	def __test_check_uri_permission(self):
		"""
		Check if the user has permission to read from or write to the file.

		Only files that users have permission to read and/or write will be
		loaded into the text editor. Determining the access permission of files
		on remote location is tricky, if not impossible. For remote URIs if
#		users have permission to a file it will be loaded to the editor in
#		readonly mode. If users provide authentication information to a URI,
#		they most likely have access to write to the file. So the URI will be
#		loaded into the text editor in read/write mode, only if the URI's scheme
#		permits writable operations. So far, the text editor considers ssh,
#		sftp, smb, dav and davs as URI schemes that support writable operations.
#		When the permission of a remote file cannot be determined, the URI is
#		loaded into the text editor in readonly mode.
#
#		@param self: Reference to the Loader instance.
#		@type self: A Loader object.
#
#		@return: Returns True if the user has permission to read the file.
#		@rtype: A Boolean object.
		"""
		from gnomevfs import FILE_INFO_FIELDS_PERMISSIONS
		result = None

		if self.fileinfo.valid_fields & FILE_INFO_FIELDS_PERMISSIONS:
			# In this branch the file is most likely local.
			FILE_INFO_FIELDS_ACCESS = 1 << 14
			if self.fileinfo.valid_fields & FILE_INFO_FIELDS_ACCESS:
				PERM_ACCESS_READABLE   = 1 << 16
				print self.fileinfo.permissions & PERM_ACCESS_READABLE
				if self.fileinfo.permissions & PERM_ACCESS_READABLE:
					PERM_ACCESS_WRITABLE   = 1 << 17
					result = True
					if not self.fileinfo.permissions & PERM_ACCESS_WRITABLE:
						self.readonly = True
				else:
					# Show an error dialog if the user does not have permission
					# to view the file.
					print "In here."
					from internationalization import msg0035
					self.__show_error_dialog(msg0035)
			else:
				# In this branch the file is most likely on a remote location.
				if not self.uri.is_local:
					result = True
					from gnomevfs import get_uri_scheme
					scheme = get_uri_scheme(str(self.uri))
					# Open the file in readonly mode if the URI scheme is not
					# writable.
					print "In right branch. Permission is possible."
					if not scheme in self.writable_vfs_schemes:
						self.readonly = True
				else:
					from internationalization import msg0361
					self.__show_error_dialog(msg0361)
		else:
			if not self.uri.is_local:
				# This branch is the most common scenario for files on remote
				# location.
				from gnomevfs import get_uri_scheme
				scheme = get_uri_scheme(str(self.uri))
				# Open the file in readonly mode if the URI scheme is not
				# writable.
				if not scheme in self.writable_vfs_schemes:
					self.readonly = True
			else:
				self.readonly = True
			result = True
		return result

	def __check_uri_permission(self):
		result = None
		if self.uri.is_local or str(self.uri).startswith("file:///"):
			from gnomevfs import get_local_path_from_uri
			local_path = get_local_path_from_uri(str(self.uri))
			from os import access, W_OK, R_OK
			value = access(local_path, R_OK)
			if value:
				result = True
				value = access(local_path, W_OK)
				if value is False:
					self.readonly = True
			else:
				# Show an error dialog if the user does not have permission
				# to view the file.
				from internationalization import msg0035
				self.__show_error_dialog(msg0035)
		else:
			result = True
			# This branch is the most common scenario for files on remote
			# location.
			from gnomevfs import get_uri_scheme
			scheme = get_uri_scheme(str(self.uri))
			# Open the file in readonly mode if the URI scheme is not
			# writable.
			if not scheme in self.writable_vfs_schemes:
				self.readonly = True
		return result

################################################################################
#
#						Text Buffer Insertion
#
################################################################################

	def insert_string_to_buffer(self, string, handle=None):
		"""
		Insert text into the editor's buffer.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@param string: Text to insert into the text editor's buffer.
		@type string: A String object.

		@param handle: An object pointing to a remote URI.
		@type handle: A gnomevfs.Handle object.
		"""
		try:
			from txtutils import decode_string_to_unicode, encode_string
			self.encoding = self.__determine_encoding()
			unicode_string = decode_string_to_unicode(string, self.encoding)
			if unicode_string is None:
				if self.use_encoding_list is False:
					# When encoding is specified by the user or an encoding
					# exists in the encoding database, fail explicitly if a
					# decoding error occurs.
					raise UnicodeDecodeError("", "", 0, 0, "")
				# True if a neutral value should be recorded in the encoding
				# metadata database, instead of a real encoding value.
				self.unicode_error_flag = True
				# When no encoding is specified, try using various encodings
				# from the encoding list, especially if UTF-8 fails.
				for encoding in self.encoding_list:
					unicode_string = decode_string_to_unicode(string, encoding)
					if unicode_string:
						self.encoding = encoding
						break
				if unicode_string is None:
					raise UnicodeDecodeError("", "", 0, 0, "")
			# Convert unicode string to utf8 encoded string. GTK+ uses utf8
			# encoded strings by default.
			utf8_string = encode_string(unicode_string, "utf8")
			if utf8_string is None:
				raise UnicodeEncodeError("", u"", 0, 0, "")
			# Insert text into the text editor's buffer.
			self.editor.textbuffer.set_text(utf8_string)
		except UnicodeDecodeError:
			# Abort the loading operation and display an error dialog.
			handle.cancel()
			self.encoding = None
			from internationalization import msg0031
			self.__show_error_dialog(msg0031)
		except UnicodeEncodeError:
			# Abort the loading operation and display an error dialog.
			handle.cancel()
			self.encoding = None
			from internationalization import msg0153
			self.__show_error_dialog(msg0153)
		return

	def get_text_from_remote_location(self, string, handle):
		"""
		Get the contents of a URI at a remote location.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@param string: Contents of a URI.
		@type string: A String object.

		@param handle: An object pointing to a URI.
		@type handle: A gnomevfs.async.Handle object.
		"""
		self.temp_buffer += string
		handle.read(4096, self.__read_cb)
		return

################################################################################
#
#								Encoding Stuff
#
################################################################################

	def __determine_encoding(self):
		"""
		Determine the encoding to use to open a document.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@param value: An encoding to use to open a document.
		@type value: A String object.
		"""
		if self.editor.encoding:
			# In this branch the encoding value is set by the user via the
			# open dialog
			return self.editor.encoding

		# Get encoding stored in database, if any.
		from encoding_metadata import get_encoding_from_database
		encoding  = get_encoding_from_database(str(self.uri))
		if encoding:
			value = encoding
		else:
			self.use_encoding_list = True
			value = "utf8"
		return value

################################################################################
#
#							Helper Methods
#
################################################################################

	def cleanup(self):
		"""
		Destroy the Loader object and reclaim memory, if possible.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.
		"""
		self = None
		del self
		from gc import collect
		collect()
		return

	def __show_error_dialog(self, message):
		"""
		Show the text editor's error dialog.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@param message: Error message for the error dialog.
		@type message: A String object.
		"""
		self.editor.emit("load-error")
		if self.uri:
			from internationalization import msg0189
			string = msg0189 % self.uri
		else:
			from internationalization import msg0362
			string = msg0362
		try:
			error_id = self.editor.feedback.set_modal_message(string, "error")
		except:
			pass
		from internationalization import msg0189
		error_message = msg0189 % str(self.uri)
		from error import ScribesErrorDialog
		from gnomevfs import format_uri_for_display
		uri = format_uri_for_display(str(self.uri))
		ScribesErrorDialog(self.editor, uri, message)
		try:
			self.editor.feedback.unset_modal_message(error_id)
		except AttributeError:
			pass
		except UnboundLocalError:
			pass
		self.__error_recovery()
		return

	def __error_recovery(self):
		"""
		Return the text editor to its default state after an error occurs.

		@param self: Reference to the Loader instance.
		@type self: A Loader object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		self.encoding = None
		self.editor.encoding = None
		self.temp_buffer = None
		self.editor.uri = self.uri = None
		self.cleanup()
		try:
			if self.editor.window.is_mapped is False:
				self.editor.emit("close-document")
		except:
			self.editor.emit("close-document")
		self = None
		from gc import collect
		collect()
		return False

	def __needs_new_window(self, editor):
		"""
		Check if the new file should be loaded in a new editor instance.

		The text editor follows a spatial and document centric paradigm. Thus,
		each file should be loaded in its own editor window. This is known as
		a single document interface. The function checks to see if the editor
		currently holds a document. It returns True if the editor already
		contains a document or is in the process of loading one, and False
		otherwise

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@param editor: Reference to the text editor instance.
		@type editor: An Editor object.

		@return: True to load file in new window, False to load in current one.
		@rtype: A Boolean object.
		"""
		value = False
		# Return True if the editor already contains a document or is in the
		# process of loading one.
		if editor.is_checking_document or editor.is_loading_document or editor.contains_document:
			value = True
		return value

	def __load_file_in_new_window(self, uri):
		"""
		Create a new instance of the text editor and load the file passed as a
		parameter into the new editor instance.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@param uri: A Uniform Resource Identifier representing an existing file.
		@type uri: A string object.
		"""
		from main import launch_new_editor
		from gtk import quit_add, main
		quit_add(0, main)
		launch_new_editor([uri])
		return
