# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates a trigger to jump to matching
pair characters.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class MatchingBracketTrigger(object):
	"""
	This class implements an object that finds matching pair characters.
	"""

	def __init__(self, editor):
		"""
		Initialize the trigger object.

		@param self: Reference to the MatchingBracketTrigger instance.
		@type self: A MatchingBracketTrigger object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__init_attributes(editor)
		self.__create_trigger()
		self.__trigger.connect("activate", self.__find_matching_bracket_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the object's data attributes.

		@param self: Reference to the MatchingBracketTrigger instance.
		@type self: A MatchingBracketTrigger object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		self.__trigger = None
		return

	def __create_trigger(self):
		"""
		Create a trigger to find matching brackets.

		@param self: Reference to the MatchingBracketTrigger instance.
		@type self: A MatchingBracketTrigger object.
		"""
		from trigger import Trigger
		self.__trigger = Trigger("find_matching_bracket")
		self.__editor.triggermanager.add_trigger(self.__trigger, "alt - B")
		return

	def __find_matching_bracket_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the MatchingBracketTrigger instance.
		@type self: A MatchingBracketTrigger object.

		@param trigger: An object that finds matching brackets.
		@type trigger: A Trigger object.
		"""
		result = self.__find_matching_bracket()
		if result:
			from cursor import get_cursor_iterator
			iterator = get_cursor_iterator(self.__editor.textbuffer)
			line = iterator.get_line() + 1
			from internationalization import msg0020
			message = msg0020 % (line)
			self.__editor.feedback.update_status_message(message, "suceed")
		else:
			from internationalization import msg0021
			self.__editor.feedback.update_status_message(msg0021, "fail")
		return

	def __find_matching_bracket(self):
		"""
		Find matching bracket, if any.

		@param self: Reference to the MatchingBracketTrigger instance.
		@type self: A MatchingBracketTrigger object.

		@return: True if a matching bracket was found.
		@rtype: A Boolean object.
		"""
		from gtksourceview import source_iter_find_matching_bracket as match
		from cursor import get_cursor_iterator
		iterator = get_cursor_iterator(self.__editor.textbuffer)
		if match(iterator):
			self.__editor.textbuffer.place_cursor(iterator)
			from cursor import move_view_to_cursor
			move_view_to_cursor(self.__editor.textview)
			return True
		return False
