# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that implements the popup menu item for
changing the case of selected text.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import ImageMenuItem

class ScribesCasePopupMenuItem(ImageMenuItem):
	"""
	This class creates the case menu item for the text editor.
	"""

	def __init__(self, editor):
		"""
		Initialize the menu item.

		@param self: Reference to the ScribesCasePopupMenuItem instance.
		@type self: A ScribesCasePopupMenuItem object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from internationalization import msg0455
		ImageMenuItem.__init__(self, msg0455)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__uppercase_menuitem.connect("activate", self.__popup_activate_cb)
		self.__lowercase_menuitem.connect("activate", self.__popup_activate_cb)
		self.__titlecase_menuitem.connect("activate", self.__popup_activate_cb)
		self.__swapcase_menuitem.connect("activate", self.__popup_activate_cb)
		self.__uppercase_menuitem.connect("map-event", self.__popup_map_event_cb)
		self.__lowercase_menuitem.connect("map-event", self.__popup_map_event_cb)
		self.__titlecase_menuitem.connect("map-event", self.__popup_map_event_cb)
		self.__swapcase_menuitem.connect("map-event", self.__popup_map_event_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the menu item's data attributes.

		@param self: Reference to the ScribesCasePopupMenuItem instance.
		@type self: A ScribesCasePopupMenuItem object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from internationalization import msg0456, msg0457, msg0458
		from internationalization import msg0459
		from utils import create_menuitem
		from gtk import Menu, Image, STOCK_SORT_DESCENDING
		self.__image = Image()
		self.__image.set_property("stock", STOCK_SORT_DESCENDING)
		self.__editor = editor
		self.__menu = Menu()
		self.__uppercase_menuitem = create_menuitem(msg0456)
		self.__lowercase_menuitem = create_menuitem(msg0457)
		self.__titlecase_menuitem = create_menuitem(msg0458)
		self.__swapcase_menuitem = create_menuitem(msg0459)
		return

	def __set_properties(self):
		"""
		Define the properties of the menu item.

		@param self: Reference to the ScribesCasePopupMenuItem instance.
		@type self: A ScribesCasePopupMenuItem object.
		"""
		self.set_image(self.__image)
		self.set_submenu(self.__menu)
		self.__menu.append(self.__uppercase_menuitem)
		self.__menu.append(self.__lowercase_menuitem)
		self.__menu.append(self.__titlecase_menuitem)
		self.__menu.append(self.__swapcase_menuitem)
		if self.__editor.is_readonly:
			self.set_property("sensitive", False)
		return

	def __popup_activate_cb(self, menuitem):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the ScribesCasePopupMenuItem instance.
		@type self: A ScribesCasePopupMenuItem object.

		@param menuitem: The popup menu's menuitem.
		@type menuitem: A gtk.MenuItem object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if menuitem == self.__uppercase_menuitem:
			self.__editor.triggermanager.trigger("uppercase")
		elif menuitem == self.__lowercase_menuitem:
			self.__editor.triggermanager.trigger("lowercase")
		elif menuitem == self.__titlecase_menuitem:
			self.__editor.triggermanager.trigger("titlecase")
		elif menuitem == self.__swapcase_menuitem:
			self.__editor.triggermanager.trigger("swapcase")
		return True

	def __popup_map_event_cb(self, menuitem, event):
		"""
		Handles callback when the "map-event" signal is emitted.

		@param self: Reference to the ScribesCasePopupMenuItem instance.
		@type self: A ScribesCasePopupMenuItem object.

		@param menuitem: The popup menu's menuitem.
		@type menuitem: A gtk.MenuItem object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		selection = self.__editor.textbuffer.get_selection_bound()
		menuitem.set_property("sensitive", False)
		if selection is None:
			menuitem.set_property("sensitive", True)
		return True
