# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module documents a class that implements the indentation popup menu item
for the text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import ImageMenuItem

class ScribesIndentPopupMenuItem(ImageMenuItem):
	"""
	This class creates the indentation popup menu item for the text editor.
	"""

	def __init__(self, scribesview):
		"""
		Initialize the popup menu item.

		@param self: Reference to the ScribesIndentPopupMenuItem instance.
		@type self: A ScribesIndentPopupMenuItem object.

		@param scribesview: The text editor's text view.
		@type scribesview: A ScribesTextView object.
		"""
		from internationalization import msg0380
		ImageMenuItem.__init__(self, msg0380)
		self.__init_attributes(scribesview)
		self.__create_wigets()
		self.__set_properties()
		self.indent_menuitem.connect("activate", self.__popup_activate_cb)
		self.unindent_menuitem.connect("activate", self.__popup_activate_cb)
		self.unindent_menuitem.connect("map-event", self.__popup_map_event_cb)

	def __init_attributes(self, scribesview):
		"""
		Initialize the popup menu item's attributes.

		@param self: Reference to the ScribesIndentPopupMenuItem instance.
		@type self: A ScribesIndentPopupMenuItem object.

		@param scribesview: The text editor's text view.
		@type scribesview: A ScribesTextView object.
		"""
		self.scribesview = scribesview
		self.editor = scribesview.editor
		self.menu = None
		self.image = None
		self.indent_menuitem = None
		self.unindent_menuitem = None
		return

	def __create_wigets(self):
		"""
		Create the popup menu item's interface.

		@param self: Reference to the ScribesIndentPopupMenuItem instance.
		@type self: A ScribesIndentPopupMenuItem object.
		"""
		from gtk import Image, STOCK_JUSTIFY_CENTER, Menu
		self.image = Image()
		self.image.set_property("stock", STOCK_JUSTIFY_CENTER)
		self.menu = Menu()
		from utils import create_menuitem
		from internationalization import msg0087, msg0086
		from gtk import STOCK_UNINDENT, STOCK_INDENT
		self.indent_menuitem = create_menuitem(msg0086, STOCK_INDENT)
		self.unindent_menuitem = create_menuitem(msg0087, STOCK_UNINDENT)
		return

	def __set_properties(self):
		"""
		Set the menu item's properties.

		@param self: Reference to the ScribesIndentPopupMenuItem instance.
		@type self: A ScribesIndentPopupMenuItem object.
		"""
		self.set_image(self.image)
		self.set_submenu(self.menu)
		self.menu.append(self.indent_menuitem)
		self.menu.append(self.unindent_menuitem)
		if self.editor.is_readonly:
			self.set_property("sensitive", False)
		return

	def __popup_activate_cb(self, menuitem):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the ScribesIndentPopupMenuItem instance.
		@type self: A ScribesIndentPopupMenuItem object.

		@param menuitem: A menuitem for the ScribesIndentPopupMenuItem.
		@type menuitem: A gtk.MenuItem object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if menuitem == self.indent_menuitem:
			self.editor.triggermanager.trigger("indent_line")
		if menuitem == self.unindent_menuitem:
			self.editor.triggermanager.trigger("unindent_line")
		return True

	def __popup_map_event_cb(self, menuitem, event):
		"""
		Handles callback when the "map-event" signal is emitted.

		This function determines whether or not the unindent menuitem should be
		enabled.

		@param self: Reference to the ScribesIndentPopupMenuItem instance.
		@type self: A ScribesIndentPopupMenuItem object.

		@param menuitem: The the unindent menu item.
		@type menuitem: A gtk.MenuItem object.

		@param event: An event that occurs when the unindent menu item is displayed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		menuitem.set_property("sensitive", False)
		from cursor import get_cursor_line
		cursor_line = get_cursor_line(self.editor.textbuffer)
		begin_position = self.editor.textbuffer.get_iter_at_line(cursor_line)
		try:
			begin_selection, end_selection = self.editor.textbuffer.get_selection_bounds()
		except ValueError:
			if begin_position.get_char() in [" ", "\t"]:
				menuitem.set_property("sensitive", True)
			else:
				menuitem.set_property("sensitive", False)
			return True
		first_selected_line = begin_selection.get_line()
		last_selected_line = end_selection.get_line()
		if first_selected_line == last_selected_line:
			if begin_position.get_char() in [" ", "\t"]:
				menuitem.set_property("sensitive", True)
			else:
				menuitem.set_property("sensitive", False)
			return True
		indentation_is_possible = False
		for line in range(first_selected_line, last_selected_line+1):
			begin_position = self.editor.textbuffer.get_iter_at_line(line)
			if begin_position.get_char() in [" ", "\t"]:
				indentation_is_possible = True
		menuitem.set_property("sensitive", indentation_is_possible)
		return True
