# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module documents the lines menu item object for the text editor's popup
menu.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import ImageMenuItem

class ScribesLinesPopupMenuItem(ImageMenuItem):
	"""
	This class creates the spaces popup menu item for the text editor.
	"""

	def __init__(self, scribesview):
		"""
		Initialize the popup menu item.

		@param self: Reference to the ScribesLinesPopupMenuItem instance.
		@type self: A ScribesLinesPopupMenuItem object.

		@param scribesview: The text editor's text view.
		@type scribesview: A ScribesTextView object.
		"""
		from internationalization import msg0395
		ImageMenuItem.__init__(self, msg0395)
		self.__init_attributes(scribesview)
		self.__creates_widgets()
		self.__set_properties()
		self.delete_line_menuitem.connect("activate", self.__popup_activate_cb)
		self.join_line_menuitem.connect("activate", self.__popup_activate_cb)
		self.free_line_above_menuitem.connect("activate", self.__popup_activate_cb)
		self.free_line_below_menuitem.connect("activate", self.__popup_activate_cb)
		self.delete_cursor_to_end_menuitem.connect("activate", self.__popup_activate_cb)
		self.delete_cursor_to_begin_menuitem.connect("activate", self.__popup_activate_cb)

	def __init_attributes(self, scribesview):
		"""
		Initialize the popup menu item's attributes.

		@param self: Reference to the ScribesLinesPopupMenuItem instance.
		@type self: A ScribesLinesPopupMenuItem object.

		@param scribesview: The text editor's text view.
		@type scribesview: A ScribesTextView object.
		"""
		self.scribesview = scribesview
		self.editor = scribesview.editor
		self.menu = None
		self.image = None
		self.delete_line_menuitem = None
		self.join_line_menuitem = None
		self.free_line_below_menuitem = None
		self.free_line_above_menuitem = None
		self.delete_cursor_to_begin_menuitem = None
		self.delete_cursor_to_end_menuitem = None
		return

	def __creates_widgets(self):
		"""
		Create the popup menu item's interface.

		@param self: Reference to the ScribesLinesPopupMenuItem instance.
		@type self: A ScribesLinesPopupMenuItem object.
		"""
		from gtk import Image, STOCK_JUSTIFY_CENTER, Menu
		self.image = Image()
		self.image.set_property("stock", STOCK_JUSTIFY_CENTER)
		self.menu = Menu()
		from utils import create_menuitem
		from internationalization import msg0396, msg0397, msg0398
		from internationalization import msg0399, msg0400, msg0401
		self.delete_line_menuitem = create_menuitem(msg0396)
		self.join_line_menuitem = create_menuitem(msg0397)
		self.free_line_below_menuitem = create_menuitem(msg0399)
		self.free_line_above_menuitem = create_menuitem(msg0398)
		self.delete_cursor_to_end_menuitem = create_menuitem(msg0400)
		self.delete_cursor_to_begin_menuitem = create_menuitem(msg0401)
		return

	def __set_properties(self):
		"""
		Set the menu item's properties.

		@param self: Reference to the ScribesLinesPopupMenuItem instance.
		@type self: A ScribesLinesPopupMenuItem object.
		"""
		self.set_image(self.image)
		self.set_submenu(self.menu)
		self.menu.append(self.join_line_menuitem)
		self.menu.append(self.delete_line_menuitem)
		self.menu.append(self.free_line_below_menuitem)
		self.menu.append(self.free_line_above_menuitem)
		self.menu.append(self.delete_cursor_to_end_menuitem)
		self.menu.append(self.delete_cursor_to_begin_menuitem)
		if self.editor.is_readonly:
			self.set_property("sensitive", False)
		return

	def __popup_activate_cb(self, menuitem):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the ScribesLinesPopupMenuItem instance.
		@type self: A ScribesLinesPopupMenuItem object.

		@param menuitem: The popup menu's menuitem.
		@type menuitem: A gtk.MenuItem object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if menuitem == self.delete_line_menuitem:
			self.editor.triggermanager.trigger("delete_line")
		elif menuitem == self.join_line_menuitem:
			self.editor.triggermanager.trigger("join_line")
		elif menuitem == self.free_line_below_menuitem:
			self.editor.triggermanager.trigger("free_line_below")
		elif menuitem == self.free_line_above_menuitem:
			self.editor.triggermanager.trigger("free_line_above")
		elif menuitem == self.delete_cursor_to_begin_menuitem:
			self.editor.triggermanager.trigger("delete_cursor_to_begin")
		elif menuitem == self.delete_cursor_to_end_menuitem:
			self.editor.triggermanager.trigger("delete_cursor_to_end")
		return True
