# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module documents the spaces menu item object for the text editor's popup
menu.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import ImageMenuItem

class ScribesSelectionPopupMenuItem(ImageMenuItem):
	"""
	This class creates the spaces popup menu item for the text editor.
	"""

	def __init__(self, scribesview):
		"""
		Initialize the popup menu item.

		@param self: Reference to the ScribesSelectionPopupMenuItem instance.
		@type self: A ScribesSelectionPopupMenuItem object.

		@param scribesview: The text editor's text view.
		@type scribesview: A ScribesTextView object.
		"""
		from internationalization import msg0392
		ImageMenuItem.__init__(self, msg0392)
		self.__init_attributes(scribesview)
		self.__creates_widgets()
		self.__set_properties()
		self.select_word_menuitem.connect("activate", self.__popup_activate_cb)
		self.select_line_menuitem.connect("activate", self.__popup_activate_cb)
		self.select_sentence_menuitem.connect("activate", self.__popup_activate_cb)
		self.select_paragraph_menuitem.connect("activate", self.__popup_activate_cb)
		self.select_word_menuitem.connect("map-event", self.__popup_word_map_event_cb)
		self.select_line_menuitem.connect("map-event", self.__popup_line_map_event_cb)
		self.select_sentence_menuitem.connect("map-event", self.__popup_sentence_map_event_cb)
		self.select_paragraph_menuitem.connect("map-event", self.__popup_sentence_map_event_cb)

	def __init_attributes(self, scribesview):
		"""
		Initialize the popup menu item's attributes.

		@param self: Reference to the ScribesSelectionPopupMenuItem instance.
		@type self: A ScribesSelectionPopupMenuItem object.

		@param scribesview: The text editor's text view.
		@type scribesview: A ScribesTextView object.
		"""
		self.scribesview = scribesview
		self.editor = scribesview.editor
		self.menu = None
		self.image = None
		self.select_word_menuitem = None
		self.select_line_menuitem = None
		self.select_sentence_menuitem = None
		self.select_paragraph_menuitem = None
		return

	def __creates_widgets(self):
		"""
		Create the popup menu item's interface.

		@param self: Reference to the ScribesSelectionPopupMenuItem instance.
		@type self: A ScribesSelectionPopupMenuItem object.
		"""
		from gtk import Image, STOCK_BOLD, Menu
		self.image = Image()
		self.image.set_property("stock", STOCK_BOLD)
		self.menu = Menu()
		from utils import create_menuitem
		from internationalization import msg0393, msg0394, msg0402
		from internationalization import msg0403
		self.select_word_menuitem = create_menuitem(msg0393)
		self.select_line_menuitem = create_menuitem(msg0394)
		self.select_sentence_menuitem = create_menuitem(msg0402)
		self.select_paragraph_menuitem = create_menuitem(msg0403)
		return

	def __set_properties(self):
		"""
		Set the menu item's properties.

		@param self: Reference to the ScribesSelectionPopupMenuItem instance.
		@type self: A ScribesSelectionPopupMenuItem object.
		"""
		self.set_image(self.image)
		self.set_submenu(self.menu)
		self.menu.append(self.select_line_menuitem)
		self.menu.append(self.select_word_menuitem)
		self.menu.append(self.select_sentence_menuitem)
		self.menu.append(self.select_paragraph_menuitem)
		if self.editor.is_readonly:
			self.set_property("sensitive", False)
		return

	def __popup_activate_cb(self, menuitem):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the ScribesSelectionPopupMenuItem instance.
		@type self: A ScribesSelectionPopupMenuItem object.

		@param menuitem: The popup menu's menuitem.
		@type menuitem: A gtk.MenuItem object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if menuitem == self.select_word_menuitem:
			self.editor.triggermanager.trigger("select_word")
		elif menuitem == self.select_line_menuitem:
			self.editor.triggermanager.trigger("select_line")
		elif menuitem == self.select_sentence_menuitem:
			self.editor.triggermanager.trigger("select_sentence")
		elif menuitem == self.select_paragraph_menuitem:
			self.editor.triggermanager.trigger("select_paragraph")
		return True

	def __popup_word_map_event_cb(self, menuitem, event):
		"""
		Handles callback when the "map-event" signal is emitted.
		"""
		menuitem.set_property("sensitive", False)
		from word import inside_word, starts_word, ends_word
		from cursor import get_cursor_iterator
		cursor_position = get_cursor_iterator(self.editor.textbuffer)
		if inside_word(cursor_position) or starts_word(cursor_position) or ends_word(cursor_position):
			menuitem.set_property("sensitive", True)
		return True

	def __popup_line_map_event_cb(self, menuitem, event):
		"""
		Handles callback when the "map-event" signal is emitted.
		"""
		menuitem.set_property("sensitive", False)
		from lines import get_line_bounds
		begin_position, end_position = get_line_bounds(self.editor.textbuffer)
		if not begin_position.get_char() in ["\n", "\x00"]:
			menuitem.set_property("sensitive", True)
		return True

	def __popup_sentence_map_event_cb(self, menuitem, event):
		"""
		Handles callback when the "map-event" signal is emitted.
		"""
		menuitem.set_property("sensitive", False)
		from cursor import get_cursor_iterator
		cursor_position = get_cursor_iterator(self.editor.textbuffer)
		if cursor_position.starts_sentence() or cursor_position.ends_sentence() or cursor_position.inside_sentence():
			menuitem.set_property("sensitive", True)
		return True
