# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module documents the spaces menu item object for the text editor's popup
menu.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import ImageMenuItem

class ScribesSpacesPopupMenuItem(ImageMenuItem):
	"""
	This class creates the spaces popup menu item for the text editor.
	"""

	def __init__(self, scribesview):
		"""
		Initialize the popup menu item.

		@param self: Reference to the ScribesSpacesPopupMenuItem instance.
		@type self: A ScribesSpacesPopupMenuItem object.

		@param scribesview: The text editor's text view.
		@type scribesview: A ScribesTextView object.
		"""
		from internationalization import msg0388
		ImageMenuItem.__init__(self, msg0388)
		self.__init_attributes(scribesview)
		self.__creates_widgets()
		self.__set_properties()
		self.tabs_to_spaces_menuitem.connect("activate", self.__popup_activate_cb)
		self.spaces_to_tabs_menuitem.connect("activate", self.__popup_activate_cb)
		self.removes_spaces_menuitem.connect("activate", self.__popup_activate_cb)
		 
	def __init_attributes(self, scribesview):
		"""
		Initialize the popup menu item's attributes.

		@param self: Reference to the ScribesSpacesPopupMenuItem instance.
		@type self: A ScribesSpacesPopupMenuItem object.

		@param scribesview: The text editor's text view.
		@type scribesview: A ScribesTextView object.
		"""
		self.scribesview = scribesview
		self.editor = scribesview.editor
		self.menu = None
		self.image = None
		self.tabs_to_spaces_menuitem = None
		self.spaces_to_tabs_menuitem = None
		self.removes_spaces_menuitem = None
		return

	def __creates_widgets(self):
		"""
		Create the popup menu item's interface.

		@param self: Reference to the ScribesSpacesPopupMenuItem instance.
		@type self: A ScribesSpacesPopupMenuItem object.
		"""
		from gtk import Image, STOCK_CLEAR, Menu
		self.image = Image()
		self.image.set_property("stock", STOCK_CLEAR)
		self.menu = Menu()
		from utils import create_menuitem
		from internationalization import msg0389, msg0390, msg0391
		self.tabs_to_spaces_menuitem = create_menuitem(msg0389)
		self.spaces_to_tabs_menuitem = create_menuitem(msg0390)
		self.removes_spaces_menuitem = create_menuitem(msg0391)
		return

	def __set_properties(self):
		"""
		Set the menu item's properties.

		@param self: Reference to the ScribesSpacesPopupMenuItem instance.
		@type self: A ScribesSpacesPopupMenuItem object.
		"""
		self.set_image(self.image)
		self.set_submenu(self.menu)
		self.menu.append(self.spaces_to_tabs_menuitem)
		self.menu.append(self.tabs_to_spaces_menuitem)
		self.menu.append(self.removes_spaces_menuitem)
		if self.editor.is_readonly:
			self.set_property("sensitive", False)
		return

	def __popup_activate_cb(self, menuitem):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the ScribesSpacesPopupMenuItem instance.
		@type self: A ScribesSpacesPopupMenuItem object.

		@param menuitem: The popup menu's menuitem.
		@type menuitem: A gtk.MenuItem object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if menuitem == self.tabs_to_spaces_menuitem:
			self.editor.triggermanager.trigger("tabs_to_spaces")
		elif menuitem == self.spaces_to_tabs_menuitem:
			self.editor.triggermanager.trigger("spaces_to_tabs")
		else:
			self.editor.triggermanager.trigger("remove_trailing_spaces")
		return True
