# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This modules documents a class that creates the preferences dialog for
the text editor. The dialog allows users to customize the behavior and
properties of the text editor's buffer or view.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from sdialog import Dialog

class PreferenceDialog(Dialog):
	"""
	This class creates a dialog that allows users to customize the
	behavior and properties of the text editor.
	"""

	def __init__(self, editor):
		"""
		Initialize the preferences dialog.

		@param self: Reference to the PreferenceDialog instance.
		@type self: A PreferenceDialog object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		Dialog.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__arrange_widgets()
		self.__close_button.connect("clicked", self.__clicked_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the dialog's data attributes.

		@param self: Reference to the PreferenceDialog instance.
		@type self: A PreferenceDialog object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		self.__status_id = None
		self.__trigger = None
		from gtk import STOCK_CLOSE, Button
		self.__close_button = Button(stock=STOCK_CLOSE, use_underline=True)
		self.__label = self.__create_labels()
		from preffontbutton import PreferencesFontButton
		self.__font_button = PreferencesFontButton(editor)
		from preftabspinbutton import TabSpinButton
		self.__tab_spin_button = TabSpinButton(editor)
		from preftabcheckbutton import TabCheckButton
		self.__tab_check_button = TabCheckButton(editor)
		from preftextwrapcheckbutton import TextWrapCheckButton
		self.__text_wrap_check_button = TextWrapCheckButton(editor)
		from prefmargincheckbutton import MarginCheckButton
		self.__margin_check_button = MarginCheckButton(editor)
		from prefspellcheckbutton import SpellCheckButton
		self.__spell_check_button = SpellCheckButton(editor)
		from prefmarginspinbutton import MarginSpinButton
		self.__margin_spin_button = MarginSpinButton(editor)
		return

	def show_dialog(self):
		"""
		Show the preferences dialog.

		@param self: Reference to the PreferenceDialog instance.
		@type self: A PreferenceDialog object.
		"""
		self.__editor.emit("show-dialog", self)
		from internationalization import msg0196
		self.__status_id = self.__editor.feedback.set_modal_message(msg0196, "preferences")
		Dialog.show_dialog(self)
		return

	def hide_dialog(self):
		"""
		Hide the preferences dialog.

		@param self: Reference to the PreferenceDialog instance.
		@type self: A PreferenceDialog object.
		"""
		self.__editor.emit("hide-dialog", self)
		self.__editor.feedback.unset_modal_message(self.__status_id)
		Dialog.hide_dialog(self)
		return

	def __set_properties(self):
		"""
		Define the default behavior of the dialog.

		@param self: Reference to the PreferenceDialog instance.
		@type self: A PreferenceDialog object.
		"""
		self.set_property("name", "scribes preferences dialog")
		from internationalization import msg0132
		self.set_property("title", msg0132)
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self.__editor.window, 2.67, 2.67)
		self.set_property("default-width", width)
		self.set_transient_for(self.__editor.window)
		return

	def __arrange_widgets(self):
		"""
		Arrange the widgets of the preferences dialog.

		@param self: Reference to the PreferenceDialog instance.
		@type self: A PreferenceDialog object.
		"""
		self.main_area.set_spacing(10)
		from gtk import Alignment, HBox
		alignment = Alignment(xalign=0.0)
		alignment.add(self.__label[0])
		self.main_area.pack_start(alignment, False, False, 0)

		hbox = HBox(homogeneous=False, spacing=10)
		alignment = Alignment(xalign=0.4, yalign=0.0, xscale=0.7, yscale=0.0)
		alignment.add(hbox)
		hbox.pack_start(self.__label[1], False, False, 0)
		hbox.pack_start(self.__font_button, True, True, 0)
		self.__label[1].set_mnemonic_widget(self.__font_button)
		self.main_area.pack_start(alignment, False, False, 0)

		alignment = Alignment(xalign=0.0)
		alignment.add(self.__label[2])
		self.main_area.pack_start(alignment, False, False, 0)

		hbox = HBox(homogeneous=False, spacing=10)
		alignment = Alignment(xalign=0.1, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		hbox.pack_start(self.__label[3], False, False, 0)
		hbox.pack_start(self.__tab_spin_button, False, False, 0)
		self.__label[3].set_mnemonic_widget(self.__tab_spin_button)
		self.main_area.pack_start(alignment, False, False, 0)

		hbox = HBox(homogeneous=False, spacing=10)
		alignment = Alignment(xalign=0.12, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		hbox.pack_start(self.__tab_check_button, False, False, 0)
		self.main_area.pack_start(alignment, False, False, 0)

		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.__label[4])
		self.main_area.pack_start(alignment, False, False, 0)

		hbox = HBox(homogeneous=False, spacing=10)
		alignment = Alignment(xalign=0.1, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		hbox.pack_start(self.__text_wrap_check_button, False, False, 0)
		self.main_area.pack_start(alignment, False, False, 0)

		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.__label[5])
		self.main_area.pack_start(alignment, True, False, 0)

		hbox = HBox(homogeneous=False, spacing=10)
		alignment = Alignment(xalign=0.1, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		hbox.pack_start(self.__margin_check_button, False, False, 0)
		self.main_area.pack_start(alignment, False, False, 0)

		hbox = HBox(homogeneous=False, spacing=10)
		alignment = Alignment(xalign=0.15, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		hbox.pack_start(self.__label[6], False, False, 0)
		hbox.pack_start(self.__margin_spin_button, False, False, 0)
		self.__label[6].set_mnemonic_widget(self.__margin_spin_button)
		self.main_area.pack_start(alignment, False, False, 0)

		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.__label[7])
		self.main_area.pack_start(alignment, True, False, 0)

		hbox = HBox(homogeneous=False, spacing=10)
		alignment = Alignment(xalign=0.1, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		hbox.pack_start(self.__spell_check_button, False, False, 0)
		self.main_area.pack_start(alignment, False, False, 0)

		self.button_area.pack_start(self.__close_button, False, False, 0)
		return

	def __clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the PreferenceDialog instance.
		@type self: A PreferenceDialog object.

		@param button: The close button.
		@type button: A Button object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_dialog()
		return True

	def __create_labels(self):
		"""
		Create all labels for the preferences dialog.

		@param self: Reference to the PreferenceDialog instance.
		@type self: A PreferenceDialog object.

		@return: A list of labels for the dialog.
		@rtype: A List object.
		"""
		from internationalization import msg0063, msg0064, msg0065
		from internationalization import msg0066, msg0067, msg0069
		from internationalization import msg0071, msg0072
		string_list = [msg0063, msg0064, msg0065, msg0066, msg0067, msg0069,
						msg0071, msg0072]
		labels = []
		from gtk import Label
		for string in string_list:
			label = Label(string)
			label.set_use_underline(True)
			label.set_use_markup(True)
			labels.append(label)
		return labels
