# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates a menu for the text editor's
preference toolbar button.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import Menu

class PreferenceMenu(Menu):
	"""
	This class creates a menu for the text editor's preference toolbar
	button.
	"""

	def __init__(self, editor):
		"""
		Initialize the menu.

		@param self: Reference to the PreferenceMenu instance.
		@type self: A PreferenceMenu object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		Menu.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.show_all()
		self.__color_editor_menuitem.connect("activate", self.__color_activate_cb)
		self.__template_editor_menuitem.connect("activate", self.__template_activate_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the menu's attributes.

		@param self: Reference to the PreferenceMenu instance.
		@type self: A PreferenceMenu object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		from internationalization import msg0296, msg0305
		from gtk import STOCK_SELECT_COLOR, STOCK_PROPERTIES
		from utils import create_menuitem
		self.__color_editor_menuitem = create_menuitem(msg0296, STOCK_SELECT_COLOR)
		self.__template_editor_menuitem = create_menuitem(msg0305, STOCK_PROPERTIES)
		return

	def __set_properties(self):
		"""
		Define the default behavior and properties of the menu.

		@param self: Reference to the PreferenceMenu instance.
		@type self: A PreferenceMenu object.
		"""
		self.append(self.__color_editor_menuitem)
		self.append(self.__template_editor_menuitem)
		from tooltips import template_button_tip, color_button_tip
		self.__editor.tip.set_tip(self.__color_editor_menuitem, color_button_tip)
		self.__editor.tip.set_tip(self.__template_editor_menuitem, template_button_tip)
		return

	def __color_activate_cb(self, menuitem):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the PreferenceMenu instance.
		@type self: A PreferenceMenu object.

		@param menuitem: The color editor menuitem.
		@type menuitem: A gtk.MenuItem object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.__editor.triggermanager.trigger("show_color_editor")
		return True

	def __template_activate_cb(self, menuitem):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the PreferenceMenu instance.
		@type self: A PreferenceMenu object.

		@param menuitem: The template editor menuitem.
		@type menuitem: A gtk.MenuItem object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.__editor.triggermanager.trigger("show_template_editor")
		return True
