# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates the text wrap check button
for the text editor's preference dialog.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import CheckButton

class TextWrapCheckButton(CheckButton):
	"""
	This class creates a check button for the text editor's preference
	dialog. The check button allows users to set the buffer's wrapping
	properties.
	"""

	def __init__(self, editor):
		"""
		Initialize the check button.

		@param self: Reference to the TextWrapCheckButton instance.
		@type self: A TextWrapCheckButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		CheckButton.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__client.notify_add("/apps/scribes/text_wrapping", self.__wrap_text_cb)
		self.connect("toggled", self.__toggled_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the button's data attributes.

		@param self: Reference to the TextWrapCheckButton instance.
		@type self: A TextWrapCheckButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		from gconf import client_get_default
		self.__client = client_get_default()
		return

	def __set_properties(self):
		"""
		Define the default behavior of the button.

		@param self: Reference to the TextWrapCheckButton instance.
		@type self: A TextWrapCheckButton object.
		"""
		wrap_text = self.__client.get_bool("/apps/scribes/text_wrapping")
		if wrap_text is None:
			wrap_text = False
		self.set_active(wrap_text)
		from internationalization import msg0068
		self.set_label(msg0068)
		self.set_use_underline(True)
		from tooltips import tw_check_button_tip
		self.__editor.tip.set_tip(self, tw_check_button_tip)
		return

	def __wrap_text_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when text wrapping properties change.

		@param self: Reference to the TextWrapCheckButton instance.
		@type self: A TextWrapCheckButton object.
		"""
		wrap_text = client.get_bool("/apps/scribes/text_wrapping")
		if wrap_text is None:
			wrap_text = False
		if wrap_text:
			if self.get_active() is False:
				self.set_active(True)
			from internationalization import msg0199
			self.__editor.feedback.update_status_message(msg0199, "succeed", 5)
		else:
			if self.get_active():
				self.set_active(False)
			from internationalization import msg0200
			self.__editor.feedback.update_status_message(msg0200, "succeed", 5)
		return

	def __toggled_cb(self, button):
		"""
		Handles callback when the "toggled" signal is emitted.

		@param self: Reference to the TextWrapCheckButton instance.
		@type self: A TextWrapCheckButton object.

		@param button: Reference to the TextWrapCheckButton.
		@type button: A TextWrapCheckButton object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		wrap_text = self.get_active()
		if wrap_text:
			if self.__client.get_bool("/apps/scribes/text_wrapping") is False:
				self.__client.set_bool("/apps/scribes/text_wrapping", True)
		else:
			if self.__client.get_bool("/apps/scribes/text_wrapping"):
				self.__client.set_bool("/apps/scribes/text_wrapping", False)
		return True
