# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that shows the text editor's print dialog.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import PrintOperation

class PrintDialog(PrintOperation):
	"""
	This class implements and object that shows the print dialog.
	"""

	def __init__(self, editor):
		"""
		Initialize the object.

		@param self: Reference to the PrintDialog instance.
		@type self: A PrintDialog object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		PrintOperation.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.connect("begin-print", self.__dialog_begin_print_cb)
		self.connect("end-print", self.__dialog_end_print_cb)
		self.connect("request-page-setup", self.__dialog_request_page_setup_cb)
		self.connect("draw-page", self.__dialog_draw_page_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the object's data attributes.

		@param self: Reference to the PrintDialog instance.
		@type self: A PrintDialog object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__editor = editor
		return

	def __set_properties(self):
		"""
		Define the default properties of the print object.

		@param self: Reference to the PrintDialog instance.
		@type self: A PrintDialog object.
		"""
		from gtk import UNIT_MM
		self.set_property("n-pages", 1)
		self.set_property("unit", UNIT_MM)
		self.set_property("show-progress", True)
		self.set_property("use-full-page", True)
		return

	def show_dialog(self):
		"""
		Show the print dialog.

		@param self: Reference to the PrintDialog instance.
		@type self: A PrintDialog object.
		"""
		from gtk import PRINT_OPERATION_ACTION_PRINT_DIALOG
		from gtk import PRINT_OPERATION_RESULT_ERROR
		from gtk import PRINT_OPERATION_RESULT_APPLY
		from gtk import PRINT_OPERATION_RESULT_IN_PROGRESS
		response = self.run(PRINT_OPERATION_ACTION_PRINT_DIALOG, self.__editor.window)
		if response == PRINT_OPERATION_RESULT_ERROR:
			print "An error occured."
		elif response == PRINT_OPERATION_RESULT_IN_PROGRESS:
			print "Printing in progress"
		else:
			print "Hide dialog"
		return

	def __dialog_begin_print_cb(self, operation, context):
		"""
		Handles callback when the "begin-print" signal is emitted.

		@param self: Reference to the PrintDialog instance.
		@type self: A PrintDialog object.

		@param operation: A print operation object.
		@type operation: A PrintOperation object.

		@param context: The context for the print operation.
		@type context: A PrintContext object.
		"""
		if self.get_print_settings():
			print "Settings are available."
			self.set_print_settings(self.get_print_settings())
		print "Begin printing"
		return

	def __dialog_request_page_setup_cb(self, operation, context, page_nr, setup):
		"""
		Handles callback when the "request-page-setup" signal is emitted.

		@param self: Reference to the PrintDialog instance.
		@type self: A PrintDialog object.

		@param operation: A print operation object.
		@type operation: A PrintOperation object.

		@param context: The context for the print operation.
		@type context: A PrintContext object.

		@param page_nr: The number of the currently printed page.
		@type page_nr: An Integer object.

		@param setup: A page setup object.
		@type setup: A gtk.PageSetup object.
		"""
		if setup:
			print "Setup is available."
			self.set_default_page_setup(setup)
		print "In request-page-setup"
		return

	def __dialog_end_print_cb(self, operation, context):
		"""
		Handles callback when the "end-print" signal is emitted.

		@param self: Reference to the PrintDialog instance.
		@type self: A PrintDialog object.

		@param operation: A print operation object.
		@type operation: A PrintOperation object.

		@param context: The context for the print operation.
		@type context: A PrintContext object.
		"""
		print "Finished printing."
		return

	def __dialog_draw_page_cb(self, operation, context, page_nr):
		"""
		Handles callback when the "draw-page" signal is emitted.

		@param self: Reference to the PrintDialog instance.
		@type self: A PrintDialog object.

		@param operation: A print operation object.
		@type operation: A PrintOperation object.

		@param context: The context for the print operation.
		@type context: A PrintContext object.

		@param page_nr: The number of the currently printed page.
		@type page_nr: An Integer object.

		@param setup: A page setup object.
		@type setup: A gtk.PageSetup object.
		"""
		cairo_context = context.get_cairo_context()

		# Draw a red rectangle, as wide as the paper (inside the margins)

		cairo_context.rectangle(0, 0, context.get_width(), 10)
		cairo_context.set_source_rgb(0.8, 0.8, 0.8)
		cairo_context.fill()

		# Draw some text
		layout = context.create_pango_layout()
		if self.__editor.uri:
			begin, end = self.__editor.textbuffer.get_bounds()
			text = self.__editor.textbuffer.get_text(begin, end)
			layout.set_text(text)
		from pango import FontDescription, ALIGN_CENTER, SCALE
		desc = FontDescription("Monospace 12")
		layout.set_font_description(desc)
		layout.set_width(context.get_width())
		layout.set_alignment(ALIGN_CENTER)
		x,layout_height = layout.get_size()
		text_height = layout_height / SCALE
		cairo_context.move_to(context.get_width() / 2,  (10 - text_height) / 2)
		cairo_context.show_layout(layout)
		print "In draw-page"
		return
