# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This modules exposes a class responsible for creating the text editor's
print dialog.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class PrintDialog(object):
	"""
	This class creates the print dialog for the text editor.
	"""

	def __init__(self, editor):
		"""
		Initialize the print dialog.

		@param self: Reference to the ScribesPrintDialog instance.
		@type self: A ScribesPrintDialog object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__init_attributes(editor)
		self.__set_properties()
		self.dialog.connect("response", self.__printdialog_response_cb)
		self.__show_dialog()

	def __init_attributes(self, editor):
		"""
		Initialize the Dialog's attributes.

		@param self: Reference to the ScribesPrintDialog instance.
		@type self: A ScribesPrintDialog object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		self.status_id = None
		from printjob import ScribesPrintJob
		printjob  = ScribesPrintJob(editor)
		self.job = printjob.get_job()
		from gnomeprint.ui import Dialog, DIALOG_RANGE
		from internationalization import msg0076
		self.dialog = Dialog(self.job, msg0076, DIALOG_RANGE)
		return

	def __set_properties(self):
		"""
		Set the dialog's properties.

		@param self: Reference to the ScribesPrintDialog instance.
		@type self: A ScribesPrintDialog object.
		"""
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self.editor.window,
															1.6, 1.929648241)
		self.dialog.set_default_size(width, height)
		self.dialog.set_icon_name("stock_print")
		self.dialog.set_transient_for(self.editor.window)
		return

	def __show_dialog(self):
		"""
		Show the print dialog.

		@param self: Reference to the ScribesPrintDialog instance.
		@type self: A ScribesPrintDialog object.
		"""
		from internationalization import msg0194
		self.status_id = self.editor.feedback.set_modal_message(msg0194, "print")
		self.editor.emit("show-dialog", self.dialog)
		self.dialog.run()
		return

	def __printdialog_response_cb(self, dialog, response):
		"""
		Handles callback when the "response" signal is emitted.

		@param self: Reference to the ScribesPrintDialog instance.
		@type self: A ScribesPrintDialog object.
		"""
		from gnomeprint.ui import DIALOG_RESPONSE_PRINT, DIALOG_RESPONSE_PREVIEW
		if response == DIALOG_RESPONSE_PREVIEW:
			from printpreview import ScribesPrintPreview
			ScribesPrintPreview(self)
		elif response == DIALOG_RESPONSE_PRINT:
			self.editor.feedback.unset_modal_message(self.status_id)
			self.job.print_()
			self.editor.emit("hide-dialog", self.dialog)
			self.dialog.hide()
		else:
			self.editor.feedback.unset_modal_message(self.status_id)
			self.editor.emit("hide-dialog", self.dialog)
			self.dialog.hide()
		return
