# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from dialog import ScribesDialog

class ScribesRemoteDialog(ScribesDialog):
	"""
	This class creates a dialog used to open files at remote locations.
	"""

	def __init__(self, editor):
		"""
		Initialize the remote dialog and set its state, behavior and properties.

		@param self: Reference to the ScribesRemoteDialog instance.
		@type self: A ScribesRemoteDialog object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		ScribesDialog.__init__(self, editor)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__create_widgets()
		self.__arrange_widgets()
		self.comboboxentry.child.connect("activate", self.__remote_activate_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the dialog's attributes.

		@param self: Reference to the ScribesRemoteDialog instance.
		@type self: A ScribesRemoteDialog object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		return

	def __set_properties(self):
		"""
		Set the dialog's properties.

		@param self: Reference to the ScribesRemoteDialog instance.
		@type self: A ScribesRemoteDialog object.
		"""
		from internationalization import msg0026
		self.set_property("title", msg0026)
		self.set_property("role", "scribes_remote_dialog")
		self.set_property("icon-name", "stock_open")
		self.set_property("name", "scribesremotedialog")
		from gtk import STOCK_OPEN, STOCK_CANCEL, RESPONSE_OK, RESPONSE_CANCEL
		self.add_button(STOCK_CANCEL, RESPONSE_CANCEL)
		self.add_button(STOCK_OPEN, RESPONSE_OK)
		self.set_default_response(RESPONSE_OK)
		self.set_property("border-width", 10)
		self.vbox.set_spacing(10)
		self.vbox.set_homogeneous(False)
		self.action_area.set_homogeneous(False)
		return

	def __create_widgets(self):
		"""
		Create the widgets for the remote dialog.

		@param self: Reference to the ScribesRemoteDialog instance.
		@type self: A ScribesRemoteDialog object.
		"""
		self.label = self.__create_labels()
		from encodingbox import ScribesEncodingComboBox
		self.combobox = ScribesEncodingComboBox(self.editor)
		from comboboxentry import ScribesComboBoxEntry
		self.comboboxentry = ScribesComboBoxEntry(self.editor)
		from utils import create_encoding_box
		self.encoding_box  = create_encoding_box(self.combobox)
		return

	def __arrange_widgets(self):
		"""
		Arrange widgets after they have been created.

		@param self: Reference to the ScribesRemoteDialog instance.
		@type self: A ScribesRemoteDialog object.
		"""
		self.vbox.pack_start(self.label[0], True, True, 0)
		self.vbox.pack_start(self.comboboxentry, True, True, 0)
		self.vbox.pack_start(self.encoding_box, True, True, 0)
		return

	def show_dialog(self):
		"""
		Show the remote dialog.

		@param self: Reference to the ScribesRemoteDialog instance.
		@type self: A ScribesRemoteDialog object.
		"""
		self.editor.emit("show-dialog", self)
		self.show_all()
		from internationalization import msg0357
		status_id = self.editor.feedback.set_modal_message(msg0357, "open")
		response = self.run()
		self.editor.emit("hide-dialog", self)
		from gobject import timeout_add
		timeout_add(1, self.editor.feedback.unset_modal_message, status_id)
		self.hide_dialog()
		from gtk import RESPONSE_OK
		if response == RESPONSE_OK:
			from gobject import timeout_add
			timeout_add(1, self.__process_text_entry)
		return

	def __create_labels(self):
	 	"""
	 	Create labels for the preferences dialog.

	 	@param self: Reference to the Preferences instance.
	 	@type self: A Preferences object.
	 	"""
	 	from internationalization import msg0356
		string_list = [msg0356]
		label_list = []
		from gtk import Label
		# Convert strings to labels.
		for string in string_list:
			label = Label(string)
			label.set_use_underline(True)
			label.set_use_markup(True)
			label_list.append(label)
		return label_list

	def __remote_activate_cb(self, entry):
		"""
		Handles callback when the comboboxentry "activate" signal is emitted.

		@param self: Reference to the ScribesRemoteDialog instance.
		@type self: A ScribesRemoteDialog object.

		@param entry: Text entry for the ScribesComboBoxEntry
		@type entry: A gtk.Entry object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		from gtk import RESPONSE_OK
		self.response(RESPONSE_OK)
		return True

	def __process_text_entry(self):
		"""
		Feed the URI provided in the text entry the text editor's loader object.

		@param self: Reference to the ScribesRemoteDialog instance.
		@type self: A ScribesRemoteDialog object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		text = self.comboboxentry.child.get_text()
		if text:
		 	self.editor.fileloader.load_uris([text.strip()])
		return False
