# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module exposes a class the documents the implementation of an object that
perform text substition operations in the text editor's buffer.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE

class Replace(GObject):
	"""
	This class implements an object that performs text substition operations in
	the text editor's buffer. The class defines the behavior and default
	properties of the object.
	"""

	__gsignals__ = {
		"replacing": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"replaced": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"replaced-all": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"cancel": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, editor):
		"""
		Initialize the replace object.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		GObject.__init__(self)
		self.__init_attributes(editor)
		self.connect("cancel", self.__replace_cancel_cb)
		self.editor.connect("hide-bar", self.__replace_hide_bar_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the replace object's attributes.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		self.cancel_operation = False
		self.searchmanager = editor.searchmanager
		self.replace_tag = self.__create_replace_tag()
		return

	def replace(self, text):
		"""
		Replace selected text in the text editor's buffer.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.
		"""
		self.emit("replacing")
		self.cancel_operation = False
		begin, end = self.searchmanager.positions[self.searchmanager.index]
		textbuffer = self.editor.textbuffer
		begin_mark = textbuffer.create_mark(None, begin, True)
		self.searchmanager.reset()
		textbuffer.begin_user_action()
		textbuffer.delete(begin, end)
		begin = textbuffer.get_iter_at_mark(begin_mark)
		textbuffer.insert_with_tags(begin, text, self.replace_tag)
		textbuffer.end_user_action()
		if begin_mark.get_deleted() is False:
			textbuffer.delete_mark(begin_mark)
		self.emit("replaced")
		return

	def replace_all(self, text):
		"""
		Replace all selected text in the text editor's buffer.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.
		"""
		self.emit("replacing")
		self.cancel_operation = False
		position_marks = []
		textbuffer = self.editor.textbuffer
		def delete_marks():
			for item in position_marks:
				if item[0].get_deleted() is False:
					textbuffer.delete_mark(item[0])
				if item[1].get_deleted() is False:
					textbuffer.delete_mark(item[1])
		from utils import response
		for position in self.searchmanager.positions:
			response()
			mark_begin = self.editor.textbuffer.create_mark(None, position[0], True)
			mark_end = self.editor.textbuffer.create_mark(None, position[1], False)
			position_marks.append((mark_begin, mark_end))
			if self.cancel_operation:
				delete_marks()
				self.searchmanager.reset()
				return
		self.searchmanager.reset()
		from cursor import move_view_to_cursor
		textbuffer.begin_user_action()
		for position in position_marks:
			response()
			if self.cancel_operation:
				delete_marks()
				return
			begin = textbuffer.get_iter_at_mark(position[0])
			end = textbuffer.get_iter_at_mark(position[1])
			textbuffer.delete(begin, end)
			begin = textbuffer.get_iter_at_mark(position[0])
			textbuffer.place_cursor(begin)
			textbuffer.insert_with_tags(begin, text, self.replace_tag)
			move_view_to_cursor(self.editor.textview)
		textbuffer.end_user_action()
		delete_marks()
		if self.cancel_operation:
			return
		self.emit("replaced-all")
		return

	def reset(self):
		"""
		Return the state of the Replace object to a neutral one.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.
		"""
		begin, end = self.editor.textbuffer.get_bounds()
		self.editor.textbuffer.remove_tag(self.replace_tag, begin, end)
		return

	def cancel(self):
		"""
		Cancel a replace operation and reset the Replace object.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.
		"""
		self.emit("cancel")
		return

	def __create_replace_tag(self):
		"""
		Create the replace tag for the Replace object.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.
		"""
		tag = self.editor.textbuffer.create_tag()
		tag.set_property("background", "green")
		tag.set_property("foreground", "blue")
		from pango import WEIGHT_HEAVY
		tag.set_property("weight", WEIGHT_HEAVY)
		return tag

	def __replace_cancel_cb(self, replace):
		"""
		Handles callback when the "cancel" signal is emitted.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.

		@param replace: The text editor's replace object.
		@type replace: A Replace object.
		"""
		self.cancel_operation = True
		self.reset()
		return

	def __replace_hide_bar_cb(self, editor, bar):
		"""
		Handles callback when the "hide-bar" signal is emitted.

		@param self: Reference to the Replace instance.
		@type self: A Replace object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: The text editor's replace bar.
		@type bar: A ScribesReplaceBar object.
		"""
		self.cancel_operation = True
		self.reset()
		return
