# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that implements the replace bar's
incremental search check button.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import CheckButton

class ScribesReplaceIncrementalButton(CheckButton):
	"""
	This class implements the replace bar's incremental search check
	button. It defines the default property and behavior of the check
	button.
	"""

	def __init__(self, replacebar):
		"""
		Initialize the check button object.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param replacebar: The text editor's replacebar
		@type replacebar: A ScribesReplaceBar object.
		"""
		from internationalization import msg0418
		CheckButton.__init__(self, msg0418, use_underline=True)
		self.__init_attributes(replacebar)
		self.connect("toggled", self.__checkbutton_toggled_cb)
		self.editor.connect("show-bar", self.__checkbutton_show_bar_cb)
		self.editor.connect("hide-bar", self.__checkbutton_hide_bar_cb)
		self.searchmanager.connect("searching", self.__checkbutton_searching_cb)
		self.searchmanager.connect("matches-found", self.__checkbutton_matches_found_cb)
		self.searchmanager.connect("no-matches-found", self.__checkbutton_no_matches_found_cb)
		self.searchmanager.connect("cancel", self.__checkbutton_cancel_cb)

	def __init_attributes(self, replacebar):
		"""
		Initialize the check button's data attributes.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param replacebar: The text editor's replacebar
		@type replacebar: A ScribesReplaceBar object.
		"""
		self.replacebar = replacebar
		self.editor = replacebar.editor
		self.searchmanager = replacebar.search_replace_manager
		self.searchentry = replacebar.findbar_entry
		return

	def __checkbutton_toggled_cb(self, togglebutton):
		"""
		Handles callback when the "toggled" signal is emitted.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param togglebutton: The findbar's case check button.
		@type togglebutton: A CheckButton object.
		"""
		self.searchmanager.reset()
		value = self.get_property("active")
		self.searchmanager.enable_incremental_searching(value)
		return True

	def __checkbutton_show_bar_cb(self, editor, bar):
		"""
		Handles callback when the "show-bar" signal is emitted.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: One of the text editor's bars.
		@type bar: A ScribesBar object.
		"""
		if bar is self.replacebar:
			value = self.get_property("active")
			self.searchmanager.enable_incremental_searching(value)
		return

	def __checkbutton_hide_bar_cb(self, editor, bar):
		"""
		Handles callback when the "show-bar" signal is emitted.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: One of the text editor's bars.
		@type bar: A ScribesBar object.
		"""
		if bar is self.replacebar:
			self.searchmanager.reset()
		return

	def __checkbutton_searching_cb(self, searchmanager):
		"""
		Handles callback when the "searching" signal is emitted.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param searchmanager: The text editor's search processor.
		@type searchmanager: A SearchProcessor object.
		"""
		self.set_property("sensitive", False)
		return

	def __checkbutton_matches_found_cb(self, searchmanager):
		"""
		Handles callback when the "matches-found" signal is emitted.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param searchmanager: The text editor's search processor.
		@type searchmanager: A SearchProcessor object.
		"""
		self.set_property("sensitive", True)
		return

	def __checkbutton_no_matches_found_cb(self, searchmanager):
		"""
		Handles callback when the "no-matches-found" signal is emitted.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param searchmanager: The text editor's search processor.
		@type searchmanager: A SearchProcessor object.
		"""
		self.set_property("sensitive", True)
		return

	def __checkbutton_cancel_cb(self, searchmanager):
		"""
		Handles callback when the "cancel" signal is emitted.

		@param self: Reference to the ScribesReplaceIncrementalButton instance.
		@type self: A ScribesReplaceIncrementalButton object.

		@param searchmanager: The text editor's search processor.
		@type searchmanager: A SearchProcessor object.
		"""
		self.set_property("sensitive", True)
		return
