# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module exposes a class responsible for saving files to both local and
remote locations.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, TYPE_NONE, SIGNAL_RUN_LAST, TYPE_STRING

class FileSaver(GObject):
	"""
	This class creates an object responsible for saving files to permanent
	storage locations, for example, a hard disk or remote locations. Documents
	are first saved to a temporary location called a swap area before they are
	transferred to their permanent location.
	"""

	# Signals for the saving object.
	__gsignals__ = {
		"saving": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"save-begin": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"save-error": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING,)),
		"save-end": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"swap-created": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"swap-error": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"swap-removed": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"transfer-begin": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"transfer-error": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING,)),
		"transfer-end": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"saved": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"error": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING,)),
	}

	def __init__(self, editor):
		"""
		Initialize the file saver object.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param editor: Reference to an editor object
		@type editor: An Editor object.
		"""
		GObject.__init__(self)
		self.__init_attributes(editor)
		self.editor.connect("close-document", self.__filesaver_close_document_cb)
		self.editor.connect("save-document", self.__filesaver_save_document_cb)
		#self.editor.connect("renamed-document", self.__filesaver_renamed_document_cb)
		self.editor.connect("enable-readonly", self.__filesaver_enable_readonly_cb)
		self.editor.connect("disable-readonly", self.__filesaver_disable_readonly_cb)
		self.textbuffer.connect("changed", self.__filesaver_changed_cb)
		self.textbuffer.connect("modified-changed", self.__filesaver_modified_changed_cb)
		self.connect("saving", self.__filesaver_saving_cb)
		self.connect("save-begin", self.__filesaver_save_begin_cb)
		self.connect("save-error", self.__filesaver_save_error_cb)
		self.connect("save-end", self.__filesaver_save_end_cb)
		self.connect("swap-created", self.__filesaver_swap_created_cb)
		self.connect("swap-removed", self.__filesaver_swap_removed_cb)
		self.connect("transfer-begin", self.__filesaver_transfer_begin_cb)
		self.connect("transfer-error", self.__filesaver_transfer_error_cb)
		self.connect("transfer-end", self.__filesaver_transfer_end_cb)
		self.connect("saved", self.__filesaver_saved_cb)
		self.connect("error", self.__filesaver_error_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the file saver attributes.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param editor: Reference to an editor object
		@type editor: An Editor object.
		"""
		self.count = 0
		# Reference to the text editor.
		self.editor = editor
		# Reference to the text editor's buffer.
		self.textbuffer = editor.textbuffer
		# Temporary uri used for saving files before being transfered to
		# permanent locations.
		self.swap_uri = None
		# Temporary file object to save to.
		self.swap_file = None
		# Temporary folder to save temporary files to.
		self.swap_folder = None
		# True if the temporary storage location and file has been created.
		self.swap_is_available = False
		# True if the file is being saved to a temporary location.
		self.is_saving = False
		# True if the file was successfully saved to a temporary location.
		self.is_saved  = False
		# True if the swap file is in the process of being transfered to a
		# permanent location.
		self.is_transferring = False
		# True if the swap file has been successfully moved to its permanent
		# location.
		self.is_transferred = False
		# True if the saving process failed for any number of reasons.
		self.failed = False
		# True if the saving process completed successfully.
		self.completed = False
		# True to show error dialog.
		self.show_error_message = True
		# True to enable automatic saving.
		self.enable_automatic_saving = True
		# True if the current document is saved.
		self.document_is_saved = True
		# True if it is the last saving operation.
		self.can_quit = False
		# Register a unique number with the editor's termination queue
		self.termination_id = self.editor.register_termination_id()
		self.save_count = 0
		self.__file_info = None
		return

################################################################################
#
#							Preliminary Checks
#
################################################################################

	def process(self, is_closing=False):
		"""
		Perform preliminary checks to determine whether or not the document can
		be saved.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param is_closing: True if the document is being saved while closing.
		@type is_closing: A Boolean object.
		"""
		if self.save_count:
			self.save_count += 1
			return
		if self.editor.uri is None:
			if is_closing:
				# Create a new file and save it if the text editor's buffer
				# contains a document but there is no document to save.
				from gobject import timeout_add
				timeout_add(10, self.__create_new_file_and_save)
				self.emit("saving")
			else:
				# Show the save dialog if the text editor's buffer is empty and
				# there is no document to save.
				self.editor.triggermanager.trigger("show_save_dialog")
			return
		can_save = self.__check_write_permissions()
		if can_save is False:
			from internationalization import msg0363
			self.emit("error", msg0363)
			return
		# Get file information then begin saving.
		from gobject import timeout_add, PRIORITY_HIGH
		timeout_add(10, self.__save)
		self.emit("saving")
		return

################################################################################
#
#					  Saving Proper
#
################################################################################

	def __save(self):
		"""
		Save the document to a temporary location and file.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.
		"""
		self.emit("save-begin")
		self.__update_file_info()
		result  = self.__create_swap_folder_and_file()
		if result is False:
			from internationalization import msg0364
			self.emit("save-error", msg0364)
			return
		# Temporary URI derived from temporary file.
		from gnomevfs import get_uri_from_local_path, URI
		uri = get_uri_from_local_path(self.swap_file.name)
		self.swap_uri = URI(uri)
		# Save the content of the text editor's buffer to the randomly
		# generated temporary file.
		from gobject import timeout_add
		timeout_add(7, self.__process_file_for_saving)
		# Transfer the randomly generated temporary file to its permanent
		# location.
		timeout_add(10, self.__begin_file_transfer)
		return

	def __update_file_info(self):
		self.__file_info = None
		if self.editor.uri.is_local or str(self.editor.uri).startswith("file:///"):
			from gnomevfs import get_file_info, FILE_INFO_FIELDS_PERMISSIONS
			try:
				fileinfo = get_file_info(self.editor.uri)
				if fileinfo.valid_fields & FILE_INFO_FIELDS_PERMISSIONS:
					self.__file_info = fileinfo
			except:
				pass
		return

	def __create_swap_folder_and_file(self):
		"""
		Create a temporary folder and file to save the document to temporarily.

		The saving object saves to a temporary file, in a temporay folder. If
		the result of the saving operation is successful, the file is transfered
		to the original location of the document.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.
		"""
		# True if the temporary folder and file is successfully created.
		value = True
		# Create a temporary folder.
		from tempfile import mkdtemp, NamedTemporaryFile
		from info import home_folder
		try:
			self.swap_folder = mkdtemp(suffix="scribes",
										prefix=".Scribes",
										dir=home_folder)
			self.emit("swap-created")
			# Create a randomly generated temporary file in the temporary folder
			# created above.
			self.swap_file = NamedTemporaryFile(mode="w+",
												suffix="Scribes",
												prefix="scribes",
												dir=self.swap_folder)
		except:
			value = False
		return value

	def __process_file_for_saving(self):
		"""
		Write the content of the text editor's buffer to a temporary file.

		This function converts to the content of the text editor's buffer to an
		encoding specified by the user, or "UTF-8" if none is specified, and
		then writes the encoded text to a temporary file. The saving process is
		asynchronous, to prevent, or reduce, GUI (b)locking.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param uri: URI pointing to a temporary file.
		@type uri: A gnomevfs.URI object.
		"""
		try:
			# Encode the content of the text editor buffer to one specified by
			# the user, or UTF-8 if none is specified.
			text = self.__get_encoded_text()
			if text is None:
				raise ValueError
			from gnomevfs import OPEN_WRITE
			from gnomevfs.async import create
			try:
				# Write to a temporary file.
				create(uri=self.swap_uri,
					callback=self.__write_cb,
					open_mode=OPEN_WRITE,
					exclusive=False,
					perm=0644,
					priority=0,
					data=text)
			except:
				from internationalization import msg0365
				self.emit("save-error", msg0365)
		except ValueError:
			pass
		return False

	def __write_cb(self, handle, result, text):
		"""
		Write encoded content of a text buffer to a temporary uri.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param handle: An object pointing to an open URI.
		@type handle: A gnomevfs.Handle object.

		@param result: The result of a writing operation.
		@type result: A gnomevfs object.

		@param text: A sequence of strings to be written to a file.
		@type text: A String object.
		"""
		try:
			handle.write(text, self.__close_cb)
		except ValueError:
			handle.cancel()
			from internationalization import msg0365
			self.emit("save-error", msg0365)
		return

	def __close_cb(self, handle, bytes, result, bytes_requested):
		"""
		Close the URI representing, or pointing, to a file.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param handle: An object pointing to a URI to be closed
		@type handle: A gnomevfs.Handle object.

		@param bytes: Number of bytes read.
		@type bytes: An Integer object.

		@param result: The result of a closing operation.
		@type result: A gnomevfs object.

		@param bytes_requested: Number of bytes_requested.
		@type bytes_requested: An Integer object.
		"""
		handle.close(self.__finalize_cb)
		return

	def __finalize_cb(self, *args):
		"""
		Perform post writing operations.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param args: Miscellaneous arguments
		@type args: A Tuple object.
		"""
		self.emit("save-end")
		return

	def __begin_file_transfer(self):
		"""
		Transfer saved file from temporary location to permanent location.

		The function is a timeout_add callback. It is called repeatedly if its
		return value is True.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@return: True to call this function again, False to stop calling the
			function.
		@rtype: A Boolean object.
		"""
		if self.failed:
			return False
		value = True
		if self.is_saved:
			self.emit("transfer-begin")
			value = False
			from gobject import timeout_add
			timeout_add(10, self.__transfer_file)
		return value

	def __transfer_file(self):
		from gnomevfs import XFER_OVERWRITE_MODE_REPLACE
		from gnomevfs import XFER_ERROR_MODE_QUERY
		from gnomevfs.async import xfer
		XFER_TARGET_DEFAULT_PERMS = 1 << 12
		try:
			xfer(source_uri_list=[self.swap_uri],
				target_uri_list=[self.editor.uri],
				xfer_options=XFER_TARGET_DEFAULT_PERMS,
				error_mode=XFER_ERROR_MODE_QUERY,
				priority = 0,
				overwrite_mode=XFER_OVERWRITE_MODE_REPLACE,
				progress_update_callback=self.__progress_update_cb,
				update_callback_data=self.swap_file,
				progress_sync_callback=self.__progress_cb)
		except:
			from internationalization import msg0366
			self.emit("transfer-error", msg0366)
		return False

	def __progress_cb(self, info):
		"""
		Provide progress information on file transfer.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param info: An object providing information on file transfer,
		@type info: A gnomevfs.XferProgressInfo object.
		"""
		if info.vfs_status:
			return False
		return  True

	def __progress_update_cb(self, handle, info, data):
		"""
		Provide progress information on file transfer.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param _reserved: Reserved parameter
		@type _reserved: An Unknown object.

		@param info: An object providing information on file transfer.
		@type info: A gnomevfs.XferProgressInfo object.

		@param data: Temporary file object.
		@type data: A File object.
		"""
		if info.vfs_status:
			# A transfer error occured.
			handle.cancel()
			from internationalization import msg0367
			self.emit("transfer-error", msg0367)
			return False
		from gnomevfs import XFER_PHASE_COMPLETED
		if info.phase == XFER_PHASE_COMPLETED:
			self.emit("transfer-end")
		return True

	def __get_encoded_text(self):
		"""
		Convert the content of the text editor buffer to a specified encoding.

		The user specifies which encoding the file should be saved in. Otherwise,
		UTF-8 is the default encoding used for saving files.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@return: An encoded string or None if the encoding process fails.
		@rtype: A String object or None.
		"""
		start, end = self.editor.textbuffer.get_bounds()
		text = self.editor.textbuffer.get_text(start, end)
		from txtutils import decode_string_to_unicode, encode_string
		try:
			unicode_text = decode_string_to_unicode(text, "utf8")
			if unicode_text is None:
				raise UnicodeDecodeError("", "", 0, 0, "")
			if self.editor.encoding is None:
				encoded_text = encode_string(unicode_text, "utf8")
			else:
				encoded_text = encode_string(unicode_text, self.editor.encoding)
			if encoded_text is None:
				raise UnicodeEncodeError("", u"", 0, 0, "")
		except UnicodeDecodeError:
			encoded_text = None
			from internationalization import msg0368
			self.emit("save-error", msg0368)
		except UnicodeEncodeError:
			encoded_text = None
			from internationalization import msg0369
			self.emit("save-error", msg0369)
		return encoded_text

	def __create_new_file_and_save(self):
		"""
		Create a new file and save it.

		This function is called for unsaved documents that have been modified.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.
		"""
		from info import desktop_folder
		from os.path import exists
		if exists(desktop_folder):
			# Save to the desktop folder if it exists.
			folder = desktop_folder
		else:
			# Save to the home folder if the desktop folder does not exists.
			from info import home_folder
			folder = home_folder
		# A count to append to unsaved documents if many unsaved documents
		# exists in folder.
		count = 1
		from internationalization import msg0025
		from dircache import listdir
		file_list = listdir(folder)
		# Calculate count to append to unsaved documents.
		while True:
			newfile = msg0025 + " " + str(count)
			if newfile in file_list:
				count += 1
			else:
				break
		newfile = folder + "/" + newfile
		from gnomevfs import make_uri_from_shell_arg, URI
		self.editor.uri = URI(make_uri_from_shell_arg(newfile))
		from gobject import timeout_add, PRIORITY_HIGH
		timeout_add(10, self.__save)
		return False

	def __check_write_permissions(self):
		"""
		Check if user has permission to write to a directory and file.

		This function does not test for write permission on remote locations.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@return: True if the user has permission to write to directory and file.
		@rtype: A Boolean object.

		"""
		value = True
		if self.editor.uri.is_local or str(self.editor.uri).startswith("file:"):
			from gnomevfs import get_local_path_from_uri
			file_path = get_local_path_from_uri(str(self.editor.uri))
			from os import access, W_OK, path
			folder_path = path.dirname(file_path)
			if access(folder_path, W_OK) is False:
				value = False
			elif access(file_path, W_OK) is False:
				if path.exists(file_path):
					value = False
				else:
					value = True
		return value

	def __remove_swap_area(self):
		"""
		Remove the temporary file and folder created during a saving operation.

		This function is used as a timeout_add callback.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@return: False to prevent this function from being called repeatedly.
		@rtype: A Boolean object.
		"""
		try:
			self.swap_file.close()
			from os import rmdir
			rmdir(self.swap_folder)
		except:
			pass
		self.swap_is_available  = False
		if self.is_saved and self.is_transferred and self.completed:
			self.emit("saved")
		return False

	def __show_error_dialog(self, message):
		"""
		Show an error dialog with an error message.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param message: An error message.
		@type message: A String object.
		"""
		from error import ScribesErrorDialog
		from gnomevfs import format_uri_for_display
		uri = format_uri_for_display(str(self.editor.uri))
		ScribesErrorDialog(self.editor, uri, message)
		return

	def __filesaver_saving_cb(self, filesaver):
		"""
		Handles callback when the saving object begins to save a document.

		This function signals the text editor that a save operation is about to
		begin. The text editor in turn sends a global signal relaying this
		message.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		# Announce to all objects that the editor is about to save the document
		# it contains.
		self.editor.emit("saving-document")
		self.save_count += 1
		return

	def __filesaver_save_begin_cb(self, filesaver):
		"""
		Handles callback when the saving object begins to save a document.

		This function is called when the saving object begins to save the
		content of the text editor to a swap file.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		self.failed = False
		self.is_saving = True
		self.is_saved = False
		self.is_transferring = False
		self.is_transferred	= False
		self.completed = False
		self.swap_is_available = False
		return

	def __filesaver_save_end_cb(self, filesaver):
		"""
		Handles callback when the saving object begins to save a document.

		This function is called when the saving object has successfully saved
		the content of the document to a swap file.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		self.is_saved = True
		self.is_saving = False
		return

	def __filesaver_save_error_cb(self, filesaver, message):
		"""
		Handles callback when an error occurs while saving a document.

		This function is called when an error occurs while saving the content
		of the text editor's buffer to a swap file.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		self.failed = True
		self.is_saving = False
		self.is_saved = False
		self.completed = False
		if self.swap_is_available:
			from gobject import timeout_add
			timeout_add(10, self.__remove_swap_area)
		self.editor.emit("save-error")
		if self.show_error_message:
			self.__show_error_dialog(message)
		return

	def __filesaver_saved_cb(self, filesaver):
		"""
		Handles callback when the saving object has successfully saved a
		document.

		This function signals the text editor that a save operation completed
		successfully. The text editor in turn sends a global signal relaying
		this message.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		# Announce to all objects that the editor finished saving a document
		# successfully.
		self.save_count -= 1
		if self.save_count:
			from gobject import timeout_add, PRIORITY_HIGH
			timeout_add(10, self.__save)
			return
		self.document_is_saved = True
		self.editor.emit("saved-document")
		#self.count += 1
		#print "[%d] saved document" % (self.count)
		self.save_count = 0
		if self.can_quit:
			self.editor.unregister_termination_id(self.termination_id)
		from gc import collect
		collect()
		return

	def __filesaver_swap_created_cb(self, filesaver):
		"""
		Handles callback when the swap (temporary) folder and file have been
		created.

		The text editor saves documents to a temporary location before the
		document is transfered to its original location. The temporary location
		is termed the swap area. This function is called when the swap area
		is created.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		self.swap_is_available = True
		return

	def __filesaver_swap_removed_cb(self, filesaver):
		"""
		Handles callback when the swap (temporary) folder and file are deleted.

		The text editor saves documents to a temporary location before the
		document is transfered to its original location. The temporary location
		is termed the swap area. This function is called when the swap area
		is deleted.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		self.swap_is_available = False
		return

	def __filesaver_transfer_begin_cb(self, filesaver):
		"""
		Handles callback temporary saved file is about to be transfered to its
		intended location.

		The text editor saves documents to a temporary location before the
		document is transfered to its original location. The temporary location
		is termed the swap area. This function is called when the transfer
		begins.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		self.is_transferring = True
		self.is_transferred = False
		self.completed = False
		return

	def __filesaver_transfer_end_cb(self, filesaver):
		"""
		Handles callback temporary saved file is transfered to its intended
		location.

		The text editor saves documents to a temporary location before the
		document is transfered to its original location. The temporary location
		is termed the swap area. This function is called when the transfer
		ends successfully.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		self.is_transferred = True
		self.is_transferring = False
		self.completed = True
		from gobject import timeout_add
		timeout_add(10, self.__remove_swap_area)
		if self.__file_info:
			try:
				from gnomevfs import set_file_info, SET_FILE_INFO_PERMISSIONS
				set_file_info(self.editor.uri, self.__file_info, SET_FILE_INFO_PERMISSIONS)
			except:
				pass
		return

	def __filesaver_transfer_error_cb(self, filesaver, message):
		"""
		Handles callback when an error occurs while transferring temporary saved
		a file to its original location.

		The text editor saves documents to a temporary location before the
		document is transferred to its original location. The temporary location
		is termed the swap area. This function is called when error occurs
		during the transfer.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.
		"""
		self.failed = True
		self.is_transferred = False
		self.is_transferring = False
		self.completed = False
		if self.swap_is_available:
			from gobject import timeout_add
			timeout_add(10, self.__remove_swap_area)
		self.editor.emit("save-error")
		if self.show_error_message:
			self.__show_error_dialog(message)
		return

	def __filesaver_error_cb(self, filesaver, message):
		"""
		Handles callback when a generic error occurs during the saving process.

		This function is called when a function unrelated to the saving process
		triggers an error, permission problems for example.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param filesaver: Reference to the FileSaver instance.
		@type filesaver: A FileSaver object.

		"""
		self.save_count = 0
		self.editor.emit("save-error")
		if self.show_error_message:
			self.__show_error_dialog(message)
		return

	def __filesaver_changed_cb(self, textbuffer):
		"""
		Handles callback when the "changed" signal is emitted.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.editor.is_loading_document:
			return False
		self.document_is_saved = False
		return False

	def __filesaver_modified_changed_cb(self, textbuffer):
		"""
		Handles callback when the "modified-changed" signal is emitted.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.editor.is_loading_document or self.editor.uri is None:
			return False
		if self.document_is_saved is False and self.enable_automatic_saving:
			from gobject import timeout_add
			timeout_add(21000, self.__save_document)
		return False

	def __save_document(self):
		"""
		Save the document.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		if self.document_is_saved is False:# and self.enable_automatic_saving:
			self.process()
		return False

	def __filesaver_save_document_cb(self, editor):
		"""
		Handles callback when the "save-document" signal is emitted.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.process()
		return

	def __filesaver_close_document_cb(self, editor):
		"""
		Handles callback when the "close-document" signal is emitted.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.can_quit = True
		if self.document_is_saved is False and self.enable_automatic_saving:
			self.process(True)
		else:
			self.editor.unregister_termination_id(self.termination_id)
		return

	def __filesaver_enable_readonly_cb(self, editor):
		"""
		Handles callback when the "enable-readonly" signal is emitted.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.enable_automatic_saving = False
		return

	def __filesaver_disable_readonly_cb(self, editor):
		"""
		Handles callback when the "disable-readonly" signal is emitted.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.enable_automatic_saving = True
		return

	def __filesaver_renamed_document_cb(self, editor):
		"""
		Handles callback when the "renamed-document" signal is emitted.

		@param self: Reference to the FileSaver instance.
		@type self: A FileSaver object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.enable_automatic_saving = True
		return
