# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that highlights regions within pair
bracket characters. Characters currently affected are "(", ")", "[", "]"
"<", ">", "{" and "}". More pair characters will be added if the need
arises.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class LexicalScopeHighlight(object):
	"""
	The class implements and object that highlights regions within pair
	characters. The following characters are supported "(", ")", "[", "]"
	"<", ">", "{" and "}"
	"""

	def __init__(self, editor):
		"""
		Initialize the highlight object.

		@param self: Reference to the LexicalScopeHighlight instance.
		@type self: A LexicalScopeHighlight object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.
		"""
		self.__init_attributes(editor)
		self.textbuffer.connect("apply-tag", self.__scope_apply_tag_cb)
		self.textbuffer.connect("remove-tag", self.__scope_remove_tag_cb)
		self.editor.connect("loading-document", self.__scope_loading_document_cb)
		self.editor.connect("loaded-document", self.__scope_loaded_document_cb)
		self.editor.connect("enable-readonly", self.__scope_loading_document_cb)
		self.editor.connect("disable-readonly", self.__scope_loaded_document_cb)
		self.editor.connect("cursor-moved", self.__scope_cursor_moved_cb)
		self.client.notify_add("/apps/scribes/scope_highlight_color", self.__scope_highlight_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the highlight object's data attributes.

		@param self: Reference to the LexicalScopeHighlight instance.
		@type self: A LexicalScopeHighlight object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.
		"""
		from gconf import client_get_default, CLIENT_PRELOAD_ONELEVEL
		self.client = client_get_default()
		self.client.add_dir("/apps/scribes", CLIENT_PRELOAD_ONELEVEL)
		self.tag_position = []
		self.can_highlight = True
		self.buffer_is_tagged = False
		self.editor  = editor
		self.textbuffer = editor.textbuffer
		begin, end = editor.textbuffer.get_bounds()
		self.highlight_tag = self.__create_highlight_tag()
		from gtksourceview import source_iter_find_matching_bracket as match
		self.match = match
		self.begin_mark = None
		self.end_mark = None
		return

	def __scope_cursor_moved_cb(self, editor):
		"""
		Handles callback when the "cursor-moved" signal is emitted.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		if self.can_highlight:
			self.__highlight_region()
		return

	def __highlight_region(self):
		"""
		Highlight region within pair characters.

		This function highlights regions withing most pair characters.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param iterator: A position within the text editor's buffer.
		@type iterator: A gtk.TextIter object.
		"""
		if self.buffer_is_tagged:
			begin = self.textbuffer.get_iter_at_mark(self.begin_mark)
			end = self.textbuffer.get_iter_at_mark(self.end_mark)
			self.textbuffer.remove_tag(self.highlight_tag, begin, end)
		iterator = self.textbuffer.get_iter_at_mark(self.textbuffer.get_insert())
		if self.match(iterator):
			cursor_iter = self.textbuffer.get_iter_at_mark(self.textbuffer.get_insert())
			self.textbuffer.apply_tag(self.highlight_tag, iterator, cursor_iter)
		return False

	def __scope_apply_tag_cb(self, textbuffer, tag, start, end):
		"""
		Handles callback when the "apply-tag" signal is emitted.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.

		@param tag: A tag in the text editor's buffer.
		@type tag: A gtk.TextTag object.

		@param start: The position of the begining of the tag.
		@type start: A gtk.TextIter object.

		@param end: The position of the end of the tag.
		@type end: A gtk.TextIter object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if tag == self.highlight_tag:
			#if self.begin_mark and self.begin_mark.get_deleted() is False:
			#	self.textbuffer.delete_mark(self.begin_mark)
			#if self.end_mark and self.end_mark.get_deleted() is False:
			#	self.textbuffer.delete_mark(self.end_mark)
			#self.begin_mark = self.textbuffer.create_mark(None, start, True)
			#self.end_mark = self.textbuffer.create_mark(None, end, False)
			if self.begin_mark is None:
				self.begin_mark = self.textbuffer.create_mark(None, start, True)
			if self.end_mark is None:
				self.end_mark = self.textbuffer.create_mark(None, end, False)
			self.textbuffer.move_mark(self.begin_mark, start)
			self.textbuffer.move_mark(self.end_mark, end)
			self.buffer_is_tagged = True
		return True

	def __scope_remove_tag_cb(self, textbuffer, tag, start, end):
		"""
		Handles callback when the "remove-tag" signal is emitted.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.

		@param tag: A tag in the text editor's buffer.
		@type tag: A gtk.TextTag object.

		@param start: The position of the begining of the tag.
		@type start: A gtk.TextIter object.

		@param end: The position of the end of the tag.
		@type end: A gtk.TextIter object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if tag == self.highlight_tag:
			self.buffer_is_tagged = False
		return True

	def __create_highlight_tag(self):
		"""
		Create the a highlight tag.

		@param self: Reference to the LexicalScopeHighlight instance.
		@type self: A LexicalScopeHighlight object.

		@return: A region highlight tag.
		@rtype: A gtk.TextTag object.
		"""
		from gtk import TextTag
		tag = TextTag("lexical_scope_tag")
		color = self.client.get_string("/apps/scribes/scope_highlight_color")
		if color is None:
			color = "#cfcfcf"
		self.textbuffer.get_tag_table().add(tag)
		tag.set_property("background", color)
		return tag

	def __scope_loading_document_cb(self, editor):
		self.can_highlight = False
		begin, end = self.textbuffer.get_bounds()
		self.textbuffer.remove_tag(self.highlight_tag, begin, end)
		return

	def __scope_loaded_document_cb(self, editor):
		self.__highlight_region()
		self.can_highlight = True
		return

	def __scope_highlight_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback #FIXME: yeap

		@param self: Reference to the LexicalScopeHighlight instance.
		@type self: A LexicalScopeHighlight object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		begin, end = self.textbuffer.get_bounds()
		self.textbuffer.remove_tag(self.highlight_tag, begin, end)
		color = self.client.get_string("/apps/scribes/scope_highlight_color")
		if color is None:
			color = "#cfcfcf"
		self.highlight_tag.set_property("background", color)
		self.__highlight_region()
		return
