# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

from dbus.service import Object, method, signal, BusName

class ScribesService(Object):
	"""
	This class creates a D-Bus object for the text editor. The object
	implements, methods, signals and services that D-Bus capable
	applications, including the text editor, can interact with.
	"""

	def __init__(self):
		"""
		Initialize the D-Bus object.

		@param self: Reference to the ScribesService instance.
		@type self: A ScribesService object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param dbus_name: The D-Bus service name for the text editor.
		@type dbus_name: A String object.

		@param object_path: The D-Bus path for the text editor.
		@type object_path: A String object.
		"""
		from info import session_bus
		service_name = "net.sourceforge.ScribesService"
		object_path = "/net/sourceforge/ScribesService"
		bus_name = BusName(service_name, bus=session_bus)
		Object.__init__(self, bus_name, object_path)
		self.__init_attributes()
		session_bus.add_signal_receiver(self.__name_change_cb,
						'NameOwnerChanged',
						'org.freedesktop.DBus',
						'org.freedesktop.DBus',
						'/org/freedesktop/DBus',
						arg0='net.sourceforge.Scribes')
		from gobject import timeout_add, PRIORITY_LOW
		timeout_add(60000, self.__quit_service_timeout, priority=PRIORITY_LOW)

	def __init_attributes(self):
		"""
		Initialize the object's data attributes.

		@param self: Reference to the ScribesService instance.
		@type self: A ScribesService object.
		"""
		self.__object_paths = []
		self.__uri_list = []
		self.__can_start_psyco = True
		return

	@method("net.sourceforge.ScribesService")
	def get_uris(self):
		return self.__get_uris()

	@method("net.sourceforge.ScribesService")
	def new_window(self):
		return self.__open_new_window()

	@method("net.sourceforge.ScribesService")
	def new_files(self, uris):
		return self.__open_new_files(uris)

	@method("net.sourceforge.ScribesService")
	def focus_uri_window(self, uri):
		return self.focus_window(uri)

	@method("net.sourceforge.ScribesService")
	def register_object_path(self, object_path):
		return self.__register_object_path(object_path)

	@method("net.sourceforge.ScribesService")
	def unregister_object_path(self, object_path):
		return self.__unregister_object_path(object_path)

	@method("net.sourceforge.ScribesService")
	def get_number_of_instances(self):
		return len(self.__object_paths)

	@method("net.sourceforge.ScribesService")
	def close_windows(self):
		return self.close_all_windows()

	@method("net.sourceforge.ScribesService")
	def update_uris(self):
		return self.__update_uris()
		
	@method("net.sourceforge.ScribesService")
	def can_start_psyco(self):
		return self.__can_start_psyco
		
	@method("net.sourceforge.ScribesService")
	def started_psyco(self):
		return self.__change_psyco_flag()

	@signal("net.sourceforge.ScribesService")
	def focus_window(self, uri):
		pass

	@signal("net.sourceforge.ScribesService")
	def close_all_windows(self):
		pass

	def __change_psyco_flag(self):
		self.__can_start_psyco = False
		return 

	def __get_uris(self):
		if not self.__uri_list:
			return ""
		return self.__uri_list

	def __update_uris(self):
		from info import session_bus
		self.__uri_list = []
		for path in self.__object_paths:
			proxy_object = session_bus.get_object("net.sourceforge.Scribes", path)
			proxy_object.get_uri(dbus_interface="net.sourceforge.ScribesIFace",
						reply_handler=self.__update_uris_reply_handler_cb,
						error_handler=self.__error_handler_cb)
		return

	def __update_uris_reply_handler_cb(self, uri):
		if uri:
			self.__uri_list.append(uri)
		return

	def __open_new_window(self):
		from info import session_bus
		proxy_object = session_bus.get_object("net.sourceforge.Scribes", self.__object_paths[0])
		proxy_object.open_new_window(dbus_interface="net.sourceforge.ScribesIFace",
							reply_handler=self.__new_window_reply_handler_cb,
							error_handler=self.__error_handler_cb)
		return

	def __open_new_files(self, uris):
		from info import session_bus
		proxy_object = session_bus.get_object("net.sourceforge.Scribes", self.__object_paths[0])
		proxy_object.open_new_files(uris,
				dbus_interface="net.sourceforge.ScribesIFace",
				reply_handler=self.__new_files_reply_handler_cb,
				error_handler=self.__error_handler_cb)
		return

	def __new_window_reply_handler_cb(self, *args):
		return

	def __new_files_reply_handler_cb(self, *args):
		return

	def __error_handler_cb(self, error):
		print error
		print "In scribesservice.py"
		return

	def __name_change_cb(self, *args):
		from info import dbus_iface
		if not "net.sourceforge.Scribes" in dbus_iface.ListNames():
			self.__quit_service()
		return

	def __quit_service(self):
		self = None
		del self
		from os import _exit
		_exit(0)
		return False

	def __register_object_path(self, object_path):
		if not object_path in self.__object_paths:
			self.__object_paths.append(object_path)
		return

	def __unregister_object_path(self, object_path):
		if object_path in self.__object_paths:
			self.__object_paths.remove(object_path)
		return

	def __quit_service_timeout(self):
		if self.__object_paths:
			return True
		self.__quit_service()
		return False
