# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a function that allow processes, especially ones,
written by third parties to be initialized after the GUI is created.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

def init_services(editor):
	"""
	Initialize auxilliary services after the GUI has been created.

	This is the preferred function to places your initialization classes
	or function in. The primarly purpose of the function is to give
	access to the editor object as well as to ensure the editor widgets
	and services have been initialized before auxilliary functions run.
	And ofcourse to allow you to initialize your service.
	If possible, try starting your class or function in a seperate
	thread.

	@param editor: Reference to the text editor.
	@type editor: An Editor object.
	"""
	from thread import start_new_thread
	from gtk.gdk import threads_enter, threads_leave
	threads_enter()

	# Trigger bookmark operations.
	from bookmarktriggers import BookmarkTriggers
	start_new_thread(BookmarkTriggers, (editor,))

	# Service that monitors and expands template triggers.
	from templatetrigger import TemplateTrigger
	start_new_thread(TemplateTrigger, (editor,))

	# Services that loads user-defined templates.
	from templateloader import TemplateLoader
	start_new_thread(TemplateLoader, (editor,))

	# Object that monitors cursor movements.
	from cursormove import CursorMovement
	start_new_thread(CursorMovement, (editor,))

	# Object that indexes words for word completion.
	from completionlist import WordCompletionListGenerator
	start_new_thread(WordCompletionListGenerator, (editor,))

	# Object that monitors the buffer for word completion.
	from wordcompletion import WordCompletion
	start_new_thread(WordCompletion, (editor,))

	# Object that performs automatic bracket operations.
	from bracket import BracketManager
	start_new_thread(BracketManager, (editor,))

	# Object that performs lexical scope highlighting.
	from scopehighlight import LexicalScopeHighlight
	start_new_thread(LexicalScopeHighlight, (editor,))

	# Trigger for the document browser.
	from documentbrowsertrigger import DocumentBrowserTrigger
	start_new_thread(DocumentBrowserTrigger, (editor,))

	# Trigger for the preference dialog.
	from preferencestrigger import PreferencesTrigger
	start_new_thread(PreferencesTrigger, (editor,))

	# Trigger for the color editor.
	from coloreditortrigger import ColorEditorTrigger
	start_new_thread(ColorEditorTrigger, (editor,))

	# Trigger for the template editor.
	from templateeditortrigger import TemplateEditorTrigger
	start_new_thread(TemplateEditorTrigger, (editor,))

	# Trigger for the open dialog.
	from opendialogtrigger import OpenDialogTrigger
	start_new_thread(OpenDialogTrigger, (editor,))

	# Trigger for the save dialog.
	from savedialogtrigger import SaveDialogTrigger
	start_new_thread(SaveDialogTrigger, (editor,))

	# Trigger for the remote dialog.
	from remotedialogtrigger import RemoteDialogTrigger
	start_new_thread(RemoteDialogTrigger, (editor,))

	# Trigger for the about dialog.
	from aboutdialogtrigger import AboutDialogTrigger
	start_new_thread(AboutDialogTrigger, (editor,))

	# Triggers to change case of a string.
	from casetrigger import CaseTrigger
	start_new_thread(CaseTrigger, (editor,))

	# Triggers to show the print dialog.
	from printtrigger import PrintTrigger
	start_new_thread(PrintTrigger, (editor,))

	# Trigger to find matching bracket.
	from matchingbrackettrigger import MatchingBracketTrigger
	start_new_thread(MatchingBracketTrigger, (editor,))

	# Trigger to initialize the search and replace manager.
	from searchtrigger import SearchReplaceTrigger
	start_new_thread(SearchReplaceTrigger, (editor,))

	# Trigger to show the findbar.
	from findbartrigger import FindBarTrigger
	start_new_thread(FindBarTrigger, (editor,))

	# Trigger to show the incremental searchbar
	from incsearchbartrigger import IncrementalSearchBarTrigger
	start_new_thread(IncrementalSearchBarTrigger, (editor,))

	# Trigger to show the replace bar.
	from replacebartrigger import ReplaceBarTrigger
	start_new_thread(ReplaceBarTrigger, (editor,))

	# Trigger to show the goto bar.
	from gotobartrigger import GotoBarTrigger
	start_new_thread(GotoBarTrigger, (editor,))

	# Trigger to search for next text in the buffer.
	from searchnexttrigger import SearchNextTrigger
	start_new_thread(SearchNextTrigger, (editor,))

	# Trigger to search for previous text in the buffer.
	from searchprevioustrigger import SearchPreviousTrigger
	start_new_thread(SearchPreviousTrigger, (editor,))

	threads_leave()
	return
