# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class responsible for inserting templates into
the text editor's buffer.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class TemplateManager(object):
	"""
	This class implements an object that inserts templates into the text
	editor's buffer.
	"""

	def __init__(self, template_trigger):
		"""
		Initialize the manager object.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param template_trigger: A trigger to insert templates into the text editor.
		@type template_trigger: A TemplateTrigger object.
		"""
		self.__init_attributes(template_trigger)
		self.trigger.connect("found-template-trigger", self.__manager_found_template_trigger_cb)
		self.editor.textview.connect("key-press-event", self.__manager_key_press_event_cb)

	def __init_attributes(self, template_trigger):
		"""
		Initialize the manager's data attributes.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param template_trigger: A trigger to insert templates into the text editor.
		@type template_trigger: A TemplateTrigger object.
		"""
		self.trigger = template_trigger
		self.editor = template_trigger.editor
		self.template_list = []
		from re import UNICODE, compile
		self.placeholderpattern = compile("\$\{[^${}]*\}", UNICODE)
		return

	def __manager_found_template_trigger_cb(self, trigger_object, trigger):
		"""
		Handles callback when the "found-template-trigger" is emitted.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param trigger_object: An object responsible for monitoring template triggers.
		@type trigger_object: A TemplateTrigger object.

		@param trigger: A template trigger.
		@type trigger: A String object.
		"""
		self.__process_template_trigger(trigger)
		from cursor import move_view_to_cursor
		move_view_to_cursor(self.editor.textview)
		return

	def __process_template_trigger(self, trigger):
		"""
		Process a template to be inserted into the text editor's buffer.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param trigger: A template trigger.
		@type trigger: A TemplateTrigger object.
		"""
		from gobject import timeout_add
		timeout_add(3, self.__insert_template, trigger)
		return

	def __insert_template(self, trigger):
		template = self.get_template(trigger)
		result = self.has_placeholders(template)
		self.editor.textbuffer.begin_user_action()
		self.remove_template_trigger(trigger)
		if result:
			# In this branch the template has placeholders
			from templateprocessor import TemplateProcessor
			template_proc = TemplateProcessor(self, template, result)
			self.template_list.append(template_proc)
		else:
			# In this branch the template has no placeholders
			self.insert_template(template)
			from internationalization import msg0464
			self.editor.feedback.update_status_message(msg0464, "succeed")
		self.editor.textbuffer.end_user_action()
		return False

	def remove_template_trigger(self, trigger):
		"""
		Remove the template trigger from the text editor's buffer.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param trigger: A template trigger.
		@type trigger: A String object.
		"""
		length = self.__get_template_trigger_length(trigger)
		from cursor import get_cursor_iterator
		cursor_position = get_cursor_iterator(self.editor.textbuffer)
		begin_position = cursor_position.copy()
		for item in range(length):
			begin_position.backward_char()
		self.editor.textbuffer.begin_user_action()
		self.editor.textbuffer.delete(begin_position, cursor_position)
		self.editor.textbuffer.end_user_action()
		return

	def __get_template_trigger_length(self, trigger):
		"""
		Get the length of a template trigger.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param trigger: A template trigger.
		@type trigger: A String object.
		"""
		if trigger.startswith("General"):
			string = trigger.replace("General", "")
			length = len(string)
		else:
			from utils import get_language
			language = get_language(str(self.editor.uri))
			language_id = language.get_id()
			string = trigger.replace(language_id, "")
			length = len(string)
		return length

	def insert_template(self, template):
		"""
		Insert template in the text editor's buffer.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param template: A template.
		@type template: A String object.
		"""
		from lines import get_beginning_spaces
		spaces = get_beginning_spaces(self.editor.textbuffer)
		if spaces:
			template = self.__indent_template(spaces, template)
		self.editor.textbuffer.begin_user_action()
		self.editor.textbuffer.insert_at_cursor(template)
		self.editor.textbuffer.end_user_action()
		return

	def __indent_template(self, spaces, template):
		"""
		Indent each line in a template if necessary.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param spaces: The indentation spaces to place in front of lines.
		@type spaces: A List object.

		@param template: The template to insert into the text editor's buffer.
		@type template: A String object.

		@return: Return a correctly indented template.
		@rtype: A String object.
		"""
		template_lines = template.split("\n")
		if len(template_lines) > 1:
			spaces = "".join(spaces)
			first_line = template_lines[0]
			other_lines = template_lines[1:]
			temp_list = []
			for line in other_lines:
				string = spaces + line
				temp_list.append(string)
			temp_list.insert(0, first_line)
			template = "\n".join(temp_list)
		return template

	def get_template(self, trigger):
		"""
		Get a template from the template dictionary.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param trigger: A template trigger.
		@type trigger: A String object.

		@return: Return a template.
		@rtype: A String object.
		"""
		if trigger.startswith("General"):
			template = self.editor.textbuffer.general_template_dictionary[trigger]
		else:
			template = self.editor.textbuffer.language_template_dictionary[trigger]
		return template

	def has_placeholders(self, template):
		"""
		Check if template has placeholders.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param template: A template to insert into the text buffer.
		@type template: A String object.

		@return: A list of placeholders if any.
		@rtype: A List object.
		"""
		# Match any strings enclosed in ${} with the exception of ${}.
		from re import findall, UNICODE
		#find placeholders
		result = findall(self.placeholderpattern, template, UNICODE)
		return result

	def __manager_key_press_event_cb(self, textview, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the TemplateManager instance.
		@type self: A TemplateManager object.

		@param textview: The text editor's view.
		@type textview: A ScribesTextView object.

		@param event: An event that occurs when keys on the keyboard are pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		from gtk import keysyms
		if event.keyval == keysyms.Tab and self.template_list:
			self.template_list[-1].next_placeholder()
			return True
		if event.keyval == keysyms.ISO_Left_Tab and self.template_list:
			self.template_list[-1].previous_placeholder()
			return True
		return False
