# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This modules documents a class that creates the template add dialog. The
dialog allows users to add templates to the editor's template database.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from templatedialog import TemplateDialog

class TemplateAddDialog(TemplateDialog):
	"""
	This class creates the add template dialog for the template editor.
	The dialog allows users to add templates to the text editor's
	template database. This class defines the default properties and
	behavior of the template dialog.
	"""

	def __init__(self, template_editor):
		"""
		Initialize the dialog.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		TemplateDialog.__init__(self, template_editor)
		self.__init_attributes()
		self.__set_properties()
		self.save_button.connect("clicked", self.dialog_clicked_cb)
		self.name_entry.connect("changed", self.__dialog_changed_cb)
		self.connect("map-event", self.__dialog_map_event_cb)

	def __init_attributes(self):
		"""
		Initialize the dialog's data attributes.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		from gtk import Button, STOCK_SAVE
		self.language_view = self.template_editor.language_view
		self.description_view = self.template_editor.description_view
		self.save_button = Button(stock=STOCK_SAVE, use_underline=True)
		return

	def __set_properties(self):
		"""
		Define the dialog's properties.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.
		"""
		from internationalization import msg0312, msg0309, msg0310
		from internationalization import msg0311
		self.set_property("title", msg0312)
		self.set_property("name", "Add Template Dialog")
		self.name_label.set_property("label", msg0309)
		self.name_label.set_mnemonic_widget(self.name_entry)
		self.description_label.set_property("label", msg0310)
		self.description_label.set_mnemonic_widget(self.description_entry)
		self.dialog_editor_label.set_property("label", msg0311)
		self.dialog_editor_label.set_mnemonic_widget(self.dialog_editor)
		from gtk import BUTTONBOX_END
		self.button_area.set_property("layout-style", BUTTONBOX_END)
		self.button_area.pack_start(self.cancel_button, False, False, 0)
		self.button_area.pack_start(self.save_button, False, False, 0)
		return

	def dialog_clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@param button: The dialog's add button.
		@type button: A gtk.Button object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		result = self.check_for_template_trigger()
		if result:
			result = self.__check_for_space_in_template_trigger()
			if result is False:
				result = self.__check_for_duplicate_trigger()
				if result is False:
					result = self.__check_for_duplicate_cursors()
					if result is False:
						self.add_template_to_database()
						self.hide_dialog()
		return True

	def check_for_template_trigger(self):
		"""
		Check if the name entry for the dialog contains text.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@return: True if the name entry contains text.
		@rtype: A Boolean object.
		"""
		if self.name_entry.get_text():
			return True
		from internationalization import msg0421
		self.editor.error_dialog.show_message(msg0421, parent_window=self)
		return False

	def __check_for_duplicate_trigger(self):
		"""
		Check if string representing the template trigger already exists
		in the template database.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@return: True if a string representing the trigger already exists.
		@rtype: A Boolean object.
		"""
		selection = self.language_view.get_selection()
		model, iterator = selection.get_selected()
		language_id = model.get_value(iterator, 1)
		template_key = language_id + self.name_entry.get_text().strip()
		result = False
		from template_metadata import open_template_database
		from template_metadata import close_template_database
		database = open_template_database()
		for key in database.keys():
			if key == template_key:
				from internationalization import msg0422
				message = msg0422 % self.name_entry.get_text()
				self.editor.error_dialog.show_message(message, parent_window=self)
				result = True
				break
		close_template_database(database)
		return result

	def add_template_to_database(self):
		"""
		Add the template entered in the add template dialog to the
		template database.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.
		"""
		selection = self.language_view.get_selection()
		model, iterator = selection.get_selected()
		language_id = model.get_value(iterator, 1)
		template_key = language_id + self.name_entry.get_text().strip()
		description = self.description_entry.get_text().strip()
		textbuffer = self.dialog_editor.get_buffer()
		begin, end = textbuffer.get_bounds()
		text = textbuffer.get_text(begin, end)
		from template_metadata import open_template_database
		from template_metadata import close_template_database
		database = open_template_database()
		database[template_key] = (description, text)
		close_template_database(database)
		self.editor.emit("updated-template-database")
		return

	def error_message(self, message):
		"""
		Show an error dialog with a message.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@param message: An error message.
		@type message: A String object.
		"""
		from gtk import MessageDialog, DIALOG_DESTROY_WITH_PARENT
		from gtk import MESSAGE_ERROR, BUTTONS_CLOSE
		dialog = MessageDialog(parent=self, flags=DIALOG_DESTROY_WITH_PARENT,
			type=MESSAGE_ERROR, buttons=BUTTONS_CLOSE,
			message_format=None)
		dialog.format_secondary_text(message)
		from internationalization import msg0133
		dialog.set_property("title", msg0133)
		dialog.set_property("role", "error_message_dialog")
		dialog.set_property("icon-name", "stock_dialog-error")
		dialog.set_property("skip-pager-hint", True)
		dialog.set_property("skip-taskbar-hint", True)
		dialog.set_property("urgency-hint", False)
		dialog.set_property("modal", True)
		from gtk import WIN_POS_CENTER_ON_PARENT
		dialog.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		dialog.set_property("resizable", True)
		dialog.run()
		dialog.hide_all()
		dialog.destroy()
		return

	def __dialog_map_event_cb(self, dialog, event):
		"""
		Handles callback when the "map-event" signal is emitted.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@param dialog: Reference to the TemplateAddDialog.
		@type dialog: A TemplateAddDialog object.

		@param event: An event that occurs when the dialog is displayed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.__set_widgets_sensitivity()
		return False

	def __dialog_changed_cb(self, entry):
		"""
		Handles callback when the "changed" signal is emitted.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@param entry: The name field.
		@type entry: A gtk.Entry object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.__set_widgets_sensitivity()
		return False

	def __set_widgets_sensitivity(self):
		"""
		Set the sensitivity of the widgets in the dialog.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.
		"""
		if self.name_entry.get_text():
			self.description_entry.set_property("sensitive", True)
			self.dialog_editor.set_property("sensitive", True)
			self.save_button.set_property("sensitive", True)
		else:
			self.description_entry.set_property("sensitive", False)
			self.dialog_editor.set_property("sensitive", False)
			self.save_button.set_property("sensitive", False)
		return

	def __check_for_space_in_template_trigger(self):
		"""
		If the template trigger has a space character show an error.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@return: True if spaces are found in the name entry field.
		@rtype: A Boolean object.
		"""
		text = self.name_entry.get_text().strip()
		for character in text:
			if character == " ":
				from internationalization import msg0423
				self.editor.error_dialog.show_message(msg0423, parent_window=self)
				return True
		return False

	def __check_for_duplicate_cursors(self):
		"""
		If the editor contains duplicate "cursor" placeholders show an
		error.

		@param self: Reference to the TemplateAddDialog instance.
		@type self: A TemplateAddDialog object.

		@return: True if duplicate cursor placeholders are found in the editor.
		@rtype: A Boolean object.
		"""
		textbuffer = self.dialog_editor.get_buffer()
		begin, end = textbuffer.get_bounds()
		text = textbuffer.get_text(begin, end)
		from re import findall, UNICODE
		matches = findall("\$\{cursor\}", text, UNICODE)
		if len(matches) > 1:
			from internationalization import msg0424
			self.editor.error_dialog.show_message(msg0424, parent_window=self)
			return True
		return False
