# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents the base class for the template editor's dialog.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import SIGNAL_RUN_LAST, TYPE_NONE
from gtk import Window

class TemplateDialog(Window):
	"""
	This class creates a base dialog for the template editor's add and
	edit dialogs. The class is designed primarily for inheritance. The
	class defines the default properties and behavior of the dialog.
	"""

	__gsignals__ = {
		"show-dialog": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"hide-dialog": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, template_editor):
		"""
		Initialize the base dialog.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribeTemplateEditor object.
		"""
		Window.__init__(self)
		self.__init_attributes(template_editor)
		self.__set_properties()
		self.__arrange_widgets()
		self.connect("show-dialog", self.__window_show_dialog_cb)
		self.connect("hide-dialog", self.__window_hide_dialog_cb)
		self.connect("key-press-event", self.__window_key_press_event_cb)
		self.connect("delete-event", self.__window_delete_event_cb)
		self.cancel_button.connect("clicked", self.__window_clicked_cb)

	def __init_attributes(self, template_editor):
		"""
		Initialize the dialog's data attributes.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribeTemplateEditor object.
		"""
		from gtk import VBox, HButtonBox, Button, STOCK_CANCEL
		self.template_editor = template_editor
		self.editor = template_editor.editor
		self.main_area = VBox()
		self.button_area = HButtonBox()
		self.spacing = 10
		self.button_area.set_spacing(self.spacing)
		self.name_entry = self.__create_entry()
		self.name_label = self.__create_label()
		self.description_entry = self.__create_entry()
		self.description_label = self.__create_label()
		from templatedialogeditor import TemplateDialogEditor
		self.dialog_editor = TemplateDialogEditor(self)
		self.dialog_editor_label = self.__create_label()
		self.cancel_button = Button(stock=STOCK_CANCEL, use_underline=True)
		return

	def __create_entry(self):
		"""
		Create a text entry widget for the dialog and define its
		properties.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@return: A text entry widget for the dialog.
		@rtype: A gtk.Entry object.
		"""
		from gtk import Entry
		entry = Entry()
		return entry

	def __create_label(self):
		"""
		Create a label for the dialog's entries and defines its
		properties.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@return: A label for a dialog entry.
		@rtype: A gtk.Label object.
		"""
		from gtk import Label
		label = Label()
		label.set_property("use-underline", True)
		return label

	def __set_properties(self):
		"""
		Set the default properties for the text editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.
		"""
		from gtk import WIN_POS_CENTER_ON_PARENT
		from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER
		self.set_type_hint(WINDOW_TYPE_HINT_DIALOG)
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self.editor.window, 2, 2.133333333)
		self.set_property("default-width", width)
		self.set_property("default-height", height)
		self.set_property("role", "template editor")
		self.set_property("modal", True)
		self.set_property("skip-taskbar-hint", True)
		self.set_property("skip-pager-hint", True)
		self.set_property("icon-name", "gnome-settings")
		self.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		self.set_property("gravity", GRAVITY_CENTER)
		self.set_property("resizable", True)
		self.set_property("border-width", self.spacing)
		self.set_transient_for(self.template_editor.window)
		return

	def __arrange_widgets(self):
		"""
		Arrange the widgets for the dialog.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.
		"""
		from gtk import Table, EXPAND, FILL, Alignment, SHRINK, VBox
		container = Table(rows=3, columns=2, homogeneous=False)
		container.set_col_spacings(self.spacing)
		container.set_row_spacings(self.spacing)
		vbox = VBox(spacing=self.spacing)
		self.add(vbox)
		vbox.pack_start(self.main_area, True, True, 0)
		vbox.pack_start(self.button_area, False, False, 0)
		self.main_area.pack_start(container, True, True, 0)
		alignment = Alignment(xalign=1.0, yalign=0.5)
		alignment.add(self.name_label)
		container.attach(child=alignment,
						left_attach=0,
						right_attach=1,
						top_attach=0,
						bottom_attach=1,
						xoptions=FILL,
						yoptions=FILL,
						xpadding=0,
						ypadding=0)

		container.attach(child=self.name_entry,
						left_attach=1,
						right_attach=2,
						top_attach=0,
						bottom_attach=1,
						xoptions=FILL,
						yoptions=FILL,
						xpadding=0,
						ypadding=0)

		alignment = Alignment(xalign=1.0, yalign=0.5)
		alignment.add(self.description_label)
		container.attach(child=alignment,
						left_attach=0,
						right_attach=1,
						top_attach=1,
						bottom_attach=2,
						xoptions=FILL,
						yoptions=FILL,
						xpadding=0,
						ypadding=0)

		container.attach(child=self.description_entry,
						left_attach=1,
						right_attach=2,
						top_attach=1,
						bottom_attach=2,
						xoptions=FILL,
						yoptions=FILL,
						xpadding=0,
						ypadding=0)

		alignment = Alignment(xalign=1.0, yalign=0.01)
		alignment.add(self.dialog_editor_label)
		container.attach(child=alignment,
						left_attach=0,
						right_attach=1,
						top_attach=2,
						bottom_attach=3,
						xoptions=FILL,
						yoptions=FILL,
						xpadding=0,
						ypadding=0)

		from utils import create_scrollwin
		scrollwin = create_scrollwin()
		scrollwin.add(self.dialog_editor)
		container.attach(child=scrollwin,
						left_attach=1,
						right_attach=2,
						top_attach=2,
						bottom_attach=3,
						xoptions=EXPAND|FILL,
						yoptions=EXPAND|FILL,
						xpadding=0,
						ypadding=0)
		return

	def __window_key_press_event_cb(self, window, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param window: The dialog for the template editor.
		@type window: A TemplateDialog object.

		@param event: An event that occurs when the keyboard is pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		from gtk import keysyms
		if event.keyval == keysyms.Escape:
			self.hide_dialog()
			return True
		return False

	def __window_delete_event_cb(self, window, event):
		"""
		Handles callback when the "delete-event" signal is emitted.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param window: The dialog for the template editor.
		@type window: A TemplateDialog object.

		@param event: An event that occurs when the keyboard is pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_dialog()
		return True

	def __window_clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param button: The cancel button for the template dialog.
		@type button: A gtk.Button object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.hide_dialog()
		return True

	def show_dialog(self):
		"""
		Show the template dialog.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.
		"""
		self.emit("show-dialog")
		return

	def hide_dialog(self):
		"""
		Show the template dialog.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.
		"""
		self.emit("hide-dialog")
		return

	def __window_show_dialog_cb(self, window):
		"""
		Handles callback when the "show-dialog" signal is emitted.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param window: Reference to the TemplateDialog.
		@type window: A TemplateDialog object.
		"""
		self.show_all()
		return

	def __window_hide_dialog_cb(self, window):
		"""
		Handles callback when the "hide-dialog" signal is emitted.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param window: Reference to the TemplateDialog.
		@type window: A TemplateDialog object.
		"""
		self.hide_all()
		self.editor.emit("updated-template-database")
		self.name_entry.set_text("")
		self.description_entry.set_text("")
		return
