# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates an editor for the template
editor's dialog.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import SIGNAL_RUN_LAST, TYPE_NONE
from gtksourceview import SourceView

class TemplateDialogEditor(SourceView):
	"""
	This class creates an editor for the template dialog. It defines
	the default properties and behavior of the editor.
	"""

	__gsignals__ = {
		"cursor-moved": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, dialog):
		"""
		Initialize the editor.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: An TemplateDialogEditor object.

		@param dialog: The dialog for the editor.
		@type dialog: A TemplateDialog object.
		"""
		SourceView.__init__(self)
		self.__init_attributes(dialog)
		self.__set_properties()
		self.template_editor.connect("show-dialog", self.__view_show_dialog_cb)
		self.template_editor.connect("hide-dialog", self.__view_hide_dialog_cb)
		self.connect("key-press-event", self.__view_key_press_event_cb)
		self.textbuffer.connect_after("insert-text", self.__view_insert_text_cb)
		self.textbuffer.connect("delete-range", self.__view_delete_range_cb)
		self.textbuffer.connect_after("delete-range", self.__view_delete_range_cb)
		self.connect("cursor-moved", self.__view_cursor_moved_cb)

	def __init_attributes(self, dialog):
		"""
		Initialize the editor's data attributes.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: An TemplateDialogEditor object.

		@param dialog: The dialog for the editor.
		@type dialog: A TemplateDialog object.
		"""
		self.is_visible = False
		self.template_editor = dialog
		from gconf import client_get_default
		self.client = client_get_default()
		from gtksourceview import SourceBuffer
		self.textbuffer = SourceBuffer()
		self.variable_region = None
		self.placeholder_region = None
		self.markers = []
		self.timeout_id = None
		self.tag = self.__create_tag()
		from cursor import get_cursor_iterator
		self.get_cursor_iterator = get_cursor_iterator
		cursor_position = self.get_cursor_iterator(self.textbuffer)
		self.mark = self.textbuffer.create_mark(None, cursor_position, True)
		return

	def __set_properties(self):
		"""
		Define the editor's properties.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: An TemplateDialogEditor object.
		"""
		#self.set_show_line_numbers(True)
		self.set_property("buffer", self.textbuffer)
		self.textbuffer.set_property("check-brackets", True)
		self.set_auto_indent(True)
		margin_position = self.client.get_int("/apps/scribes/margin_position")
		if margin_position is None:
			margin_position = 72
		self.set_margin(margin_position)
		show_margin = self.client.get_bool("/apps/scribes/margin")
		if show_margin is None:
			show_margin = False
		self.set_show_margin(show_margin)
		tab_width = self.client.get_int("/apps/scribes/tab")
		if tab_width is None:
			tab_width = 4
		self.set_tabs_width(tab_width)
		use_tabs = self.client.get_bool("/apps/scribes/use_tabs")
		if use_tabs is None:
			use_tabs = True
		self.set_insert_spaces_instead_of_tabs(not use_tabs)
		self.set_highlight_current_line(True)
		gconf_font = self.client.get_string("/apps/scribes/font")
		if gconf_font is None:
			gconf_font = "Monospace 12"
		from pango import FontDescription
		font = FontDescription(gconf_font)
		self.modify_font(font)
		wrap_mode_bool = self.client.get_bool("/apps/scribes/text_wrapping")
		use_theme_colors = self.client.get_bool("/apps/scribes/use_theme_colors")
		if use_theme_colors is None:
			use_theme_colors = True
		from gtk import WRAP_WORD, WRAP_NONE
		if wrap_mode_bool:
			self.set_wrap_mode(WRAP_WORD)
		else:
			self.set_wrap_mode(WRAP_NONE)
		if use_theme_colors is False:
			# Use foreground and background colors specified by the user stored
			# in GConf, the GNOME configuration database.
			from gconf import VALUE_STRING
			fgcolor = self.client.get_string("/apps/scribes/fgcolor")
			bgcolor = self.client.get_string("/apps/scribes/bgcolor")
			if fgcolor is None:
				fgcolor = "#000000"
			if bgcolor is None:
				bgcolor = "#ffffff"
			from gtk.gdk import color_parse
			foreground_color = color_parse(fgcolor)
			background_color = color_parse(bgcolor)
			from gtk import STATE_NORMAL
			self.modify_base(STATE_NORMAL, background_color)
			self.modify_text(STATE_NORMAL, foreground_color)
		return

	def __view_key_press_event_cb(self, textview, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param textview: The template editor's text view.
		@type textview: A TemplateDialogEditor object.

		@param event: An event that occurs when the keyboard is pressed.
		@type event: A gtk.Event object.
		"""
		result = self.__anaylze_input(event.keyval)
		return result

	def __view_insert_text_cb(self, textbuffer, iterator, text, length):
		"""
		Handles callback when the "insert-text" signal is emitted.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param textbuffer: The buffer for the dialog.
		@type textbuffer: A SourceBuffer object.

		@param iterator: The position where text is to be inserted.
		@type iterator: A gtk.TextIter object.

		@param text: The text to be inserted into the buffer.
		@type text: A String object.

		@param length: The length of text to be inserted into the buffer.
		@type length: An Integer object.
		"""
		if self.placeholder_region:
			begin = self.textbuffer.get_iter_at_mark(self.variable_region[0])
			end = self.textbuffer.get_iter_at_mark(self.variable_region[1])
			self.textbuffer.apply_tag(self.tag, begin, end)
		return False

	def __view_cursor_moved_cb(self, dialog):
		"""
		Handles callback when the "cursor-moved" signal is emitted.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param dialog: Reference the the TemplateDialogEditor.
		@type dialog: A TemplateDialogEditor object.
		"""
		if self.placeholder_region:
			begin = self.textbuffer.get_iter_at_mark(self.placeholder_region[0])
			end = self.textbuffer.get_iter_at_mark(self.placeholder_region[1])
			iterator = self.get_cursor_iterator(self.textbuffer)
			if iterator.in_range(begin, end) is False:
				self.clear_tags_and_marks()
		return False

	def __anaylze_input(self, keyval):
		"""
		Anaylze the text type into the editor's buffer.

		This function monitors the following characters: "$", "{", "}".
		When "$" is entered into the buffer, a placeholder is
		automatically inserted into the editor's buffer. If the cursor
		is within the placeholder, typing "}" moves the cursor to the
		end of the placeholder. "$" and "{" are prevented from being
		entered when when the cursor is within the placeholder.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param keyval: A value representing a character/
		@type keyval: An Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		result = False
		from gtk import keysyms
		if keyval == keysyms.dollar:
			self.__process_dollar_character()
			result = True
		elif keyval == keysyms.braceleft:
			result = self.__process_braceleft_character()
		elif keyval == keysyms.braceright:
			result = self.__process_braceright_character()
		return result

	def __process_dollar_character(self):
		"""
		Insert placeholder into the editor's buffer.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param keyval: A value representing a keyboard characters.
		@type keyval: An Integer object.
		"""
		if self.placeholder_region is None:
			selection = self.textbuffer.get_selection_bounds()
			if selection:
				self.__enclose_selection(selection)
				return
			from cursor import get_cursor_iterator
			iterator = get_cursor_iterator(self.textbuffer)
			first_mark = self.textbuffer.create_mark(None, iterator, True)
			self.textbuffer.insert_at_cursor("${placeholder}")
			iterator = get_cursor_iterator(self.textbuffer)
			last_mark = self.textbuffer.create_mark(None, iterator, False)
			iterator = self.textbuffer.get_iter_at_mark(first_mark)
			iterator.forward_chars(2)
			second_mark = self.textbuffer.create_mark(None, iterator, True)
			iterator = self.textbuffer.get_iter_at_mark(last_mark)
			iterator.backward_char()
			third_mark = self.textbuffer.create_mark(None, iterator, False)
			self.variable_region = second_mark, third_mark
			self.placeholder_region = first_mark, last_mark
			self.markers = [first_mark, second_mark, third_mark, last_mark]
			begin = self.textbuffer.get_iter_at_mark(second_mark)
			end = self.textbuffer.get_iter_at_mark(third_mark)
			self.textbuffer.select_range(begin, end)
		return

	def __enclose_selection(self, selection):
		"""
		Enclose selected text in placeholders.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param selection: The position of selected text in buffer.
		@type selection: A Tuple object.
		"""
		text = self.textbuffer.get_text(selection[0], selection[1])
		self.textbuffer.delete(selection[0], selection[1])
		string = "${" + text + "}"
		self.textbuffer.insert_at_cursor(string)
		return

	def __process_braceleft_character(self):
		"""
		Prevent insertion of the opening parenthesis if the cursor is
		within the a placeholder region.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param keyval: A value representing a keyboard characters.
		@type keyval: An Integer object.
		"""
		result = False
		if self.placeholder_region:
			result = True
		return result

	def __process_braceright_character(self):
		"""
		Move the cursor to the end of the placeholder region if the
		cursor is within the placeholder region.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param keyval: A value representing a keyboard characters.
		@type keyval: An Integer object.
		"""
		result = False
		if self.placeholder_region:
			iterator = self.textbuffer.get_iter_at_mark(self.placeholder_region[1])
			self.textbuffer.place_cursor(iterator)
			result = True
		return result

	def __create_tag(self):
		"""
		Create a tag for the placeholder.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.
		"""
		tag = self.textbuffer.create_tag()
		tag.set_property("background", "#ADD8E6")
		return tag

	def clear_tags_and_marks(self):
		"""
		Remove tags and marks from the textbuffer.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.
		"""
		self.placeholder_region = None
		self.variable_region = None
		if self.markers:
			for mark in self.markers:
				if mark.get_deleted() is False:
					self.textbuffer.delete_mark(mark)
		begin, end = self.textbuffer.get_bounds()
		self.textbuffer.remove_tag(self.tag, begin, end)
		return

	def __view_show_dialog_cb(self, dialog):
		"""
		Handles callback when the "show-dialog" signal is emitted.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param dialog: The text editor's template dialog.
		@type dialog: A TemplateDialogEditor object.
		"""
		from gobject import timeout_add
		self.timeout_id = timeout_add(50, self.__check_cursor_position)
		selection = self.template_editor.language_view.get_selection()
		model, iterator = selection.get_selected()
		language_id = model.get_value(iterator, 1)
		if language_id == "General":
			self.textbuffer.set_property("highlight", False)
		else:
			self.textbuffer.set_property("highlight", True)
			self.__activate_syntax_highlight(language_id)
		self.clear_tags_and_marks()
		self.is_visible = True
		return

	def __view_hide_dialog_cb(self, dialog):
		"""
		Handles callback when the "hide-dialog" signal is emitted.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.

		@param dialog: The text editor's template dialog.
		@type dialog: A TemplateDialogEditor object.
		"""
		if self.timeout_id:
			from gobject import source_remove
			source_remove(self.timeout_id)
			self.timeout_id = None
		self.is_visible = False
		self.clear_tags_and_marks()
		self.textbuffer.set_text("")
		return

	def __activate_syntax_highlight(self, language_id):
		"""
		Toggle syntax highlight on for a particular language.

		@param self: Reference to the TemplateEditorTextView instance.
		@type self: A TemplateEditorTextView object.

		@param language_id: An identification for a particular language.
		@type language_id: A String object.
		"""
		from gtksourceview import SourceLanguage, SourceLanguagesManager
		manager = SourceLanguagesManager()
		languages = manager.get_available_languages()
		for language in languages:
			if language.get_id() == language_id:
				break
		from syntax import activate_syntax_highlight
		activate_syntax_highlight(self.textbuffer, language)
		return

	def __check_cursor_position(self):
		"""
		Check if the cursor position has moved.

		@param self: Reference to the TemplateDialogEditor instance.
		@type self: A TemplateDialogEditor object.
		"""
		iterator = self.textbuffer.get_iter_at_mark(self.mark)
		cursor_position = self.get_cursor_iterator(self.textbuffer)
		if iterator.equal(cursor_position):
			return True
		self.emit("cursor-moved")
		cursor_position = self.get_cursor_iterator(self.textbuffer)
		self.mark = self.textbuffer.create_mark(None, cursor_position, True)
		return True

	def __view_delete_range_cb(self, *args):
		self.emit("cursor-moved")
		return False
