# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates a template editor for the
text editor's buffer.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE

class ScribesTemplateEditor(GObject):
	"""
	This class creates a template editor for the text editor. The template
	editor allows users to view, add, remove, edit, import and export
	templates or snippets. The template editor is basically a frontend
	to the template database.
	"""

	__gsignals__ = {
		"show-template-editor": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"hide-template-editor": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, editor):
		"""
		Initialize the template editor.

		@param self: Reference to the ScribesTemplateEditor instance.
		@type self: A ScribesTemplateEditor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		GObject.__init__(self)
		self.__init_attributes(editor)
		self.__create_widgets()
		self.__arrange_widgets()

	def __init_attributes(self, editor):
		"""
		Initialize the template editor's data attributes.

		@param self: Reference to the ScribesTemplateEditor instance.
		@type self: A ScribesTemplateEditor object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		return

	def __create_widgets(self):
		"""
		Create the graphic user interface components for the editor.

		@param self: Reference to the ScribesTemplateEditor instance.
		@type self: A ScribesTemplateEditor object.
		"""
		from templateeditorwindow import TemplateEditorWindow
		self.window = TemplateEditorWindow(self)
		from templateeditorlangview import TemplateEditorLanguageView
		self.language_view = TemplateEditorLanguageView(self)
		from templateeditordescriptionview import TemplateEditorDescriptionView
		self.description_view = TemplateEditorDescriptionView(self)
		from templateeditortextview import TemplateEditorTextView
		self.textview = TemplateEditorTextView(self)
		from templateeditoraddbutton import TemplateEditorAddButton
		self.addbutton = TemplateEditorAddButton(self)
		from templateeditoreditbutton import TemplateEditorEditButton
		self.editbutton = TemplateEditorEditButton(self)
		from templateeditorremovebutton import TemplateEditorRemoveButton
		self.removebutton = TemplateEditorRemoveButton(self)
		from templateeditorimportbutton import TemplateEditorImportButton
		self.importbutton = TemplateEditorImportButton(self)
		from templateeditorexportbutton import TemplateEditorExportButton
		self.exportbutton = TemplateEditorExportButton(self)
		return

	def __arrange_widgets(self):
		"""
		Arrange the graphic user interface components for the editor.

		@param self: Reference to the ScribesTemplateEditor instance.
		@type self: A ScribesTemplateEditor object.
		"""
		from gtk import HBox, VBox, POLICY_AUTOMATIC, POLICY_NEVER
		from utils import create_scrollwin
		maincontainer = VBox(spacing=10)
		self.window.add(maincontainer)
		viewcontainer = HBox(spacing=10)
		maincontainer.pack_start(viewcontainer, True, True, 0)
		scrollwin = create_scrollwin()
		scrollwin.set_policy(POLICY_NEVER, POLICY_AUTOMATIC)
		scrollwin.add(self.language_view)
		viewcontainer.pack_start(scrollwin, False, False, 0)
		rightviewcontainer = VBox(spacing=10)
		viewcontainer.pack_start(rightviewcontainer, True, True, 0)
		upperviewcontainer = HBox(spacing=10)
		rightviewcontainer.pack_start(upperviewcontainer, True, True, 0)
		scrollwin = create_scrollwin()
		scrollwin.add(self.description_view)
		upperviewcontainer.pack_start(scrollwin, True, True, 0)
		scrollwin = create_scrollwin()
		scrollwin.add(self.textview)
		rightviewcontainer.pack_start(scrollwin, True, True, 0)
		buttoncontainer = VBox(spacing=7)
		upperviewcontainer.pack_start(buttoncontainer, False, True, 0)
		buttoncontainer.pack_start(self.addbutton, False, False, 0)
		buttoncontainer.pack_start(self.editbutton, False, False, 0)
		buttoncontainer.pack_start(self.removebutton, False, False, 0)
		buttoncontainer.pack_start(self.importbutton, False, False, 0)
		buttoncontainer.pack_start(self.exportbutton, False, False, 0)
		return

	def show_template_editor(self):
		"""
		Show the template editor.

		@param self: Reference to the ScribesTemplateEditor instance.
		@type self: A ScribesTemplateEditor object.
		"""
		self.emit("show-template-editor")
		return

	def hide_template_editor(self):
		"""
		Hide the template editor.

		@param self: Reference to the ScribesTemplateEditor instance.
		@type self: A ScribesTemplateEditor object.
		"""
		self.emit("hide-template-editor")
		return

	def remove_template_from_database(self):
		"""
		Remove the template selected in the Description tree view from
		the template database.

		@param self: Reference to the ScribesTemplateEditor instance.
		@type self: A ScribesTemplateEditor object.
		"""
		selection = self.language_view.get_selection()
		model, iterator = selection.get_selected()
		language_id = model.get_value(iterator, 1)
		selection = self.description_view.get_selection()
		model, paths = selection.get_selected_rows()
		if not paths:
			return True
		from template_metadata import open_template_database
		from template_metadata import close_template_database
		database = open_template_database()
		for path in paths:
			iterator = model.get_iter(path)
			trigger = model.get_value(iterator, 0)
			del database[language_id+trigger]
		close_template_database(database)
		self.editor.emit("updated-template-database")
		return
