# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates the name and description view
for the template editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import TreeView

class TemplateEditorDescriptionView(TreeView):
	"""
	This class creates the description tree view for the template editor.
	The view lists the name of each template trigger and an associated
	description for it per language. The class defines the default
	properties and behavior of the tree view.
	"""

	def __init__(self, template_editor):
		"""
		Initialize the description tree view.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		TreeView.__init__(self)
		self.__init_attributes(template_editor)
		self.__set_properties()
		self.language_view.connect("cursor-changed", self.__view_cursor_changed_cb)
		self.connect("cursor-changed", self.__view_cursor_changed_1_cb)
		self.connect("key-press-event", self.__view_key_press_event_cb)
		self.connect("map-event", self.__view_map_event_cb)
		self.connect("row-activated", self.__view_row_activate_cb)
		self.connect("drag-data-get", self.__view_drag_data_get_cb)
		self.editor.connect("updated-template-database", self.__view_updated_template_database)
		self.editor.connect("loaded-document", self.__reset_flag)
		self.editor.connect("renamed-document", self.__reset_flag)

	def __init_attributes(self, template_editor):
		"""
		Initialize the view's data attributes.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		self.is_first_time = True
		self.template_editor = template_editor
		self.template_editor.description_view = self
		self.editor = template_editor.editor
		self.language_view = template_editor.language_view
		self.model = self.__create_model()
		self.name_renderer = self.__create_renderer()
		self.description_renderer = self.__create_renderer()
		self.name_column = self.__create_name_column()
		self.description_column = self.__create_description_column()
		self.count = 0
		self.__path = (0,)
		return

	def __set_properties(self):
		"""
		Set the description view's default properties.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.
		"""
		from gtk import SELECTION_MULTIPLE
		selection = self.get_selection()
		selection.set_mode(SELECTION_MULTIPLE)
		from gtk.gdk import BUTTON1_MASK, ACTION_COPY, ACTION_DEFAULT
		targets = [("text/plain", 0, 123), ("STRING", 0, 123)]
		self.enable_model_drag_source(BUTTON1_MASK, targets, ACTION_COPY|ACTION_DEFAULT)
		self.set_property("model", self.model)
		self.set_property("rules-hint", True)
		self.set_property("search-column", 0)
		self.set_property("headers-clickable", True)
		self.append_column(self.name_column)
		self.append_column(self.description_column)
		self.name_column.clicked()
		return

	def __create_model(self):
		"""
		Create the model for the template editor's description view.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@return: A model for the description view.
		@rtype: A gtk.ListStore object.
		"""
		from gtk import ListStore
		model = ListStore(str, str)
		return model

	def __create_renderer(self):
		"""
		Create the renderer for the description view's column

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@return: A renderer for the description view.
		@rtype: A gtk.CellRendererText object.
		"""
		from gtk import CellRendererText
		renderer = CellRendererText()
		return renderer

	def __create_name_column(self):
		"""
		Create the column for the template editor's description view.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@return: A column for the description view.
		@rtype: A gtk.TreeViewColumn object.
		"""
		from gtk import TreeViewColumn, TREE_VIEW_COLUMN_GROW_ONLY
		from gtk import SORT_ASCENDING
		from internationalization import msg0307
		column = TreeViewColumn(msg0307, self.name_renderer, text=0)
		column.set_property("expand", False)
		column.set_property("sizing", TREE_VIEW_COLUMN_GROW_ONLY)
		column.set_property("clickable", True)
		column.set_sort_column_id(0)
		column.set_property("sort-indicator", True)
		column.set_property("sort-order", SORT_ASCENDING)
		return column

	def __create_description_column(self):
		"""
		Create the column for the template editor's description view.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@return: A column for the description view.
		@rtype: A gtk.TreeViewColumn object.
		"""
		from gtk import TreeViewColumn, TREE_VIEW_COLUMN_GROW_ONLY
		from gtk import SORT_ASCENDING
		from internationalization import msg0308
		column = TreeViewColumn(msg0308, self.description_renderer, text=1)
		column.set_property("expand", True)
		column.set_property("sizing", TREE_VIEW_COLUMN_GROW_ONLY)
		return column

	def __populate_model(self):
		"""
		Populate the description view's model.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.
		"""
		boolean = False
		self.model.clear()
		# Get the name of the selected row in the Language TreeView.
		selection = self.language_view.get_selection()
		model, iterator = selection.get_selected()
		try:
			language_id = model.get_value(iterator, 1)
		except TypeError:
			path, column = self.language_view.get_cursor()
			selection.select_path(path)
			model, iterator = selection.get_selected()
			language_id = model.get_value(iterator, 1)
		from template_metadata import open_template_database
		from template_metadata import close_template_database
		database = open_template_database()
		for key in database.keys():
			if key.startswith(language_id):
				boolean = True
				name = key.replace(language_id, "")
				description = database[key][0]
				self.model.append([name, description])
		close_template_database(database)
		self.set_property("sensitive", boolean)
		return

	def __view_cursor_changed_cb(self, language_view):
		"""
		Handles callback when the "cursor-changed" signal is emitted.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param language_view: The template editor's language view.
		@type language_view: A TemplateEditorLanguageView object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.__populate_model()
		self.__path = (0,)
		selection = self.get_selection()
		selection.select_path(self.__path)
		return False

	def __view_cursor_changed_1_cb(self, description_view):
		"""
		Handles callback when the "cursor-changed" signal is emitted.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param description_view: The template editor's description view.
		@type description_view: A TemplateEditorDescriptionView object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		selection = self.get_selection()
		model, paths = selection.get_selected_rows()
		self.__path = (0,)
		if paths:
			self.__path = paths[0]
		return False

	def __view_key_press_event_cb(self, treeview, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param treeview: The description tree view.
		@type treeview: A TemplateEditorDescriptionView object.

		@param event: An event that occurs when the keyboard is pressed.
		@type event: A gtk.Event object.
		"""
		from gtk import keysyms
		if event.keyval == keysyms.F6:
			self.language_view.grab_focus()
		if event.keyval == keysyms.Delete:
			self.template_editor.removebutton.activate()
		return False

	def __view_row_activate_cb(self, *args):
		"""
		Handles callback when the "row-activated" signal is emitted.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param *args: Some arguments.
		@type *args: A Tuple object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.template_editor.editbutton.activate()
		return True

	def __view_updated_template_database(self, editor):
		"""
		Handles callback when the "updated-template-database" signal is emitted.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__populate_model()
		self.name_column.queue_resize()
		self.description_column.queue_resize()
		self.columns_autosize()
		if len(self.model):
			self.__select_path()
		else:
			selection = self.language_view.get_selection()
			model, iterator = selection.get_selected()
			selection.select_iter(iterator)
			self.language_view.grab_focus()
		return

	def __select_path(self):
		"""
		Determine the appropriate row to select.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.
		"""
		selection = self.get_selection()
		model = self.get_model()
		length = len(model)
		try:
			if self.__path[0] > (length - 1):
				selection.select_path((length - 1),)
			else:
				selection.select_path(self.__path)
		except TypeError:
			self.__path = (0,)
			selection.select_path(self.__path)
		self.scroll_to_cell(self.__path, self.get_column(0), True, 0.5, 0.0)
		self.grab_focus()
		return

	def __view_map_event_cb(self, treeview, event):
		"""
		Handles callback when the "map-event" signal is emitted.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param treeview: Reference to the TemplateEditorDescriptionView.
		@type treeview: A TemplateEditorDescriptionView object.

		@param event: An event that occurs when the view is displayed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.is_first_time:
			self.__populate_model()
			selection = self.get_selection()
			selection.select_path(0)
		self.grab_focus()
		self.is_first_time = False
		return False

	def __reset_flag(self, editor):
		"""
		Reset the flag that indicates whether or not the view is being
		displayed for the first time.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_first_time = True
		return

	def __view_drag_data_get_cb(self, treeview, context, data, info, time):
		"""
		Handles callback when the "drag-data-get" signal is emitted.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param treeview: Reference to the TemplateEditorDescriptionView
		@type treeview: A TemplateEditorDescriptionView object.

		@param context: An object representing context data.
		@type context: A gtk.DragContext object.

		@param data: An object representing selection data.
		@type data: A gtk.SelectionData object.

		@param info: A unique identification number for the text editor.
		@type info: An Integer object.

		@param time: The time of the drag and drop operation.
		@type time: An Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		templates = self.__get_selected_templates()
		from writexmltemplate import create_template_string
		string = create_template_string(templates)
		data.set(data.target, 8, string)
		return False

	def __get_selected_templates(self):
		"""
		Get selected templates in the description view.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@return: Return list containing information about selected templates.
		@rtype: A List object.
		"""
		templates = []
		language_id = self.__get_language_id()
		selection = self.get_selection()
		model, paths = selection.get_selected_rows()
		for path in paths:
			trigger = self.__get_template_trigger(path)
			description = self.__get_template_description(path)
			template = self.__get_template(language_id+trigger)
			templates.append((language_id, trigger, description, template))
		return templates

	def __get_language_id(self, column=1):
		"""
		Get the language identification for the template.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@return: The language identification for the template.
		@rtype: A String object.
		"""
		selection = self.language_view.get_selection()
		model, iterator = selection.get_selected()
		language_id = model.get_value(iterator, column)
		return language_id

	def __get_template_trigger(self, path):
		"""
		Get the template trigger.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@param path: A row in the description view.
		@type path: A Path object.

		@return: The template trigger.
		@rtype: A String object.
		"""
		model = self.get_model()
		iterator = model.get_iter(path)
		trigger = model.get_value(iterator, 0)
		return trigger

	def __get_template_description(self, path):
		"""
		Get the template description.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@param path: A row in the description view.
		@type path: A Path object.

		@return: A description for the template.
		@rtype: A String object.
		"""
		model = self.get_model()
		iterator = model.get_iter(path)
		description = model.get_value(iterator, 1)
		return description

	def __get_template(self, template_key):
		"""
		Get the template from the template database.

		@param self: Reference to the ExportDialog instance.
		@type self: A ExportDialog object.

		@param template_key: The template key for a template value.
		@type template_key: A String object.

		@return: A template for the text editor.
		@rtype: A String object.
		"""
		from template_metadata import open_template_database
		from template_metadata import close_template_database
		database = open_template_database()
		template = database[template_key][1]
		close_template_database(database)
		return template
