# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates the edit button for the text
editor's template editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import Button

class TemplateEditorExportButton(Button):
	"""
	This class creates the export button for the template editor.
	Pressing the button activates a dialog that allows you to add new templates
	for the text editor. The class defines the default properties and
	behavior of the add button.
	"""

	def __init__(self, template_editor):
		"""
		Initialize the button.

		@param self: Reference to the TemplateEditorAddButton instance.
		@type self: A TemplateEditorAddButton object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A TemplateEditorAddButton object.
		"""
		Button.__init__(self)
		self.__init_attributes(template_editor)
		self.__create_icon_and_text()
		self.__set_properties()
		self.language_view.connect("cursor-changed", self.__button_cursor_changed_cb)
		self.description_view.connect_after("cursor-changed", self.__button_cursor_changed_cb)
		self.connect("clicked", self.__button_clicked_cb)
		self.editor.connect("updated-template-database", self.__button_update_database_cb)

	def __init_attributes(self, template_editor):
		"""
		Initialize the button's attributes.

		@param self: Reference to the TemplateEditorAddButton instance.
		@type self: A TemplateEditorAddButton object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A TemplateEditorAddButton object.
		"""
		self.template_editor = template_editor
		self.editor = template_editor.editor
		self.language_view = template_editor.language_view
		self.description_view = template_editor.description_view
		self.preview = template_editor.textview
		return

	def __create_icon_and_text(self):
		"""
		Create the text and icon for the import button.

		@param self: Reference to the TemplateEditorImportButton instance.
		@type self: A TemplateEditorImportButton object.
		"""
		from utils import create_button
		from gtk import STOCK_UNDO
		from internationalization import msg0419
		hbox = create_button(STOCK_UNDO, msg0419)
		self.add(hbox)
		return

	def __set_properties(self):
		"""
		Define the default properties of the button.

		@param self: Reference to the TemplateEditorExportButton instance.
		@type self: A TemplateEditorExportButton object.
		"""
		self.set_property("sensitive", False)
		return

	def __button_cursor_changed_cb(self, treeview):
		"""
		Handles callback when the "cursor-changed" signal is emitted.

		@param self: Reference to the TemplateEditorExportButton instance.
		@type self: A TemplateEditorExportButton object.

		@param treeview: The template editor's description treeview.
		@type treeview: A TemplateEditorDescriptionView object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.set_property("sensitive", False)
		if len(self.description_view.get_model()):
			self.set_property("sensitive", True)
		return False

	def __button_update_database_cb(self, editor):
		"""
		Handles callback when the "updated-template-database" signal is emitted.

		@param self: Reference to the TemplateEditorExportButton instance.
		@type self: A TemplateEditorExportButton object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.set_property("sensitive", False)
		if len(self.description_view.get_model()):
			self.set_property("sensitive", True)
		return

	def __button_clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the TemplateEditorExportButton instance.
		@type self: A TemplateEditorExportButton object.

		@param button: Reference to the TemplateEditorExportButton.
		@type button: A TemplateEditorExportButton object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		try:
			self.dialog.show_dialog()
		except:
			from exportdialog import ExportDialog
			self.dialog = ExportDialog(self)
			self.dialog.show_dialog()
		return False
