# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This modules documents a class that creates the language tree view for
the text editor's template editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE
from gtk import TreeView

class TemplateEditorLanguageView(TreeView):
	"""
	This class creates the language tree view for the template editor.
	It defines the default property and behavior of the tree view
	object.
	"""

	__gsignals__ = {
		"populated-model": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, template_editor):
		"""
		Initialize the view.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		TreeView.__init__(self)
		self.__init_attributes(template_editor)
		self.__set_properties()
		self.__populate_model()
		self.connect("map-event", self.__view_map_event_cb)
		self.connect("drag-data-get", self.__view_drag_data_get_cb)
		self.connect("drag-data-received", self.__view_drag_data_received_cb)
		self.editor.connect("loaded-document", self.__view_document_cb)
		self.editor.connect("renamed-document", self.__view_document_cb)
		self.connect("key-press-event", self.__view_key_press_event_cb)

	def __init_attributes(self, template_editor):
		"""
		Initialize the view's data attributes.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		self.template_editor = template_editor
		self.template_editor.language_view = self
		self.editor = template_editor.editor
		self.is_first_time = True
		self.model = self.__create_model()
		self.renderer = self.__create_renderer()
		self.column = self.__create_column()
		return

	def __set_properties(self):
		"""
		Set the language view's default properties.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.
		"""
		from gtk.gdk import ACTION_DEFAULT, BUTTON1_MASK
		from gtk import TARGET_SAME_APP
		self.enable_model_drag_source(BUTTON1_MASK, [("STRING", 0, 123)], ACTION_DEFAULT)
		self.enable_model_drag_dest([("STRING", TARGET_SAME_APP, 124)], ACTION_DEFAULT)
		self.set_property("model", self.model)
		self.set_property("rules-hint", True)
		self.set_property("search-column", 0)
		self.set_property("headers-clickable", True)
		self.append_column(self.column)
		self.column.clicked()
		return

	def __populate_model(self):
		"""
		Populate the language view's model.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.
		"""
		from gtksourceview import SourceLanguagesManager
		language_list = SourceLanguagesManager().get_available_languages()
		language_names = [(name.get_name(), name.get_id()) for name in language_list]
		self.model.append(["General", "General"])
		for name, id in language_names:
			self.model.append([name, id])
		return

	def __create_model(self):
		"""
		Create the model for the template editor's language view.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@return: A model for the language view.
		@rtype: A gtk.ListStore object.
		"""
		from gtk import ListStore
		model = ListStore(str, str)
		return model

	def __create_column(self):
		"""
		Create the column for the template editor's language view.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@return: A column for the language view.
		@rtype: A gtk.TreeViewColumn object.
		"""
		from gtk import TreeViewColumn, TREE_VIEW_COLUMN_GROW_ONLY
		from gtk import SORT_ASCENDING
		from internationalization import msg0306
		column = TreeViewColumn(msg0306, self.renderer, text=0)
		column.set_property("expand", False)
		column.set_property("sizing", TREE_VIEW_COLUMN_GROW_ONLY)
		column.set_property("clickable", True)
		column.set_sort_column_id(0)
		column.set_property("sort-indicator", True)
		column.set_property("sort-order", SORT_ASCENDING)
		return column

	def __create_renderer(self):
		"""
		Create the renderer for the language view's column

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@return: A renderer for the language view.
		@rtype: A gtk.CellRendererText object.
		"""
		from gtk import CellRendererText
		renderer = CellRendererText()
		return renderer

	def __view_document_cb(self, editor):
		"""
		Handles callback when the "renamed-document" or "loaded-document"
		signal is emitted.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_first_time = True
		return

	def __view_map_event_cb(self, treeview, event):
		"""
		Handles callback when the "map-event" signal is emitted.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param treeview: The language view.
		@type treeview: A TemplateEditorLanguageView object.

		@param event: An event that occurs when the view is displayed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.is_first_time:
			if self.editor.uri:
				from utils import get_language
				language = get_language(str(self.editor.uri))
				if language:
					self.__select_language(language.get_id())
				else:
					self.__select_language("General")
			else:
				self.__select_language("General")
#		self.grab_focus()
		self.is_first_time = False
		return False

	def __select_language(self, language_id):
		"""
		Select a row corresponding to the language in the tree view.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param language_id: An identifier representing a language.
		@type language_id: A String object.
		"""
		model = self.get_model()
		iterator = model.get_iter_first()
		language = model.get_value(iterator, 1)
		if language == language_id:
			selection = self.get_selection()
			selection.select_iter(iterator)
			path = model.get_path(iterator)
			self.scroll_to_cell(path, self.get_column(0), True, 0.5, 0.0)
		else:
			while True:
				iterator = model.iter_next(iterator)
				if iterator:
					language = model.get_value(iterator, 1)
					if language == language_id:
						selection = self.get_selection()
						selection.select_iter(iterator)
						path = model.get_path(iterator)
						self.scroll_to_cell(path, self.get_column(0), True, 0.5, 0.0)
						break
				else:
					break
		return

	def __view_key_press_event_cb(self, treeview, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param treeview: The description tree view.
		@type treeview: A TemplateEditorLanguageView object.

		@param event: An event that occurs when the keyboard is pressed.
		@type event: A gtk.Event object.
		"""
		from gtk import keysyms
		if event.keyval == keysyms.F6:
			self.template_editor.description_view.grab_focus()
			return False
		return False

	def __view_drag_data_get_cb(self, treeview, context, data, info, time):
		"""
		Handles callback when the "drag-data-get" signal is emitted.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param treeview: Reference to the TemplateEditorLanguageView.
		@type treeview: A TemplateEditorLanguageView object.

		@param context: An object containing data about a drag selection.
		@type context: A gtk.DragContextData object.

		@param data: Data representing the drag selection.
		@type data: A gtk.SelectionData object.

		@param info: A unique identification for the text editor.
		@type info: An Integer object.

		@param time: The time the drop operation occurred.
		@type time: An Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		selection = self.get_selection()
		model, iterator = selection.get_selected()
		language_id = model.get_value(iterator, 1)
		templates = self.__get_templates(language_id)
		if templates:
			from writexmltemplate import create_template_string
			string = create_template_string(templates)
			data.set(data.target, 8, string)
		return True

	def __get_templates(self, language_id):
		"""
		Get templates associated with the language identifier from the
		template database.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@return: Templates associated with a language identifier.
		@rtype: A List object.
		"""
		templates = []
		from template_metadata import open_template_database
		from template_metadata import close_template_database
		database = open_template_database()
		for key in database.keys():
			if key.startswith(language_id):
				trigger = key[len(language_id):]
				description = database[key][0]
				template = database[key][1]
				templates.append((language_id, trigger, description, template))
		close_template_database(database)
		return templates

	def __view_drag_data_received_cb(self, treeview, context, x, y, data, info, time):
		"""
		Handles callback when the "drag-data-received" signal is emitted.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param treeview: Reference to the TemplateEditorLanguageView.
		@type treeview: A TemplateEditorLanguageView object.

		@param context: An object containing data about a drag selection.
		@type context: A gtk.DragContextData object.

		@param x: The x-cordinate of the drop.
		@type x: An Integer object.

		@param y: The y-cordinate of the drop.
		@type y: An Integer object.

		@param selection_data: Data representing the drag selection.
		@type selection_data: A gtk.SelectionData object.

		@param info: A unique identification for the text editor.
		@type info: An Integer object.

		@param time: The time the drop operation occurred.
		@type time: An Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		path, position = self.get_dest_row_at_pos(x, y)
		from gtk import TREE_VIEW_DROP_BEFORE, TREE_VIEW_DROP_AFTER
		if position in [TREE_VIEW_DROP_BEFORE, TREE_VIEW_DROP_AFTER]:
			return True
		language_id = self.__get_drop_row(path)
		string = data.data
		from readxmltemplate import get_template_from_string
		templates = get_template_from_string(string)
		if templates:
			new_templates = self.__create_new_templates(templates, language_id)
			from writexmltemplate import create_template_string
			template_string = create_template_string(new_templates)
			from importtemplate import import_template_from_string
			result = import_template_from_string(template_string)
			if result:
				selection = self.get_selection()
				selection.select_path(path)
				self.editor.emit("updated-template-database")
		return True

	def __create_new_templates(self, templates, language_id):
		"""
		Create a new template from the old one.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param templates: The old template for the text editor.
		@type templates: A List object.

		@param language_id: The templates language identifier.
		@type language_id: A String object.

		@return: New template format to be imported.
		@rtype: A List object.
		"""
		new_templates = []
		for template in templates:
			new_template = self.__convert_template(template, language_id)
			new_templates.append(new_template)
		return new_templates

	def __convert_template(self, template, language_id):
		"""
		Convert old to new template.

		This function just changes the language identifier of the old
		template with the new one.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param template: The old template.
		@type template: A Tuple object.

		@param language_id: An identifier representing the language for the template.
		@type language_id: A String object.

		@return: The new template format.
		@rtype: A Tuple object.
		"""
		key = template[0]
		model = self.get_model()
		trigger = self.__get_trigger_from_template_key(key)
		template = (language_id, trigger, template[1], template[2])
		return template

	def __get_trigger_from_template_key(self, key):
		"""
		Get the name of the trigger from the template key.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param key: The name of the key associated with a template.
		@type key: A String object.

		@return: The trigger for a template.
		@rtype: A String object.
		"""
		if key.startswith("General"):
			return key[len("General"):]
		from gtksourceview import SourceLanguagesManager
		manager = SourceLanguagesManager()
		languages = manager.get_available_languages()
		for language in languages:
			if key.startswith(language.get_id()):
				trigger = key[len(language.get_id()):]
				break
		return trigger

	def __get_drop_row(self, path):
		"""
		Get the language of the row drag and drop occurred.

		@param self: Reference to the TemplateEditorLanguageView instance.
		@type self: A TemplateEditorLanguageView object.

		@param path: The row a drag and drop occurred.
		@type path: A Tuple object.

		@return: The language identifier of the row a drag and drop occurred.
		@rtype: A String object.
		"""
		model = self.get_model()
		iterator = model.get_iter(path)
		language_id = model.get_value(iterator, 1)
		return language_id
