# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates the remove button for the text
editor's template editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import Button

class TemplateEditorRemoveButton(Button):
	"""
	This class creates the add button for the template editor. Pressing
	the button activates a dialog that allows you to add new templates
	for the text editor. The class defines the default properties and
	behavior of the add button.
	"""

	def __init__(self, template_editor):
		"""
		Initialize the button.

		@param self: Reference to the TemplateEditorRemoveButton instance.
		@type self: A TemplateEditorRemoveButton object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		from gtk import STOCK_REMOVE
		Button.__init__(self, stock=STOCK_REMOVE, use_underline=True)
		self.__init_attributes(template_editor)
		self.__set_properties()
		self.language_view.connect("cursor-changed", self.__button_cursor_changed_cb)
		self.description_view.connect("cursor-changed", self.__button_cursor_changed_cb)
		self.connect("clicked", self.__button_clicked_cb)

	def __init_attributes(self, template_editor):
		"""
		Initialize the button's attributes.

		@param self: Reference to the TemplateEditorRemoveButton instance.
		@type self: A TemplateEditorRemoveButton object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		self.template_editor = template_editor
		self.editor = template_editor.editor
		self.description_view = template_editor.description_view
		self.language_view = template_editor.language_view
		return

	def __set_properties(self):
		"""
		Define the default properties of the button.

		@param self: Reference to the TemplateEditorRemoveButton instance.
		@type self: A TemplateEditorRemoveButton object.
		"""
		self.set_property("sensitive", False)
		return

	def __button_cursor_changed_cb(self, treeview):
		"""
		Handles callback when the "cursor-changed" signal is emitted.

		@param self: Reference to the TemplateEditorRemoveButton instance.
		@type self: A TemplateEditorRemoveButton object.

		@param treeview: The template editor's treeview.
		@type treeview: A TemplateEditorLanguageView object.
		"""
		self.set_property("sensitive", False)
		if len(self.description_view.get_model()):
			self.set_property("sensitive", True)
		return True

	def __button_clicked_cb(self, button):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the TemplateEditorRemoveButton instance.
		@type self: A TemplateEditorRemoveButton object.

		@param button: Reference to the TemplateEditorRemoveButton.
		@type button: A TemplateEditorRemoveButton object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.template_editor.remove_template_from_database()
		return True
