# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates a text view for the text
editor's template editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtksourceview import SourceView, SourceBuffer

class TemplateEditorTextView(SourceView):
	"""
	This class creates the text view for the template editor. It defines
	the default properties and behavior of the text view.
	"""

	def __init__(self, template_editor):
		"""
		Initialize the text view.

		@param self: Reference to the TemplateEditorTextView instance.
		@type self: A TemplateEditorTextView object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A TemplateEditorTextView object.
		"""
		SourceView.__init__(self, SourceBuffer())
		self.__init_attributes(template_editor)
		self.__set_properties()
		self.description_view.connect_after("cursor-changed", self.__view_cursor_changed_cb)
		self.language_view.connect("cursor-changed", self.__view_cursor_changed_cb)
		self.description_view.connect_after("select-all", self.__view_cursor_changed_cb)
		self.connect("button-press-event", self.__view_button_press_event_cb)

	def __init_attributes(self, template_editor):
		"""
		Initialize the text view's data attributes.

		@param self: Reference to the TemplateEditorTextView instance.
		@type self: A TemplateEditorTextView object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A TemplateEditorTextView object.
		"""
		self.template_editor = template_editor
		self.template_editor.textview = self
		self.description_view = template_editor.description_view
		self.language_view = template_editor.language_view
		self.textbuffer = self.get_buffer()
		from gconf import client_get_default
		self.client = client_get_default()
		return

	def __set_properties(self):
		"""
		Define the text view's default properties.

		@param self: Reference to the TemplateEditorTextView instance.
		@type self: A TemplateEditorTextView object.
		"""
		self.set_property("auto-indent", False)
		self.set_property("show-line-markers", False)
		self.set_property("show-line-numbers", False)
		self.set_property("show-margin", False)
		self.textbuffer.set_property("highlight", False)
		self.textbuffer.set_property("check-brackets", False)
		self.set_property("cursor-visible", False)
		self.set_property("editable", False)
		self.set_property("sensitive", False)
		tab_width = self.client.get_int("/apps/scribes/tab")
		if tab_width is None:
			tab_width = 4
		self.set_tabs_width(tab_width)
		use_tabs = self.client.get_bool("/apps/scribes/use_tabs")
		if use_tabs is None:
			use_tabs = True
		self.set_insert_spaces_instead_of_tabs(not use_tabs)
		gconf_font = self.client.get_string("/apps/scribes/font")
		if gconf_font is None:
			gconf_font = "Monospace 12"
		from pango import FontDescription
		font = FontDescription(gconf_font)
		self.modify_font(font)
		wrap_mode_bool = self.client.get_bool("/apps/scribes/text_wrapping")
		if wrap_mode_bool is None:
			wrap_mode_bool = False
		use_theme_colors = self.client.get_bool("/apps/scribes/use_theme_colors")
		if use_theme_colors is None:
			use_theme_colors = True
		from gtk import WRAP_WORD, WRAP_NONE
		if wrap_mode_bool:
			self.set_wrap_mode(WRAP_WORD)
		else:
			self.set_wrap_mode(WRAP_NONE)
		if use_theme_colors is False:
			# Use foreground and background colors specified by the user stored
			# in GConf, the GNOME configuration database.
			from gconf import VALUE_STRING
			fgcolor = self.client.get_string("/apps/scribes/fgcolor")
			bgcolor = self.client.get_string("/apps/scribes/bgcolor")
			if fgcolor is None:
				fgcolor = "#000000"
			if bgcolor is None:
				bgcolor = "#ffffff"
			from gtk.gdk import color_parse
			foreground_color = color_parse(fgcolor)
			background_color = color_parse(bgcolor)
			from gtk import STATE_NORMAL
			self.modify_base(STATE_NORMAL, background_color)
			self.modify_text(STATE_NORMAL, foreground_color)
		return

	def __view_cursor_changed_cb(self, description_view):
		"""
		Handles callback when the "cursor-changed" signal is emitted.

		@param self: Reference to the TemplateEditorTextView instance.
		@type self: A TemplateEditorTextView object.

		@param description_view: The template editor's description view.
		@type description_view: A TemplateEditorDescriptionView object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.textbuffer.set_text("")
		if not len(self.description_view.get_model()):
			self.set_property("sensitive", False)
			return False
		description_selection = self.description_view.get_selection()
		if description_selection.count_selected_rows() > 1:
			self.textbuffer.set_text("")
			self.textbuffer.set_property("highlight", False)
			self.set_property("sensitive", False)
			return False
		model, paths = description_selection.get_selected_rows()
		iterator = model.get_iter(paths[0])
		trigger_selection = model.get_value(iterator, 0)
		language_selection = self.__get_selection(self.language_view, 1)
		if language_selection != "General":
			self.__activate_syntax_highlight(language_selection)
		else:
			self.textbuffer.set_property("highlight", False)
		template_key = language_selection + trigger_selection
		from template_metadata import open_template_database
		from template_metadata import close_template_database
		database = open_template_database()
		for key in database.keys():
			if key == template_key:
				template = database[key][1]
				self.textbuffer.set_text(template)
				break
		close_template_database(database)
		self.set_property("sensitive", True)
		return False

	def __get_selection(self, treeview, column=0):
		"""
		Get the value of the selected row in a tree view column.

		@param self: Reference to the TemplateEditorTextView instance.
		@type self: A TemplateEditorTextView object.

		@param treeview: A tree view widget.
		@type treeview: A gtk.TreeView object.

		@param value: The value of a selected row in a tree view.
		@type value: A String object.
		"""
		# Get the name of the selected row in the Language TreeView.
		selection = treeview.get_selection()
		model, iterator = selection.get_selected()
		try:
			value = model.get_value(iterator, column)
		except TypeError:
			path, view_column = treeview.get_cursor()
			selection.select_path(path)
			model, iterator = selection.get_selected()
			value = model.get_value(iterator, column)
		return value

	def __activate_syntax_highlight(self, language_id):
		"""
		Toggle syntax highlight on for a particular language.

		@param self: Reference to the TemplateEditorTextView instance.
		@type self: A TemplateEditorTextView object.

		@param language_id: An identification for a particular language.
		@type language_id: A String object.
		"""
		from gtksourceview import SourceLanguage, SourceLanguagesManager
		manager = SourceLanguagesManager()
		languages = manager.get_available_languages()
		for language in languages:
			if language.get_id() == language_id:
				break
		from syntax import activate_syntax_highlight
		activate_syntax_highlight(self.textbuffer, language)
		return

	def __view_button_press_event_cb(self, *args):
		self.template_editor.editbutton.activate()
		return True
