# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates the window for the text
editor's template editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import Window

class TemplateEditorWindow(Window):
	"""
	This class creates a window object for the template editor. It
	defines the default properties and behavior of the window object.
	"""

	def __init__(self, template_editor):
		"""
		Initialize the window object.

		@param self: Reference to the TemplateEditorWindow instance.
		@type self: A TemplateEditorWindow object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A TemplateEditorWindow object.
		"""
		Window.__init__(self)
		self.__init_attributes(template_editor)
		self.__set_properties()
		self.connect("delete-event", self.__window_delete_event_cb)
		self.connect("key-press-event", self.__window_key_press_event_cb)
		self.connect("drag-data-received", self.__window_drag_data_received_cb)
		self.template_editor.connect("show-template-editor", self.__window_show_template_editor_cb)
		self.template_editor.connect("hide-template-editor", self.__window_hide_template_editor_cb)

	def __init_attributes(self, template_editor):
		"""
		Initialize the window object.

		@param self: Reference to the TemplateEditorWindow instance.
		@type self: A TemplateEditorWindow object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A TemplateEditorWindow object.
		"""
		self.editor = template_editor.editor
		self.template_editor = template_editor
		self.is_visible = False
		self.template_editor.window = self
		return

	def __set_properties(self):
		"""
		Set the window's default properties.

		@param self: Reference to the TemplateEditorWindow instance.
		@type self: A TemplateEditorWindow object.
		"""
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self.editor.window, 1.6, 1.6)
		self.set_property("default-width", width)
		self.set_property("default-height", height)
		self.set_property("border-width", 10)
		from gtk import WIN_POS_CENTER_ON_PARENT, DEST_DEFAULT_ALL
		self.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		self.set_property("icon-name", "gnome-settings")
		self.set_property("role", "template_window")
		self.set_property("name", "scribes_template_editor_window")
		from internationalization import msg0305
		self.set_property("title", msg0305)
		self.set_property("destroy-with-parent", True)
		self.set_transient_for(self.editor.window)
		targets = [("text/uri-list", 0, 111)]
		from gtk.gdk import ACTION_COPY
		self.drag_dest_set(DEST_DEFAULT_ALL, targets, ACTION_COPY)
		return

	def __window_delete_event_cb(self, window, event):
		"""
		Handles callback when the "delete-event" signal is emitted.

		@param self: Reference to the TemplateEditorWindow instance.
		@type self: A TemplateEditorWindow object.

		@param window: The template editor's window.
		@type window: A TemplateEditorWindow object.

		@param event: An event that occurs when the close button is pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.template_editor.hide_template_editor()
		return True

	def __window_key_press_event_cb(self, window, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the TemplateEditorWindow instance.
		@type self: A TemplateEditorWindow object.

		@param window: The template editor's window.
		@type window: A TemplateEditorWindow object.

		@param event: An event that occurs when the close button is pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.is_visible:
			from gtk import keysyms
			if event.keyval == keysyms.Escape:
				self.template_editor.hide_template_editor()
				return True
		return False

	def __window_show_template_editor_cb(self, template_editor):
		"""
		Handles callback when the "show-template-editor" signal is
		emitted.

		@param self: Reference to the TemplateEditorWindow instance.
		@type self: A TemplateEditorWindow object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		if self.is_visible is False:
			self.show_all()
			self.is_visible = True
		self.present()
		return

	def __window_hide_template_editor_cb(self, template_editor):
		"""
		Handles callback when the "hide-template-editor" signal is
		emitted.

		@param self: Reference to the TemplateEditorWindow instance.
		@type self: A TemplateEditorWindow object.

		@param template_editor: The text editor's template editor.
		@type template_editor: A ScribesTemplateEditor object.
		"""
		self.hide_all()
		self.is_visible = False
		return

	def __window_drag_data_received_cb(self, window, context, x, y, selection_data, info, time):
		"""
		Handles callback when the "drag-data-received" signal is emitted.

		@param self: Reference to the TemplateEditorWindow instance.
		@type self: A TemplateEditorWindow object.

		@param window: Reference to the TemplateEditorWindow.
		@type window: A TemplateEditorWindow object.

		@param context: An object containing data about a drag selection.
		@type context: A gtk.DragContextData object.

		@param x: The x-cordinate of the drop.
		@type x: An Integer object.

		@param y: The y-cordinate of the drop.
		@type y: An Integer object.

		@param selection_data: Data representing the drag selection.
		@type selection_data: A gtk.SelectionData object.

		@param info: A unique identification for the text editor.
		@type info: An Integer object.

		@param time: The time the drop operation occurred.
		@type time: An Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if info == 111:
			self.present()
			uris = selection_data.get_uris()
			for uri in uris:
				if uri.startswith("file:///"):
					from gnomevfs import get_local_path_from_uri
					local_path = get_local_path_from_uri(uri)
					from importtemplate import import_template_from_file
					self.set_property("sensitive", False)
					from utils import response
					response()
					if import_template_from_file(local_path):
						self.editor.emit("updated-template-database")
						context.finish(True, False, time)
					else:
						context.finish(False, False, time)
						from internationalization import msg0425
						self.template_editor.add_dialog.error_message(msg0425)
					self.set_property("sensitive", True)
		return True
