# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that loads templates into the text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class TemplateLoader(object):
	"""
	This class implements an object that loads templates into the text
	editor.
	"""

	def __init__(self, editor):
		"""
		Initialize the loader object.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__init_attributes(editor)
		self.__load_general_templates()
		self.editor.connect("loaded-document", self.__loader_loaded_document_cb)
		self.editor.connect("renamed-document", self.__loader_renamed_document_cb)
		self.editor.connect("updated-template-database", self.__loader_updated_database_cb)
		self.__reload_language_templates()

	def __init_attributes(self, editor):
		"""
		Initialize the loader's data attributes.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		return

	def __loader_gui_created_cb(self, editor):
		"""
		Handles callback when the "gui-created" signal is emitted.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__load_general_templates()
		return

	def __loader_loaded_document_cb(self, editor):
		"""
		Handles callback when the "loaded-document" signal is emitted.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__reload_language_templates()
		return

	def __loader_renamed_document_cb(self, editor):
		"""
		Handles callback when the "renamed-document" signal is emitted.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__reload_language_templates()
		return

	def __load_general_templates(self):
		"""
		Load generic templates into the text editor.

		This templates are always loaded.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.
		"""
		from template_metadata import open_template_database, close_template_database
		database = open_template_database()
		textbuffer = self.editor.textbuffer
		for elements in database.keys():
			if elements.startswith("General"):
				textbuffer.general_template_dictionary[elements] = database[elements][1]
		close_template_database(database)
		return

	def __load_language_templates(self):
		"""
		Load language templates into the text editor.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.
		"""
		if self.editor.uri is None:
			return
		from utils import get_language
		language = get_language(str(self.editor.uri))
		if not language:
			return
		language_id = language.get_id()
		from template_metadata import open_template_database, close_template_database
		database = open_template_database()
		textbuffer = self.editor.textbuffer
		for elements in database.keys():
			if elements.startswith(language_id):
				textbuffer.language_template_dictionary[elements] = database[elements][1]
		close_template_database(database)
		return

	def __reload_language_templates(self):
		"""
		Reload language templates into the text editor.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.
		"""
		textbuffer = self.editor.textbuffer
		textbuffer.language_template_dictionary = {}
		self.__load_language_templates()
		return

	def __reload_general_templates(self):
		"""
		Reload generic templates into the text editor.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.
		"""
		textbuffer = self.editor.textbuffer
		textbuffer.general_template_dictionary = {}
		self.__load_general_templates()
		return

	def __loader_updated_database_cb(self, editor):
		"""
		Handles callback when the "updated-template-database" signal is
		emitted.

		@param self: Reference to the TemplateLoader instance.
		@type self: A TemplateLoader object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__reload_general_templates()
		self.__reload_language_templates()
		return
