# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates an object that processes
templates containing placeholders.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class TemplateProcessor(object):
	"""
	This class creates an object that inserts templates with placeholders
	into the text editor's buffer and provides mechanisms for navigating
	between the placeholders.
	"""

	def __init__(self, template_manager, template, placeholders):
		"""
		Initialize the template processor.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.

		@param template_manager: An object that manages templates.
		@type template_manager: A TemplateManager object.

		@param template: A template containing placeholders.
		@type template: A String object.

		@param placeholders: A list of placeholders.
		@type placeholders: A List object.
		"""
		self.__init_attributes(template_manager, placeholders)
		# Generate a list of mirrors, duplicate placeholders, if any.
		self.__generate_mirror_list()

		# Mark the start and end of the template region.
		from cursor import get_cursor_iterator
		cursor_position = get_cursor_iterator(self.__editor.textbuffer)
		self.__start_template_position = self.__editor.textbuffer.create_mark(None, cursor_position, True)
		self.__end_template_position = self.__editor.textbuffer.create_mark(None, cursor_position, False)

		# Insert template into the text editor's buffer.
		self.__editor.textbuffer.begin_user_action()
		self.__template_manager.insert_template(template)
		# Check for special placeholders.
		if self.__has_special_placeholders():
			# Expand special placeholders in the buffer and tag them.
			self.__expand_special_placeholders()
		# Check for normal placeholders.
		if self.__has_placeholders():
			# Mark and tag placeholders in the buffer.
			self.__mark_placeholders_in_buffer()
			self.__tag_placeholders_in_buffer()
			self.__editor.textbuffer.end_user_action()
			# Determine the last placeholder index.
			self.__determine_last_placeholder()
			self.__select_first_placeholder()
			# Monitor signals.
			self.__cursor_movement_id = self.__editor.connect("cursor-moved",
									self.__processor_cursor_moved_cb)
			self.__text_id = self.__editor.textbuffer.connect("insert-text",
									self.__processor_insert_text_cb)
			self.__text_id2 = self.__editor.textbuffer.connect_after("insert-text",
									self.__processor_insert_text_after_cb)
			# Feedback to the status bar.
			from internationalization import msg0467
			self.__status_id = self.__editor.feedback.set_modal_message(msg0467, "info")
		else:
			self.__editor.textbuffer.end_user_action()
			from gobject import timeout_add
			timeout_add(7, self.__destroy)

	def __init_attributes(self, template_manager, placeholders):
		"""
		Initialize the processor's data attributes.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.

		@param template_manager: An object that manages templates.
		@type template_manager: A TemplateManager object.

		@param placeholders: A list of placeholders.
		@type placeholders: A List object.
		"""
		# Reference to the text editor.
		self.__editor = template_manager.editor
		# A manager that oversees all templates.
		self.__template_manager = template_manager
		# A list of placeholders for the template.
		self.__placeholders = placeholders
		# A tag used for unmodified placeholders.
		self.__pre_modification_highlight_tag = self.__create_pre_modification_tag()
		# A tag used for modified placeholders.
		self.__post_modification_highlight_tag = self.__create_post_modification_tag()
		# A tag used for placeholders under modification.
		self.__modification_highlight_tag = self.__create_modification_tag()
		# A tag used for special placeholders.
		self.__special_highlight_tag = self.__create_special_tag()
		# The beginning position of the template in the buffer.
		self.__start_template_position = None
		# The end position of the template in the buffer.
		self.__end_template_position = None
		# A list of placeholders special to the text holder.
		self.__special_placeholders = [
			"${time}", "${timestring}", "${timestamp}", "${date}",
			"${day}", "${month}", "${year}"
		]
		# A dictionary containing indexes as keys and placeholders as
		# values.
		self.__placeholder_dictionary = {}
		# A dictionary containing indexes as keys and duplicate
		# placeholders as mirrors.
		self.__mirror_dictionary = {}
		# A list of duplicate placeholders, or mirrors.
		self.__mirror_list = []
		# True if the template has mirrors.
		self.__has_mirrors = False
		# A value that keeps track of the cursor position relative to
		# placeholders in the buffer.
		self.__index = 0
		# An index representing the last placeholder.
		self.__last_index = None
		self.__cursor_movement_id = None
		self.__text_id = None
		self.__text_id2 = None
		self.__status_id = None
		return

########################################################################
#
#					Navigation Methods
#
########################################################################

	def next_placeholder(self):
		"""
		Jump to the next placeholder in the template.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		begin_mark, end_mark, value = self.__placeholder_dictionary[self.__index]
		if begin_mark and not end_mark and value:
			# In this branch the index is most likely on the last
			# special placeholder, "cursor." Thus exit template mode.
			self.__destroy()
			return
		# Tag the current placeholder before moving to the next one.
		begin = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
		end = self.__editor.textbuffer.get_iter_at_mark(end_mark)
		self.__editor.textbuffer.remove_tag(self.__pre_modification_highlight_tag, begin, end)
		self.__editor.textbuffer.remove_tag(self.__modification_highlight_tag, begin, end)
		self.__editor.textbuffer.apply_tag(self.__post_modification_highlight_tag, begin, end)
		# Update mirrors if any.
		if (begin_mark, end_mark) in self.__mirror_dictionary.keys():
			self.__update_mirrors()
		from cursor import move_view_to_cursor
		self.__index = self.__calculate_next_index()
		# Jump to, or select, the placeholder associated with the new
		# index.
		if self.__index:
			begin_mark, end_mark, value = self.__placeholder_dictionary[self.__index]
			if begin_mark and end_mark and value:
				# In this branch the placeholder is not "cursor"
				begin = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
				end = self.__editor.textbuffer.get_iter_at_mark(end_mark)
				self.__editor.textbuffer.select_range(begin, end)
			else:
				# In this branch the placeholder is likely "cursor"
				iterator = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
				self.__editor.textbuffer.place_cursor(iterator)
			move_view_to_cursor(self.__editor.textview)
			from internationalization import msg0465
			self.__editor.feedback.update_status_message(msg0465, "succeed")
		else:
			iterator = self.__editor.textbuffer.get_iter_at_mark(self.__end_template_position)
			self.__editor.textbuffer.place_cursor(iterator)
			move_view_to_cursor(self.__editor.textview)
			self.__destroy()
		return

	def previous_placeholder(self):
		"""
		Move to the previous placeholder if possible.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		# Tag the current placeholder before moving to the next one.
		begin_mark, end_mark, value = self.__placeholder_dictionary[self.__index]
		if begin_mark and end_mark:
			if (begin_mark, end_mark) in self.__mirror_dictionary.keys():
				self.__update_mirrors()
			begin = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
			end = self.__editor.textbuffer.get_iter_at_mark(end_mark)
			self.__editor.textbuffer.remove_tag(self.__pre_modification_highlight_tag, begin, end)
			self.__editor.textbuffer.remove_tag(self.__modification_highlight_tag, begin, end)
			self.__editor.textbuffer.apply_tag(self.__post_modification_highlight_tag, begin, end)
		# Determine the previous placeholder index.
		self.__index = self.__calculate_previous_index()
		if self.__index is None:
			return
		# Select the previous placeholder.
		begin_mark, end_mark, value = self.__placeholder_dictionary[self.__index]
		if begin_mark and end_mark and value:
			begin = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
			end = self.__editor.textbuffer.get_iter_at_mark(end_mark)
			self.__editor.textbuffer.select_range(begin, end)
		else:
			iterator = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
			self.__editor.textbuffer.place_cursor(iterator)
		from cursor import move_view_to_cursor
		move_view_to_cursor(self.__editor.textview)
		from internationalization import msg0466
		self.__editor.feedback.update_status_message(msg0466, "succeed")
		return

########################################################################
#
#					Helper Methods for Placeholders
#
########################################################################

	def __mark_placeholders_in_buffer(self):
		"""
		Mark the position of placeholders in the text editor's buffer and
		escape them.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		placeholders = []
		for placeholder in self.__placeholders:
			if not placeholder in self.__special_placeholders:
				placeholders.append(placeholder)
		start_position = self.__editor.textbuffer.get_iter_at_mark(self.__start_template_position)
		from gtk import TEXT_SEARCH_VISIBLE_ONLY
		from cursor import get_cursor_iterator
		# Search for special placeholders, replace them with their
		# determined value and tag them.
		count = 0
		mirror_list = []
		for placeholder in placeholders:
			end_position = self.__editor.textbuffer.get_iter_at_mark(self.__end_template_position)
			begin, end = start_position.forward_search(placeholder, TEXT_SEARCH_VISIBLE_ONLY, end_position)
			self.__editor.textbuffer.place_cursor(begin)
			self.__editor.textbuffer.delete(begin, end)
			cursor_position = get_cursor_iterator(self.__editor.textbuffer)
			if placeholder == "${cursor}":
				emark = self.__editor.textbuffer.create_mark(None, cursor_position, False)
				emark.set_visible(True)
				self.__placeholder_dictionary["cursor"] = (emark,)
			else:
				bmark = self.__editor.textbuffer.create_mark(None, cursor_position, True)
				emark = self.__editor.textbuffer.create_mark(None, cursor_position, False)
				self.__placeholder_dictionary[count] = (bmark, emark, True)
				escaped_placeholder = self.__escape_placeholder(placeholder)
				if escaped_placeholder:
					if self.__has_mirrors:
						if escaped_placeholder in self.__mirror_list:
							self.__mirror_dictionary[(bmark, emark)] = escaped_placeholder
							if escaped_placeholder in mirror_list:
								self.__placeholder_dictionary[count] = (bmark, emark, False)
							else:
								mirror_list.append(escaped_placeholder)
				else:
					escaped_placeholder = " "
				self.__editor.textbuffer.insert_at_cursor(escaped_placeholder)
				count += 1
			start_position = self.__editor.textbuffer.get_iter_at_mark(emark)
		return

	def __tag_placeholders_in_buffer(self):
		"""
		Tag all placeholders in the text editor's buffer with pre
		modification highlight tags.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		for position in self.__placeholder_dictionary.values():
			if len(position) > 2:
				begin = self.__editor.textbuffer.get_iter_at_mark(position[0])
				end = self.__editor.textbuffer.get_iter_at_mark(position[1])
				self.__editor.textbuffer.apply_tag(self.__pre_modification_highlight_tag, begin, end)
		return

	def __select_first_placeholder(self):
		"""
		Select the first placeholder.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		begin_mark, end_mark, value = self.__placeholder_dictionary[0]
		if begin_mark and end_mark and value:
			begin = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
			end = self.__editor.textbuffer.get_iter_at_mark(end_mark)
			self.__editor.textbuffer.select_range(begin, end)
		return

	def __expand_special_placeholders(self):
		"""
		Replace special placeholders in the text editor's buffer with
		their determined value and tag them.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		special_placeholders = []
		# Populate a list with special placeholders.
		for placeholder in self.__placeholders:
			if placeholder in self.__special_placeholders:
				special_placeholders.append(placeholder)
		start_position = self.__editor.textbuffer.get_iter_at_mark(self.__start_template_position)
		from gtk import TEXT_SEARCH_VISIBLE_ONLY
		from cursor import get_cursor_iterator
		# Search for special placeholders, replace them with their
		# determined value and tag them.
		for placeholder in special_placeholders:
			end_position = self.__editor.textbuffer.get_iter_at_mark(self.__end_template_position)
			begin, end = start_position.forward_search(placeholder, TEXT_SEARCH_VISIBLE_ONLY, end_position)
			self.__editor.textbuffer.place_cursor(begin)
			self.__editor.textbuffer.delete(begin, end)
			new_placeholder = self.__replace_special_placeholder(placeholder)
			cursor_position = get_cursor_iterator(self.__editor.textbuffer)
			emark = self.__editor.textbuffer.create_mark(None, cursor_position, False)
			self.__editor.textbuffer.insert_with_tags(cursor_position, new_placeholder,
												self.__special_highlight_tag)
			start_position = self.__editor.textbuffer.get_iter_at_mark(emark)
		return

	def __replace_special_placeholder(self, placeholder):
		"""
		Replaces a variable/'special placeholder' with it's value.

		Any variables added here need to be added
		to self.special_placeholders in self.__init_attributes

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		if placeholder == "${day}":
			from time import localtime
			thetime = localtime()
			return str(thetime[2])
		if placeholder == "${month}":
			from time import localtime
			thetime = localtime()
			return str(thetime[1])
		if placeholder == "${year}":
			from time import localtime
			thetime = localtime()
			return str(thetime[0])
		if placeholder == "${date}":
			from time import localtime
			thetime = localtime()
			return "%d:%d:%d" % (thetime[0], thetime[1], thetime[2])
		if placeholder == "${time}":
			from time import localtime
			thetime = localtime()
			return "%d:%d:%d" % (thetime[3], thetime[4], thetime[5])
		if placeholder == "${timestring}":
			from time import ctime
			return ctime()
		if placeholder == "${timestamp}":
			from time import localtime
			thetime = localtime()
			return "[%d-%d-%d] %d:%d:%d" % (thetime[0], thetime[1], thetime[2], thetime[3], thetime[4], thetime[5])

	def __escape_placeholder(self, placeholder):
		"""
		Remove foreign characters from placeholders.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.

		@param placeholder: A placeholder.
		@type placeholder: A String object.

		@return: A placeholder with removed foreign characters.
		@rtype: A String object.
		"""
		placeholder = placeholder.lstrip("$")
		placeholder = placeholder.lstrip("{")
		placeholder = placeholder.rstrip("}")
		return placeholder

	def __has_placeholders(self):
		"""
		Check if the template has normal placeholders.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		for placeholder in self.__placeholders:
			if not placeholder in self.__special_placeholders:
				return True
		return False

	def __has_special_placeholders(self):
		"""
		Check if the template has special placeholders.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		for placeholder in self.__placeholders:
			if placeholder in self.__special_placeholders:
				return True
		return False

	def __get_modified_placeholder(self):
		"""
		Return the position of a placeholder under modification.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.

		@return: The position of the placeholder being modified.
		@rtype: A List object.
		"""
		placeholders = self.__placeholder_dictionary.values()
		from cursor import get_cursor_iterator
		for placeholder in placeholders:
			if placeholder[0] and not placeholder[1] and placeholder[2]:
				# Ignore the "cursor" special placeholder.
				continue
			begin = self.__editor.textbuffer.get_iter_at_mark(placeholder[0])
			end = self.__editor.textbuffer.get_iter_at_mark(placeholder[1])
			iterator = get_cursor_iterator(self.__editor.textbuffer)
			if iterator.in_range(begin, end) or iterator.equal(begin) or iterator.equal(end):
				return placeholder
		return None

########################################################################
#
#				Method for Index Manipulations
#
########################################################################

	def __determine_last_placeholder(self):
		"""
		Set the index of the last placeholder.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		if self.__placeholder_dictionary.has_key("cursor"):
			cursor_marker = self.__placeholder_dictionary["cursor"]
			if len(self.__placeholder_dictionary) == 1:
				# In this branch cursor is the only placeholder.
				del self.__placeholder_dictionary["cursor"]
				self.__last_index = 0
				self.__placeholder_dictionary[self.__last_index] = cursor_marker[0], None, True
			elif len(self.__placeholder_dictionary) > 1:
				del self.__placeholder_dictionary["cursor"]
				self.__last_index = max(self.__placeholder_dictionary.keys()) + 1
				self.__placeholder_dictionary[self.__last_index] = cursor_marker[0], None, True
		else:
			self.__last_index = max(self.__placeholder_dictionary.keys())
			if self.__has_mirrors:
				navigating_index_list = []
				for key, value in self.__placeholder_dictionary.items():
					if value[2]:
						navigating_index_list.append(key)
				if navigating_index_list:
					self.__last_index = max(navigating_index_list)
		return

	def __calculate_next_index(self):
		"""
		Determine the next placeholder index to jump to.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.

		@return: The next placeholder index to jump to.
		@rtype: An Integer object.
		"""
		indexes = self.__placeholder_dictionary.keys()
		indexes.sort()
		for index in indexes:
			if index > self.__index:
				if self.__placeholder_dictionary[index][2]:
					return index
		return None

	def __calculate_previous_index(self):
		"""
		Determine the previous placeholder index to jump to.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.

		@return: The next placeholder index to jump to.
		@rtype: An Integer object.
		"""
		indexes = self.__placeholder_dictionary.keys()
		indexes.sort()
		indexes.reverse()
		for index in indexes:
			if not self.__index:
				self.__index = max(indexes)
				if self.__placeholder_dictionary[self.__index][2]:
					return self.__index
			if index < self.__index:
				if self.__placeholder_dictionary[index][2]:
					return index
		return None

	def __update_index(self, begin_mark, end_mark):
		"""
		Update the current placeholder index.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		for value in range(self.__last_index + 1):
			marks = self.__placeholder_dictionary[value]
			if begin_mark in marks and end_mark in marks:
				self.__index = value
				break
		return

########################################################################
#
#							Mirrors
#
########################################################################

	def __generate_mirror_list(self):
		"""
		Generate a list of duplicate placeholders, or mirrors.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		for placeholder in self.__placeholders:
			# Ignore special placeholders.
			if placeholder in self.__special_placeholders:
				continue
			# Check for duplicate placeholders.
			if self.__placeholders.count(placeholder) > 1:
				self.__has_mirrors = True
				placeholder = self.__escape_placeholder(placeholder)
				if not placeholder in self.__mirror_list:
					self.__mirror_list.append(placeholder)
		return

	def __update_mirrors(self):
		"""
		Update other mirrors when one mirror is edited.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		begin_mark, end_mark, value = self.__placeholder_dictionary[self.__index]
		mirror_list = []
		for key, value in self.__mirror_dictionary.items():
			if value == self.__mirror_dictionary[(begin_mark, end_mark)]:
				if key != (begin_mark, end_mark):
					mirror_list.append(key)
		if mirror_list:
			self.__editor.textbuffer.handler_block(self.__text_id2)
			for placeholder in mirror_list:
				begin_position = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
				end_position = self.__editor.textbuffer.get_iter_at_mark(end_mark)
				begin = self.__editor.textbuffer.get_iter_at_mark(placeholder[0])
				end = self.__editor.textbuffer.get_iter_at_mark(placeholder[1])
				self.__editor.textbuffer.delete(begin, end)
				begin_position = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
				end_position = self.__editor.textbuffer.get_iter_at_mark(end_mark)
				begin = self.__editor.textbuffer.get_iter_at_mark(placeholder[0])
				end = self.__editor.textbuffer.get_iter_at_mark(placeholder[1])
				self.__editor.textbuffer.insert_range(begin, begin_position, end_position)
				begin_position = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
				end_position = self.__editor.textbuffer.get_iter_at_mark(end_mark)
				begin = self.__editor.textbuffer.get_iter_at_mark(placeholder[0])
				end = self.__editor.textbuffer.get_iter_at_mark(placeholder[1])
				self.__editor.textbuffer.remove_tag(self.__pre_modification_highlight_tag, begin, end)
				self.__editor.textbuffer.remove_tag(self.__modification_highlight_tag, begin, end)
				self.__editor.textbuffer.apply_tag(self.__post_modification_highlight_tag, begin, end)
			self.__editor.textbuffer.handler_unblock(self.__text_id2)
		return

########################################################################
#
#						Tags for Placeholders
#
########################################################################

	def __create_pre_modification_tag(self):
		"""
		Create a tag for unmodified placeholders.

		@param self: Reference to the TemplateProcessor.
		@type self: A TemplateProcessor object.

		@return: A highlight tag.
		@rtype: A gtk.TextTag object.
		"""
		highlight_tag = self.__editor.textbuffer.create_tag()
		highlight_tag.set_property("background", "yellow")
		highlight_tag.set_property("foreground", "blue")
		from pango import WEIGHT_HEAVY
		highlight_tag.set_property("weight", WEIGHT_HEAVY)
		return highlight_tag

	def __create_post_modification_tag(self):
		"""
		Create a tag for modified placeholders.

		@param self: Reference to the TemplateProcessor.
		@type self: A TemplateProcessor object.

		@return: A highlight tag.
		@rtype: A gtk.TextTag object.
		"""
		highlight_tag = self.__editor.textbuffer.create_tag()
		highlight_tag.set_property("foreground", "blue")
		from pango import WEIGHT_HEAVY, STYLE_ITALIC
		highlight_tag.set_property("weight", WEIGHT_HEAVY)
		highlight_tag.set_property("style", STYLE_ITALIC)
		return highlight_tag

	def __create_modification_tag(self):
		"""
		Create a tag for placeholders being modified.

		@param self: Reference to the TemplateProcessor.
		@type self: A TemplateProcessor object.

		@return: A highlight tag.
		@rtype: A gtk.TextTag object.
		"""
		highlight_tag = self.__editor.textbuffer.create_tag()
		highlight_tag.set_property("background", "#ADD8E6")
		highlight_tag.set_property("foreground", "#CB5A30")
		from pango import WEIGHT_HEAVY
		highlight_tag.set_property("weight", WEIGHT_HEAVY)
		return highlight_tag

	def __create_special_tag(self):
		"""
		Create a tag for special placeholders.

		@param self: Reference to the TemplateProcessor.
		@type self: A TemplateProcessor object.

		@return: A highlight tag.
		@rtype: A gtk.TextTag object.
		"""
		highlight_tag = self.__editor.textbuffer.create_tag()
		highlight_tag.set_property("foreground", "pink")
		from pango import WEIGHT_HEAVY
		highlight_tag.set_property("weight", WEIGHT_HEAVY)
		return highlight_tag

########################################################################
#
#					Event and Signal Handlers
#
########################################################################

	def __processor_cursor_moved_cb(self, editor):
		"""
		Handles callback when the "cursor-moved" signal is emitted.

		@param self: Reference to the class instance.
		@type self: A class object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		# Destroy this template object if the cursor moves outside the
		# template region.
		from cursor import get_cursor_iterator
		cursor_position = get_cursor_iterator(self.__editor.textbuffer)
		begin_position = self.__editor.textbuffer.get_iter_at_mark(self.__start_template_position)
		end_position = self.__editor.textbuffer.get_iter_at_mark(self.__end_template_position)
		if not cursor_position.in_range(begin_position, end_position):
			if cursor_position.equal(begin_position) and cursor_position.equal(end_position):
				self.__destroy()
			elif cursor_position.equal(end_position):
				pass
			else:
				self.__destroy()
		return

	def __processor_insert_text_cb(self, textbuffer, iterator, text, length):
		"""
		Handles callback when the "insert-text" signal is emitted.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.

		@param iterator: The position to insert text.
		@type iterator: A gtk.TextIter object.

		@param text: Text to be entered in the text editor's buffer.
		@type text: A String object.

		@param length: The length of text to be entered in the buffer.
		@type length: An Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.__index != self.__last_index:
			# Ignore every other index other than the last one.
			return False
		begin_mark, end_mark, value = self.__placeholder_dictionary[self.__index]
		if (begin_mark, end_mark) in self.__mirror_dictionary.keys():
			# Ignore mirrors.
			return False
		# Exit template mode when the user types over the last placeholder.
		if begin_mark and end_mark and value:
			begin = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
			end = self.__editor.textbuffer.get_iter_at_mark(end_mark)
			if iterator.in_range(begin, end) or iterator.equal(begin) or iterator.equal(end):
				self.__destroy()
		else:
			iterator = self.__editor.textbuffer.get_iter_at_mark(begin_mark)
			if iterator.equal(iterator):
				self.__destroy()
		return False

	def __processor_insert_text_after_cb(self, textbuffer, iterator, text, length):
		"""
		Handles callback when the "insert-text" signal is emitted.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A ScribesTextBuffer object.

		@param iterator: The position to insert text.
		@type iterator: A gtk.TextIter object.

		@param text: Text to be entered in the text editor's buffer.
		@type text: A String object.

		@param length: The length of text to be entered in the buffer.
		@type length: An Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		# Check if a placeholder is being modified.
		placeholder = self.__get_modified_placeholder()
		if placeholder:
			# Tag placeholder that is being modified.
			begin_position = self.__editor.textbuffer.get_iter_at_mark(placeholder[0])
			end_position = self.__editor.textbuffer.get_iter_at_mark(placeholder[1])
			self.__editor.textbuffer.apply_tag(self.__modification_highlight_tag,
											begin_position, end_position)
			self.__update_index(placeholder[0], placeholder[1])
		return False

########################################################################
#
#						Destroy Method
#
########################################################################

	def __destroy(self):
		"""
		Destroy the template object, remove all marks and tags.

		@param self: Reference to the TemplateProcessor instance.
		@type self: A TemplateProcessor object.
		"""
		# Disconnect all signals associated with this template object.
		if self.__cursor_movement_id:
			self.__editor.disconnect(self.__cursor_movement_id)
		if self.__text_id:
			self.__editor.textbuffer.disconnect(self.__text_id)
		if self.__text_id2:
			self.__editor.textbuffer.disconnect(self.__text_id2)
		# Remove this template object from the template manager.
		try:
			self.__template_manager.template_list.remove(self)
		except ValueError:
			pass
		# Remove all tags associated with this template object.
		begin_position = self.__editor.textbuffer.get_iter_at_mark(self.__start_template_position)
		end_position = self.__editor.textbuffer.get_iter_at_mark(self.__end_template_position)
		self.__editor.textbuffer.remove_tag(self.__pre_modification_highlight_tag,
											begin_position, end_position)
		self.__editor.textbuffer.remove_tag(self.__post_modification_highlight_tag,
											begin_position, end_position)
		self.__editor.textbuffer.remove_tag(self.__modification_highlight_tag,
											begin_position, end_position)
		self.__editor.textbuffer.remove_tag(self.__special_highlight_tag,
											begin_position, end_position)
		# Remove all marks associated with this template object.
		if self.__start_template_position.get_deleted() is False:
			self.__editor.textbuffer.delete_mark(self.__start_template_position)
		if self.__end_template_position.get_deleted() is False:
			self.__editor.textbuffer.delete_mark(self.__end_template_position)
		if self.__placeholder_dictionary:
			for placeholders in self.__placeholder_dictionary.values():
				for marker in placeholders:
					if marker in [False, True, None]:
						continue
					if marker.get_deleted() is False:
						self.__editor.textbuffer.delete_mark(marker)
		if self.__status_id:
			self.__editor.feedback.unset_modal_message(self.__status_id, False)
		from internationalization import msg0464
		self.__editor.feedback.update_status_message(msg0464, "succeed")
		# Destroy this template object and call the garbage collector.
		self = None
		del self
		from gc import collect
		collect()
		return False
