# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that inserts the templates into the text
editor's buffer.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_PYOBJECT

class TemplateTrigger(GObject):
	"""
	This class creates an object that monitors and activates template
	triggers.
	"""
	__gsignals__ = {
		"found-template-trigger": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT,)),
	}

	def __init__(self, editor):
		"""
		Initialize the template trigger monitor.

		@param self: Reference to the TemplateTrigger instance.
		@type self: A TemplateTrigger object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		GObject.__init__(self)
		self.__init_attributes(editor)
		self.editor.textview.connect("key-press-event", self.__trigger_key_press_event_cb)
		from template import TemplateManager
		from thread import start_new_thread
		start_new_thread(TemplateManager, (self,))

	def __init_attributes(self, editor):
		"""
		Initialize the template triggers' data attributes.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		return

	def __trigger_key_press_event_cb(self, textview, event):
		"""
		Handle callback when the "key-press-event" signal is emitted.

		@param self: Reference to the TemplateTrigger instance.
		@type self: A TemplateTrigger object.

		@param textview: The text editor's view.
		@type textview: A ScribesTextView object.

		@param event: An event that occurs when the keyboard keys are pressed.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		from gtk import keysyms
		if event.keyval == keysyms.Tab:
			result = self.__run_template_trigger_check()
			if result:
				return True
		return False

	def __run_template_trigger_check(self):
		"""
		Check the buffer for template triggers.

		@param self: Reference to the TemplateTrigger instance.
		@type self: A TemplateTrigger object.

		@return: True if a template trigger is found.
		@rtype: A Boolean object.
		"""
		from cursor import word_to_cursor, get_template_trigger
		word = word_to_cursor(self.editor.textbuffer)
		if word:
			if self.editor.uri is None:
				template_trigger = "General" + word
				if template_trigger in self.editor.textbuffer.general_template_dictionary.keys():
					self.emit("found-template-trigger", template_trigger)
					return True
				return False
			from utils import get_language
			language = get_language(str(self.editor.uri))
			if language is None:
				template_trigger = "General" + word
				if template_trigger in self.editor.textbuffer.general_template_dictionary.keys():
					self.emit("found-template-trigger", template_trigger)
					return True
				return False
			language_id = language.get_id()
			template_trigger = language_id + word
			if template_trigger in self.editor.textbuffer.language_template_dictionary.keys():
				self.emit("found-template-trigger", template_trigger)
				return True
			template_trigger = "General" + word
			if template_trigger in self.editor.textbuffer.general_template_dictionary.keys():
				self.emit("found-template-trigger", template_trigger)
				return True
		return False
