# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module implements a class that is responsible for creating the text
editor's buffer.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtksourceview import SourceBuffer

class ScribesTextBuffer(SourceBuffer):
	"""
	This class creates the buffer object for text editor instances. It defines
	the state, properties and behavior of buffer objects. The class inherits
	from gtksourceview.SourceBuffer. See the PyGTK, or GTK+, manual for more
	details on gtksourceview.SourceBuffer class.
	"""

	def __init__(self, editor):
		"""
		Initialize the buffer object for the text editor.

		@param self: Reference to a ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from utils import response
		response()
		SourceBuffer.__init__(self)
		self.init_attributes(editor)
		self.__set_properties()
		from save import FileSaver
		from thread import start_new_thread
		start_new_thread(FileSaver, (self.editor,))
		self.connect("changed", self.__buffer_changed_cb)
		editor.connect("loading-document", self.__buffer_loading_document_cb)
		editor.connect("loaded-document", self.__buffer_loaded_document_cb)
		editor.connect("load-error", self.__buffer_load_error_cb)
		editor.connect("saved-document", self.__buffer_saved_document_cb)
		editor.connect("enable-readonly", self.__buffer_enable_readonly_cb)
		editor.connect("disable-readonly", self.__buffer_disable_readonly_cb)
		editor.connect("close-document", self.__buffer_close_document_cb)
		editor.connect("renamed-document", self.__buffer_renamed_document_cb)

	def init_attributes(self, editor):
		"""
		Initialize the attributes of the text editor's buffer.

		This function contains some attributes not available in the
		gtksourceview.SourceBuffer class. The attributes provided solely for
		purpose of the text editor.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor = editor
		self.editor.textbuffer = self
		# Register a unique number with the editor's termination queue
		self.termination_id = self.editor.register_termination_id()
		self.undoable_action = False
		self.word_completion_dictionary = {}
		self.general_template_dictionary = {}
		self.language_template_dictionary = {}
		# It does nothing. Only used for testing.
		self.count = 0
		return

	def __set_properties(self):
		"""
		Set the editor's buffer properties.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.
		"""
		self.set_check_brackets(False)
		self.set_max_undo_levels(0)
		self.set_text("")
		self.set_modified(False)
		return

################################################################################
#
#							Signal Handlers
#
################################################################################

	def __buffer_changed_cb(self, textbuffer):
		"""
		Handles callback when text is inserted into the text editor's buffer.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A gtk.SourceBuffer object.

		@return: True to prevent propagation of signals to parent widgets.
		@rtype: A Boolean object.
		"""
		if self.editor.is_loading_document:
			return False
		self.editor.contains_document = True
		# True if the file does not need to be saved.
		self.editor.saved_document = False
		return False

	def __buffer_modified_changed_cb(self, textbuffer):
		"""
		Handles callback when the text editor's buffer's state has changed.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param textbuffer: The text editor's buffer.
		@type textbuffer: A gtk.SourceBuffer object.

		@return: True to prevent propagation of signals to parent widgets.
		@rtype: A Boolean object.
		"""
		if self.editor.is_loading_document:
			return False
		if self.editor.saved_document or self.editor.uri is None:
			return False
		return False

	def __buffer_loading_document_cb(self, editor):
		"""
		Handles callback when the text editor is in the process of loading a
		document into the text editor's buffer.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: An instance of the text editor's.
		@type editor: An Editor object.
		"""
		self.begin_not_undoable_action()
		self.undoable_action = True
		from gobject import timeout_add
		timeout_add(20, self.__activate_sytnax_colors)
		return

	def __buffer_loaded_document_cb(self, editor):
		"""
		Handles callback when the text editor has finished loading documents
		into the text editor's buffer.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		self.end_not_undoable_action()
		self.undoable_action = False
		from gobject import timeout_add
		timeout_add(20, self.__set_cursor_positon)
		self.set_modified(False)
		return

	def __buffer_saved_document_cb(self, editor):
		"""
		Handles callback when the text editor's buffer has finished saving the
		contents of the text editor's buffer.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: An instance of the text editor's.
		@type editor: An Editor object.
		"""
		self.set_modified(False)
		return

	def __buffer_enable_readonly_cb(self, editor):
		"""
		Handles callback when the text editor is switched to readonly mode.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		self.set_check_brackets(False)
		return

	def __buffer_disable_readonly_cb(self, editor):
		"""
		Handles callback when the text editor is switched from readonly to
		readwrite mode.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		self.set_check_brackets(True)
		return

	def __buffer_load_error_cb(self, editor):
		"""
		Handles callback when the text editor fails to load a document.

		This function resets the buffer to a usable state.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: An instance of the text editor
		@type editor: An Editor object.
		"""
		if self.undoable_action:
			self.end_not_undoable_action()
		self.__set_properties()
		return

	def __buffer_close_document_cb(self, editor):
		"""
		Handles callback when the "quit" signal is emitted.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from gobject import timeout_add, PRIORITY_HIGH
		timeout_add(10, self.__update_cursor_metadata, str(self.editor.uri))
		return

	def __buffer_renamed_document_cb(self, editor):
		"""
		Handles callback when the name of the document is renamed.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from gobject import timeout_add
		timeout_add(7, self.__activate_sytnax_colors)
		self.set_modified(False)
		self.set_check_brackets(True)
		return

################################################################################
#
#						Helper Methods
#
################################################################################

	def __activate_sytnax_colors(self):
		"""
		Activate syntax highlight colors for the text editor's buffer.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		if self.editor.uri is None:
			# Call this function until the URI of the document has been
			# determined.
			return True
		# Get the source language of the URI, or document.
		from utils import get_language
		language = get_language(str(self.editor.uri))
		# Activate syntax highlight for the language.
		from gobject import timeout_add
		from syntax import activate_syntax_highlight
		timeout_add(10, activate_syntax_highlight, self, language)
		return False

	def __update_cursor_metadata(self, uri):
		"""
		Update the cursor database with information about the cursor position in
		the text editor's buffer.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param uri: A universal resource identifier representing, or pointing
			to, a text document.
		@type uri: A String object.
		"""
		if self.editor.uri:
			from cursor_metadata import update_cursor_position_in_database
			from cursor import get_cursor_line, get_cursor_index
			cursor_line = get_cursor_line(self)
			cursor_index = get_cursor_index(self)
			cursor_position = cursor_line, cursor_index
			update_cursor_position_in_database(uri, cursor_position)
		self.editor.unregister_termination_id(self.termination_id)
		return False

	def __set_cursor_positon(self):
		"""
		Determine where to place the cursor when a file is loaded.

		The editor stores the last cursor position in a database so that
		when a file is loaded the cursor is placed at the last position in the
		file.

		However, it is possible that another software program alters the state
		of the file. Thus making the information in the editor's database
		obsolete. In such a case, the editor tries to determine where to place
		the cursor.

		If the changes are too drastic, the editor places the cursor at the
		begining of the file. If the changes are minor, the editor places the
		cursor at an approximate position close the the last position the cursor
		was last seen.

		@param self: Reference to the Loader object.
		@type self: A Loader object.
		"""
		try:
			from cursor_metadata import get_cursor_position_from_database
			position = get_cursor_position_from_database(str(self.editor.uri))
			cursor_line, cursor_index = position[0] + 1, position[1]
		except TypeError:
			cursor_line, cursor_index = 1, 0
		start_iterator = self.get_start_iter()
		number_of_lines = self.get_line_count()
		if cursor_line > number_of_lines:
			self.place_cursor(start_iterator)
			return
		iterator = self.get_iter_at_line(cursor_line - 1)
		line_index = iterator.get_bytes_in_line()
		if cursor_index > line_index:
			iterator.set_line_index(line_index)
		else:
			iterator.set_line_index(cursor_index)
		self.place_cursor(iterator)
		from cursor import move_view_to_cursor
		move_view_to_cursor(self.editor.textview)
		return False
