# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module implements a class that creates a container for the text editor's
buffer.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtksourceview import SourceView

class ScribesTextView(SourceView):
	"""
	This class creates the container object that holds the text editor's buffer.
	It defines the state, properties and behavior of the container object,
	popular called the text editor's view. The view also defines the look and
	feel of the buffer. The color of text in the buffer, the size of fonts, the
	width of tabs are all defined by the view. As such the view is the only
	user customizable part of the text editor. The class inherits from
	gtksourceview.SourceView. See the PyGTK, or GTK+, manual for details on the
	gtksourceview.SourceView class.
	"""

	def __init__(self, editor):
		"""
		Initialize the text editor's buffer container.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: Reference to the text editor instance.
		@type editor: An Editor object.
		"""
		from utils import response
		response()
		SourceView.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		# Monitor signals and events.
		self.connect("map-event", self.__map_event_cb)
		self.connect("focus-in-event", self.__view_focus_in_event_cb)
		self.connect("drag-motion", self.__drag_motion_cb)
		self.connect("drag-drop", self.__drag_drop_cb)
		self.connect("drag-data-received", self.__drag_data_received_cb)
#		self.connect("drag-data-get", self.__drag_data_get_cb)
		self.connect("populate-popup", self.__view_populate_popup_cb)
		self.connect("copy-clipboard", self.__view_copy_clipboard_cb)
		self.connect("cut-clipboard", self.__view_cut_clipboard_cb)
		self.connect("paste-clipboard", self.__view_paste_clipboard_cb)
		self.connect_after("paste-clipboard", self.__view_paste_clipboard_after_cb)
		self.editor.connect("loading-document", self.__view_loading_document_cb)
		self.editor.connect("loaded-document", self.__view_loaded_document_cb)
		self.editor.connect("load-error", self.__view_load_error_cb)
		self.editor.connect("enable-readonly", self.__view_enable_readonly_cb)
		self.editor.connect("disable-readonly", self.__view_disable_readonly_cb)
		self.editor.connect("renamed-document", self.__view_renamed_document_cb)
		self.editor.connect("hide-dialog", self.__view_hide_dialog_cb)
		self.editor.connect("saved-document", self.__view_saved_document_cb)
		self.editor.connect("enable-fullscreen", self.__view_enable_fullscreen_cb)
		self.editor.connect("disable-fullscreen", self.__view_disable_fullscreen_cb)
		self.editor.connect("show-bar", self.__view_show_bar_cb)
		self.editor.connect("hide-bar", self.__view_hide_bar_cb)
		self.connect("button-press-event", self.__view_button_press_event_cb)

		# GConf notification monitors.
		self.client.notify_add("/apps/scribes/font", self.__font_changed_cb)
		self.client.notify_add("/apps/scribes/tab", self.__tab_width_cb)
		self.client.notify_add("/apps/scribes/text_wrapping", self.__text_wrapping_cb)
		self.client.notify_add("/apps/scribes/margin", self.__show_margin_cb)
		self.client.notify_add("/apps/scribes/margin_position", self.__margin_position_cb)
		self.client.notify_add("/apps/scribes/spell_check", self.__spell_check_cb)
		self.client.notify_add("/apps/scribes/use_theme_colors", self.__themes_cb)
		self.client.notify_add("/apps/scribes/fgcolor", self.__foreground_cb)
		self.client.notify_add("/apps/scribes/bgcolor", self.__background_cb)
		self.client.notify_add("/apps/scribes/use_tabs", self.__use_tabs_cb)
		self.client.notify_add("/apps/scribes/SyntaxHighlight", self.__syntax_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the buffer container's attributes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		# Initialize gconf, the gnome configuration system
		from gconf import client_get_default, CLIENT_PRELOAD_ONELEVEL
		self.client = client_get_default()
		self.client.add_dir("/apps/scribes", CLIENT_PRELOAD_ONELEVEL)
		self.editor = editor
		self.editor.textview = self
		self.is_first_time = True
		self.is_pasting = False
		self.spell_checker = None
		self.bar_is_visible = False
		self.file_size = 0
		self.bar = None
		from utils import response
		self.response = response
		return

	def __set_properties(self):
		"""
		Set default textview properties.

		Much of the properties of the view can be defined by users. So the
		properties of the view are stored in GNOME's configuration database,
		GConf. During the text editor's initialization process,

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: Reference to the text editor instance.
		@type editor: An Editor object.
		"""
		# Drag and drop setup.
		targets = [("text/uri-list", 0, 80),
				("GTK_TEXT_BUFFER_CONTENTS", 0, 82), ("UTF8_STRING", 0, 83),
					("COMPOUND_TEXT", 0, 84), ("text/plain", 0, 81),]
		from gtk import DEST_DEFAULT_ALL
		from gtk.gdk import ACTION_COPY
		self.drag_dest_set(DEST_DEFAULT_ALL, targets, ACTION_COPY)
		self.set_property("buffer", self.editor.textbuffer)
		from gtk import RESIZE_PARENT
		self.set_resize_mode(RESIZE_PARENT)
		self.set_show_line_numbers(True)
		self.set_auto_indent(True)
		spell_check = self.client.get_bool("/apps/scribes/spell_check")
		if spell_check:
			try:
				from gobject import GError
				from gtkspell import Spell
				from locale import getdefaultlocale
				self.spell_checker = Spell(self, getdefaultlocale()[0])
			except GError:
				pass
		margin_position = self.client.get_int("/apps/scribes/margin_position")
		if margin_position is None:
			margin_position = 72
		self.set_margin(margin_position)
		show_margin = self.client.get_bool("/apps/scribes/margin")
		if show_margin is None:
			show_margin = False
		self.set_show_margin(show_margin)
		tab_width = self.client.get_int("/apps/scribes/tab")
		if tab_width is None:
			tab_width = 4
		self.set_tabs_width(tab_width)
		use_tabs = self.client.get_bool("/apps/scribes/use_tabs")
		if use_tabs is None:
			use_tabs = True
		self.set_insert_spaces_instead_of_tabs(not use_tabs)
		self.set_highlight_current_line(True)
		gconf_font = self.client.get_string("/apps/scribes/font")
		if gconf_font is None:
			gconf_font = "Monospace 12"
		from pango import FontDescription
		font = FontDescription(gconf_font)
		self.modify_font(font)
		wrap_mode_bool = self.client.get_bool("/apps/scribes/text_wrapping")
		use_theme_colors = self.client.get_bool("/apps/scribes/use_theme_colors")
		if use_theme_colors is None:
			use_theme_colors = True
		from gtk import WRAP_WORD, WRAP_NONE
		if wrap_mode_bool:
			self.set_wrap_mode(WRAP_WORD)
		else:
			self.set_wrap_mode(WRAP_NONE)
		if use_theme_colors is False:
			# Use foreground and background colors specified by the user stored
			# in GConf, the GNOME configuration database.
			from gconf import VALUE_STRING
			fgcolor = self.client.get_string("/apps/scribes/fgcolor")
			bgcolor = self.client.get_string("/apps/scribes/bgcolor")
			from gtk.gdk import color_parse
			if fgcolor is None:
				fgcolor = "#000000"
			if bgcolor is None:
				bgcolor = "#ffffff"
			foreground_color = color_parse(fgcolor)
			background_color = color_parse(bgcolor)
			from gtk import STATE_NORMAL
			self.modify_base(STATE_NORMAL, background_color)
			self.modify_text(STATE_NORMAL, foreground_color)
		return

################################################################################
#
#						Signal and Event Handlers
#
################################################################################

	def __map_event_cb(self, widget, event):
		"""
		Handles callback when the textview widget is drawn to screen.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param widget: Reference to the text editor's buffer container.
		@type widget: A gtk.SourceView object.

		@param event: An event that occurs when the widgets are displayed on
			screen.
		@type event: A gtk.Event object.
		"""
		self.__refresh_view()
		if self.editor.is_loading_document:
			from cursor import show_busy_textview_cursor
			show_busy_textview_cursor(self)
		return False

	def __drag_motion_cb(self, textview, context, x, y, time):
		"""
		Handles callback when a drag operation begins.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param textview: The text editor's view, or buffer container.
		@type textview: A gtk.SourceView object.

		@param context: The type of information being dragged.
		@type context: A gtk.Context object.

		@param x: The xcordinate of the cursor.
		@type x: An Integer object.

		@param y: The ycordinate of the cursor.
		@type y: An Integer object.

		@param time: The time the drag operation began.
		@type time: An Integer object.

		@return: False to propagate signals to parent widgets, True otherwise.
		@rtype: A Boolean object.
		"""
		if "text/uri-list" in context.targets:
			return True
		return False

	def __drag_drop_cb(self, textview, context, x, y, time):
		"""
		Handles callback when a drop operation begins.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param textview: The text editor's view, or buffer container.
		@type textview: A gtk.SourceView object.

		@param context: The type of information being dragged.
		@type context: A gtk.Context object.

		@param x: The xcordinate of the cursor.
		@type x: An Integer object.

		@param y: The ycordinate of the cursor.
		@type y: An Integer object.

		@param time: The time the drag operation began.
		@type time: An Integer object.

		@return: False to propagate signals to parent widgets, True otherwise.
		@rtype: A Boolean object.
		"""
		if "text/uri-list" in context.targets:
			return True
		return False

	def __drag_data_received_cb(self, textview, context, xcord, ycord,
								selection_data, info, timestamp):
		"""
		Handles callback when a drop operation finishes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param textview: The text editor's view, or buffer container.
		@type textview: A gtk.SourceView object.

		@param context: The type of information being dragged.
		@type context: A gtk.Context object.

		@param xcord: The xcordinate of the cursor.
		@type xcord: An Integer object.

		@param ycord: The ycordinate of the cursor.
		@type ycord: An Integer object.

		@param selection_data: A data selection
		@type selection_data: A gtk.Selection object.

		@param info: ?
		@type info: ? ? object.

		@param timestamp: The time the drag operation began.
		@type timestamp: An Integer object.

		@return: False to propagate signals to parent widgets, True otherwise.
		@rtype: A Boolean object.
		"""
		if not "text/uri-list" in context.targets:
			return False
		if info == 80:
			# Load file
			uri_list = list(selection_data.get_uris())
			if self.editor.uri:
				if str(self.editor.uri) in [uri_list]:
					uri_list.remove(str(self.editor.uri))
					if not uri_list:
						self.editor.window.hide()
						self.editor.window.present()
						context.finish(True, False, timestamp)
						return True
			self.editor.fileloader.load_uris(uri_list)
			context.finish(True, False, timestamp)
		return True

	def __drag_data_get_cb(self, textview, context, data, info, time):
		"""
		Handles callback when the "drag-data-get" signal is emitted.

		@param self: Reference to the TemplateEditorDescriptionView instance.
		@type self: A TemplateEditorDescriptionView object.

		@param treeview: Reference to the TemplateEditorDescriptionView
		@type treeview: A TemplateEditorDescriptionView object.

		@param context: An object representing context data.
		@type context: A gtk.DragContext object.

		@param data: An object representing selection data.
		@type data: A gtk.SelectionData object.

		@param info: A unique identification number for the text editor.
		@type info: An Integer object.

		@param time: The time of the drag and drop operation.
		@type time: An Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		selection = self.get_buffer().get_selection_bounds()
		if selection:
			string = self.get_buffer().get_text(selection[0], selection[1])
			data.set(data.target, 8, string)
		return False

	def __view_loading_document_cb(self, editor):
		"""
		Handles callback when the text editor is loading a document into the
		text editor's buffer.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		self.set_property("editable", False)
		self.set_property("highlight-current-line", False)
		self.set_property("cursor-visible", False)
		self.set_property("sensitive", False)
		self.__refresh_view()
		return

	def __view_loaded_document_cb(self, editor):
		"""
		Handles callback when the text editor has finished loading a document
		into the text editor's buffer.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		self.set_property("sensitive", True)
		self.set_property("highlight-current-line", True)
		self.set_property("cursor-visible", True)
		self.set_property("editable", True)
		from cursor import show_textview_cursor
		show_textview_cursor(self)
		self.__refresh_view()
		if self.editor.uri:
			from utils import get_file_size
			self.file_size = get_file_size(str(self.editor.uri))
		return

	def __view_load_error_cb(self, editor):
		"""
		Handles callback when the text editor has finished loading a document
		into the text editor's buffer.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		self.set_property("sensitive", True)
		self.set_property("cursor-visible", True)
		self.set_property("highlight-current-line", True)
		self.set_property("editable", True)
		from cursor import show_textview_cursor, move_view_to_cursor
		show_textview_cursor(self)
		move_view_to_cursor(self)
		self.__refresh_view()
		return

	def __view_enable_readonly_cb(self, editor):
		"""
		Handles callback when the text editor is switched to readonly mode.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		self.set_property("editable", False)
		self.set_property("highlight-current-line", False)
		self.set_property("cursor-visible", False)
		self.set_property("show-line-numbers", False)
		return

	def __view_disable_readonly_cb(self, editor):
		"""
		Handles callback when the text editor is switched from readonly to
		readwrite mode.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		self.set_property("editable", True)
		self.set_property("highlight-current-line", True)
		self.set_property("cursor-visible", True)
		self.set_property("show-line-numbers", True)
		self.__refresh_view()
		return

	def __view_renamed_document_cb(self, editor):
		"""
		Handles callback when the name of the document is renamed.

		@param self: Reference to the ScribesTextBuffer instance.
		@type self: A ScribesTextBuffer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.set_property("cursor-visible", True)
		self.set_property("editable", True)
		self.set_property("highlight-current-line", True)
		self.set_property("cursor-visible", True)
		self.set_property("show-line-numbers", True)
		self.__refresh_view()
		if self.editor.uri:
			from utils import get_file_size
			self.file_size = get_file_size(str(self.editor.uri))
		return

	def __view_hide_dialog_cb(self, editor, dialog):
		"""
		Handles callback when the "hide-dialog" signal is emitted.

		This function returns focus back to the text editor's view, or buffer
		container, when dialogs connected to the "hide-dialog" signal are
		closed.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: Reference to the text editor
		@type editor: An Editor object.
		"""
		self.__refresh_view()
		return

	def __view_focus_in_event_cb(self, textview, event):
		"""
		Handles callback when the text editor's windows "focus-in-event" signal
		is emitted.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param window: The text editor's buffer container, or view.
		@type window: A ScribesTextView object.

		@param event: An event triggered when the text editor's window is focused.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.bar_is_visible is False:
			self.__refresh_view()
		return False

	def __view_saved_document_cb(self, editor):
		#self.__refresh()
		from utils import response
		response()
		return

	def __view_enable_fullscreen_cb(self, editor):
		"""
		Handles callback when the "enable-fullscreen" signal is emitted.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__refresh_view()
		return

	def __view_disable_fullscreen_cb(self, editor):
		"""
		Handles callback when the "disable-fullscreen" signal is emitted.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.__refresh_view()
		return

	def __view_show_bar_cb(self, editor, bar):
		"""
		Handles callback when the "hide-bar" signal is emitted.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: The text editor's bar.
		@type bar: A ScribesBar object.
		"""
		self.bar = bar
		self.bar_is_visible = True
		self.set_property("editable", False)
		self.set_property("cursor-visible", False)
		return

	def __view_hide_bar_cb(self, editor, bar):
		"""
		Handles callback when the "hide-bar" signal is emitted.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: The text editor's bar.
		@type bar: A ScribesBar object.
		"""
		self.bar_is_visible = False
		self.set_property("editable", True)
		self.set_property("cursor-visible", True)
		self.__refresh_view()
		return

	def __view_button_press_event_cb(self, textview, event):
		"""
		Handles callback when the "button-press-event" signal is emitted.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param textview: The text editor's view.
		@type textview: A ScribesTextView object.

		@param event: An event that occurs when the mouse buttons are pressed.
		@type event: A gtk.Event object.

		@param bar: The text editor's bar.
		@type bar: A ScribesBar object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		if self.bar_is_visible:
			self.bar.hide_bar()
			return True
		return False

	def __view_populate_popup_cb(self, textview, menu):
		"""
		Handles callback when the "populate-popup" signal is emitted.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param textview: The text editor's buffer container.
		@type textview: A ScribesTextView object.

		@param menu: The text editor's popup menu.
		@type menu: A gtk.Menu object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		from gtk import MenuItem, SeparatorMenuItem
		menu.insert(SeparatorMenuItem(), 4)
		from popupindent import ScribesIndentPopupMenuItem
		indentation_menuitem = ScribesIndentPopupMenuItem(self)
		menu.insert(indentation_menuitem, 5)
		from popupspaces import ScribesSpacesPopupMenuItem
		space_menuitem = ScribesSpacesPopupMenuItem(self)
		menu.insert(space_menuitem, 6)
		from popupselection import ScribesSelectionPopupMenuItem
		selection_menuitem = ScribesSelectionPopupMenuItem(self)
		menu.insert(selection_menuitem, 7)
		from popuplines import ScribesLinesPopupMenuItem
		lines_menuitem = ScribesLinesPopupMenuItem(self)
		menu.insert(lines_menuitem, 8)
		from popupbookmarks import ScribesBookmarksPopupMenuItem
		bookmarks_menuitem = ScribesBookmarksPopupMenuItem(self)
		menu.insert(bookmarks_menuitem, 9)
		from popupcase import ScribesCasePopupMenuItem
		case_menuitem = ScribesCasePopupMenuItem(self.editor)
		menu.insert(case_menuitem, 10)
		menu.append(SeparatorMenuItem())
		from popupabout import ScribesAboutPopupMenuItem
		about_menuitem = ScribesAboutPopupMenuItem(self.editor)
		menu.append(about_menuitem)
		menu.show_all()
		return True

	def __view_copy_clipboard_cb(self, textview):
		textbuffer = self.get_buffer()
		feedback = self.editor.feedback
		# Get selection bounds
		selection = textbuffer.get_selection_bounds()

		# Depending on whether or not text is selected and copied, send appropriate
		# feedback back to the user via the statusbar.
		if selection:
			from internationalization import msg0089
			feedback.update_status_message(msg0089, "copy")
		else:
			from internationalization import msg0090
			feedback.update_status_message(msg0090, "fail")
		return False

	def __view_cut_clipboard_cb(self, textview):
		feedback = self.editor.feedback
		if self.editor.is_readonly:
			from internationalization import msg0145
			feedback.update_status_message(msg0145, "fail")
			return False
		textbuffer = self.get_buffer()

		# Get selection bounds
		selection = textbuffer.get_selection_bounds()

		# Depending on whether or not text is selected and copied, send appropriate
		# feedback back to the user via the statusbar.
		if selection:
			from internationalization import msg0091
			feedback.update_status_message(msg0091, "cut")
		else:
			from internationalization import msg0092
			feedback.update_status_message(msg0092, "fail")
		return False

	def __view_paste_clipboard_cb(self, textview):
		self.is_pasting = True
		feedback = self.editor.feedback
		if self.editor.is_readonly:
			from internationalization import msg0145
			feedback.update_status_message(msg0145, "fail")
			return False

		# Check to see if there is text in the clipboard manager.
		from gtk import clipboard_get
		clipboard = clipboard_get()
		selection = clipboard.wait_for_text()

		# Depending on whether or not there is text in the clipboard manager, send
		# appropriate feedback back to the user via the statusbar.
		if selection:
			from internationalization import msg0093
			feedback.update_status_message(msg0093, "paste")
		else:
			from internationalization import msg0094
			feedback.update_status_message(msg0094, "fail")
		return False

	def __view_paste_clipboard_after_cb(self, textview):
		self.is_pasting = False
		return False

################################################################################
#
#						GConf Notification Handlers
#
################################################################################

	def __font_changed_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when the font of the text editor changes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		from pango import FontDescription
		new_font = FontDescription(entry.value.to_string())
		self.modify_font(new_font)
		self.__refresh_view()
		return

	def __tab_width_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when the tab width of the text editor changes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		self.set_tabs_width(int(entry.value.to_string()))
		self.__refresh_view()
		return

	def __text_wrapping_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when the text wrapping mode of the text editor changes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		from gtk import WRAP_NONE, WRAP_WORD
		if entry.value.get_bool():
			self.set_wrap_mode(WRAP_WORD)
		else:
			self.set_wrap_mode(WRAP_NONE)
		self.__refresh_view()
		return

	def __margin_position_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when the right margin position of the text editor
		changes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		self.set_margin(int(entry.value.to_string()))
		self.__refresh_view()
		return

	def __show_margin_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when the right margin of the text editor is hidden or
		displayed.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		self.set_show_margin(entry.value.get_bool())
		self.__refresh_view()
		return

	def __spell_check_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when spell checking is toggled on or off in the text
		editor.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.

		"""
		if entry.value.get_bool():
			from gobject import GError
			try:
				from gtkspell import Spell
				from locale import getdefaultlocale
				self.spell_checker = Spell(self, getdefaultlocale()[0])
			except GError:
				pass
		else:
			self.spell_checker.detach()
		self.__refresh_view()
		return

	def __themes_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when of the text editor's view theme changes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		# Set the foreground and background color of the text editor's buffer.
		if entry.value.get_bool():
			# Use foreground and background colors specified by the GTK+ theme.
			from gtk import RcStyle
			style = RcStyle()
			self.modify_style(style)
			#  Reset font to value in GConf
			value = client.get_string("/apps/scribes/font")
			from pango import FontDescription
			font = FontDescription(value)
			self.modify_font(font)
		else:
			# Use foreground and background colors specified by the user stored in
			# GConf, the GNOME configuration database.
			color = client.get_string("/apps/scribes/fgcolor")
			from gtk.gdk import color_parse
			foreground_color = color_parse(color)
			color = client.get_string("/apps/scribes/bgcolor")
			background_color = color_parse(color)
			from gtk import STATE_NORMAL
			self.modify_base(STATE_NORMAL, background_color)
			self.modify_text(STATE_NORMAL, foreground_color)
		self.__refresh_view()
		return

	def __foreground_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when the color text within the text editor changes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		color = client.get_string("/apps/scribes/fgcolor")
		from gtk.gdk import color_parse
		foreground_color = color_parse(color)
		from gtk import STATE_NORMAL
		self.modify_text(STATE_NORMAL, foreground_color)
		self.__refresh_view()
		return

	def __background_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback when the background color of the text editor changes.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		color = client.get_string("/apps/scribes/bgcolor")
		from gtk.gdk import color_parse
		background_color = color_parse(color)
		from gtk import STATE_NORMAL
		self.modify_base(STATE_NORMAL, background_color)
		self.__refresh_view()
		return

	def __use_tabs_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback #FIXME: yeap

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		use_tabs = client.get_bool("/apps/scribes/use_tabs")
		self.set_insert_spaces_instead_of_tabs(not use_tabs)
		self.__refresh_view()
		return

	def __syntax_cb(self, client, cnxn_id, entry, data):
		"""
		Handles callback #FIXME: yeap

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.

		@param client: A client used to query the GConf daemon and database
		@type client: A gconf.Client object.

		@param cnxn_id: The identification number for the GConf client.
		@type cnxn_id: An Integer object.

		@param entry: An entry from the GConf database.
		@type entry: A gconf.Entry object.

		@param data: Optional data
		@type data: Any type object.
		"""
		from syntax import activate_syntax_highlight
		from utils import get_language
		language = None
		if self.editor.uri:
			language = get_language(str(self.editor.uri))
		activate_syntax_highlight(self.get_buffer(), language)
		return

	def __refresh_view(self):
		"""
		Redraw elements of the view and its children.

		@param self: Reference to the ScribesTextView instance.
		@type self: A ScribesTextView object.
		"""
		self.queue_resize()
		self.queue_draw()
		self.resize_children()
		self.grab_focus()
		try:
			self.window.process_updates(True)
		except:
			pass
		return

	def __refresh(self):
		self.queue_resize()
		self.queue_draw()
		self.resize_children()
		try:
			self.window.process_updates(True)
		except:
			pass
		return
