# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module implements a class responsible for creating a toolbar object for
text editor instances.

GTKUIManager is not used because previous attempts have proven that it is
inflexible for the project's purposes.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import Toolbar

class ScribesToolbar(Toolbar):
	"""
	This class creates the toolbar object for text editor instances. The toolbar
	object houses toolitems, or toolbar buttons, that allow users to perform
	actions such as launching a new instance of the text editor, opening a
	document, or renaming a document, printing the contents of the text editor
	buffer and so on. It inherits from gtk.Toolbar.
	"""

	def __init__(self, editor):
	 	"""
	 	Initialize a toolbar object for the text editor and populate it with
	 	toolitems.

	 	@param self: Reference to the toolbar instance.
	 	@type self: A ScribesToolbar object.
	 	"""
	 	from utils import response
	 	response()
		Toolbar.__init__(self)
		self.__set_properties()
		self.__populate_toolbar(editor)
		self.resize_children()
		self.show_all()

	def __set_properties(self):
		"""
		Setup the default properties of the toolbar for the text editor.

		@param self: Reference to the ScribesToolbar instance.
		@type self: A ScribesToolbar object.
		"""
		from gtk import ORIENTATION_HORIZONTAL, TOOLBAR_ICONS, RESIZE_PARENT
		self.set_property("orientation", ORIENTATION_HORIZONTAL)
		self.set_property("toolbar-style", TOOLBAR_ICONS)
		self.set_property("show-arrow", True)
		self.set_reallocate_redraws(False)
		self.set_property("resize-mode", RESIZE_PARENT)
		self.set_redraw_on_allocate(False)
		return False

	def __populate_toolbar(self, editor):
		"""
		Populate the text editor's toolbar with toolitems.

		@param self: Reference to the ScribesToolbar instance.
		@type self: A ScribesToolbar object.
		"""
		from toolbuttons import NewToolButton, OpenToolButton, SaveToolButton
		from toolbuttons import PrintToolButton, UndoToolButton
		from toolbuttons import RedoToolButton, GotoToolButton, SearchToolButton
		from toolbuttons import ReplaceToolButton, PrefToolButton
		from toolbuttons import HelpToolButton, Spinner
		# Create the new file toolbutton.
		editor.newtoolbutton = NewToolButton(editor)
		self.insert(editor.newtoolbutton, 0)

		# Create the open file toolbutton.
		editor.opentoolbutton = OpenToolButton(editor)
		self.insert(editor.opentoolbutton, 1)

		# Create the save file toolbutton.
		editor.savetoolbutton = SaveToolButton(editor)
		self.insert(editor.savetoolbutton, 2)

		# Create a toolbar separator.
		from gtk import SeparatorToolItem
		separator = SeparatorToolItem()
		separator.set_draw(True)
		self.insert(separator, 3)

		# Create the print toolbutton.
		editor.printtoolbutton = PrintToolButton(editor)
		self.insert(editor.printtoolbutton, 4)

		# Create a toolbar separator.
		separator = SeparatorToolItem()
		separator.set_draw(True)
		self.insert(separator, 5)

		# Create the undo toolbutton.
		editor.undotoolbutton = UndoToolButton(editor)
		self.insert(editor.undotoolbutton, 6)

		# Create the redo toolbutton.
		editor.redotoolbutton = RedoToolButton(editor)
		self.insert(editor.redotoolbutton, 7)

		# Create a separator.
		separator = SeparatorToolItem()
		separator.set_draw(True)
		self.insert(separator, 8)

		# Create the goto line toolbutton.
		editor.gototoolbutton = GotoToolButton(editor)
		self.insert(editor.gototoolbutton, 9)

		# Create the find toolbutton.
		editor.searchtoolbutton = SearchToolButton(editor)
		self.insert(editor.searchtoolbutton, 10)

		# Create the replace toolbutton.
		editor.replacetoolbutton = ReplaceToolButton(editor)
		self.insert(editor.replacetoolbutton, 11)

		# Create a separator.
		separator = SeparatorToolItem()
		separator.set_draw(True)
		self.insert(separator, 12)

		# Create the preferences menu toolbutton.
		editor.preftoolbutton = PrefToolButton(editor)
		self.insert(editor.preftoolbutton, 13)

		# Create the help toolbutton.
		editor.helptoolbutton = HelpToolButton(editor)
		self.insert(editor.helptoolbutton, 14)

		# Create a separator.
		separator = SeparatorToolItem()
		separator.set_expand(True)
		separator.set_draw(False)
		self.insert(separator, 15)

		# Create the Spinner.
		editor.spinner = Spinner()
		self.insert(editor.spinner, 16)
		return False
