# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module exposes a class that creates the toolbar container for the text
editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import HBox

class ScribesToolbarContainer(HBox):
	"""
	This class creates the toolbar container object for the text editor. The
	toolbar container houses the text editor's toolbar object.
	"""

	def __init__(self, editor):
	 	"""
	 	Initialize the text editor's toolbar container and set its default
	 	state and properties.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
	 	"""
		from utils import response
		response()
	 	HBox.__init__(self)
	 	self.__init_attributes(editor)
		self.__set_properties()
		self.editor.connect("show-dialog", self.__toolbarcontainer_show_dialog_cb)
		self.editor.connect("hide-dialog", self.__toolbarcontainer_hide_dialog_cb)
		self.editor.connect("show-bar", self.__toolbarcontainer_show_bar_cb)
		self.editor.connect("hide-bar", self.__toolbarcontainer_hide_bar_cb) 
		self.editor.connect("enable-fullscreen", self.__toolbarcontainer_enable_fullscreen_cb)
		self.editor.connect("disable-fullscreen", self.__toolbarcontainer_disable_fullscreen_cb)
		self.resize_children()

	def __init_attributes(self, editor):
	 	"""
	 	Initialize the text editor's toolbar container and set its default
	 	state and properties.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
	 	"""
		self.editor = editor
		# Initialize the fullscreen button.
		self.fsbutton = None
		self.timer = None
		self.handler_id = None
		return

	def __set_properties(self):
		"""
		Set the toolbar container's properties.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.
		"""
		from gtk import RESIZE_PARENT
		self.set_property("resize-mode", RESIZE_PARENT)
		self.set_property("name", "scribestoolbarcontainer")
		return

	def __toolbarcontainer_show_dialog_cb(self, editor, dialog):
	 	"""
	 	Handles callback when the "show-dialog" signal is emitted.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
	 	"""
		self.set_property("sensitive", False)
		return

	def __toolbarcontainer_hide_dialog_cb(self, editor, dialog):
	 	"""
	 	Handles callback when the "hide-dialog" signal is emitted.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
	 	"""
		self.set_property("sensitive", True)
		return

	def __toolbarcontainer_show_bar_cb(self, editor, bar): 
		self.set_property("sensitive", False)  
		return  
		
	def __toolbarcontainer_hide_bar_cb(self, editor, bar): 
		self.set_property("sensitive", True)  
		return  

	def __toolbarcontainer_enable_fullscreen_cb(self, editor):
		"""
		Handles callback when the "enable-fullscreen" signal is emitted.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from gobject import timeout_add
		self.timer = timeout_add(5000, self.__hide_toolbarcontainer)
		try:
			self.fsbutton.show_all()
			self.pack_end(self.fsbutton, False, False, 0)
		except:
			from fsbutton import ScribesFullscreenButton
			self.fsbutton = ScribesFullscreenButton(self.editor)
			self.fsbutton.show_all()
			self.pack_end(self.fsbutton, False, False, 0)
		self.queue_resize()
		self.resize_children()
		self.handler_id = self.editor.window.connect("motion-notify-event",
								self.__toolbarcontainer_motion_notify_event_cb)
		return

	def __toolbarcontainer_disable_fullscreen_cb(self, editor):
		"""
		Handles callback when the "disable-fullscreen" signal is emitted.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.editor.window.disconnect(self.handler_id)
		from gobject import source_remove
		source_remove(self.timer)
		self.fsbutton.hide_all()
		self.remove(self.fsbutton)
		self.show_all()
		self.queue_resize()
		self.resize_children()
		return

	def __toolbarcontainer_motion_notify_event_cb(self, window, event):
		"""
		Handles callback when the "motion-notify-event" signal is emitted.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.

		@param window: The window for the text editor.
		@type window: A ScribesWindow object.

		@param event: An event that occurs when the mouse moves.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		window.window.get_pointer()
		self.show_all()
		from gobject import source_remove
		source_remove(self.timer)
		from gobject import timeout_add
		self.timer = timeout_add(5000, self.__hide_toolbarcontainer)
		return False

	def __hide_toolbarcontainer(self):
		"""
		Hide the toolbar container.

		@param self: Reference to the ScribesToolbarContainer instance.
		@type self: A ScribesToolbarContainer object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		if self.editor.window.is_fullscreen:
			self.hide_all()
		return False
