# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module implements a class responsible for creating a window object for
text editor instances.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import Window

class ScribesWindow(Window):
	"""
	This class creates the window object for text editor instances. It defines
	the state, properties and behavior of window objects. The window object
	allows users to minimize, maximize, iconify, shade, move and close instances
	of the text editor. It also provides the base, or root, container for all
	the text editor's graphic user interface components. The class inherits from
	gtk.Window. See the PyGTK, or GTK+, manual for detailed information on the
	gtk.Window class.
	"""

	def __init__(self, editor):
		"""
		Initialize the text editor's window object.

		This function sets the text editor's window object's properties and
		connects the window to GTK+ signals and events.

		@param self: Reference to the text editor's window instance.
		@type self: A ScribesWindow object.
		"""
		from gtk import window_set_auto_startup_notification
		window_set_auto_startup_notification(True)
		from utils import response
		response()
		Window.__init__(self)
		self.__init_attributes(editor)
		self.__set_properties()
		self.__signal_id_1 = self.connect("delete-event", self.__window_delete_event_cb)
		self.__signal_id_2 = self.connect("map-event", self.__window_map_event_cb, editor)
		self.__signal_id_3 = self.connect("window-state-event", self.__window_state_event_cb)
		self.focus_out_id = self.connect("focus-out-event", self.__window_focus_out_event_cb)
		self.__signal_id_4 = self.editor.connect("gui-created", self.__window_gui_created_cb)
		self.__signal_id_5 = self.editor.connect("loading-document", self.__window_loading_document_cb)
		self.__signal_id_6 = self.editor.connect("loaded-document", self.__window_loaded_document_cb)
		self.__signal_id_7 = self.editor.connect("load-error", self.__window_load_error_cb)
		self.__signal_id_8 = self.editor.connect("enable-readonly", self.__window_enable_readonly_cb)
		self.__signal_id_9 = self.editor.connect("disable-readonly", self.__window_disable_readonly_cb)
		self.__signal_id_10 = self.editor.connect("saved-document", self.__window_saved_document_cb)
		self.__signal_id_11 = self.editor.connect("close-document", self.__window_close_document_cb)
		self.__signal_id_12 = self.editor.connect("enable-fullscreen", self.__window_enable_fullscreen_cb)
		self.__signal_id_13 = self.editor.connect("disable-fullscreen", self.__window_disable_fullscreen_cb)
		self.__signal_id_14 = self.editor.connect("show-bar", self.__window_show_bar_cb)
		self.__signal_id_15 = self.editor.connect("hide-bar", self.__window_hide_bar_cb)
		self.__signal_id_16 = self.connect("key-press-event", self.__window_key_press_event_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the text editor's window attributes.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.
		"""
		self.editor = editor
		self.editor.window = self
		self.bar_is_visible = False
		self.bar = None
		self.is_fullscreen = False
		# The flag is used to indicate whether or not the window is been mapped
		# to screen for the first time. The flag is True if it is been mapped
		# to screen for the first time, and False otherwise.
		self.is_first_time = True
		# True if the window has been drawn to screen, mapped.
		self.is_mapped = False
		# True if the window is maximized.
		self.is_maximized = False
		# Maximize value before hiding the window.
		self.maximize_value = None
		# The width of the window in pixels.
		self.width = None
		# The height of the window in pixels.
		self.height = None
		# The x coordinate, or position, of the window.
		self.xcoordinate = None
		# The y coordinate, or position, of the window.
		self.ycoordinate = None
		self.error_flag = False
		self.__signal_id_1 = None
		self.__signal_id_2 = None
		self.__signal_id_3 = None
		self.__signal_id_4 = None
		self.__signal_id_5 = None
		self.__signal_id_6 = None
		self.__signal_id_7 = None
		self.__signal_id_8 = None
		self.__signal_id_9 = None
		self.__signal_id_10 = None
		self.__signal_id_11 = None
		self.__signal_id_12 = None
		self.__signal_id_13 = None
		self.__signal_id_14 = None
		self.__signal_id_15 = None
		self.__signal_id_16 = None
		# Register a unique number with the editor's termination queue
		self.termination_id = self.editor.register_termination_id()
		return

	def __set_properties(self):
		"""
		Set the default text editor's window properties.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.
		"""
		from utils import calculate_resolution_independence
		width, height = calculate_resolution_independence(self, 1.462857143,
															1.536)
		from internationalization import msg0025
		self.set_property("title", msg0025)
		self.set_property("icon-name", "text-editor")
		self.set_property("default-height", height)
		self.set_property("default-width", width)
		self.set_property("role", "main_window")
		self.set_property("urgency-hint", True)
		self.set_property("border-width", 1)
		self.set_property("name", "Text Editor Window")
		self.set_focus_on_map(True)
		return

################################################################################
#
#					Signal and Event Callback Handlers
#
################################################################################

	def __window_delete_event_cb(self, widget, event):
		"""
		Handles callback the "delete-event" signal is emitted.

		This function quits the text editor.

		@param self: Reference to the text editor's instance.
		@type self: A ScribesWindow object.

		@param widget: Reference to the text editor's instance.
		@type widget: A gtk.Window object.

		@param event: An event to destroy a window object.
		@type event: A gtk.Event object.

		@param editor: Reference to the text editor instance.
		@type editor: An Editor object.

		@return: True to prevent propagation of the signal to parent widgets.
		@rtype: A Boolean object.
		"""
		self.editor.triggermanager.trigger("close_file")
		return True

	def __window_map_event_cb(self, widget, event, editor):
		"""
		Handles callback when the "map-event" signal is emitted.

		If the text editor's window is displayed for the first time, this
		function binds the text editor's accelerators to the window and resizes
		children widgets.

		@param self: Reference to the text editor's instance.
		@type self: A ScribesWindow object.

		@param widget: Reference to the text editor's instance.
		@type widget: A gtk.Window object.

		@param event: An event that shows the editor window.
		@type event: A gtk.Event object.

		@return: True to prevent propagation of the signal to parent widgets.
		@rtype: A Boolean object.
		"""
		if self.is_first_time:
			self.resize_children()
			self.window.process_updates(True)
			self.is_first_time = False
			self.is_mapped = True
		return True

	def __window_focus_out_event_cb(self, window, event):
		"""
		Handles callback when the "focus-out-event" signal is emitted.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param window: The text editor's window.
		@type window: A ScribesWindow object.

		@param event: An event that occurs when the window looses focus.
		@type event: A gtk.Event object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		# Save a document when the text editor's window loses focus.
		if self.editor.uri and self.editor.saved_document is False and self.editor.is_readonly is False:
				self.editor.triggermanager.trigger("save_file")
		return False

	def __window_state_event_cb(self, window, event):
		"""
		Handles callback when the "window-state-event" is emitted.

		This function keeps track of the state of the window.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param window: Reference to the text editor window.
		@type window: A gtk.Window object.

		@param event: An event that shows the editor window.
		@type event: A gtk.Event object.

		@return: True to prevent propagation of the signal to parent widgets.
		@rtype: A Boolean object.
		"""
		self.queue_resize()
		self.resize_children()
		from gtk.gdk import WINDOW_STATE_MAXIMIZED, WINDOW_STATE_FULLSCREEN
		if event.new_window_state in (WINDOW_STATE_MAXIMIZED, WINDOW_STATE_FULLSCREEN):
			self.is_maximized = True
		else:
			self.is_maximized = False
		return True

	def __window_loading_document_cb(self, editor):
		"""
		Handles callback when the text editor is in the process of loading a
		document.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		# Set the titlebar to show the file is currently being loaded.
		from internationalization import msg0335
		title = self.editor.uri.short_name
		self.set_title(msg0335 % title)
		if self.is_mapped and self.editor.is_loading_document:
			# Recall the previous state of the document's window when it was
			# last closed.
			from gobject import timeout_add
			timeout_add(10, self.__show_window_while_loading)
		return

	def __window_loaded_document_cb(self, editor):
		"""
		Handles callback when the text editor has finished loading a document.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		# Set the titlebar to show the name of the loaded file.
		title = self.editor.uri.short_name
		self.set_title(title)
		self.present()
		return

	def __window_load_error_cb(self, editor):
		"""
		Handles callback when the "load-error" signal is emitted.

		This function is called when an error occurs while loading a document.
		The function quits the text editor, if the an error occurs before the
		text editor window is displayed. This is done to prevent the text
		editor's process from running indefinitely.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		from internationalization import msg0025
		self.error_flag = True
		self.set_title(msg0025)
		if self.is_mapped is False:
			# If an error occurs before the text editor is displayed, quit the
			# application.
			from gtk import main_quit
			main_quit()
		return

	def __window_enable_readonly_cb(self, editor):
		"""
		Handles callback when the text editor is in readonly mode.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		title = self.editor.uri.short_name
		from internationalization import msg0034
		self.set_title(title + msg0034)
		return

	def __window_disable_readonly_cb(self, editor):
		"""
		Handles callback when the "disable-readonly" signal is emitted.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		title = self.editor.uri.short_name
		self.set_title(title)
		return

	def __window_gui_created_cb(self, editor):
		"""
		Handles callback when the "gui-created" signal is emitted.

		This function displays the text editor on the screen.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: An instance of the text editor.
		@type editor: An Editor object.
		"""
		from gobject import timeout_add
		timeout_add(1, self.__show_window)
		self.editor.textbuffer.connect("changed", self.__window_changed_cb)
		return

	def __window_changed_cb(self, textbuffer):
		"""
		Handles callback when the text editor's buffer "changed" signal is
		emitted.

		This function prepends an asterik to the name of the document in the
		titebar, when text is entered in the text editor's buffer.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param textbuffer: Reference to the text editor's buffer.
		@type textbuffer: A gtksourceview.SourceBuffer object.

		@return: True to propagate signals to parent widgets.
		@rtype: A Boolean object.
		"""
		if self.editor.is_loading_document:
			return True
		if self.editor.uri:
			if self.get_title().startswith("*") is False:
				title = self.editor.uri.short_name
				self.set_title("*" + title)
		return True

	def __window_enable_fullscreen_cb(self, editor):
		"""
		Handles callback when the "enable-fullscreen" signal is emitted.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_fullscreen = True
		self.fullscreen()
		self.resize_children()
		self.queue_resize()
		return

	def __window_disable_fullscreen_cb(self, editor):
		"""
		Handles callback when "disable-fullscreen" signal is emitted.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		self.is_fullscreen = False
		self.unfullscreen()
		self.resize_children()
		self.queue_resize()
		return

	def __window_show_bar_cb(self, editor, bar):
		"""
		Handles callback when the "show-bar" signal is emitted.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: The bar for the text editor.
		@type bar: A ScribesBar object.
		"""
		self.bar = bar
		self.bar_is_visible = True
		return

	def __window_hide_bar_cb(self, editor, bar):
		"""
		Handles callback when the "hide-bar" signal is emitted.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.

		@param bar: The bar for the text editor.
		@type bar: A ScribesBar object.
		"""
		self.bar_is_visible = False
		self.window.process_updates(True)
		return

	def __window_key_press_event_cb(self, window, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		This function hides the text editor's bar.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param window: The text editor's window.
		@type window: A ScribesWindow object.

		@param event: An event that happens when the keyboard keys are pressed.
		@type event: A gtk.Event object.

		@param bar: The bar for the text editor.
		@type bar: A ScribesBar object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		from gtk import keysyms
		if event.keyval == keysyms.Escape and self.bar_is_visible:
			self.bar.hide_bar()
			return True
		if event.keyval == keysyms.Escape and self.editor.uri is None and self.editor.contains_document is False:
			self.editor.triggermanager.trigger("close_file")
		from gtk.gdk import CONTROL_MASK, SHIFT_MASK
		if event.state & CONTROL_MASK and event.state & SHIFT_MASK:
			if event.keyval == keysyms.W and self.editor.uri is None:
				self.hide_all()
				self.editor.emit("close-document-no-save")
		return False

	def __show_window(self):
		"""
		Show the text editor's window.

		When the text editor is loading a document, the window is shown only
		after metadata information about the document's most recent window size
		and position have retrieved from the position database. This information
		is used to calculate the documents window position and size before
		displaying it.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@return: True to call this function again, False otherwise.
		@rtype: A Boolean object.
		"""
		if self.editor.will_load_document:
			if self.editor.uri:
				from gobject import timeout_add, PRIORITY_HIGH
				timeout_add(1, self.__show_window_while_loading, priority=PRIORITY_HIGH)
			else:
				return True
		else:
			if self.editor.load_error is False:
				self.show_all()
		return False

	def __show_window_while_loading(self):
		"""
		Show the text editor's window.

		When the text editor is loading a document, show the window after
		metadata information about the document's most recent window size and
		position has been retrieved.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.
		"""
		try:
			# Get window position from the position database, if possible.
			from position_metadata import get_window_position_from_database
			maximize, width, height, xcoordinate, ycoordinate = \
				get_window_position_from_database(str(self.editor.uri))
			if maximize:
				self.maximize()
			else:
				if self.is_mapped is False:
					self.move(xcoordinate, ycoordinate)
				self.resize(width, height)
		except TypeError:
			pass
		if self.editor.load_error is False:
			self.show_all()
		return False

	def __window_saved_document_cb(self, editor):
		"""
		Handles callback when the contents of the text editor's buffer have
		been saved.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: An instance of the text editor's buffer.
		@type editor: An Editor object.
		"""
		self.set_title(editor.uri.short_name)
		return

	def __window_close_document_cb(self, editor):
		"""
		Record metadata information about the text editor's window and hide it.

		This function gets the window's position and size, records the
		information in a metadata database, and then hides the text editor's
		window.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param editor: An instance of the text editor's buffer.
		@type editor: An Editor object.
		"""
		self.disconnect(self.__signal_id_1)
		self.disconnect(self.__signal_id_2)
		self.disconnect(self.__signal_id_3)
		self.disconnect(self.__signal_id_16)
		self.editor.disconnect(self.__signal_id_4)
		self.editor.disconnect(self.__signal_id_5)
		self.editor.disconnect(self.__signal_id_6)
		self.editor.disconnect(self.__signal_id_7)
		self.editor.disconnect(self.__signal_id_8)
		self.editor.disconnect(self.__signal_id_9)
		self.editor.disconnect(self.__signal_id_10)
		self.editor.disconnect(self.__signal_id_11)
		self.editor.disconnect(self.__signal_id_12)
		self.editor.disconnect(self.__signal_id_13)
		self.editor.disconnect(self.__signal_id_14)
		self.editor.disconnect(self.__signal_id_15)
		self.disconnect(self.focus_out_id)
		# Get the text editor's window size and position.
		self.xcoordinate, self.ycoordinate = self.get_position()
		self.width, self.height = self.get_size()
		self.maximize_value = self.is_maximized
		self.hide_all()
		# Update the metadata database with the size and position of the window.
		from gobject import timeout_add
		timeout_add(10, self.__update_position_metadata, str(editor.uri))
		return

################################################################################
#
#					Metadata Information Management
#
################################################################################

	def __update_position_metadata(self, uri):
		"""
		Update the position database with information about the text editor's
		window position.

		@param self: Reference to the ScribesWindow instance.
		@type self: A ScribesWindow object.

		@param uri: A universal resource identifier representing, or pointing
			to, a text document.
		@type uri: A String object.
		"""
		from position_metadata import update_window_position_in_database
		if self.editor.uri:
			if self.maximize_value:
				window_position = (True, None, None, None, None)
				update_window_position_in_database(uri, window_position)
			else:
				window_position = (False, self.width, self.height, self.xcoordinate,
								self.ycoordinate)
				update_window_position_in_database(uri, window_position)
		self.editor.unregister_termination_id(self.termination_id)
		self.destroy()
		del self.editor, self
		return False

	def __garbage_collector(self):
		"""
		Call the garbage collector every so often.

		@param editor: Reference to the text editor.
		@type editor: An Editor object.
		"""
		from gc import collect
		from utils import response
		response()
		collect()
		return True
