# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that provides word completion functionality
for the text editor.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_OBJECT

class WordCompletion(GObject):
	"""
	This class implements the word completion object for the text editor.
	The completion object compares the first entered into the text
	editor's buffer with a list from an automatically generated
	completion dictionary and then determines if the text entered in the
	buffer has possible completion strings. The completion object then
	generates a list of possible completion strings.
	"""

	__gsignals__ = {
		"matches-found": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"no-matches-found": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, editor):
		"""
		Initialize the word completion list generator.

		@param self: Reference to the WordCompletion instance.
		@type self: A WordCompletion object.

		@param textbuffer: The text editor's buffer
		@type textbuffer: A ScribesTextBuffer object.
		"""
		GObject.__init__(self)
		self.__init_attributes(editor)
		from thread import start_new_thread
		from completionmanager import CompletionManager
		start_new_thread(CompletionManager, (self,))
		self.textbuffer.connect_after("insert-text", self.__completion_insert_text_cb)
		self.editor.textview.connect("undo", self.__completion_undo_cb)
		self.editor.textview.connect("redo", self.__completion_redo_cb)
		self.editor.textview.connect("paste-clipboard", self.__completion_paste_clipboard_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the generator's attributes.

		@param self: Reference to the WordCompletion instance.
		@type self: A WordCompletion object.

		@param textbuffer: The text editor's buffer
		@type textbuffer: A ScribesTextBuffer object.
		"""
		self.textbuffer = editor.textbuffer
		self.editor = editor
		self.completion_list = []
		return

	def __completion_insert_text_cb(self, textbuffer, iterator, text, length):
		"""
		Handles callback when the "insert-text" signal is emitted.

		@param self: Reference to the WordCompletion instance.
		@type self: A WordCompletion object.

		@param textbuffer: The text editor's buffer
		@type textbuffer: A ScribesTextBuffer object.
		"""
		if length > 1:
			self.emit("no-matches-found")
		else:
			self.__run_word_completion_check()
		return False

	def __completion_undo_cb(self, sourceview):
		"""
		Handles callback when the "undo" signal is emitted.

		@param self: Reference to the WordCompletion instance.
		@type self: A WordCompletion object.

		@param sourceview: The text editor's view.
		@type sourceview: A ScribesTextView object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.emit("no-matches-found")
		return True

	def __completion_redo_cb(self, sourceview):
		"""
		Handles callback when the "redo" signal is emitted.

		@param self: Reference to the WordCompletion instance.
		@type self: A WordCompletion object.

		@param sourceview: The text editor's view.
		@type sourceview: A ScribesTextView object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.emit("no-matches-found")
		return True

	def __completion_paste_clipboard_cb(self, textview):
		"""
		Handles callback when the "paste-clipboard" signal is emitted.

		@param self: Reference to the WordCompletion instance.
		@type self: A WordCompletion object.

		@param sourceview: The text editor's view.
		@type sourceview: A ScribesTextView object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.emit("no-matches-found")
		return True

	def __run_word_completion_check(self):
		"""
		Search the text buffer for potential word completion matches.

		@param self: Reference to the WordCompletion instance.
		@type self: A WordCompletion object.
		"""
		from cursor import get_word_before_cursor
		word = get_word_before_cursor(self.textbuffer)
		if word is None:
			self.emit("no-matches-found")
			return
		dictionary = self.textbuffer.word_completion_dictionary
		found_matches = self.__search_for_matches(word, dictionary)
		if found_matches:
			self.completion_list = found_matches
			self.emit("matches-found")
		else:
			self.emit("no-matches-found")
		return

	def __search_for_matches(self, word, dictionary):
		"""
		Search the contents of the text editor's buffer for potential word
		completion matches.

		@param word: A string after a space character preceding the cursor.
		@type word: A string object.

		@param unicode_string: The contents of the text editor's buffer in unicode.
		@type unicode_string: A unicode string object.

		@return: A list containing potential words for word completion.
		@rtype: A list object.
		"""
		match_list = [list(items) for items in dictionary.items() \
				if items[0].startswith(word) and items[0] != word]
		if match_list:
			wank_list = []
			from utils import response
			for items in match_list:
				items.reverse()
				wank_list.append(items)
			wank_list.sort()
			wank_list.reverse()
			word_completion_list = [items[1] for items in wank_list]
		else:
			word_completion_list = None
		return word_completion_list
